@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/calma/CalmaRdcl.c,v 1.5 2010/06/25 13:59:51 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/calma/CalmaRdcl.c,v 1.5 2010/06/25 13:59:24 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -96,6 +96,7 @@ calmaSetPosition(sname)
     int nbytes, rtype;
     char *strname = NULL;
     int strRecSize = 0;
+    bool found = FALSE;
 
     originalPos = ftello(calmaInputFile);
 
@@ -104,13 +105,14 @@ calmaSetPosition(sname)
         do
         {
              READRH(nbytes, rtype);      /* Read header	*/
-             if (nbytes <= 0) goto complete;
+             if (nbytes <= 0) break;
 
              /* Skip no of bytes in record header until
               * we reach to next structure record.
               */
              fseek(calmaInputFile, nbytes - CALMAHEADERLENGTH, SEEK_CUR);
         } while (rtype != CALMA_BGNSTR);
+        if (nbytes <= 0) break;
 
         calmaReadStringRecord(CALMA_STRNAME, &strname);
         if ((strcmp(sname, strname)) == 0)
@@ -123,13 +125,26 @@ calmaSetPosition(sname)
              if (strRecSize & 01) strRecSize++;
              fseek(calmaInputFile, -(nbytes + strRecSize + CALMAHEADERLENGTH),
 				SEEK_CUR);
-             break;
+	     freeMagic(strname);
+	     return originalPos;
         }
+	freeMagic(strname);
      }
 
-complete:
-     /* Done with strname */
-     if (strname != NULL) freeMagic(strname);
+     // Ran out of file.  It's possible that we were seeking ahead to a
+     // definition that called something that was defined between it and
+     // our previous position, so we will rewind the file and try again.
+     // If that doesn't work, then the cell is not defined in the file.
+
+     if (originalPos != 0)
+     {
+	rewind(calmaInputFile);
+	calmaSetPosition(sname);
+	return originalPos;
+     }
+
+     calmaReadError("Cell \"%s\" is used but not defined in this file.\n", sname);
+
      return originalPos;
 }
 
@@ -341,15 +356,8 @@ calmaParseStructure(filename)
 	if (nsrefs > osrefs && (nsrefs % 100) == 0)
 	    TxPrintf("    %d uses\n", nsrefs);
 	osrefs = nsrefs;
-#ifdef NONMANHATTAN
 	calmaNonManhattan = 0;
     }
-#else
-    }
-    if (calmaNonManhattan)
-	TxPrintf("    %d non-Manhattan paths converted to stair-steps.\n",
-		calmaNonManhattan);
-#endif
 
     /* Make sure it ends with an ENDSTR record */
     if (!calmaSkipExact(CALMA_ENDSTR)) goto syntaxerror;
@@ -362,12 +370,12 @@ calmaParseStructure(filename)
 
 	char *fpcopy = (char *)mallocMagic(20);
 	char *fncopy = StrDup(NULL, filename);
-	sprintf(fpcopy, "%d", filepos);
+	sprintf(fpcopy, "%"DLONG_PREFIX"d", (dlong) filepos);
 	DBPropPut(cifReadCellDef, "GDS_START", (ClientData)fpcopy);
 
 	fpcopy = (char *)mallocMagic(20);
 	filepos = ftello(calmaInputFile);
-	sprintf(fpcopy, "%d", filepos);
+	sprintf(fpcopy, "%"DLONG_PREFIX"d", (dlong) filepos);
 	DBPropPut(cifReadCellDef, "GDS_END", (ClientData)fpcopy);
 
 	DBPropPut(cifReadCellDef, "GDS_FILE", (ClientData)fncopy);
@@ -403,10 +411,11 @@ calmaParseStructure(filename)
     DBReComputeBbox(cifReadCellDef);
 
     /* Don't bother to register with DRC if we're going to delete the	*/
-    /* cell, or if the cell is read-only.				*/
+    /* cell, or if the cell is read-only, or if "calma drcnocheck true"	*/
+    /* has been issued.							*/
 
     if (!CalmaFlattenUses || (npaths >= 10) || (nsrefs != 0))
-	if (!CalmaReadOnly)
+	if (!CalmaReadOnly && !CalmaNoDRCCheck)
 	    DRCCheckThis(cifReadCellDef, TT_CHECKPAINT, &cifReadCellDef->cd_bbox);
 
     DBWAreaChanged(cifReadCellDef, &cifReadCellDef->cd_bbox,
@@ -903,12 +912,8 @@ gdsCopyPaintFunc(tile, gdsCopyRec)
     else
 	TiToRect(tile, &targetRect);
 
-#ifdef NONMANHATTAN
     DBNMPaintPlane(plane, TiGetTypeExact(tile), &targetRect, CIFPaintTable,
 		(PaintUndoInfo *)NULL);
-#else
-    DBPaintPlane(plane, &targetRect, CIFPaintTable, (PaintUndoInfo *)NULL);
-#endif
     
     return 0;
 }
