@@ -18,11 +18,12 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/calma/CalmaRdpt.c,v 1.3 2006/12/13 04:42:49 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/calma/CalmaRdpt.c,v 1.7 2010/08/25 17:33:54 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
-#include <stdlib.h>                 /* for abs() */
+#include <stdlib.h>             /* for abs() */
+#include <string.h>		/* for strlen() */
 #include <sys/types.h>
 
 #include <netinet/in.h>
@@ -49,10 +50,8 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/
 
 extern int calmaNonManhattan;
 
-#ifdef NONMANHATTAN
 extern int CalmaPolygonCount;
 extern HashTable calmaDefInitHash;
-#endif
 
 extern void calmaLayerError();
 bool calmaReadPath();
@@ -207,9 +206,7 @@ calmaElementBoundary()
     LinkedRect *rp;
     Plane *plane;
     CellUse *use;
-#ifdef NONMANHATTAN
     CellDef *savedef, *newdef = NULL;
-#endif
 
     /* Skip CALMA_ELFLAGS, CALMA_PLEX */
     calmaSkipSet(calmaElementIgnore);
@@ -246,7 +243,6 @@ calmaElementBoundary()
 
     /* Convert the polygon to rectangles. */
 
-#ifdef NONMANHATTAN
     if (CalmaSubcellPolygons && (calmaNonManhattan > 0))
     {
 	/* Place the polygon in its own subcell */
@@ -276,19 +272,8 @@ calmaElementBoundary()
     }
 
     rp = CIFPolyToRects(pathheadp, plane, CIFPaintTable, (PaintUndoInfo *)NULL);
-#else
-    rp = CIFPolyToRects(pathheadp);
-#endif
     CIFFreePath(pathheadp);
 
-#ifndef NONMANHATTAN
-    /* The non-Manhattan geometry polygon parsing algorithm */
-    /* typically leaves behind degenerate paths, so they    */
-    /* should not be considered erroneous.                  */
-    if (rp == NULL)
-	calmaReadError("Can't convert boundary/box into rects; ignored.\n");
-#endif
-
     /* Paint the rectangles (if any) */
     for (; rp != NULL ; rp = rp->r_next)
     {
@@ -297,7 +282,6 @@ calmaElementBoundary()
 	freeMagic((char *) rp);
     }
 
-#ifdef NONMANHATTAN
     if (cifCurReadPlanes == cifEditCellPlanes)
     {
 	CIFPaintCurrent();
@@ -315,7 +299,6 @@ calmaElementBoundary()
 	DBSetTrans(use, &GeoIdentityTransform);
 	DBPlaceCell(use, cifReadCellDef);
     }
-#endif
 }
 
 /*
@@ -444,10 +427,8 @@ calmaElementPath()
     Rect segment;
     Plane *plane;
     int first,last;
-#ifdef NONMANHATTAN
     CellUse *use;
     CellDef *savedef, *newdef = NULL;
-#endif
 
     /* Skip CALMA_ELFLAGS, CALMA_PLEX */
     calmaSkipSet(calmaElementIgnore);
@@ -587,7 +568,6 @@ calmaElementPath()
     {
 	plane = cifCurReadPlanes[ciftype];
 
-#ifdef NONMANHATTAN
 	if (CalmaSubcellPolygons && (calmaNonManhattan > 0))
 	{
 	    /* Place the polygon in its own subcell */
@@ -615,13 +595,11 @@ calmaElementPath()
 		    plane = cifCurReadPlanes[ciftype];
 	    }
 	}
-#endif /* NONMANHATTAN */
 
 	CIFPaintWirePath(pathheadp, width,
 		(pathtype == CALMAPATH_SQUAREFLUSH) ? FALSE : TRUE,
 		plane, CIFPaintTable, (PaintUndoInfo *)NULL);
 
-#ifdef NONMANHATTAN
 	if (cifCurReadPlanes == cifEditCellPlanes)
 	{
 	    CIFPaintCurrent();
@@ -639,7 +617,6 @@ calmaElementPath()
 	    DBSetTrans(use, &GeoIdentityTransform);
 	    DBPlaceCell(use, cifReadCellDef);
 	}
-#endif /* NONMANHATTAN */
     }
 }
 
@@ -649,14 +626,12 @@ calmaElementPath()
  * calmaElementText --
  *
  * Read labels.
- * CURRENTLY UNIMPLEMENTED.
  *
  * Results:
  *	None.
  *
  * Side effects:
- *	Currently, none.
- *	Eventually, may add labels to our label list.
+ *	Add labels to our label list.
  *
  * ----------------------------------------------------------------------------
  */
@@ -664,13 +639,15 @@ calmaElementPath()
 void
 calmaElementText()
 {
-    static int ignore[] = { CALMA_PRESENTATION, CALMA_PATHTYPE, CALMA_WIDTH,
-			    CALMA_STRANS, CALMA_MAG, CALMA_ANGLE, -1 };
+    static int ignore[] = { CALMA_PATHTYPE, CALMA_WIDTH, -1 };
     char *textbody = NULL;
     int nbytes, rtype;
     int layer, textt, cifnum;
     TileType type;
     Rect r;
+    unsigned short textpres;
+    double dval;
+    int size, angle, font, pos;
 
     /* Skip CALMA_ELFLAGS, CALMA_PLEX */
     calmaSkipSet(calmaElementIgnore);
@@ -690,8 +667,87 @@ calmaElementText()
     }
     else type = cifCurReadStyle->crs_labelLayer[cifnum];
 
-    /* Skip presentation, pathtype, width, and transform */
-    calmaSkipSet(ignore);
+    font = -1;
+    angle = 0;
+
+    /* Default size is 1um */
+    size = (int)((800 * cifCurReadStyle->crs_multiplier)
+				/ cifCurReadStyle->crs_scaleFactor);
+    /* Default position is bottom-right (but what the spec calls "top-left"!) */
+    pos = GEO_SOUTHEAST;
+
+    /* Parse presentation and magnitude/angle part of transform */
+    /* Skip pathtype and width */
+
+    PEEKRH(nbytes, rtype);
+    if (nbytes > 0 && rtype == CALMA_PRESENTATION)
+    {
+	calmaReadI2Record(CALMA_PRESENTATION, &textpres);
+	font = (textpres >> 4) & 0x03;
+	switch (textpres & 0x000f)
+	{
+	    case 0x000a:
+		pos = GEO_NORTHWEST;
+		break;
+	    case 0x0006:
+		pos = GEO_WEST;
+		break;
+	    case 0x0002:
+		pos = GEO_SOUTHWEST;
+		break;
+	    case 0x0009:
+		pos = GEO_NORTH;
+		break;
+	    case 0x0005:
+		pos = GEO_CENTER;
+		break;
+	    case 0x0001:
+		pos = GEO_SOUTH;
+		break;
+	    case 0x0008:
+		pos = GEO_NORTHEAST;
+		break;
+	    case 0x0004:
+		pos = GEO_EAST;
+		break;
+	    case 0x0000:
+		pos = GEO_SOUTHEAST;
+		break;
+	}
+    }
+    else if (nbytes > 0 && rtype != CALMA_STRANS)
+	calmaSkipSet(ignore);
+
+    READRH(nbytes, rtype);
+    if (nbytes > 0 && rtype == CALMA_STRANS)
+    {
+	/* We don't handle the strans record for text */
+	calmaSkipBytes(nbytes - CALMAHEADERLENGTH);
+
+	READRH(nbytes, rtype);
+	if (nbytes > 0 && rtype == CALMA_MAG)
+	{
+	    calmaReadR8(&dval);
+	    /* Assume that MAG is the label size in microns	*/
+	    /* "size" is the label size in 8 * (database units) */
+	    size = (int)((dval * 800 * cifCurReadStyle->crs_multiplier)
+				/ cifCurReadStyle->crs_scaleFactor);
+	}
+	else
+	    UNREADRH(nbytes, rtype);
+
+	READRH(nbytes, rtype);
+	if (nbytes > 0 && rtype == CALMA_ANGLE)
+	{
+	    calmaReadR8(&dval);
+	    angle = (int)dval;
+	}
+	else
+	    UNREADRH(nbytes, rtype);
+    }
+    else
+	UNREADRH(nbytes, rtype);
+
 
     /* Coordinates of text */
     READRH(nbytes, rtype)
@@ -724,11 +780,16 @@ calmaElementText()
 	static bool algmsg = FALSE;
 	bool changed = FALSE;
 	char *cp;
+	char *savstring;
 	for (cp = textbody; *cp; cp++)
 	{
 	    if (*cp <= ' ' | *cp > '~') 
 	    {
-		changed = TRUE;
+		if (!changed)
+		{
+		    savstring = StrDup(NULL, textbody);
+		    changed = TRUE;
+		}
 		if (*cp == '\r' && *(cp+1) == '\0')
 		    *cp = '\0';
 		else if (*cp == '\r') 
@@ -740,21 +801,48 @@ calmaElementText()
 	    }
 	}
 	if (changed) {
-	    calmaReadError("Warning:  weird characters fixed in label '%s'\n",
-			textbody);
+	    calmaReadError("Warning:  improper characters fixed in label '%s'\n",
+			savstring);
 	    if (!algmsg) {
 		algmsg = TRUE;
 		calmaReadError("    (algorithm used:  trailing <CR> dropped, "
 				"<CR> and ' ' changed to '_', \n"
 				"    other non-printables changed to '?')\n");
 	    }
+	    calmaReadError("	modified label is '%s'\n", textbody);
+	    freeMagic(savstring);
 	}
     }
 
     /* Place the label */
-    if (type >= 0)
+    if (strlen(textbody) == 0)
     {
-    	 (void) DBPutLabel(cifReadCellDef, &r, -1, textbody, type, 0);
+	calmaReadError("Warning:  Ignoring empty string label at (%d, %d)\n",
+		r.r_ll.p_x * cifCurReadStyle->crs_scaleFactor,
+		r.r_ll.p_y * cifCurReadStyle->crs_scaleFactor);
+    }
+    else if (type < 0)
+    {
+	calmaReadError("Warning:  label \"%s\" at (%d, %d) is on unhandled"
+		" layer:purpose pair %d:%d and will be discarded.\n", textbody,
+		r.r_ll.p_x * cifCurReadStyle->crs_scaleFactor,
+		r.r_ll.p_y * cifCurReadStyle->crs_scaleFactor, layer, textt);
+    }
+    else
+    {
+	int flags;
+
+	if (cifnum >= 0 && (cifCurReadStyle->crs_layers[cifnum]->crl_flags
+			& CIFR_TEXTLABELS))
+	    flags = LABEL_STICKY;
+	else
+	    flags = 0;
+
+	if (font < 0)
+	    DBPutLabel(cifReadCellDef, &r, pos, textbody, type, flags);
+	else
+	    DBPutFontLabel(cifReadCellDef, &r, font, size, angle,
+			&GeoOrigin, pos, textbody, type, flags);
     }
 
     /* done with textbody */
@@ -865,12 +953,6 @@ calmaReadPath(pathheadpp, iscale)
 	    pathtailp = newpathp;
 	}
     }
-
-#ifndef NONMANHATTAN
-    if (nonManhattan && (iscale != 0))
-	CIFMakeManhattanPath(*pathheadpp);
-#endif
-
     return (*pathheadpp != NULL);
 }
 
@@ -907,7 +989,7 @@ calmaLayerError(mesg, layer, dt)
     clt.clt_layer = layer;
     clt.clt_type = dt;
     he = HashFind(&calmaLayerHash, (char *) &clt);
-    if (!HashGetValue(he))
+    if (HashGetValue(he) == NULL)
     {
 	HashSetValue(he, (ClientData) 1);
 	calmaReadError("%s, layer=%d type=%d\n", mesg, layer, dt);
