@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) ="$Header: /usr/cvsroot/magic-7.5/calma/CalmaWrite.c,v 1.12 2010/12/22 16:28:28 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) ="$Header: /usr/cvsroot/magic-8.0/calma/CalmaWrite.c,v 1.8 2010/12/22 16:29:06 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -444,7 +444,7 @@ calmaProcessDef(def, outf)
 	char *buffer, *offptr;
 	size_t defsize, numbytes;
 	off_t cellstart, cellend;
-	dlong cval;
+ 	dlong cval;
 	FILE *fi;
 
 	/* Use PaOpen() so the paths searched are the same as were	*/
@@ -460,13 +460,13 @@ calmaProcessDef(def, outf)
 	    if (!isReadOnly)
 		TxError("Calma output error:  Can't find GDS file for vendor cell."
 	    			"  Using magic's internal definition\n");
-	    else
+	    else 
 		def->cd_flags |= CDVENDORGDS;
 	}
 	else
 	{
 	    offptr = (char *)DBPropGet(def, "GDS_END", NULL);
-	    sscanf(offptr, "%lld", &cval);
+	    sscanf(offptr, "%"DLONG_PREFIX"d", &cval);
 	    cellend = (off_t)cval;
 	    offptr = (char *)DBPropGet(def, "GDS_BEGIN", &oldStyle);
 	    if (!oldStyle)
@@ -485,7 +485,7 @@ calmaProcessDef(def, outf)
 		calmaOutStructName(CALMA_STRNAME, def, outf);
 	    }
 
-	    sscanf(offptr, "%lld", &cval);
+	    sscanf(offptr, "%"DLONG_PREFIX"d", &cval);
 	    cellstart = (off_t)cval;
 	    fseek(fi, cellstart, SEEK_SET);
 
@@ -1702,7 +1702,6 @@ calmaMergePaintFunc(tile, cos)
 	if (t->ti_client != (ClientData)GDS_PENDING) continue;
 	t->ti_client = (ClientData)GDS_PROCESSED;
 
-#ifdef NONMANHATTAN
 	split_type = -1;
 	if (IsSplit(t))
 	{
@@ -1727,7 +1726,6 @@ calmaMergePaintFunc(tile, cos)
 	    }
 	}
 	else
-#endif
 	    num_points = 3;
 
 	/* Create a new linked boundary structure with 4 unknown edges.	*/
@@ -1760,7 +1758,6 @@ calmaMergePaintFunc(tile, cos)
 	/* Initialize the "edge" record with the corner points of the	*/
 	/* tile.							*/
 
-#ifdef NONMANHATTAN
 	if (IsSplit(t))
 	{
 	    switch (split_type)
@@ -1826,7 +1823,6 @@ calmaMergePaintFunc(tile, cos)
 	}
 	else
 	{
-#endif
 	    lb->lb_start.p_x = urx;
 	    lb->lb_start.p_y = ury;
 	    lb->lb_type = LB_INIT;
@@ -1845,23 +1841,18 @@ calmaMergePaintFunc(tile, cos)
 	    lb = lb->lb_next;
 
 	    num_points = 0;
-
-#ifdef NONMANHATTAN
 	}
 	if (split_type == 0x1) goto left_search;
-#endif
 
 	/* Search the tile boundary for connected and unconnected tiles.	*/
 	/* Generate segments in a counterclockwise cycle.			*/
 
-#ifdef NONMANHATTAN
 	if (split_type == 0x2)
 	{
 	    intedges += calmaAddSegment(&lb, TRUE, RIGHT(t), TOP(t),
 			LEFT(t), BOTTOM(t));
 	    goto bottom_search;
 	}
-#endif
 
 	/* Search top */
 
@@ -1875,14 +1866,12 @@ calmaMergePaintFunc(tile, cos)
 	    if (!is_ext) PUSHTILE(tp);
 	}
 
-#ifdef NONMANHATTAN
 	if (split_type == 0x3)
 	{
 	    intedges += calmaAddSegment(&lb, TRUE, LEFT(t), TOP(t),
 			RIGHT(t), BOTTOM(t));
 	    goto right_search;
 	}
-#endif
 
 	/* Search left */
 
@@ -1897,14 +1886,12 @@ left_search:
 	    if (!is_ext) PUSHTILE(tp);
 	}
 
-#ifdef NONMANHATTAN
 	if (split_type == 0x0)
 	{
 	    intedges += calmaAddSegment(&lb, TRUE, LEFT(t), BOTTOM(t),
 			RIGHT(t), TOP(t));
 	    goto done_searches;
 	}
-#endif
 
 	/* Search bottom */
 
@@ -1919,14 +1906,12 @@ bottom_search:
 	    if (!is_ext) PUSHTILE(tp);
 	}
 
-#ifdef NONMANHATTAN
 	if (split_type == 0x1)
 	{
 	    intedges += calmaAddSegment(&lb, TRUE, RIGHT(t), BOTTOM(t),
 			LEFT(t), TOP(t));
 	    goto done_searches;
 	}
-#endif
 	/* Search right */
 
 right_search:
@@ -2039,8 +2024,6 @@ calmaWritePaintFunc(tile, cos)
     calmaOutRH(6, CALMA_DATATYPE, CALMA_I2, f);
     calmaOutI2(calmaPaintLayerType, f);
 
-#ifdef NONMANHATTAN
-
     /* The inefficient use of CALMA_BOUNDARY for rectangles actually	*/
     /* makes it easy to implement triangles, since they must be defined */
     /* by CALMA_BOUNDARY.						*/
@@ -2080,19 +2063,14 @@ calmaWritePaintFunc(tile, cos)
     }
     else
     {
-#endif
-
-    /* Coordinates */
-    calmaOutRH(44, CALMA_XY, CALMA_I4, f);
-    calmaOutI4(r.r_xbot, f); calmaOutI4(r.r_ybot, f);
-    calmaOutI4(r.r_xtop, f); calmaOutI4(r.r_ybot, f);
-    calmaOutI4(r.r_xtop, f); calmaOutI4(r.r_ytop, f);
-    calmaOutI4(r.r_xbot, f); calmaOutI4(r.r_ytop, f);
-    calmaOutI4(r.r_xbot, f); calmaOutI4(r.r_ybot, f);
-
-#ifdef NONMANHATTAN
+	/* Coordinates */
+	calmaOutRH(44, CALMA_XY, CALMA_I4, f);
+	calmaOutI4(r.r_xbot, f); calmaOutI4(r.r_ybot, f);
+	calmaOutI4(r.r_xtop, f); calmaOutI4(r.r_ybot, f);
+	calmaOutI4(r.r_xtop, f); calmaOutI4(r.r_ytop, f);
+	calmaOutI4(r.r_xbot, f); calmaOutI4(r.r_ytop, f);
+	calmaOutI4(r.r_xbot, f); calmaOutI4(r.r_ybot, f);
     }
-#endif
 
     /* End of element */
     calmaOutRH(4, CALMA_ENDEL, CALMA_NODATA, f);
@@ -2146,6 +2124,68 @@ calmaWriteLabelFunc(lab, type, f)
     calmaOutRH(6, CALMA_TEXTTYPE, CALMA_I2, f);
     calmaOutI2(CIFCurStyle->cs_layers[type]->cl_calmatype, f);
 
+    if (lab->lab_font >= 0)
+    {
+	unsigned short textpres = 0;
+
+	/* A bit of a hack here.  Magic can have any number of fonts,	*/
+	/* but GDS only allows four of them.  So we just crop the font	*/
+	/* index to two bits.  We provide no other font information, so	*/
+	/* this is highly implementation-dependent.  But it allows us	*/
+	/* to retain font information when reading and writing our own	*/
+	/* GDS files.							*/
+
+	textpres = (lab->lab_font & 0x03) << 4;
+
+	switch(lab->lab_just)
+	{
+	    case GEO_SOUTH:
+		textpres |= 0x0001;
+		break;
+	    case GEO_SOUTHEAST:
+		textpres |= 0x0000;
+		break;
+	    case GEO_EAST:
+		textpres |= 0x0004;
+		break;
+	    case GEO_NORTHEAST:
+		textpres |= 0x0008;
+		break;
+	    case GEO_NORTH:
+		textpres |= 0x0009;
+		break;
+	    case GEO_NORTHWEST:
+		textpres |= 0x000a;
+		break;
+	    case GEO_WEST:
+		textpres |= 0x0006;
+		break;
+	    case GEO_SOUTHWEST:
+		textpres |= 0x0002;
+		break;
+	    case GEO_CENTER:
+		textpres |= 0x0005;
+		break;
+	}
+
+	calmaOutRH(6, CALMA_PRESENTATION, CALMA_BITARRAY, f);
+	calmaOutI2(textpres, f);
+
+	calmaOutRH(6, CALMA_STRANS, CALMA_BITARRAY, f);
+	calmaOutI2(0, f);	/* Any need for these bits? */
+
+	calmaOutRH(12, CALMA_MAG, CALMA_R8, f);
+	calmaOutR8(((double)lab->lab_size / 800)
+		* (double)CIFCurStyle->cs_scaleFactor
+		/ (double)CIFCurStyle->cs_expander, f);
+
+	if (lab->lab_rotate != 0)
+	{
+	    calmaOutRH(12, CALMA_ANGLE, CALMA_R8, f);
+	    calmaOutR8((double)lab->lab_rotate, f);
+	}
+    }
+
     p.p_x = (lab->lab_rect.r_xbot + lab->lab_rect.r_xtop) * calmaWriteScale / 2;
     p.p_y = (lab->lab_rect.r_ybot + lab->lab_rect.r_ytop) * calmaWriteScale / 2;
     calmaOutRH(12, CALMA_XY, CALMA_I4, f);
