@@ -19,7 +19,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/cif/CIFrdcl.c,v 1.2 2006/04/15 01:04:45 anonymous Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/cif/CIFrdcl.c,v 1.5 2010/08/25 17:33:55 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -35,6 +35,8 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/
 #include "database/database.h"
 #include "cif/CIFint.h"
 #include "cif/CIFread.h"
+#include "cif/cif.h"
+#include "calma/calma.h"
 #include "utils/utils.h"
 #include "windows/windows.h"
 #include "dbwind/dbwind.h"
@@ -502,7 +504,7 @@ CIFParseStart()
 void
 CIFPaintCurrent()
 {
-    Plane *plane;
+    Plane *plane, *swapplane;
     int i;
 
     for (i = 0; i < cifCurReadStyle->crs_nLayers; i += 1)
@@ -516,11 +518,21 @@ CIFPaintCurrent()
 	/* Generate a paint/erase table, then paint from the CIF
 	 * plane into the current Magic cell.
 	 */
-
 	type = cifCurReadStyle->crs_layers[i]->crl_magicType;
 
-	(void) DBSrPaintArea((Tile *) NULL, plane, &TiPlaneRect,
-		&CIFSolidBits, cifPaintCurrentFunc, (ClientData) type);
+	if (cifCurReadStyle->crs_layers[i]->crl_flags & CIFR_TEMPLAYER)
+	{
+	    /* Swap planes */
+	    swapplane = cifCurReadPlanes[type];
+	    cifCurReadPlanes[type] = plane;
+	    plane = swapplane;
+	}
+	else
+	{
+	    DBSrPaintArea((Tile *) NULL, plane, &TiPlaneRect,
+			&CIFSolidBits, cifPaintCurrentFunc,
+			(ClientData)type);
+	}
 	
 	/* Recycle the plane, which was dynamically allocated. */
 
@@ -590,13 +602,8 @@ cifPaintCurrentFunc(tile, type)
     for (pNum = PL_PAINTBASE; pNum < DBNumPlanes; pNum++)
 	if (DBPaintOnPlane(type, pNum))
 	{
-#ifdef NONMANHATTAN
 	    DBNMPaintPlane(cifReadCellDef->cd_planes[pNum], TiGetTypeExact(tile),
 		    &area, DBStdPaintTbl(type, pNum), (PaintUndoInfo *) NULL);
-#else
-	    DBPaintPlane(cifReadCellDef->cd_planes[pNum], &area,
-		    DBStdPaintTbl(type, pNum), (PaintUndoInfo *) NULL);
-#endif
 	}
 
     return  0;		/* To keep the search alive. */
@@ -895,7 +902,7 @@ cifParseUser94()
     Rect rectangle;
     char *name = NULL;
     TileType type;
-    int layer;
+    int layer, flags, i;
     int savescale;
 
     (void) StrDup(&name, cifParseName());
@@ -926,21 +933,36 @@ cifParseUser94()
     CIFSkipBlanks();
     if (PEEK() != ';')
     {
-	char *name = cifParseName();
-	layer = CIFReadNameToType(name, FALSE);
+	char *lname = cifParseName();
+	layer = CIFReadNameToType(lname, FALSE);
 	if (layer < 0)
 	{
 	    CIFReadError("label attached to unknown layer %s.\n",
-		    name);
+		    lname);
 	    type = TT_SPACE;
 	}
-	else type = cifCurReadStyle->crs_labelLayer[layer];
+	else {
+	    type = cifCurReadStyle->crs_labelLayer[layer];
+	}
     } else {
 	type = cifCurLabelType;
+
+	/* Should do this better, by defining cifCurLabelFlags. . . */
+	layer = -1;
+	for (i = 0; i < cifCurReadStyle->crs_nLayers; i++)
+	    if (cifCurReadStyle->crs_labelLayer[i] == type) {
+		layer = i;
+		break;
+	    }
     }
     if (type >=0 )
     {
-    	 (void) DBPutLabel(cifReadCellDef, &rectangle, -1, name, type, 0);
+	if (layer >= 0 && (cifCurReadStyle->crs_layers[layer]->crl_flags
+		& CIFR_TEXTLABELS))
+	    flags = LABEL_STICKY;
+	else
+	    flags = 0;
+    	(void) DBPutLabel(cifReadCellDef, &rectangle, -1, name, type, flags);
     }
     freeMagic(name);
     return TRUE;
@@ -972,7 +994,7 @@ cifParseUser95()
     Point size, center;
     char *name = NULL;
     TileType type;
-    int layer;
+    int layer, i;
     int savescale;
 
     (void) StrDup(&name, cifParseName());
@@ -1038,10 +1060,24 @@ cifParseUser95()
 	}
 	else type = cifCurReadStyle->crs_labelLayer[layer];
     }
-    else type = TT_SPACE;
+    else {
+	type = TT_SPACE;
+	layer = -1;
+	for (i = 0; i < cifCurReadStyle->crs_nLayers; i++)
+	    if (cifCurReadStyle->crs_labelLayer[i] == type) {
+		layer = i;
+		break;
+	    }
+    }
     if (type >=0 )
     {
-    	 (void) DBPutLabel(cifReadCellDef, &rectangle, -1, name, type, 0);
+	int flags;
+	if (layer >= 0 && (cifCurReadStyle->crs_layers[layer]->crl_flags
+		& CIFR_TEXTLABELS))
+	    flags = LABEL_STICKY;
+	else
+	    flags = 0;
+    	(void) DBPutLabel(cifReadCellDef, &rectangle, -1, name, type, flags);
     }
 
     freeMagic(name);
@@ -1118,7 +1154,8 @@ CIFParseUser()
  */
 
 void
-CIFReadCellCleanup()
+CIFReadCellCleanup(type)
+    int type;		 // 0 = CIF, 1 = GDS, because routine is used by both
 {
     HashEntry *h;
     HashSearch hs;
@@ -1127,7 +1164,10 @@ CIFReadCellCleanup()
 
     if (cifSubcellBeingRead)
     {
-	CIFReadError("CIF ended partway through a symbol definition.\n");
+	if (type == 0)
+	    CIFReadError("CIF ended partway through a symbol definition.\n");
+	else
+	    calmaReadError("GDS ended partway through a symbol definition.\n");
 	(void) CIFParseFinish();
     }
 
@@ -1140,12 +1180,18 @@ CIFReadCellCleanup()
 	def = (CellDef *) HashGetValue(h);
 	if (def == NULL)
 	{
-	    CIFReadError("cell table has NULL entry (Magic error).\n");
+	    if (type == 0)
+		CIFReadError("cell table has NULL entry (Magic error).\n");
+	    else
+		calmaReadError("cell table has NULL entry (Magic error).\n");
 	    continue;
 	}
 	if (!(def->cd_flags & CDAVAILABLE))
 	{
-	    CIFReadError("cell %s was used but not defined.\n", def->cd_name);
+	    if (type == 0)
+		CIFReadError("cell %s was used but not defined.\n", def->cd_name);
+	    else
+		calmaReadError("cell %s was used but not defined.\n", def->cd_name);
         }
 	if (def->cd_flags & CDPROCESSEDGDS)
 	{
@@ -1168,25 +1214,43 @@ CIFReadCellCleanup()
 		    TiFreePlane(gdsplanes[pNum]);
 		}
 	    }
-	    freeMagic(def->cd_client);
+	    freeMagic((char *)def->cd_client);
 
 	    if (def->cd_parents != (CellUse *)NULL)
 	    {
 		CellUse *cu_p = def->cd_parents;
-		TxError("GDS read warning:  Cell %s has parent %s\n", savename,
-				cu_p->cu_id);
+		if (type == 0)
+		    CIFReadError("CIF read warning:  Cell %s has parent %s\n",
+				savename, cu_p->cu_id);
+		else
+		    calmaReadError("GDS read warning:  Cell %s has parent %s\n",
+				savename, cu_p->cu_id);
 		def->cd_parents = (CellUse *)NULL;
 	    }
 	    if (DBCellDeleteDef(def) == FALSE)
-		TxError("GDS read error:  Unable to delete cell %s\n", savename);
+	    {
+		if (type == 0)
+		    CIFReadError("CIF read error:  Unable to delete cell %s\n",
+				savename);
+		else
+		    calmaReadError("GDS read error:  Unable to delete cell %s\n",
+				savename);
+	    }
 	    else
-		TxPrintf("GDS read:  Removed flattened cell %s\n", savename);
+	    {
+		if (type == 0)
+		    CIFReadError("CIF read:  Removed flattened cell %s\n", savename);
+		else
+		    calmaReadError("GDS read:  Removed flattened cell %s\n", savename);
+	    }
 	    UndoEnable();
 	    freeMagic(savename);
 	}
 	else
 	{
-	    DRCCheckThis(def, TT_CHECKPAINT, &def->cd_bbox);
+	    if ((type == 0 && CIFNoDRCCheck == FALSE) ||
+			(type == 1 && CalmaNoDRCCheck == FALSE))
+		DRCCheckThis(def, TT_CHECKPAINT, &def->cd_bbox);
 	    DBWAreaChanged(def, &def->cd_bbox, DBW_ALLWINDOWS, &DBAllButSpaceBits);
 	    DBCellSetModified(def, TRUE);
 	}
