@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/cif/CIFrdtech.c,v 1.4 2010/09/15 15:45:09 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/cif/CIFrdtech.c,v 1.4 2010/09/15 15:45:30 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -680,6 +680,51 @@ CIFReadTechLine(sectionName, argc, argv)
 	return TRUE;
     }
 
+    /* Process templayer lines next.  (templayers in cifinput added 5/3/09) */
+
+    if (strcmp(argv[0], "templayer") == 0)
+    {
+	TileType type;
+
+	cifCurReadLayer = NULL;
+	cifCurReadOp = NULL;
+	if (cifCurReadStyle->crs_nLayers == MAXCIFRLAYERS)
+	{
+	    TechError("Can't handle more than %d layers per style.\n",
+		    MAXCIFRLAYERS);
+	    TechError("Your local Magic wizard can increase the table size.\n");
+	    goto errorReturn;
+	}
+        if ((argc != 2) && (argc != 3)) goto wrongNumArgs;
+	type = CIFReadNameToType(argv[1], TRUE);
+	if (type < 0) goto errorReturn;
+
+	cifCurReadLayer = (CIFReadLayer *) mallocMagic(sizeof(CIFReadLayer));
+	cifCurReadStyle->crs_layers[cifCurReadStyle->crs_nLayers]
+		= cifCurReadLayer;
+	cifCurReadStyle->crs_nLayers += 1;
+	cifCurReadLayer->crl_magicType = type;
+	cifCurReadLayer->crl_ops = NULL;
+	cifCurReadLayer->crl_flags = CIFR_TEMPLAYER | CIFR_SIMPLE;
+
+	/* Handle a special case of a list of layer names on the
+	 * layer line.  Turn them into an OR operation.
+	 */
+	
+	if (argc == 3)
+	{
+	    cifCurReadOp = (CIFOp *) mallocMagic(sizeof(CIFOp));
+	    cifCurReadOp->co_opcode = CIFOP_OR;
+	    cifCurReadOp->co_client = (ClientData)NULL;
+	    CIFParseReadLayers(argv[2], &cifCurReadOp->co_cifMask);
+	    TTMaskZero(&cifCurReadOp->co_paintMask);
+	    cifCurReadOp->co_next = NULL;
+	    cifCurReadOp->co_distance = 0;
+	    cifCurReadLayer->crl_ops = cifCurReadOp;
+	}
+	return TRUE;
+    }
+
     /* Process mapping between CIF layers and calma layers/types */
     if ((strcmp(argv[0], "calma") == 0) || (strncmp(argv[0], "gds", 3) == 0))
     {
@@ -725,13 +770,26 @@ CIFReadTechLine(sectionName, argc, argv)
 	    TechError("Must define layer before giving labels it holds.\n");
 	    goto errorReturn;
 	}
-	if (argc != 2) goto wrongNumArgs;
+	if (argc != 2)
+	{
+	    if (argc == 3)
+	    {
+		if (strcmp(argv[2], "text"))
+		    goto wrongNumArgs;
+	    }
+	    else
+		goto wrongNumArgs;
+	}
 	CIFParseReadLayers(argv[1], &mask);
 	for (i=0; i<MAXCIFRLAYERS; i+=1)
 	{
 	    if (TTMaskHasType(&mask,i))
+	    {
 		cifCurReadStyle->crs_labelLayer[i]
 			= cifCurReadLayer->crl_magicType;
+		if (argc == 3)
+		    cifCurReadStyle->crs_layers[i]->crl_flags |= CIFR_TEXTLABELS;
+	    }
 	}
 	return TRUE;
     }
@@ -1178,9 +1236,6 @@ cifParseCalmaNums(str, numArray, numNums)
  *   original value.  This is important if we are still in the process of
  *   reading CIF or GDS input.
  *
- * Return value:
- *   Return the multiplier value on success, or 0 on failure.
- *
  * ----------------------------------------------------------------------------
  */
 
