@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/cif/CIFtech.c,v 1.15 2010/10/20 20:31:16 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/cif/CIFtech.c,v 1.7 2010/10/20 20:34:19 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -99,8 +99,18 @@ cifTechFreeStyle()
 		{
 		    if (op->co_client != (ClientData)NULL)
 		    {
-			if ((op->co_opcode != CIFOP_OR) && (op->co_opcode != CIFOP_BBOX))
-			    freeMagic((char *)op->co_client);
+			switch (op->co_opcode)
+			{
+			    case CIFOP_OR:
+			    case CIFOP_BBOX:
+			    case CIFOP_MAXRECT:
+				/* These options use co_client to hold a single	*/
+				/* integer value, so it is not allocated.	*/
+				break;
+			    default:
+				freeMagic((char *)op->co_client);
+				break;
+			}
 		    }
 		    freeMagic((char *)op);
 		}
@@ -953,6 +963,10 @@ CIFTechLine(sectionName, argc, argv)
 		CIFCurStyle->cs_flags |= CWF_PERMISSIVE_LABELS;
 	    else if (strcmp(argv[i], "grow-euclidean") == 0)
 		CIFCurStyle->cs_flags |= CWF_GROW_EUCLIDEAN;
+	    else if (strcmp(argv[i], "see-vendor") == 0)
+		CIFCurStyle->cs_flags |= CWF_SEE_VENDOR;
+	    else if (strcmp(argv[i], "no-errors") == 0)
+		CIFCurStyle->cs_flags |= CWF_NO_ERRORS;
 	}
 	return TRUE;
     }
@@ -1002,6 +1016,10 @@ CIFTechLine(sectionName, argc, argv)
 	newOp->co_opcode = CIFOP_SLOTS;
     else if (strcmp(argv[0], "bbox") == 0)
 	newOp->co_opcode = CIFOP_BBOX;
+    else if (strcmp(argv[0], "net") == 0)
+	newOp->co_opcode = CIFOP_NET;
+    else if (strcmp(argv[0], "maxrect") == 0)
+	newOp->co_opcode = CIFOP_MAXRECT;
     else
     {
 	TechError("Unknown statement \"%s\".\n", argv[0]);
@@ -1119,6 +1137,26 @@ bloatCheck:
 	    TechError("Not all bloat layers fall in the same plane.\n");
 	    bloatDone: break;
 
+	case CIFOP_NET:
+	    if (argc != 3) goto wrongNumArgs;
+	    newOp->co_client = (ClientData)StrDup((char **)NULL, argv[1]);
+	    cifParseLayers(argv[2], CIFCurStyle, &newOp->co_paintMask,
+		&newOp->co_cifMask, FALSE);
+	    break;
+
+	case CIFOP_MAXRECT:
+	    if (argc == 2)
+	    {
+		if (!strncmp(argv[1], "ext", 3))
+		    newOp->co_client = (ClientData)1;
+		else if (strncmp(argv[1], "int", 3))
+		    TechError("Maxrect takes only one optional argument "
+				"\"external\" or \"internal\" (default).\n");
+	    }
+	    else if (argc != 1)
+		goto wrongNumArgs;
+	    break;
+
 	case CIFOP_BBOX:
 	    if (argc == 2)
 	    {
@@ -1679,17 +1717,28 @@ CIFTechFinal()
 		}
 		/* Presence of op->co_opcode in CIFOP_OR indicates a copy */
 		/* of the SquaresData pointer from a following operator	  */
-		/* CIFOP_BBOX uses the co_client field as a flag field.	  */
-		else if ((op->co_opcode != CIFOP_OR) && (op->co_opcode != CIFOP_BBOX))
+		/* CIFOP_BBOX and CIFOP_MAXRECT uses the co_client field  */
+		/* as a flag field, while CIFOP_NET uses it for a string. */
+		else
 		{
-		    bloats = (BloatData *)op->co_client;
-		    for (j = 0; j < TT_MAXTYPES; j++)
+		    switch (op->co_opcode)
 		    {
-			if (bloats->bl_distance[j] != 0)
-			{
-			    c = FindGCF(style->cs_scaleFactor, bloats->bl_distance[j]);
-			    if (c < minReduce) minReduce = c;
-			}
+			case CIFOP_OR:
+			case CIFOP_BBOX:
+			case CIFOP_MAXRECT:
+			case CIFOP_NET:
+			    break;
+			default:
+			    bloats = (BloatData *)op->co_client;
+			    for (j = 0; j < TT_MAXTYPES; j++)
+			    {
+				if (bloats->bl_distance[j] != 0)
+				{
+				    c = FindGCF(style->cs_scaleFactor,
+						bloats->bl_distance[j]);
+				    if (c < minReduce) minReduce = c;
+				}
+			    }
 		    }
 		}
 	    }
@@ -1869,6 +1918,12 @@ CIFLoadStyle(stylename)
  *   to the "squares" function operating on the indicated type.  This value
  *   is computed as the (cut size) + 2 * (cut border).
  *
+ *   9/12/2013:  Added "squares-grid" and "slots" to the functions understood
+ *   by the routine.  "squares-grid" behaves the same as "squares".  "slots"
+ *   can be used for contact cuts with differing metal overlap on different
+ *   sides.  Normally this would define a square slot;  this routine finds
+ *   the minimum cut size for the slot.
+ *
  * Results:
  *	Contact minimum dimension, in CIF/GDS units
  *
@@ -1890,6 +1945,7 @@ CIFGetContactSize(type, edge, spacing, border)
     CIFOp *op, *sop;
     int i;
     SquaresData *squares;
+    SlotsData *slots;
 
     if (style == NULL)
     {
@@ -1905,7 +1961,8 @@ CIFGetContactSize(type, edge, spacing, border)
 	    if (TTMaskHasType(&op->co_paintMask, type))
 		for (sop = op->co_next; sop != NULL; sop = sop->co_next)
 		{
-		    if (sop->co_opcode == CIFOP_SQUARES)
+		    if (sop->co_opcode == CIFOP_SQUARES ||
+				sop->co_opcode == CIFOP_SQUARES_G)
 		    {
 			squares = (SquaresData *)sop->co_client;
 			if (edge != NULL) *edge = squares->sq_size;
@@ -1913,6 +1970,15 @@ CIFGetContactSize(type, edge, spacing, border)
 			if (spacing != NULL) *spacing = squares->sq_sep;
 			return (squares->sq_size + (squares->sq_border << 1));
 		    }
+		    else if (sop->co_opcode == CIFOP_SLOTS)
+		    {
+			slots = (SlotsData *)sop->co_client;
+			if (edge != NULL) *edge = slots->sl_ssize;
+			if (border != NULL) *border = slots->sl_sborder;
+			if (spacing != NULL) *spacing = slots->sl_ssep;
+			return (slots->sl_ssize + (slots->sl_sborder << 1));
+		    }
+
 		    /* Anything other than an OR function will break	*/
 		    /* the relationship between magic layers and cuts.	*/
 		    else if (sop->co_opcode != CIFOP_OR)
@@ -2115,18 +2181,27 @@ CIFTechOutputScale(n, d)
 		/* Presence of op->co_opcode in CIFOP_OR indicates a copy */
 		/* of the SquaresData pointer from a following operator	  */
 		/* CIFOP_BBOX uses the co_client field as a flag field.	  */
-		else if ((op->co_opcode != CIFOP_OR) && (op->co_opcode != CIFOP_BBOX))
+		else
 		{
-		    bloats = (BloatData *)op->co_client;
-		    for (j = 0; j < TT_MAXTYPES; j++)
+		    switch (op->co_opcode)
 		    {
-			if (bloats->bl_distance[j] != 0)
-			{
-			    bloats->bl_distance[j] *= d;
-		            lgcf = FindGCF(abs(bloats->bl_distance[j]),
-					ostyle->cs_expander);
-			    lexpand = FindGCF(lexpand, lgcf);
-			}
+			case CIFOP_OR:
+			case CIFOP_BBOX:
+			case CIFOP_MAXRECT:
+			case CIFOP_NET:
+			    break;
+			default:
+			    bloats = (BloatData *)op->co_client;
+			    for (j = 0; j < TT_MAXTYPES; j++)
+			    {
+				if (bloats->bl_distance[j] != 0)
+				{
+				    bloats->bl_distance[j] *= d;
+		        	    lgcf = FindGCF(abs(bloats->bl_distance[j]),
+						ostyle->cs_expander);
+				    lexpand = FindGCF(lexpand, lgcf);
+				}
+			    }
 		    }
 		}
 	    }
@@ -2212,7 +2287,8 @@ CIFTechOutputScale(n, d)
 			break;
 		    default:
 			/* op->co_opcode in CIFOP_OR is a pointer copy	*/
-			/* and in CIFOP_BBOX is a flag.			*/
+			/* and in CIFOP_BBOX and CIFOP_MAXRECT is a	*/
+			/* flag, and in CIFOP_NET is a string.		*/
 			break;
 		}
 	    }
