@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/cif/CIFwrite.c,v 1.1.1.1 2006/04/10 22:03:14 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/cif/CIFwrite.c,v 1.2 2010/06/24 12:37:15 tim Exp $";
 #endif  /* not lint */
 
 #include <stdlib.h>
@@ -51,10 +51,7 @@ extern int cifWriteUseFunc();
 extern void cifOutPreamble();
 extern void cifOut();
 extern void cifOutFunc();
-
-#ifdef NONMANHATTAN
 extern int GrClipTriangle();
-#endif
 
 /* Current cell number in CIF numbering scheme */
 
@@ -583,7 +580,6 @@ cifWritePaintFunc(tile, f)
 
     TiToRect(tile, &r);
 
-#ifdef NONMANHATTAN
     if (IsSplit(tile))
     {
 	Point points[5];
@@ -603,19 +599,18 @@ cifWritePaintFunc(tile, f)
 	fprintf(f, ";\n");
     }
     else
-#endif
 
-    /* The only tricky thing here is that we MUST scale the rectangle
-     * up by a factor of two to avoid round-off errors in computing
-     * its center point (what a bogosity in CIF!!).  This is compensated
-     * by shrinking by a factor of two in the "DS" statement.
-     */
+	/* The only tricky thing here is that we MUST scale the rectangle
+	 * up by a factor of two to avoid round-off errors in computing
+	 * its center point (what a bogosity in CIF!!).  This is compensated
+	 * by shrinking by a factor of two in the "DS" statement.
+	 */
 
-    fprintf(f, "    B %d %d %d %d;\n",
-	(2*cifPaintScale*(r.r_xtop - r.r_xbot))/CIFCurStyle->cs_reducer,
-	(2*cifPaintScale*(r.r_ytop - r.r_ybot))/CIFCurStyle->cs_reducer,
-	(cifPaintScale*(r.r_xtop + r.r_xbot))/CIFCurStyle->cs_reducer,
-	(cifPaintScale*(r.r_ytop + r.r_ybot))/CIFCurStyle->cs_reducer);
+	fprintf(f, "    B %d %d %d %d;\n",
+		(2*cifPaintScale*(r.r_xtop - r.r_xbot))/CIFCurStyle->cs_reducer,
+		(2*cifPaintScale*(r.r_ytop - r.r_ybot))/CIFCurStyle->cs_reducer,
+		(cifPaintScale*(r.r_xtop + r.r_xbot))/CIFCurStyle->cs_reducer,
+		(cifPaintScale*(r.r_ytop + r.r_ybot))/CIFCurStyle->cs_reducer);
     CIFRects += 1;
     return 0;
 }
