@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/commands/CmdE.c,v 1.8 2010/06/17 14:40:52 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/commands/CmdE.c,v 1.4 2010/06/17 14:38:33 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -117,6 +117,7 @@ CmdEdit(w, cmd)
     cmdFoundNewEdit = FALSE;
     csave = EditRootDef;
     usave = EditCellUse;
+    EditCellUse = NULL;
 
     (void) SelEnumCells(FALSE, (bool *) NULL, (SearchContext *) NULL,
 	    cmdEditEnumFunc, (ClientData) &pointArea);
@@ -605,7 +606,7 @@ CmdErase(w, cmd)
     MagWindow *w;
     TxCommand *cmd;
 {
-    Rect editRect;
+    Rect editRect, areaReturn;
     TileTypeBitMask mask;
     extern int cmdEraseCellsFunc();
 
@@ -648,7 +649,8 @@ CmdErase(w, cmd)
     DBEraseValid(EditCellUse->cu_def, &editRect, &mask, 0);
 
     /* Erase labels. */
-    (void) DBEraseLabel(EditCellUse->cu_def, &editRect, &mask);
+    areaReturn = editRect;
+    DBEraseLabel(EditCellUse->cu_def, &editRect, &mask, &areaReturn);
 
     /* Erase subcells. */
     if (TTMaskHasType(&mask, L_CELL))
@@ -692,7 +694,7 @@ CmdErase(w, cmd)
 	DRCCheckThis (EditCellUse->cu_def, TT_CHECKPAINT, &editRect);
     TTMaskClearType(&mask, L_CELL);
     SelectClear();
-    DBWAreaChanged(EditCellUse->cu_def, &editRect, DBW_ALLWINDOWS, &mask);
+    DBWAreaChanged(EditCellUse->cu_def, &areaReturn, DBW_ALLWINDOWS, &mask);
     DBReComputeBbox(EditCellUse->cu_def);
 }
 
