@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/commands/CmdFI.c,v 1.7 2008/02/01 17:46:46 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/commands/CmdFI.c,v 1.4 2010/06/24 12:37:15 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -105,13 +105,9 @@ feedPolyFunc(tile, arg)
     TiToRect(tile, &area);
 
     DBWFeedbackAdd(&area, arg->text, arg->def, FEEDMAGNIFY,
-#ifdef NONMANHATTAN
         arg->style |
                 (TiGetTypeExact(tile) & (TT_DIAGONAL | TT_DIRECTION | TT_SIDE)));
         /* (preserve information about the geometry of a diagonal tile) */
-#else
-        arg->style);
-#endif
     return 0;
 }
 
@@ -1509,101 +1505,45 @@ cmdIdFunc(selUse, use, transform, newId)
     return 1;
 }
 
-/*
- * ----------------------------------------------------------------------------
- *
- * CmdGoto --
- *
- * Implements goto command
- *
- * Usage:
- *	goto nodename [-nocomplain]
- *
- * Results:
- *	None.  In TCL version, returns the material type
- *	of the node, so that the node can be uniquely determined
- *	by an automated script.   If the node cannot be found, a
- *	null list is returned.
- *
- * Side Effects:
- *	changes box location.
- *
- * Notes:
- *	Due to the way global node names are handled, "getnode" and
- *	"goto" are not necessarily reversible if "getnode globals on"
- *	is set!  In this case, use the glob option for findlabel
- *	to determine any valid local label belonging to the global
- *	node (a node cannot be global unless it is labeled).  This
- *	method, although awkward, can be much faster for large layouts
- *	due to the time required for "getnode" to uniquely determine
- *	the name of a large network such as power or ground.
- *
- * ----------------------------------------------------------------------------
- */
-
-void
-CmdGoto(w, cmd)
-    MagWindow *w;
-    TxCommand *cmd;
+TileType
+CmdFindNetProc(nodename, use, rect, warn_not_found)
+    char *nodename;
+    CellUse *use;
+    Rect *rect;
+    bool warn_not_found;
 {
-    char		*nodename = cmd->tx_argv[1];
     char		*s,*s2;
-    SearchContext	scx,scx2;
-    CellUse		*use;
+    SearchContext	scx, scx2;
     Transform		trans, newtrans, tmp;
     Label		*label;
-    Rect		localrect, rect;
-    CellDef		*rootdef;
-    int			pnum, xpos, ypos, locargc;
+    Rect		localrect;
+    int			pnum, xpos, ypos;
     char 		*xstr, *ystr;
     bool		locvalid;
-    bool		nocomplain = FALSE;
     TileType		ttype;
-     
-    windCheckOnlyWindow(&w, DBWclientID);
-    if ((w == (MagWindow *) NULL) || (w->w_client != DBWclientID))
-    {
-	TxError("Put the cursor in a layout window\n");
-	return;
-    }
-    scx.scx_use = (CellUse  *) w->w_surfaceID;
-    rootdef = scx.scx_use->cu_def;
-    scx.scx_trans = GeoIdentityTransform;
 
-    locargc = cmd->tx_argc;
-    if (locargc == 3)
-    {
-	if (!strncmp(cmd->tx_argv[2], "-nocom", 5))
-	{
-	    nocomplain = TRUE;
-	    locargc--;
-	}
-    }
-    if (locargc != 2)
-    {
-     	TxError("usage: goto nodename [-nocomplain]\n");
-	return;
-    }
+    scx.scx_use = use;
+    scx.scx_trans = GeoIdentityTransform;
     s = nodename;
     trans = GeoIdentityTransform;
-    use = scx.scx_use;
-    while (s2 = strchr(s,'/'))
+    while (s2 = strchr(s, '/'))
     {
      	*s2 = '\0';
-	DBTreeFindUse(s,scx.scx_use,&scx2);
+	DBTreeFindUse(s, scx.scx_use, &scx2);
 	use = scx2.scx_use;
 	if (use == NULL)
 	{
-	    TxError("Couldn't find use %s\n",s);
-	    return;
+	    if (warn_not_found)
+		TxError("Couldn't find use %s\n", s);
+	    return TT_SPACE; 
 	}
-	GeoTransTrans(DBGetArrayTransform(use,scx2.scx_x,scx2.scx_y)
-	  		,&use->cu_transform,&tmp);
-	GeoTransTrans(&tmp,&trans,&newtrans);
+	GeoTransTrans(DBGetArrayTransform(use, scx2.scx_x, scx2.scx_y),
+	  		&use->cu_transform, &tmp);
+	GeoTransTrans(&tmp, &trans, &newtrans);
 	trans = newtrans;
 	scx = scx2;
         *s2 = '/';
-	s = s2+1;
+	s = s2 + 1;
     }
 
     /* If this node name is in the format of automatically-generated */
@@ -1693,9 +1633,10 @@ CmdGoto(w, cmd)
     }
     else
     {
-	for (label = scx.scx_use->cu_def->cd_labels;label;label=label->lab_next)
+	for (label = scx.scx_use->cu_def->cd_labels; label;
+			label = label->lab_next)
 	{
-	    if (strcmp(label->lab_text,s) == 0) break;
+	    if (!strcmp(label->lab_text, s)) break;
 	}
 	if (label)
 	{
@@ -1704,14 +1645,88 @@ CmdGoto(w, cmd)
 	}
 	else
 	{
-	    if (!nocomplain)
-     		TxError("Couldn't find label %s\n",s);
-	    return;
+	    if (warn_not_found)
+		TxError("Couldn't find label %s\n", s);
+	    return TT_SPACE;
 	}
     }
-    GeoTransRect(&trans, &localrect, &rect);
-    ToolMoveBox(TOOL_BL, &rect.r_ll, FALSE, rootdef);
-    ToolMoveCorner(TOOL_TR, &rect.r_ur, FALSE, rootdef);
+    GeoTransRect(&trans, &localrect, rect);
+    return ttype;
+}
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * CmdGoto --
+ *
+ * Implements goto command
+ *
+ * Usage:
+ *	goto nodename [-nocomplain]
+ *
+ * Results:
+ *	None.  In TCL version, returns the material type
+ *	of the node, so that the node can be uniquely determined
+ *	by an automated script.   If the node cannot be found, a
+ *	null list is returned.
+ *
+ * Side Effects:
+ *	changes box location.
+ *
+ * Notes:
+ *	Due to the way global node names are handled, "getnode" and
+ *	"goto" are not necessarily reversible if "getnode globals on"
+ *	is set!  In this case, use the glob option for findlabel
+ *	to determine any valid local label belonging to the global
+ *	node (a node cannot be global unless it is labeled).  This
+ *	method, although awkward, can be much faster for large layouts
+ *	due to the time required for "getnode" to uniquely determine
+ *	the name of a large network such as power or ground.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+void
+CmdGoto(w, cmd)
+    MagWindow *w;
+    TxCommand *cmd;
+{
+    char	*s, *nodename = cmd->tx_argv[1];
+    Rect	rect;
+    CellUse	*use;
+    int		locargc;
+    bool	nocomplain = FALSE;
+    TileType	ttype;
+     
+    windCheckOnlyWindow(&w, DBWclientID);
+    if ((w == (MagWindow *) NULL) || (w->w_client != DBWclientID))
+    {
+	TxError("Put the cursor in a layout window\n");
+	return;
+    }
+
+    locargc = cmd->tx_argc;
+    if (locargc == 3)
+    {
+	if (!strncmp(cmd->tx_argv[2], "-nocom", 5))
+	{
+	    nocomplain = TRUE;
+	    locargc--;
+	}
+    }
+    if (locargc != 2)
+    {
+     	TxError("usage: goto nodename [-nocomplain]\n");
+	return;
+    }
+
+    /* CmdFindNetProc() does all the work */
+    use = (CellUse *)w->w_surfaceID;
+    ttype = CmdFindNetProc(nodename, use, &rect, !nocomplain);
+    if (ttype == TT_SPACE) return;
+
+    ToolMoveBox(TOOL_BL, &rect.r_ll, FALSE, use->cu_def);
+    ToolMoveCorner(TOOL_TR, &rect.r_ur, FALSE, use->cu_def);
 
     /* Return the tile type so we know what we're looking at if there	*/
     /* are multiple layers drawn at the indicated point.		*/ 
@@ -1736,7 +1751,6 @@ findTile(tile, rtype)
 {
     TileType ttype;
 
-#ifdef NONMANHATTAN
     if (IsSplit(tile))  
     {   
 	if (SplitSide(tile))
@@ -1745,7 +1759,6 @@ findTile(tile, rtype)
 	    ttype = SplitLeftType(tile);
     }
     else
-#endif  
 	ttype = TiGetTypeExact(tile);
     *rtype = ttype;
     return 1;			/* stop search */
@@ -1772,8 +1785,8 @@ FlatCopyAllLabels(scx, mask, xMask, targetUse)
     tpath.tp_first = tpath.tp_next = pathstring;
     tpath.tp_last = pathstring + FLATTERMSIZE;
 
-    DBTreeSrLabels(scx, mask, xMask, &tpath, flatCopyAllLabels,
-			(ClientData) targetUse);
+    DBTreeSrLabels(scx, mask, xMask, &tpath, TF_LABEL_ATTACH,
+			flatCopyAllLabels, (ClientData) targetUse);
 }
 
 int
@@ -1805,19 +1818,22 @@ flatCopyAllLabels(scx, lab, tpath, targetUse)
      * contains the heirarchical names.
      */
 
-    /* (void) DBEraseLabelsByContent(def, &labTargetRect, targetPos,
-        -1, lab->lab_text); */
+    /* (void) DBEraseLabelsByContent(def, &labTargetRect, -1, lab->lab_text); */
 
     /* (Added 9/10/04) Make sure that the target flattened label is
      * not a port; possibly we should retain ports taken from the
      * top level cell, but certainly not any others.
      */
 
+    /* To-do Feb. 2008:  Translate target rotation and offset */
+
     n = tpath->tp_next;
     f = tpath->tp_first;
     c = *n;
     strcpy(n, lab->lab_text);
-    DBPutLabel(def, &labTargetRect, targetPos, f, lab->lab_type, 0);
+    DBPutFontLabel(def, &labTargetRect, lab->lab_font, lab->lab_size,
+		lab->lab_rotate, &lab->lab_offset, targetPos,
+		f, lab->lab_type, 0);
     *n = c;
 
     return 0;
