@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/commands/CmdRS.c,v 1.15 2008/12/03 16:34:18 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/commands/CmdRS.c,v 1.13 2010/06/24 12:37:15 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -33,6 +33,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/
 #include "tiles/tile.h"
 #include "utils/hash.h"
 #include "database/database.h"
+#include "database/fonts.h"
 #include "windows/windows.h"
 #include "dbwind/dbwind.h"
 #include "utils/main.h"
@@ -678,11 +679,12 @@ CmdSelect(w, cmd)
 #define SEL_MOVE	 7
 #define SEL_PICK	 8
 #define SEL_SAVE	 9
-#define SEL_BOX		10 
+#define SEL_BOX		10
 #define SEL_CHUNK	11
 #define SEL_REGION	12
 #define SEL_NET		13
-#define SEL_DEFAULT	14
+#define SEL_SHORT	14
+#define SEL_DEFAULT	15
 
     static char *cmdSelectOption[] =
     {
@@ -700,6 +702,7 @@ CmdSelect(w, cmd)
 	"chunk",
 	"region",
 	"net",
+	"short",
 	NULL
     };
     static char *cmdSelectMsg[] =
@@ -712,7 +715,7 @@ CmdSelect(w, cmd)
 	"[more | less] visible [layers]  [de]select all visible info under box in layers",
 	"[more | less | top] cell [name] [de]select cell under cursor, or \"name\"",
 	"clear                           clear selection",
-	"flat                            flatten the contents of the selection",
+	"flat				 flatten the contents of the selection",
 	"help                            print this message",
 	"keep                            copy the selection to the layout",
 	"move                            move selection to a location in the layout",
@@ -721,6 +724,7 @@ CmdSelect(w, cmd)
 	"[more | less] box | chunk | region | net [layers]\n"
 	"				 [de]select chunk/region/net specified by\n"
 	"				 the lower left corner of the current box",
+	"short name1 name2		 find shorting path between two labels",
 	NULL
     };
 
@@ -760,6 +764,7 @@ CmdSelect(w, cmd)
     Transform trans, rootTrans, tmp1;
     Point p, rootPoint;
     Rect r, selarea;
+    ExtRectList *rlist;
     int option;
     bool layerspec;
     bool degenerate;
@@ -813,6 +818,14 @@ CmdSelect(w, cmd)
 	    optionArgs = &cmd->tx_argv[2];
 	    cmd->tx_argc--;
 	}
+	else if (!strncmp(cmd->tx_argv[1], "same", arg1len))
+	{
+	    /* Force this to be the same as the last selection command,	*/
+	    /* even if there were other commands in between.		*/
+	    lastCommand = TxCommandNumber - 1;
+	    optionArgs = &cmd->tx_argv[2];
+	    cmd->tx_argc--;
+	}
 
 	else if (!strncmp(cmd->tx_argv[1], "top", arg1len)) 
 	{
@@ -960,7 +973,7 @@ CmdSelect(w, cmd)
 	 *--------------------------------------------------------------------
 	 */
 
-	 case SEL_FLAT:
+	case SEL_FLAT:
 	    if ((more) || (less) || (cmd->tx_argc > 2)) goto usageError;
 	    SelectFlat();
 	    return;
@@ -987,6 +1000,45 @@ CmdSelect(w, cmd)
 	    DBUpdateStamps();
 	    cmdSaveCell(SelectDef, cmd->tx_argv[2], FALSE, FALSE);
 	    return;
+
+	/*--------------------------------------------------------------------
+	 * Given a net selection and two labels, determine the shortest
+	 * connecting path between the two labels.
+	 *--------------------------------------------------------------------
+	 */
+	case SEL_SHORT:
+	    if (cmd->tx_argc != 4) goto usageError;
+	    rlist = SelectShort(cmd->tx_argv[2], cmd->tx_argv[3]);
+
+	    if (rlist == NULL)
+	    {
+		TxError("No shorting path found between source and destination!\n");
+		return;
+	    }
+
+	    /* Delete selection and replace with contents of rlist */
+
+	    /* (To do:  Alternately just return a list of the contents	*/
+	    /* of rlist)						*/
+	    SelectClear();
+
+	    while (rlist != NULL)
+	    {
+		/* Paint rlist back into SelectDef */
+		DBPaint(SelectDef, &rlist->r_r, rlist->r_type);
+
+		/* cleanup as we go */
+		freeMagic(rlist);
+		rlist = rlist->r_next;
+	    }
+
+	    /* Force erase and redraw of the selection */
+	    DBReComputeBbox(SelectDef);
+	    DBWAreaChanged(SelectDef, &SelectDef->cd_extended, DBW_ALLWINDOWS,
+			(TileTypeBitMask *)NULL);
+	    GeoTransRect(&SelectUse->cu_transform, &SelectDef->cd_bbox, &selarea);
+	    DBWHLRedraw(SelectRootDef, &selarea, FALSE);
+	    break;
 	
 	case SEL_BOX: case SEL_CHUNK: case SEL_REGION: case SEL_NET:
 	    if (cmd->tx_argc > 3) goto usageError;
@@ -1216,7 +1268,10 @@ Okay:
 			level = SEL_NET;
 		}
 		if (level == SEL_NET)
+		{
 		    SelectNet(&scx, type, crec->dbw_bitmask, (Rect *) NULL, less);
+		}
+
 		return;
 	    }
 
@@ -1343,7 +1398,588 @@ Okay:
 	    return;
     }
 }
-
+
+/*
+ *-----------------------------------------------------------------------
+ * Callback functions used with CmdSetLabel to change individual labels
+ *-----------------------------------------------------------------------
+ */
+
+int
+cmdLabelTextFunc(label, cellUse, transform, text)
+    Label *label;
+    CellUse *cellUse;
+    Transform *transform;
+    char *text;
+{
+    Label *newlab;
+    CellDef *cellDef = cellUse->cu_def;
+#ifdef MAGIC_WRAPPER
+    Tcl_Obj *lobj;
+#endif
+
+    if (text == NULL)
+    {
+#ifdef MAGIC_WRAPPER
+	lobj = Tcl_GetObjResult(magicinterp);
+	Tcl_ListObjAppendElement(magicinterp, lobj,
+			Tcl_NewStringObj(label->lab_text, -1));
+	Tcl_SetObjResult(magicinterp, lobj);
+#else
+	TxPrintf("%s\n", label->lab_text);
+#endif
+    }
+    else if (cellDef == EditRootDef)
+    {
+	if (strcmp(text, label->lab_text))
+	{
+	    newlab = DBPutFontLabel(cellDef, &label->lab_rect, label->lab_font,
+			label->lab_size, label->lab_rotate, &label->lab_offset,
+			label->lab_just, text, label->lab_type, label->lab_flags);
+	    DBEraseLabelsByContent(cellDef, &label->lab_rect, -1, label->lab_text);
+	    DBWLabelChanged(cellDef, newlab, DBW_ALLWINDOWS);
+	}
+    }
+    return 0;
+}
+
+int
+cmdLabelRotateFunc(label, cellUse, transform, value)
+    Label *label;
+    CellUse *cellUse;
+    Transform *transform;
+    int *value;
+{
+    CellDef *cellDef = cellUse->cu_def;
+#ifdef MAGIC_WRAPPER
+    Tcl_Obj *lobj;
+#endif
+
+    if (value == NULL)
+    {
+#ifdef MAGIC_WRAPPER
+	lobj = Tcl_GetObjResult(magicinterp);
+	Tcl_ListObjAppendElement(magicinterp, lobj,
+			Tcl_NewIntObj(label->lab_rotate));
+	Tcl_SetObjResult(magicinterp, lobj);
+#else
+	TxPrintf("%d\n", label->lab_rotate);
+#endif
+    }
+    else if (cellDef == EditRootDef)
+    {
+	DBUndoEraseLabel(cellDef, label);
+	DBWLabelChanged(cellDef, label, DBW_ALLWINDOWS);
+	label->lab_rotate = *value;
+	DBFontLabelSetBBox(label);
+	DBUndoPutLabel(cellDef, label);
+	DBWLabelChanged(cellDef, label, DBW_ALLWINDOWS);
+    }
+    return 0;
+}
+
+int
+cmdLabelSizeFunc(label, cellUse, transform, value)
+    Label *label;
+    CellUse *cellUse;
+    Transform *transform;
+    int *value;
+{
+    CellDef *cellDef = cellUse->cu_def;
+#ifdef MAGIC_WRAPPER
+    Tcl_Obj *lobj;
+#endif
+
+    if (value == NULL)
+    {
+#ifdef MAGIC_WRAPPER
+	lobj = Tcl_GetObjResult(magicinterp);
+	Tcl_ListObjAppendElement(magicinterp, lobj,
+			Tcl_NewDoubleObj((double)label->lab_size / 8.0));
+	Tcl_SetObjResult(magicinterp, lobj);
+#else
+	TxPrintf("%g\n", (double)label->lab_size / 8.0);
+#endif
+    }
+    else if (cellDef == EditRootDef)
+    {
+	DBUndoEraseLabel(cellDef, label);
+	DBWLabelChanged(cellDef, label, DBW_ALLWINDOWS);
+	label->lab_size = *value;
+	DBFontLabelSetBBox(label);
+	DBUndoPutLabel(cellDef, label);
+	DBWLabelChanged(cellDef, label, DBW_ALLWINDOWS);
+    }
+    return 0;
+}
+
+int
+cmdLabelJustFunc(label, cellUse, transform, value)
+    Label *label;
+    CellUse *cellUse;
+    Transform *transform;
+    int *value;
+{
+    CellDef *cellDef = cellUse->cu_def;
+#ifdef MAGIC_WRAPPER
+    Tcl_Obj *lobj;
+#endif
+
+    if (value == NULL)
+    {
+#ifdef MAGIC_WRAPPER
+	lobj = Tcl_GetObjResult(magicinterp);
+	Tcl_ListObjAppendElement(magicinterp, lobj,
+			Tcl_NewStringObj(GeoPosToName(label->lab_just), -1));
+	Tcl_SetObjResult(magicinterp, lobj);
+#else
+	TxPrintf("%s\n", GeoPosToName(label->lab_just));
+#endif
+    }
+    else if (cellDef == EditRootDef)
+    {
+	DBUndoEraseLabel(cellDef, label);
+	DBWLabelChanged(cellDef, label, DBW_ALLWINDOWS);
+	label->lab_just = *value;
+	DBFontLabelSetBBox(label);
+	DBUndoPutLabel(cellDef, label);
+	DBWLabelChanged(cellDef, label, DBW_ALLWINDOWS);
+    }
+    return 0;
+}
+
+int
+cmdLabelLayerFunc(label, cellUse, transform, value)
+    Label *label;
+    CellUse *cellUse;
+    Transform *transform;
+    int *value;
+{
+    CellDef *cellDef = cellUse->cu_def;
+    TileType ttype;
+#ifdef MAGIC_WRAPPER
+    Tcl_Obj *lobj;
+#endif
+
+    if (value == NULL)
+    {
+#ifdef MAGIC_WRAPPER
+	lobj = Tcl_GetObjResult(magicinterp);
+	Tcl_ListObjAppendElement(magicinterp, lobj,
+			Tcl_NewStringObj(DBTypeLongNameTbl[label->lab_type], -1));
+	Tcl_SetObjResult(magicinterp, lobj);
+#else
+	TxPrintf("%s\n", DBTypeLongNameTbl[label->lab_type]);
+#endif
+    }
+    else if (cellDef == EditRootDef)
+    {
+	ttype = (TileType)(*value);
+	if (label->lab_type != ttype)
+	{
+	    DBUndoEraseLabel(cellDef, label);
+	    label->lab_type = ttype;
+	    DBUndoPutLabel(cellDef, label);
+	    DBCellSetModified(cellDef, TRUE);
+	}
+    }
+    return 0;
+}
+
+int
+cmdLabelStickyFunc(label, cellUse, transform, value)
+    Label *label;
+    CellUse *cellUse;
+    Transform *transform;
+    int *value;
+{
+    CellDef *cellDef = cellUse->cu_def;
+    int newvalue;
+#ifdef MAGIC_WRAPPER
+    Tcl_Obj *lobj;
+#endif
+
+    if (value == NULL)
+    {
+#ifdef MAGIC_WRAPPER
+	lobj = Tcl_GetObjResult(magicinterp);
+	Tcl_ListObjAppendElement(magicinterp, lobj,
+			Tcl_NewBooleanObj((label->lab_flags & LABEL_STICKY) ? 1 : 0));
+	Tcl_SetObjResult(magicinterp, lobj);
+#else
+	TxPrintf("%s\n", (label->lab_flags & LABEL_STICKY) ? "true" : "false");
+#endif
+    }
+    else if (cellDef == EditRootDef)
+    {
+	newvalue = label->lab_flags;
+	newvalue &= ~LABEL_STICKY;
+	newvalue |= *value;
+	if (newvalue != label->lab_flags)
+	{
+	    /* Label does not change appearance, just need to record the change */
+	    DBUndoEraseLabel(cellDef, label);
+	    label->lab_flags = newvalue;
+	    DBUndoPutLabel(cellDef, label);
+	}
+    }
+    return 0;
+}
+
+int
+cmdLabelOffsetFunc(label, cellUse, transform, point)
+    Label *label;
+    CellUse *cellUse;
+    Transform *transform;
+    Point *point;
+{
+    CellDef *cellDef = cellUse->cu_def;
+#ifdef MAGIC_WRAPPER
+    Tcl_Obj *lobj, *pobj;
+#endif
+
+    if (point == NULL)
+    {
+#ifdef MAGIC_WRAPPER
+	lobj = Tcl_GetObjResult(magicinterp);
+	pobj = Tcl_NewListObj(0, NULL);
+	Tcl_ListObjAppendElement(magicinterp, lobj, pobj);
+	Tcl_ListObjAppendElement(magicinterp, pobj,
+			Tcl_NewDoubleObj((double)label->lab_offset.p_x / 8.0));
+	Tcl_ListObjAppendElement(magicinterp, pobj,
+			Tcl_NewDoubleObj((double)label->lab_offset.p_y / 8.0));
+	Tcl_SetObjResult(magicinterp, lobj);
+#else
+	TxPrintf("%g %g\n", (double)(label->lab_offset.p_x) / 8.0,
+		(double)(label->lab_offset.p_y) / 8.0);
+#endif
+    }
+    else if (cellDef == EditRootDef)
+    {
+	DBUndoEraseLabel(cellDef, label);
+	DBWLabelChanged(cellDef, label, DBW_ALLWINDOWS);
+	label->lab_offset = *point;
+	DBFontLabelSetBBox(label);
+	DBUndoPutLabel(cellDef, label);
+	DBWLabelChanged(cellDef, label, DBW_ALLWINDOWS);
+    }
+    return 0;
+}
+
+int
+cmdLabelFontFunc(label, cellUse, transform, font)
+    Label *label;
+    CellUse *cellUse;
+    Transform *transform;
+    int *font;
+{
+    CellDef *cellDef = cellUse->cu_def;
+#ifdef MAGIC_WRAPPER
+    Tcl_Obj *lobj;
+#endif
+
+    if (font == NULL)
+    {
+#ifdef MAGIC_WRAPPER
+	lobj = Tcl_GetObjResult(magicinterp);
+	if (label->lab_font == -1)
+	    Tcl_ListObjAppendElement(magicinterp, lobj, Tcl_NewStringObj("default", 7));
+	else
+	    Tcl_ListObjAppendElement(magicinterp, lobj,
+			Tcl_NewStringObj(DBFontList[label->lab_font]->mf_name, -1));
+	Tcl_SetObjResult(magicinterp, lobj);
+#else
+	if (label->lab_font == -1)
+	    TxPrintf("default\n");
+	else
+	    TxPrintf("%s\n", DBFontList[label->lab_font]->mf_name);
+#endif
+    }
+    else if (cellDef == EditRootDef)
+    {
+	DBUndoEraseLabel(cellDef, label);
+	DBWLabelChanged(cellDef, label, DBW_ALLWINDOWS);
+	label->lab_font = *font;
+	if ((*font > -1) && (label->lab_size == 0)) label->lab_size = DBLambda[1];
+	DBFontLabelSetBBox(label);
+	DBUndoPutLabel(cellDef, label);
+	DBWLabelChanged(cellDef, label, DBW_ALLWINDOWS);
+    }
+    return 0;
+}
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * CmdSetLabel --
+ *
+ * Implement the "setlabel" command.
+ * Query or change properties of a (selected) label in the edit cell
+ *
+ * Usage:
+ *	setlabel option [name]
+ *
+ *
+ * Option may be one of:
+ *	text
+ *	font
+ *	justify
+ *	size
+ *	offset
+ *	rotate
+ *	layer
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Modified label entry.  The "setlabel font" command may load font
+ *	information if the requested font is not already in the font list.
+ *	"setlabel font <name>" can be used without any select to load fonts
+ *	from a startup script.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+#define SETLABEL_TEXT		0
+#define SETLABEL_FONT		1
+#define SETLABEL_FONTLIST	2
+#define SETLABEL_JUSTIFY	3
+#define SETLABEL_SIZE		4
+#define SETLABEL_OFFSET		5
+#define SETLABEL_ROTATE		6
+#define SETLABEL_STICKY		7
+#define SETLABEL_LAYER		8
+
+void
+CmdSetLabel(w, cmd)
+    MagWindow *w;
+    TxCommand *cmd;
+{
+    int pos = -1, font = -1, size = 0, rotate = 0, flags = 0;
+    Point offset;
+    TileType ttype;
+    int option;
+#ifdef MAGIC_WRAPPER
+    Tcl_Obj *lobj;
+#endif
+
+    static char *cmdLabelYesNo[] = { "no", "false", "off", "yes", "true", "on", 0 };
+    static char *cmdLabelSetOption[] =
+    {
+	"text <text>		change/get label text",
+	"font <name>		change/get label font",
+	"fontlist		list available fonts",
+	"justify <position>	change/get label justification",
+	"size <value>		change/get label size",
+	"offset <x> <y>		change/get label offset",
+	"rotate <degrees>	change/get label rotation",
+	"sticky [true|false]	change/get sticky property",
+	"layer <type>		change/get layer type",
+	NULL
+    };
+
+    if (cmd->tx_argc < 2 || cmd->tx_argc > 4)
+    {
+	TxError("%s <option> [<value>]\n", cmd->tx_argv[0]);
+	return;
+    }
+
+    option = Lookup(cmd->tx_argv[1], cmdLabelSetOption);
+
+    switch (option)
+    {
+	case SETLABEL_FONTLIST:
+#ifdef MAGIC_WRAPPER
+	    lobj = Tcl_NewListObj(0, NULL);
+	    for (font = 0; font < DBNumFonts; font++)
+		Tcl_ListObjAppendElement(magicinterp, lobj,
+			Tcl_NewStringObj(DBFontList[font]->mf_name, -1));
+	    Tcl_SetObjResult(magicinterp, lobj);
+#else
+	    for (font = 0; font < DBNumFonts; font++)
+		TxPrintf("%s ", DBFontList[font]->mf_name);
+	    TxPrintf("\n");
+#endif
+	    break;
+	
+	case SETLABEL_TEXT:
+	    if (EditCellUse)
+	    {
+		SelEnumLabels(&DBAllTypeBits, FALSE, (bool *)NULL,
+			cmdLabelTextFunc, (cmd->tx_argc == 3) ?
+			(ClientData)cmd->tx_argv[2] : (ClientData)NULL);
+		SelectTransform(&GeoIdentityTransform);
+	    }
+	    break;
+
+	case SETLABEL_FONT:
+	    if (cmd->tx_argc >= 2 && cmd->tx_argc <= 4)
+	    {
+		if ((cmd->tx_argc == 3) && StrIsInt(cmd->tx_argv[2]))
+		{
+		    int font = atoi(cmd->tx_argv[2]);
+		    if (font < -1 || font >= DBNumFonts)
+		    {
+			if (DBNumFonts == 0)
+			    TxError("No vector outline fonts are loaded.\n");
+			else
+			    TxError("Font index out of range (0 to %d)\n",
+					DBNumFonts - 1);
+		    }
+		    else if (font == -1)
+			TxPrintf("default\n");
+		    else
+			TxPrintf("%s\n", DBFontList[font]->mf_name);
+		}
+		else if ((cmd->tx_argc == 3 || cmd->tx_argc == 4) &&
+				!StrIsInt(cmd->tx_argv[2]))
+		{
+		    font = DBNameToFont(cmd->tx_argv[2]);
+		    if (font < -1)
+		    {
+			float scale = 1.0;
+			if ((cmd->tx_argc == 4) && StrIsNumeric(cmd->tx_argv[3]))
+			    scale = (float)atof(cmd->tx_argv[3]);
+			if (DBLoadFont(cmd->tx_argv[2], scale) == 0)
+			    TxPrintf("%s\n", DBFontList[DBNumFonts - 1]->mf_name);
+			font = DBNameToFont(cmd->tx_argv[2]);
+			if (font < -1) break;
+		    }
+		}
+	    
+		if (EditCellUse)
+		{
+		    SelEnumLabels(&DBAllTypeBits, FALSE, (bool *)NULL,
+				cmdLabelFontFunc, (cmd->tx_argc == 3) ?
+				(ClientData)&font : (ClientData)NULL);
+		    SelectTransform(&GeoIdentityTransform);
+		}
+	    }
+	    break;
+
+	case SETLABEL_JUSTIFY:
+	    if (cmd->tx_argc == 3)
+	    {
+		pos = GeoNameToPos(cmd->tx_argv[2], FALSE, TRUE);
+		if (pos < 0) break;
+	    }
+	    if (EditCellUse)
+	    {
+		SelEnumLabels(&DBAllTypeBits, FALSE, (bool *)NULL,
+			cmdLabelJustFunc, (cmd->tx_argc == 3) ?
+			(ClientData)&pos : (ClientData)NULL);
+		SelectTransform(&GeoIdentityTransform);
+	    }
+	    break;
+
+	case SETLABEL_SIZE:
+	    if (cmd->tx_argc == 3)
+	    {
+		if (StrIsNumeric(cmd->tx_argv[2]))
+		    size = cmdScaleCoord(w, cmd->tx_argv[2], TRUE, TRUE, 8);
+		if (size <= 0) break;
+	    }
+	    if (EditCellUse)
+	    {
+		SelEnumLabels(&DBAllTypeBits, FALSE, (bool *)NULL,
+			cmdLabelSizeFunc, (cmd->tx_argc == 3) ?
+			(ClientData)&size : (ClientData)NULL);
+		SelectTransform(&GeoIdentityTransform);
+	    }
+	    break;
+
+	case SETLABEL_OFFSET:
+	    if (cmd->tx_argc == 3)
+	    {
+		char *yp;
+		if ((yp = strchr(cmd->tx_argv[2], ' ')) != NULL)
+		{
+		    offset.p_x = cmdScaleCoord(w, cmd->tx_argv[2], TRUE, TRUE, 8);
+		    offset.p_y = cmdScaleCoord(w, yp, TRUE, FALSE, 8);
+		}
+		else
+		{
+		    TxError("Usage:  setlabel offset <x> <y>\n");
+		    return;
+		}
+	    }
+	    else if (cmd->tx_argc == 4)
+	    {
+		offset.p_x = cmdScaleCoord(w, cmd->tx_argv[2], TRUE, TRUE, 8);
+		offset.p_y = cmdScaleCoord(w, cmd->tx_argv[3], TRUE, FALSE, 8);
+	    }	
+	    if (EditCellUse)
+	    {
+		SelEnumLabels(&DBAllTypeBits, FALSE, (bool *)NULL,
+			cmdLabelOffsetFunc, (cmd->tx_argc != 2) ?
+			(ClientData)&offset : (ClientData)NULL);
+		SelectTransform(&GeoIdentityTransform);
+	    }
+	    break;
+
+	case SETLABEL_ROTATE:
+	    if (cmd->tx_argc == 3)
+	    {
+		if (StrIsInt(cmd->tx_argv[2]))
+		    rotate = atoi(cmd->tx_argv[2]);
+	    }
+	    if (EditCellUse)
+	    {
+		SelEnumLabels(&DBAllTypeBits, FALSE, (bool *)NULL,
+			cmdLabelRotateFunc, (cmd->tx_argc == 3) ?
+			(ClientData)&rotate : (ClientData)NULL);
+		SelectTransform(&GeoIdentityTransform);
+	    }
+	    break;
+
+	case SETLABEL_STICKY:
+	    if (cmd->tx_argc == 3)
+	    {
+		option = Lookup(cmd->tx_argv[2], cmdLabelYesNo);
+		if (option < 0)
+		{
+		    TxError("Unknown sticky option \"%s\"\n", cmd->tx_argv[2]);
+		    break;
+		}
+		flags = (option < 3) ? 0 : LABEL_STICKY;
+	    }
+	    if (EditCellUse)
+	    {
+		SelEnumLabels(&DBAllTypeBits, FALSE, (bool *)NULL,
+			cmdLabelStickyFunc, (cmd->tx_argc == 3) ?
+			(ClientData)&flags : (ClientData)NULL);
+		SelectTransform(&GeoIdentityTransform);
+	    }
+	    break;
+
+	case SETLABEL_LAYER:
+	    if (cmd->tx_argc == 3)
+	    {
+		if (!strcasecmp(cmd->tx_argv[2], "default"))
+		    ttype = -1;
+		else
+		{
+		    ttype = DBTechNoisyNameType(cmd->tx_argv[2]);
+		    if (ttype < 0) break;
+		}
+	    }
+	    if (EditCellUse)
+	    {
+		SelEnumLabels(&DBAllTypeBits, FALSE, (bool *)NULL,
+			cmdLabelLayerFunc, (cmd->tx_argc == 3) ?
+			(ClientData)&ttype : (ClientData)NULL);
+		SelectTransform(&GeoIdentityTransform);
+	    }
+	    break;
+
+
+	default:
+	    TxError("Unknown setlabel option \"%s\"\n", cmd->tx_argv[1]);
+	    break;
+    }
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1685,7 +2321,6 @@ printit:
 }
 
 
-#ifdef NONMANHATTAN
 
 /*
  * ----------------------------------------------------------------------------
@@ -1708,7 +2343,7 @@ CmdSplit(w, cmd)
     MagWindow *w;
     TxCommand *cmd;
 {
-    Rect editRect;
+    Rect editRect, expRect;
     TileTypeBitMask mask1, mask2, *cmask;
     TileType t, dir, side, diag;
     int pNum, direction;
@@ -1767,6 +2402,9 @@ CmdSplit(w, cmd)
 			ui.pu_pNum = pNum;
 			DBNMPaintPlane(EditCellUse->cu_def->cd_planes[pNum],
 				diag, &editRect, DBStdPaintTbl(t, pNum), &ui);
+			GEO_EXPAND(&editRect, 1, &expRect);
+			DBMergeNMTiles(EditCellUse->cu_def->cd_planes[pNum],
+				&expRect, &ui);
 		    }
 	    }
 	}
@@ -1800,7 +2438,7 @@ CmdSplitErase(w, cmd)
     MagWindow *w;
     TxCommand *cmd;
 {
-    Rect editRect;
+    Rect editRect, expRect;
     TileTypeBitMask mask;
     TileType t, dir, side, diag;
     int pNum, direction;
@@ -1853,6 +2491,9 @@ CmdSplitErase(w, cmd)
 		    ui.pu_pNum = pNum;
 		    DBNMPaintPlane(EditCellUse->cu_def->cd_planes[pNum],
 				diag, &editRect, DBStdEraseTbl(t, pNum), &ui);
+		    GEO_EXPAND(&editRect, 1, &expRect);
+		    DBMergeNMTiles(EditCellUse->cu_def->cd_planes[pNum],
+				&expRect, &ui);
 		}
 	}
     }
@@ -1863,8 +2504,6 @@ CmdSplitErase(w, cmd)
     DRCCheckThis (EditCellUse->cu_def, TT_CHECKPAINT, &editRect);
 }
 
-#endif
-
 
 /*
  * ----------------------------------------------------------------------------
@@ -1897,7 +2536,7 @@ CmdStretch(w, cmd)
     CellDef *rootDef;
     int xdelta, ydelta;
 
-    if (cmd->tx_argc > 4)
+    if (cmd->tx_argc > 3)
     {
 	TxError("Usage: %s [direction [amount]]\n", cmd->tx_argv[0]);
 	return;
