@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/commands/CmdTZ.c,v 1.22 2009/05/13 15:01:23 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/commands/CmdTZ.c,v 1.8 2010/06/24 12:37:15 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -99,6 +99,32 @@ checkForPaintFunc(cellDef, arg)
     return 0;
 }
 
+/*
+ * ----------------------------------------------------------------------------
+ * CmdCheckForPaintFunc --
+ *
+ *   This routine searches the database for any paint belonging to a
+ *   cell other than an internally-used one.  A FALSE result means that
+ *   no technology-dependent paint exists in the layout, and therefore
+ *   it is safe to change technologies.
+ *
+ * Results:
+ *	TRUE if technology-dependent paint exists in the database, FALSE
+ *	if not.
+ *
+ * Side Effects:
+ *	None.
+ * ----------------------------------------------------------------------------
+ */
+
+bool
+CmdCheckForPaintFunc()
+{
+    if (DBCellSrDefs(0, checkForPaintFunc, (ClientData)&DBNumPlanes))
+	return TRUE;
+    return FALSE;
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -307,6 +333,12 @@ CmdTech(w, cmd)
 		    TTMaskAndMask(&lockedLayers, &DBAllButSpaceAndDRCBits);
 		    DBTechPrintTypes(&lockedLayers, TRUE);
 		}
+		else if (!strcmp(cmd->tx_argv[2], "revert"))
+		{
+		    // Copy DBTechActiveLayerBits back to DBActiveLayerBits
+		    TTMaskZero(&DBActiveLayerBits);
+		    TTMaskSetMask(&DBActiveLayerBits, &DBTechActiveLayerBits);
+		}
 		else
 		    DBTechPrintCanonicalType(cmd->tx_argv[2]);
 	    }
@@ -1053,11 +1085,9 @@ cmdWhatPaintFunc(rect, type, mask)
     TileType type;		/* Type of this piece of paint. */
     TileTypeBitMask *mask;	/* Place to OR in type's bit. */
 {
-#ifdef NONMANHATTAN 
     if (type & TT_DIAGONAL)
 	type = (type & TT_SIDE) ? (type & TT_RIGHTMASK) >> 14 :
 		(type & TT_LEFTMASK);
-#endif
     TTMaskSetType(mask, type);
     return 0;
 }
