@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/database/DBcellbox.c,v 1.2 2008/01/04 20:34:50 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/database/DBcellbox.c,v 1.3 2008/12/11 18:36:43 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -198,7 +198,8 @@ DBCellSetModified(cellDef, ismod)
  *
  * Side effects:
  *	Sets cellUse->cu_bbox to be the bounding box for the indicated CellUse
- *	in coordinates of that CellUse's parent.
+ *	in coordinates of that CellUse's parent,  and cellUse->cu_extended
+ *	to the bounding box of the cell use including all rendered text labels.
  *
  * ----------------------------------------------------------------------------
  */
@@ -207,8 +208,8 @@ void
 DBComputeUseBbox(use)
     CellUse *use;
 {
-    Rect *box;
-    Rect childRect;
+    Rect *box, *extended;
+    Rect childRect, childExtend;
     int xdelta, ydelta;
 
     xdelta = use->cu_xsep * (use->cu_xhi - use->cu_xlo);
@@ -217,29 +218,40 @@ DBComputeUseBbox(use)
     if (ydelta < 0) ydelta = (-ydelta);
 
     box = &(use->cu_def->cd_bbox);
+    extended = &(use->cu_def->cd_extended);
+
     if (use->cu_xsep < 0)
     {
 	childRect.r_xbot = box->r_xbot - xdelta;
 	childRect.r_xtop = box->r_xtop;
+	childExtend.r_xbot = extended->r_xbot - xdelta;
+	childExtend.r_xtop = extended->r_xtop;
     }
     else
     {
 	childRect.r_xbot = box->r_xbot;
 	childRect.r_xtop = box->r_xtop + xdelta;
+	childExtend.r_xbot = extended->r_xbot;
+	childExtend.r_xtop = extended->r_xtop + xdelta;
     }
 
     if (use->cu_ysep < 0)
     {
 	childRect.r_ybot = box->r_ybot - ydelta;
 	childRect.r_ytop = box->r_ytop;
+	childExtend.r_ybot = extended->r_ybot - ydelta;
+	childExtend.r_ytop = extended->r_ytop;
     }
     else
     {
 	childRect.r_ybot = box->r_ybot;
 	childRect.r_ytop = box->r_ytop + ydelta;
+	childExtend.r_ybot = extended->r_ybot;
+	childExtend.r_ytop = extended->r_ytop + ydelta;
     }
 
     GeoTransRect(&use->cu_transform, &childRect, &use->cu_bbox);
+    GeoTransRect(&use->cu_transform, &childExtend, &use->cu_extended);
 }
 
 /*
@@ -596,7 +608,7 @@ dbReComputeBboxFunc(cellDef, boundProc, recurseProc)
     void (*recurseProc)();
 {
     bool degenerate;
-    Rect rect, area, *box;
+    Rect rect, area, extended, *box;
     Rect redisplayArea;
     CellUse *use;
     CellDef *parent;
@@ -609,8 +621,13 @@ dbReComputeBboxFunc(cellDef, boundProc, recurseProc)
 
     if (cellDef->cd_flags & CDFIXEDBBOX) return;
 
-    foundAny = FALSE;
-    for (pNum = 0; pNum < DBNumPlanes; pNum++)
+    /*
+     * Include area of subcells separately
+     */
+    if ((foundAny = DBBoundCellPlane(cellDef, TRUE, &rect)) > 0)
+	area = rect;
+
+    for (pNum = PL_PAINTBASE; pNum < DBNumPlanes; pNum++)
 	if (pNum != PL_DRC_CHECK)
 	    if ((*boundProc)(cellDef->cd_planes[pNum], &rect))
 	    {
@@ -622,7 +639,7 @@ dbReComputeBboxFunc(cellDef, boundProc, recurseProc)
 	    }
 
     /*
-     * Include the area of labels too.
+     * Include the area of labels, too.
      */
     for (label = cellDef->cd_labels; label != NULL;  label = label->lab_next)
     {
@@ -644,6 +661,14 @@ dbReComputeBboxFunc(cellDef, boundProc, recurseProc)
 	}
     }
 
+    extended = area;
+    if (foundAny)
+    {
+	for (label = cellDef->cd_labels; label != NULL;  label = label->lab_next)
+	    if (label->lab_font >= 0)
+		GeoInclude(&label->lab_bbox, &extended);
+    }
+
     /*
      * If there is nothing at all, produce a 1x1 box with its
      * lower left corner at the origin.
@@ -667,6 +692,8 @@ dbReComputeBboxFunc(cellDef, boundProc, recurseProc)
     if (area.r_ybot == area.r_ytop)
 	area.r_ytop = area.r_ybot + 1;
 
+    if (degenerate) extended = area;
+
     /* Did the bounding box change?  If not then there's no need to
      * recompute the parents.  If the cell has no material, then
      * we must always propagate upwards.  This is because DBClearCellDef
@@ -674,7 +701,7 @@ dbReComputeBboxFunc(cellDef, boundProc, recurseProc)
      * (it isn't safe for DBClearCellDef to propagate).  At this point
      * the propagation must be done.
      */
-    box = &cellDef->cd_bbox;
+    box = &cellDef->cd_extended;
     if ((area.r_xbot == box->r_xbot && area.r_ybot == box->r_ybot
 	    && area.r_xtop == box->r_xtop && area.r_ytop == box->r_ytop)
 	    && !degenerate)
@@ -706,6 +733,7 @@ dbReComputeBboxFunc(cellDef, boundProc, recurseProc)
 	}
 
     cellDef->cd_bbox = area;
+    cellDef->cd_extended = extended;
 
     /*
      * Relink each of the uses into the subcell tile planes
@@ -716,14 +744,14 @@ dbReComputeBboxFunc(cellDef, boundProc, recurseProc)
 
     for (use = cellDef->cd_parents; use != NULL; use = use->cu_nextuse)
     {
-	redisplayArea = use->cu_bbox;
+	redisplayArea = use->cu_extended;
 	DBComputeUseBbox(use);
 	if ((parent = use->cu_parent) != (CellDef *) NULL)
 	{
 	    parent->cd_flags |= CDBOXESCHANGED;
 	    DBPlaceCell(use, parent);
 	    (*recurseProc)(parent);
-	    (void) GeoInclude(&use->cu_bbox, &redisplayArea);
+	    (void) GeoInclude(&use->cu_extended, &redisplayArea);
 	    DBWAreaChanged(parent, &redisplayArea, (int) ~use->cu_expandMask,
 		&DBAllButSpaceBits);
 	}
