@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/database/DBcellcopy.c,v 1.10 2009/04/28 12:47:25 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/database/DBcellcopy.c,v 1.13 2010/06/24 12:37:15 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -46,7 +46,7 @@ static PaintResultType (*dbCurPaintTbl)[NT][NT] = DBPaintResultTbl;
  * The following variable points to the version of DBPaintPlane used
  * for painting during yanks.  This is occasionally switched by clients
  * such as the design-rule checker that need to use, for example,
- * DBPaintPlaneMergeOnce instead of the standard version.
+ * DBPaintPlaneMark instead of the standard version.
  */
 static void (*dbCurPaintPlane)() = DBPaintPlaneWrapper;
 
@@ -89,7 +89,7 @@ struct copyLabelArg
  *
  * DBPaintPlaneWrapper --
  *
- *    Simple wrapper to DBPaintPlane.  Argument "type" is ignored.
+ *    Simple wrapper to DBPaintPlane.
  *    Note that this function is passed as a pointer on occasion, so
  *    it cannot be replaced with a macro!
  *
@@ -104,20 +104,41 @@ DBPaintPlaneWrapper(def, pNum, type, area, undo)
     Rect *area;
     PaintUndoInfo *undo;
 {
-#ifdef NONMANHATTAN
     TileType loctype = type & TT_LEFTMASK;
-#else
-    TileType loctype = type;
-#endif
+    Rect expand;
 
     undo->pu_pNum = pNum;
-#ifdef NONMANHATTAN
     DBNMPaintPlane(def->cd_planes[pNum], type, area,
 		dbCurPaintTbl[pNum][loctype], undo);
-#else
-    DBPaintPlane(def->cd_planes[pNum], area,
-		dbCurPaintTbl[pNum][loctype], undo);
-#endif
+    GEO_EXPAND(area, 1, &expand);
+    DBMergeNMTiles(def->cd_planes[pNum], &expand, undo);
+}
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * DBPaintPlaneMark --
+ *
+ *    Another wrapper function to DBPaintPlane.  This one is used for
+ *    hierarchical DRC, and ensures that tiles are never painted twice
+ *    on the same pass, so as not to cause false overlap errors.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+void
+DBPaintPlaneMark(def, pNum, type, area, undo)
+    CellDef *def;
+    int pNum;
+    TileType type;
+    Rect *area;
+    PaintUndoInfo *undo;
+{
+    TileType loctype = type & TT_LEFTMASK;
+
+    undo->pu_pNum = pNum;
+    DBNMPaintPlane0(def->cd_planes[pNum], type, area,
+		dbCurPaintTbl[pNum][loctype], undo, TRUE);
 }
 
 /*
@@ -141,11 +162,7 @@ DBPaintPlaneActive(def, pNum, type, area, undo)
     Rect *area;
     PaintUndoInfo *undo;
 {
-#ifdef NONMANHATTAN
     TileType loctype = type & TT_LEFTMASK;
-#else
-    TileType loctype = type;
-#endif
     TileType t;
 
     if (DBIsContact(loctype))
@@ -169,7 +186,6 @@ DBPaintPlaneActive(def, pNum, type, area, undo)
 	DBPaintPlaneWrapper(def, pNum, type, area, undo);
 }
 
-#ifdef NONMANHATTAN
 /*
  *-----------------------------------------------------------------------------
  *
@@ -208,7 +224,6 @@ DBCellCopyManhattanPaint(scx, mask, xMask, targetUse)
 
     (void) DBTreeSrTiles(scx, mask, xMask, dbCopyManhattanPaint, (ClientData) &arg);
 }
-#endif
 
 
 /*
@@ -251,7 +266,7 @@ DBCellCopyAllPaint(scx, mask, xMask, targetUse)
     locMask = *mask;
     DBMaskAddStacking(&locMask);
 
-    (void) DBTreeSrTiles(scx, &locMask, xMask, dbCopyAllPaint, (ClientData) &arg);
+    DBTreeSrTiles(scx, &locMask, xMask, dbCopyAllPaint, (ClientData) &arg);
 }
 
 /*
@@ -305,7 +320,8 @@ DBCellCopyAllLabels(scx, mask, xMask, targetUse, pArea)
 	pArea->r_xtop = -1;
     }
     (void) DBTreeSrLabels(scx, mask, xMask, (TerminalPath *) 0,
-			dbCopyAllLabels, (ClientData) &arg);
+			TF_LABEL_ATTACH, dbCopyAllLabels,
+			(ClientData) &arg);
 }
 
     /*ARGSUSED*/
@@ -317,25 +333,37 @@ dbCopyAllLabels(scx, lab, tpath, arg)
     struct copyLabelArg *arg;
 {
     Rect labTargetRect;
-    int targetPos;
+    Point labOffset;
+    int targetPos, labRotate;
     CellDef *def;
 
     def = arg->cla_targetUse->cu_def;
     if (!GEO_LABEL_IN_AREA(&lab->lab_rect, &(scx->scx_area))) return 0;
     GeoTransRect(&scx->scx_trans, &lab->lab_rect, &labTargetRect);
     targetPos = GeoTransPos(&scx->scx_trans, lab->lab_just);
+    GeoTransPointDelta(&scx->scx_trans, &lab->lab_offset, &labOffset);
+    labRotate = GeoTransAngle(&scx->scx_trans, lab->lab_rotate);
 
     /* Eliminate duplicate labels.  Don't pay any attention to layers
      * in deciding on duplicates:  if text and position match, it's a
      * duplicate.
      */
 
-    (void) DBEraseLabelsByContent(def, &labTargetRect, targetPos,
-        -1, lab->lab_text);
-    (void) DBPutLabel(def, &labTargetRect, targetPos,
+    DBEraseLabelsByContent(def, &labTargetRect, -1, lab->lab_text);
+    DBPutFontLabel(def, &labTargetRect, lab->lab_font,
+		lab->lab_size, labRotate, &labOffset, targetPos,
 		lab->lab_text, lab->lab_type, lab->lab_flags);
     if (arg->cla_bbox != NULL)
-	(void) GeoIncludeAll(&labTargetRect, arg->cla_bbox);
+    {
+	GeoIncludeAll(&labTargetRect, arg->cla_bbox);
+
+	/* Rendered font labels include the bounding box of the text itself */
+	if (lab->lab_font >= 0)
+	{
+	    GeoTransRect(&scx->scx_trans, &lab->lab_bbox, &labTargetRect);
+	    GeoIncludeAll(&labTargetRect, arg->cla_bbox);
+	}
+    }
     return 0;
 }
 
@@ -439,7 +467,8 @@ DBCellCopyLabels(scx, mask, xMask, targetUse, pArea)
     CellDef *def = targetUse->cu_def;
     Rect labTargetRect;
     Rect *rect = &scx->scx_area;
-    int targetPos;
+    int targetPos, labRotate;
+    Point labOffset;
     CellUse *sourceUse = scx->scx_use;
 
     if (pArea != NULL)
@@ -458,22 +487,24 @@ DBCellCopyLabels(scx, mask, xMask, targetUse, pArea)
 	{
 	    GeoTransRect(&scx->scx_trans, &lab->lab_rect, &labTargetRect);
 	    targetPos = GeoTransPos(&scx->scx_trans, lab->lab_just);
+	    GeoTransPointDelta(&scx->scx_trans, &lab->lab_offset, &labOffset);
+	    labRotate = GeoTransAngle(&scx->scx_trans, lab->lab_rotate);
+
 
 	    /* Eliminate duplicate labels.  Don't pay any attention to
 	     * type when deciding on duplicates, since types can change
 	     * later and then we'd have a duplicate.
 	     */
 
-	    (void) DBEraseLabelsByContent(def, &labTargetRect, targetPos,
-		    -1, lab->lab_text);
-	    (void) DBPutLabel(def, &labTargetRect, targetPos,
-		    lab->lab_text, lab->lab_type, lab->lab_flags, TRUE);
+	    DBEraseLabelsByContent(def, &labTargetRect, -1, lab->lab_text);
+	    DBPutFontLabel(def, &labTargetRect, lab->lab_font,
+			lab->lab_size, labRotate, &labOffset, targetPos,
+			lab->lab_text, lab->lab_type, lab->lab_flags);
 	    if (pArea != NULL)
 		(void) GeoIncludeAll(&labTargetRect, pArea);
 	}
 }
 
-#ifdef NONMANHATTAN
 /***
  *** Filter function for paint: Ignores diagonal (split) tiles for
  *** purposes of selection searches.
@@ -520,7 +551,6 @@ dbCopyManhattanPaint(tile, cxp)
     (*dbCurPaintPlane)(def, pNum, type, &targetRect, &ui);
     return (0);
 }
-#endif
 
 
 /***
@@ -548,7 +578,6 @@ dbCopyAllPaint(tile, cxp)
      * a sanity check.
      */
 
-#ifdef NONMANHATTAN
     bool splittile = FALSE;
     TileType dinfo = 0;
     
@@ -559,7 +588,6 @@ dbCopyAllPaint(tile, cxp)
 	type = (SplitSide(tile)) ? SplitRightType(tile) :
 			SplitLeftType(tile);
     }
-#endif
 
     if (type == TT_SPACE)
 	return 0;
@@ -608,7 +636,6 @@ dbCopyAllPaint(tile, cxp)
 
     def->cd_flags |= CDMODIFIED|CDGETNEWSTAMP;
 
-#ifdef NONMANHATTAN
     /* Nonmanhattan geometry requires slightly different handling. */
     /* Paint the whole tile and clip by erasing areas outside the  */
     /* clipping rectangle.					   */
@@ -699,10 +726,8 @@ topbottom:
 	}
     }
     else
-#endif
-
-    /* Clip against the target area */
-    GEOCLIP(&targetRect, &arg->caa_rect);
+	/* Clip against the target area */
+	GEOCLIP(&targetRect, &arg->caa_rect);
 
 splitdone:
 
