@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/database/DBcellname.c,v 1.8 2009/08/03 13:43:58 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/database/DBcellname.c,v 1.5 2009/07/27 00:56:56 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -1007,7 +1007,7 @@ DBUsePrint(CellName, who, dolist)
 		    {
 			/* DBTreeFindUse() should really take a def, not a use */
 			fakeuse.cu_def = celldef;
-			DBTreeFindUse(CellName, celluse, &scx);
+			DBTreeFindUse(CellName, &fakeuse, &scx);
 			if (scx.scx_use != NULL) break;
 		    }
 		}
@@ -1276,6 +1276,7 @@ DBCellDefAlloc()
     cellDef->cd_bbox.r_ybot = 0;
     cellDef->cd_bbox.r_xtop = 1;
     cellDef->cd_bbox.r_ytop = 1;
+    cellDef->cd_extended = cellDef->cd_bbox;
     cellDef->cd_name = (char *) NULL;
     cellDef->cd_file = (char *) NULL;
 #ifdef FILE_LOCKS
