@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/database/DBcellsrch.c,v 1.7 2010/08/15 14:35:27 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/database/DBcellsrch.c,v 1.7 2010/09/15 21:53:22 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -121,16 +121,12 @@ DBTreeSrTiles(scx, mask, xMask, func, cdarg)
     filter.tf_arg = cdarg;
     filter.tf_mask = mask;
     filter.tf_xmask = xMask;
-#ifdef NONMANHATTAN
     filter.tf_dinfo = 0;
-#endif
     filter.tf_planes = DBTechTypesToPlanes(mask);
 
     return dbCellTileSrFunc(scx, &filter);
 }
 
-#ifdef NONMANHATTAN
-
 /*
  * DBTreeSrNMTiles --
  *	This is a variant of the above in which the search is over
@@ -173,8 +169,6 @@ DBTreeSrNMTiles(scx, dinfo, mask, xMask, func, cdarg)
     return dbCellTileSrFunc(scx, &filter);
 }
 
-#endif
-
 /*
  * dbCellTileSrFunc --
  *
@@ -208,18 +202,16 @@ dbCellTileSrFunc(scx, fp)
 	if (PlaneMaskHasPlane(fp->tf_planes, pNum))
 	{
 	    context.tc_plane = pNum;
-#ifdef NONMANHATTAN
 	    if (fp->tf_dinfo & TT_DIAGONAL)
 	    {
+		// TileType dinfo = DBTransformDiagonal(fp->tf_dinfo, &scx->scx_trans);
 		TileType dinfo = DBInvTransformDiagonal(fp->tf_dinfo, &scx->scx_trans);
 		if (DBSrPaintNMArea((Tile *) NULL, def->cd_planes[pNum],
 			dinfo, &scx->scx_area, fp->tf_mask,
 			fp->tf_func, (ClientData) &context))
 		    return 1;
 	    }
-	    else
-#endif
-	    if (DBSrPaintArea((Tile *) NULL, def->cd_planes[pNum],
+	    else if (DBSrPaintArea((Tile *) NULL, def->cd_planes[pNum],
 		    &scx->scx_area, fp->tf_mask,
 		    fp->tf_func, (ClientData) &context))
 		return 1;
@@ -493,7 +485,7 @@ DBNoTreeSrTiles(scx, mask, xMask, func, cdarg)
  */
 
 int
-DBTreeSrLabels(scx, mask, xMask, tpath, func, cdarg)
+DBTreeSrLabels(scx, mask, xMask, tpath, flags, func, cdarg)
     SearchContext *scx;		/* Pointer to search context specifying
 				 * a cell use to search, an area in the
 				 * coordinates of the cell's def, and a
@@ -514,6 +506,11 @@ DBTreeSrLabels(scx, mask, xMask, tpath, func, cdarg)
 				 * new pathname components as we encounter
 				 * them.
 				 */
+    unsigned char flags;	/* Flags to denote whether labels should be
+				 * searched according to the area of the
+				 * attachment, the area of the label itself,
+				 * or both.
+				 */
     int (*func)();		/* Function to apply at each qualifying tile */
     ClientData cdarg;		/* Client data for above function */
 {
@@ -523,6 +520,7 @@ DBTreeSrLabels(scx, mask, xMask, tpath, func, cdarg)
     CellUse *cellUse = scx->scx_use;
     CellDef *def = cellUse->cu_def;
     TreeFilter filter;
+    bool is_touching;
     int dbCellLabelSrFunc();
 
     ASSERT(def != (CellDef *) NULL, "DBTreeSrLabels");
@@ -533,7 +531,13 @@ DBTreeSrLabels(scx, mask, xMask, tpath, func, cdarg)
     for (lab = def->cd_labels; lab; lab = lab->lab_next)
     {
 	if (SigInterruptPending) break;
-	if (GEO_TOUCH(&lab->lab_rect, r) && TTMaskHasType(mask, lab->lab_type))
+	is_touching = FALSE;
+	if ((lab->lab_font < 0) || (flags & TF_LABEL_ATTACH))
+	    is_touching = GEO_TOUCH(&lab->lab_rect, r);
+	if (!is_touching && (flags & TF_LABEL_DISPLAY) && (lab->lab_font >= 0))
+	    is_touching = GEO_TOUCH(&lab->lab_bbox, r);
+
+	if (is_touching && TTMaskHasType(mask, lab->lab_type))
 	    if ((*func)(scx, lab, tpath, cdarg))
 		return (1);
     }
@@ -543,6 +547,7 @@ DBTreeSrLabels(scx, mask, xMask, tpath, func, cdarg)
     filter.tf_mask = mask;
     filter.tf_xmask = xMask;
     filter.tf_tpath = tpath;
+    filter.tf_flags = flags;
     /* filter.tf_planes is unused */
 
     /* Visit each child CellUse recursively.
@@ -582,6 +587,7 @@ dbCellLabelSrFunc(scx, fp)
     CellDef *def = scx->scx_use->cu_def;
     char *tnext;
     int result;
+    bool has_overlap;
 
     ASSERT(def != (CellDef *) NULL, "dbCellLabelSrFunc");
     if (!DBDescendSubcell(scx->scx_use, fp->tf_xmask)) return 0;
@@ -606,8 +612,14 @@ dbCellLabelSrFunc(scx, fp)
 
     result = 0;
     for (lab = def->cd_labels; lab; lab = lab->lab_next)
-	if (GEO_OVERLAP(&lab->lab_rect, r)
-		&& TTMaskHasType(mask, lab->lab_type))
+    {
+	has_overlap = FALSE;
+	if ((lab->lab_font < 0) || (fp->tf_flags & TF_LABEL_ATTACH))
+	    has_overlap = GEO_OVERLAP(&lab->lab_rect, r);
+	if (!has_overlap && (fp->tf_flags & TF_LABEL_DISPLAY) && (lab->lab_font >= 0))
+	    has_overlap = GEO_OVERLAP(&lab->lab_bbox, r);
+
+	if (has_overlap && TTMaskHasType(mask, lab->lab_type))
 	{
 	    if ((*fp->tf_func)(scx, lab, fp->tf_tpath, fp->tf_arg))
 	    {
@@ -615,6 +627,7 @@ dbCellLabelSrFunc(scx, fp)
 		goto cleanup;
 	    }
 	}
+    }
 
     /* Now visit each child use recursively */
     if (DBCellSrArea(scx, dbCellLabelSrFunc, (ClientData) fp))
@@ -803,13 +816,11 @@ dbSeeTypesAllSrFunc(tile, cxp)
     TiToRect(tile, &tileRect);
     if (GEO_OVERLAP((&tileRect), area))
     {
-#ifdef NONMANHATTAN 
 	if (IsSplit(tile))
 	    TTMaskSetType(mask, SplitSide(tile) ?
 			SplitRightType(tile) : SplitLeftType(tile));
         else
-#endif
-	TTMaskSetType(mask, TiGetType(tile));
+	    TTMaskSetType(mask, TiGetType(tile));
     }
     return 0;
 }
@@ -1504,9 +1515,7 @@ dbTileScaleFunc(tile, scvals)
 {
     TileType type;
     Rect targetRect;
-#ifdef NONMANHATTAN
     TileType exact;
-#endif
 
     TiToRect(tile, &targetRect);
 
@@ -1525,7 +1534,6 @@ dbTileScaleFunc(tile, scvals)
     }
 
     type = TiGetTypeExact(tile);
-#ifdef NONMANHATTAN
     exact = type;
     if (IsSplit(tile))
 	type = (SplitSide(tile)) ? SplitRightType(tile) : SplitLeftType(tile);
@@ -1533,12 +1541,6 @@ dbTileScaleFunc(tile, scvals)
 		((scvals->doCIF) ? CIFPaintTable :
 		DBStdPaintTbl(type, scvals->pnum)),
 		(PaintUndoInfo *)NULL);
-#else
-    DBPaintPlane(scvals->ptarget, &targetRect,
-		((scvals->doCIF) ? CIFPaintTable :
-		DBStdPaintTbl(type, scvals->pnum)),
-		(PaintUndoInfo *)NULL);
-#endif
     return 0;
 }
 
@@ -1664,6 +1666,10 @@ dbScaleCell(cellDef, scalen, scaled)
 	DBScalePoint(&bbox->r_ll, scalen, scaled);
 	DBScalePoint(&bbox->r_ur, scalen, scaled);
 
+	bbox = &use->cu_extended;
+	DBScalePoint(&bbox->r_ll, scalen, scaled);
+	DBScalePoint(&bbox->r_ur, scalen, scaled);
+
 	DBScaleValue(&use->cu_transform.t_c, scalen, scaled);
 	DBScaleValue(&use->cu_transform.t_f, scalen, scaled);
 
@@ -1726,14 +1732,28 @@ dbScaleCell(cellDef, scalen, scaled)
     /* Check consistency of several global pointers	*/
     /* WARNING:  This list may not be complete!		*/
 
-    /* Also scale the position of all labels */
+    /* Also scale the position of all labels. */
+    /* If labels are the rendered-font type, scale the size as well */
 
     if (cellDef->cd_labels)
     {
+	int i;
 	for (lab = cellDef->cd_labels; lab; lab = lab->lab_next)
 	{
 	    DBScalePoint(&lab->lab_rect.r_ll, scalen, scaled);
 	    DBScalePoint(&lab->lab_rect.r_ur, scalen, scaled);
+
+	    if (lab->lab_font >= 0)
+	    {
+		DBScalePoint(&lab->lab_offset, scalen, scaled);
+		DBScaleValue(&lab->lab_size, scalen, scaled);
+
+		DBScalePoint(&lab->lab_bbox.r_ll, scalen, scaled);
+		DBScalePoint(&lab->lab_bbox.r_ur, scalen, scaled);
+
+		for (i = 0; i < 4; i++)
+                    DBScalePoint(&lab->lab_corners[i], scalen, scaled);
+	    }
 	}
     }
 
@@ -1743,6 +1763,8 @@ donecell:
 
     DBScalePoint(&cellDef->cd_bbox.r_ll, scalen, scaled);
     DBScalePoint(&cellDef->cd_bbox.r_ur, scalen, scaled);
+    DBScalePoint(&cellDef->cd_extended.r_ll, scalen, scaled);
+    DBScalePoint(&cellDef->cd_extended.r_ur, scalen, scaled);
 
     return 0;
 }
