@@ -20,10 +20,11 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/database/DBconnect.c,v 1.5 2010/09/15 18:15:51 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/database/DBconnect.c,v 1.6 2010/09/15 18:15:40 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
+#include <string.h>		// for memcpy()
 #include "utils/magic.h"
 #include "utils/geometry.h"
 #include "tiles/tile.h"
@@ -71,9 +72,7 @@ typedef struct
 {
     Rect		area;		/* Area to process */
     TileTypeBitMask	*connectMask;	/* Connection mask for search */
-#ifdef NONMANHATTAN
     TileType		dinfo;		/* Info about triangular search areas */
-#endif
 } conSrArea;
 
 struct conSrArg2
@@ -93,8 +92,6 @@ struct conSrArg2
 
 #define CSA2_LIST_START_SIZE 256
 
-#ifdef NONMANHATTAN
-
 /*
  *-----------------------------------------------------------------
  * DBTransformDiagonal --
@@ -139,12 +136,12 @@ DBTransformDiagonal(oldtype, trans)
  *-----------------------------------------------------------------
  * DBInvTransformDiagonal --
  *
- *	This is the same as the routine above, but finds the
- *	tranformation of a diagonal in a parent cell as seen
- *	from the child's coordinate system.  This comes down
- *	to merely swapping transform components b and d, since
- *	translation is not considered.
- *
+ *	This is equivalent to the routine above, but inverted, so
+ *	that the result is correct for the orientation of the
+ *	triangle in the coordinate system of the child cell,
+ *	rather than the parent cell (which comes down to merely
+ *	swapping transform positions b and d, since translation
+ *	isn't considered).
  *-----------------------------------------------------------------
  */
 
@@ -154,11 +151,13 @@ DBInvTransformDiagonal(oldtype, trans)
     Transform *trans;
 {
     TileType dinfo;
-    int o1, o2, o3, dir, side;
-	
+    int o1, o2, o3;
+    int dir, side;
+
     o1 = ((trans->t_e > 0) || (trans->t_b > 0)) ? 1 : 0;
     o2 = ((trans->t_a > 0) || (trans->t_d > 0)) ? 1 : 0;
     o3 = (trans->t_a != 0) ? 1 : 0;
+
     dir = (oldtype & TT_DIRECTION) ? 1 : 0;
     side = ((oldtype & TT_SIDE) ? 1 : 0) ^ o2 ^ (dir | o3);
     dir ^= o1 ^ o2;
@@ -170,8 +169,6 @@ DBInvTransformDiagonal(oldtype, trans)
     return dinfo;
 }
 
-#endif
-
 
 /*
  * ----------------------------------------------------------------------------
@@ -265,6 +262,7 @@ DBSrConnect(def, startArea, mask, connect, bounds, func, clientData)
     if (startTile == NULL) return 0;
     /* The following lets us call DBSrConnect recursively */
     else if (startTile->ti_client == (ClientData)1) return 0;
+   
 
     /* Pass 1.  During this pass the client function gets called. */
 
@@ -374,7 +372,6 @@ dbSrConnectFunc(tile, csa)
      * recursively.
      */
     
-#ifdef NONMANHATTAN
     if (IsSplit(tile))
     {
 	if (SplitSide(tile))
@@ -383,26 +380,21 @@ dbSrConnectFunc(tile, csa)
 	    loctype = SplitLeftType(tile);
     }
     else
-#endif
-    loctype = TiGetTypeExact(tile);
+	loctype = TiGetTypeExact(tile);
     connectMask = &csa->csa_connect[loctype];
 
     /* Left side: */
 
-#ifdef NONMANHATTAN
     if (IsSplit(tile) && SplitSide(tile)) goto bottomside;
-#endif
 
     for (t2 = BL(tile); BOTTOM(t2) < tileArea.r_ytop; t2 = RT(t2))
     {
-#ifdef NONMANHATTAN
 	if (IsSplit(t2))
 	{
 	    checktype = SplitRightType(t2);
 	}
 	else
-#endif
-	checktype = TiGetTypeExact(t2);
+	    checktype = TiGetTypeExact(t2);
 	if (TTMaskHasType(connectMask, checktype))
 	{
 	    if (csa->csa_clear)
@@ -417,21 +409,17 @@ dbSrConnectFunc(tile, csa)
     /* Bottom side: */
 
 bottomside:
-#ifdef NONMANHATTAN
     if (IsSplit(tile) && (!(SplitSide(tile) ^ SplitDirection(tile))))
 	goto rightside;
-#endif
 
     for (t2 = LB(tile); LEFT(t2) < tileArea.r_xtop; t2 = TR(t2))
     {
-#ifdef NONMANHATTAN
 	if (IsSplit(t2))
 	{
 	    checktype = SplitTopType(t2);
 	}
 	else
-#endif
-	checktype = TiGetTypeExact(t2);
+	    checktype = TiGetTypeExact(t2);
 	if (TTMaskHasType(connectMask, checktype))
 	{
 	    if (csa->csa_clear)
@@ -446,20 +434,16 @@ bottomside:
     /* Right side: */
 
 rightside:
-#ifdef NONMANHATTAN
     if (IsSplit(tile) && !SplitSide(tile)) goto topside;
-#endif
 
     for (t2 = TR(tile); ; t2 = LB(t2))
     {
-#ifdef NONMANHATTAN
 	if (IsSplit(t2))
 	{
 	    checktype = SplitLeftType(t2);
 	}
 	else
-#endif
-	checktype = TiGetTypeExact(t2);
+	    checktype = TiGetTypeExact(t2);
 	if (TTMaskHasType(connectMask, checktype))
 	{
 	    if (csa->csa_clear)
@@ -475,20 +459,16 @@ rightside:
     /* Top side: */
 topside:
 
-#ifdef NONMANHATTAN
     if (IsSplit(tile) && (SplitSide(tile) ^ SplitDirection(tile))) goto donesides;
-#endif
 
     for (t2 = RT(tile); ; t2 = BL(t2))
     {
-#ifdef NONMANHATTAN
 	if (IsSplit(t2))
 	{
 	    checktype = SplitBottomType(t2);
 	}
 	else
-#endif
-	checktype = TiGetTypeExact(t2);
+	    checktype = TiGetTypeExact(t2);
 	if (TTMaskHasType(connectMask, checktype))
 	{
 	    if (csa->csa_clear)
@@ -519,8 +499,8 @@ donesides:
 	GEO_EXPAND(&newArea, 1, &newArea);
 	for (i = PL_TECHDEPBASE; i < DBNumPlanes; i++)
 	{
+	    if (!PlaneMaskHasPlane(planes, i)) continue;
 	    newcsa.csa_plane = i;
-#ifdef NONMANHATTAN
 	    if (IsSplit(tile))
 	    {
 		if (DBSrPaintNMArea((Tile *) NULL, csa->csa_def->cd_planes[i],
@@ -528,9 +508,7 @@ donesides:
 			dbSrConnectFunc, (ClientData) &newcsa) != 0)
 		    return 1;
 	    }
-	    else
-#endif
-	    if (DBSrPaintArea((Tile *) NULL, csa->csa_def->cd_planes[i],
+	    else if (DBSrPaintArea((Tile *) NULL, csa->csa_def->cd_planes[i],
 			&newArea, connectMask, dbSrConnectFunc,
 			(ClientData) &newcsa) != 0) return 1;
 	}
@@ -598,14 +576,18 @@ dbcConnectLabelFunc(scx, lab, tpath, csa2)
 {
     CellDef *def = csa2->csa2_use->cu_def;
     Rect r;
-    int pos;
+    Point offset;
+    int pos, rotate;
     int dbcConnectFunc();		/* Forward declaration */
 
     GeoTransRect(&scx->scx_trans, &lab->lab_rect, &r);
     pos = GeoTransPos(&scx->scx_trans, lab->lab_just);
+    GeoTransPointDelta(&scx->scx_trans, &lab->lab_offset, &offset);
+    rotate = GeoTransAngle(&scx->scx_trans, lab->lab_rotate);
 
-    DBEraseLabelsByContent(def, &r, pos, -1, lab->lab_text);
-    DBPutLabel(def, &r, pos, lab->lab_text, lab->lab_type, lab->lab_flags);
+    DBEraseLabelsByContent(def, &r, -1, lab->lab_text);
+    DBPutFontLabel(def, &r, lab->lab_font, lab->lab_size, rotate, &offset,
+		pos, lab->lab_text, lab->lab_type, lab->lab_flags);
 
     if (lab->lab_flags & PORT_DIR_MASK)
     {
@@ -673,9 +655,7 @@ dbcConnectFunc(tile, cx)
     SearchContext *scx = cx->tc_scx;
     SearchContext scx2;
     TileType loctype = TiGetTypeExact(tile);
-#ifdef NONMANHATTAN
     TileType dinfo = 0;
-#endif
     int pNum = cx->tc_plane;
     CellDef *def;
 
@@ -705,13 +685,11 @@ dbcConnectFunc(tile, cx)
     GeoClip(&newarea, csa2->csa2_bounds);
     if (GEO_RECTNULL(&newarea)) return 0;
 
-#ifdef NONMANHATTAN
     if (IsSplit(tile))
     {
 	dinfo = DBTransformDiagonal(loctype, &scx->scx_trans);
 	loctype = (SplitSide(tile)) ? SplitRightType(tile) : SplitLeftType(tile);
     }
-#endif
 
     /* See if the destination cell contains stuff over the whole
      * current area (on its home plane) that is connected to it.
@@ -727,32 +705,37 @@ dbcConnectFunc(tile, cx)
 
     if (DBIsContact(loctype))
     {
-	TileType ctype;
-	TileTypeBitMask *cMask, *rMask = DBResidueMask(loctype);
+//	TileType ctype;
+//	TileTypeBitMask *cMask, *rMask = DBResidueMask(loctype);
 
-	TTMaskSetOnlyType(&notConnectMask, loctype);
+//	TTMaskSetOnlyType(&notConnectMask, loctype);
 
 	/* Different contact types may share residues (6/18/04) */
 	/* Use TTMaskIntersect(), not TTMaskEqual()---types	*/
 	/* which otherwise stack may be in separate cells	*/
 	/* (12/1/05)						*/
 
-	for (ctype = TT_TECHDEPBASE; ctype < DBNumUserLayers; ctype++)
-	{
-	    cMask = DBResidueMask(ctype);
-	    if (TTMaskIntersect(rMask, cMask))
-		TTMaskSetType(&notConnectMask, ctype);
-	}
+//	for (ctype = TT_TECHDEPBASE; ctype < DBNumUserLayers; ctype++)
+//	{
+//	    if (DBIsContact(ctype))
+//	    {
+//		cMask = DBResidueMask(ctype);
+//		if (TTMaskIntersect(rMask, cMask))
+//		    TTMaskSetType(&notConnectMask, ctype);
+//	    }
+//	}
 
 	/* The mask of contact types must include all stacked contacts */
-	for (ctype = DBNumUserLayers; ctype < DBNumTypes; ctype++)
-	{
-	    cMask = DBResidueMask(ctype);
-	    if (TTMaskHasType(cMask, loctype))
-		TTMaskSetType(&notConnectMask, ctype);
-	}
+//	for (ctype = DBNumUserLayers; ctype < DBNumTypes; ctype++)
+//	{
+//	    cMask = DBResidueMask(ctype);
+//	    if (TTMaskHasType(cMask, loctype))
+//		TTMaskSetType(&notConnectMask, ctype);
+//	}
+//	TTMaskCom(&notConnectMask);
 
-	TTMaskCom(&notConnectMask);
+	TTMaskZero(&notConnectMask);
+	TTMaskSetMask(&notConnectMask, &DBNotConnectTbl[loctype]);
     }
     else
     {
@@ -763,31 +746,19 @@ dbcConnectFunc(tile, cx)
     /* which have not already been painted.			*/
 
     def = csa2->csa2_use->cu_def;
-#ifdef NONMANHATTAN
     if (DBSrPaintNMArea((Tile *) NULL, def->cd_planes[pNum],
 		dinfo, &newarea, &notConnectMask, dbcUnconnectFunc,
 		(ClientData) NULL) == 0)
 	return 0;
-#else
-    if (DBSrPaintArea((Tile *) NULL, def->cd_planes[pNum],
-		&newarea, &notConnectMask, dbcUnconnectFunc,
-		(ClientData) NULL) == 0)
-	return 0;
-#endif
 
     /* Paint this tile into the destination cell.  This
      * marks its area has having been processed.  Then recycle
      * the storage for the current list element.
      */
 
-#ifdef NONMANHATTAN
     DBNMPaintPlane(def->cd_planes[pNum], dinfo,
 		&newarea, DBStdPaintTbl(loctype, pNum),
 		(PaintUndoInfo *) NULL);
-#else
-    DBPaintPlane(def->cd_planes[pNum], &newarea,
-	    DBStdPaintTbl(loctype, pNum), (PaintUndoInfo *) NULL);
-#endif
 
     /* Copy information from original context into new search context */
     scx2 = *csa2->csa2_topscx;
@@ -797,7 +768,8 @@ dbcConnectFunc(tile, cx)
     /* tile area and plane, and add them to the destination.	*/
 
     DBTreeSrLabels(&scx2, connectMask, csa2->csa2_xMask, NULL,
-			dbcConnectLabelFunc, (ClientData) csa2);
+			TF_LABEL_ATTACH, dbcConnectLabelFunc,
+			(ClientData) csa2);
 
     /* Since the whole area of this tile hasn't been recorded,
      * we must process its area to find any other tiles that
@@ -808,7 +780,6 @@ dbcConnectFunc(tile, cx)
      * even just touch.
      */
 
-#ifdef NONMANHATTAN
     /* Only extend those sides bordering the diagonal tile */
 	  
     if (dinfo & TT_DIAGONAL)
@@ -825,14 +796,11 @@ dbcConnectFunc(tile, cx)
     }
     else
     {
-#endif
-    newarea.r_ybot -= 1;
-    newarea.r_ytop += 1;
-    newarea.r_xbot -= 1;
-    newarea.r_xtop += 1;
-#ifdef NONMANHATTAN
+	newarea.r_ybot -= 1;
+	newarea.r_ytop += 1;
+	newarea.r_xbot -= 1;
+	newarea.r_xtop += 1;
     }
-#endif
 
     /* Register the area and connection mask as needing to be processed */
 
@@ -847,24 +815,21 @@ dbcConnectFunc(tile, cx)
 	csa2->csa2_size *= 2;
 
 	newlist = (conSrArea *)mallocMagic(csa2->csa2_size * sizeof(conSrArea));
-	for (i = 0; i < lastsize; i++)
-	{
-	    newlist[i].area = csa2->csa2_list[i].area;
-	    newlist[i].connectMask = csa2->csa2_list[i].connectMask;
-#ifdef NONMANHATTAN
-	    newlist[i].dinfo = csa2->csa2_list[i].dinfo;
-#endif
-	    
-	}
+	memcpy((void *)newlist, (void *)csa2->csa2_list,
+			(size_t)lastsize * sizeof(conSrArea));
+	// for (i = 0; i < lastsize; i++)
+	// {
+	//     newlist[i].area = csa2->csa2_list[i].area;
+	//     newlist[i].connectMask = csa2->csa2_list[i].connectMask;
+	//     newlist[i].dinfo = csa2->csa2_list[i].dinfo;
+	// }
 	freeMagic((char *)csa2->csa2_list);
 	csa2->csa2_list = newlist;
     }
 
     csa2->csa2_list[csa2->csa2_top].area = newarea;
     csa2->csa2_list[csa2->csa2_top].connectMask = connectMask;
-#ifdef NONMANHATTAN
     csa2->csa2_list[csa2->csa2_top].dinfo = dinfo;
-#endif
 
     return 0;
 }
@@ -927,9 +892,7 @@ DBTreeCopyConnect(scx, mask, xMask, connect, area, destUse)
 {
     struct conSrArg2 csa2;
     TileTypeBitMask *newmask;
-#ifdef NONMANHATTAN
     TileType newtype;
-#endif
 
     csa2.csa2_use = destUse;
     csa2.csa2_xMask = xMask;
@@ -951,18 +914,14 @@ DBTreeCopyConnect(scx, mask, xMask, connect, area, destUse)
     {
 	newmask = csa2.csa2_list[csa2.csa2_top].connectMask;
 	scx->scx_area = csa2.csa2_list[csa2.csa2_top].area;
-#ifdef NONMANHATTAN
 	newtype = csa2.csa2_list[csa2.csa2_top].dinfo;
-#endif
 	csa2.csa2_top--;
 
-#ifdef NONMANHATTAN
 	if (newtype & TT_DIAGONAL)
 	    DBTreeSrNMTiles(scx, newtype, newmask, xMask, dbcConnectFunc,
 			(ClientData) &csa2);
 	else
-#endif
-	DBTreeSrTiles(scx, newmask, xMask, dbcConnectFunc, (ClientData) &csa2);
+	    DBTreeSrTiles(scx, newmask, xMask, dbcConnectFunc, (ClientData) &csa2);
     }
     freeMagic((char *)csa2.csa2_list);
 
