@@ -17,13 +17,13 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/database/DBio.c,v 1.16 2009/01/15 15:57:11 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/database/DBio.c,v 1.5 2010/06/24 12:37:15 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#ifdef __STDC__
+#ifdef STDC
 #include <unistd.h>
 #endif
 #include <ctype.h>
@@ -225,9 +225,9 @@ file_is_not_writeable(name)
  *
  *	7. If the cell contains labels, then the labels are preceded
  *	by the line "<< labels >>".  Each label is one line of the form
- *		rlabel <layer> <xbot> <ybot> <xtop> <ytop> <position> <text>
+ *		rlabel <layer> [s] <xbot> <ybot> <xtop> <ytop> <position> <text>
  *		 or
- *		flabel <layer> <xbot> <ybot> <xtop> <ytop> <position> <fontname>
+ *		flabel <layer> [s] <xbot> <ybot> <xtop> <ytop> <position> <fontname>
  *			<size> <rotation> <offsetx> <offsety> <text>
  *		 or
  *		label <layer> <x> <y> <position> <text>
@@ -492,41 +492,25 @@ dbCellReadDef(f, cellDef, name, ignoreTech)
 	 * in the file beginning with 'r'.
 	 */
 nextrect:
-#ifdef NONMANHATTAN 
 	while (((c = getc(f)) == 'r') || (c == 't'))
 	{
 	    TileType dinfo;
 	    int dir;
-#else
-	while (((c = getc(f)) == 'r') || (c == 't'))
-	{
-#endif
 	    /*
 	     * GetRect actually reads the rest of the line up to
 	     * a trailing newline or EOF.
 	     */
 	    if (c == 't')
 	    {
-#ifdef NONMANHATTAN  
 		if ((dir = GetRect(f, 3, rp, n, d)) == 0) goto badfile;
 		dir >>= 1;
 		dinfo = TT_DIAGONAL | ((dir & 0x2) ? TT_SIDE : 0) |
 			((((dir & 0x2) >> 1) ^ (dir & 0x1)) ?
 			TT_DIRECTION : 0);
-#else
-		if (!GetRect(f, 3, rp, n, d)) goto badfile;
-		TxPrintf("Error: cell %s contains non-manhattan geometry\n",
-			cellDef->cd_name);
-		TxPrintf("unsupported by this magic executable.  Skipping.\n");
-		fflush(stdout);
-		continue;
-#endif
 	    }
             else  
 	    {
-#ifdef NONMANHATTAN  
 		dinfo = 0;
-#endif
 		if (!GetRect(f, 4, rp, n, d)) goto badfile;
 	    }
 
@@ -561,10 +545,8 @@ nextrect:
 		    if (TTMaskHasType(rmask, rtype))
 		    {
 			loctype = rtype;
-#ifdef NONMANHATTAN
 			if (dinfo & TT_SIDE) loctype <<= 14;
 			loctype |= dinfo;
-#endif
 			DBPaint(cellDef, rp, loctype);
 		    }
 		}
@@ -995,7 +977,7 @@ dbReadOpen(cellDef, name, setFileName)
 			 * cell was loaded.
 			 */
 {
-    FILE *f;
+    FILE *f = NULL;
     char *filename, *realname;
     bool is_locked;
 
@@ -1273,7 +1255,7 @@ badTransform:
     subCellDef = DBCellLookDef(cellname);
     if (subCellDef == (CellDef *) NULL)
     {
-	subCellDef = DBCellNewDef(cellname, (char *) NULL);
+	subCellDef = DBCellNewDef(cellname, (char *)NULL);
 	subCellDef->cd_timestamp = childStamp;
 
 	/* Make sure rectangle is non-degenerate */
@@ -1289,6 +1271,7 @@ badTransform:
 		    r.r_xbot, r.r_ybot, r.r_xtop, r.r_ytop);
 	}
 	subCellDef->cd_bbox = r;
+	subCellDef->cd_extended = r;
     }
     else if (DBIsAncestor(subCellDef, cellDef))
     {
@@ -1630,8 +1613,10 @@ dbReadLabels(cellDef, line, len, f, scalen, scaled)
 {
     char layername[50], text[1024], port_use[50], port_class[50];
     TileType type;
-    int ntok, orient;
+    int ntok, orient, size, rotate, font, flags;
+    Point offset;
     Rect r;
+    char stickyflag[2];
 
     /* Get first label line */
     if (dbFgets(line, len, f) == NULL) return (FALSE);
@@ -1654,28 +1639,78 @@ dbReadLabels(cellDef, line, len, f, scalen, scaled)
 	 */
 	if (line[0] == 'r')
 	{
-	    if (sscanf(line, "rlabel %49s %d %d %d %d %d %99[^\n]",
-		    layername, &r.r_xbot, &r.r_ybot, &r.r_xtop, &r.r_ytop,
-			       &orient, text) != 7)
+	    if (sscanf(line, "rlabel %*49s %1s", stickyflag) == 1)
 	    {
-		TxError("Skipping bad \"rlabel\" line: %s", line);
+		font = -1;
+		if (*stickyflag == 's')
+		{
+		    flags = LABEL_STICKY;
+		    if (sscanf(line, "rlabel %49s %c %d %d %d %d %d %99[^\n]",
+			layername, &stickyflag[0], &r.r_xbot, &r.r_ybot,
+			&r.r_xtop, &r.r_ytop, &orient, text) != 8)
+		    {
+			TxError("Skipping bad \"rlabel\" line: %s", line);
+			goto nextlabel;
+		    }
+		}
+		else
+		{
+		    flags = 0;
+		    if (sscanf(line, "rlabel %49s %d %d %d %d %d %99[^\n]",
+			layername, &r.r_xbot, &r.r_ybot, &r.r_xtop, &r.r_ytop,
+			&orient, text) != 7)
+		    {
+			TxError("Skipping bad \"rlabel\" line: %s", line);
+			goto nextlabel;
+		    }
+		}
+	    }
+	    else
+	    {
+		TxError("Skipping bad \"flabel\" line: %s", line);
 		goto nextlabel;
 	    }
 	}
 	else if (line[0] == 'f')
 	{
-	    /* Forward-compatibility with magic v. 8.0+ font labels.	*/
-	    /* We ignore the extra information and process them like	*/
-	    /* plain labels.						*/
-
-	    if (sscanf(line,
-		    "flabel %49s %d %d %d %d %d %*255s %*d %*d %*d %*d %99[^\n]",
-		    layername, &r.r_xbot, &r.r_ybot, &r.r_xtop, &r.r_ytop,
-				&orient, text) != 7)
+	    char fontname[256];
+	    if (sscanf(line, "flabel %*49s %1s", stickyflag) == 1)
+	    {
+		if (*stickyflag == 's')
+		{
+		    flags = LABEL_STICKY;
+		    if (sscanf(line,
+			    "flabel %49s %c %d %d %d %d %d %255s %d %d %d %d %99[^\n]",
+			    layername, &stickyflag[0], &r.r_xbot, &r.r_ybot, &r.r_xtop,
+			    &r.r_ytop, &orient, fontname, &size, &rotate, &offset.p_x,
+			    &offset.p_y, text) != 13)
+		    {
+			TxError("Skipping bad \"flabel\" line: %s", line);
+			goto nextlabel;
+	    	    }
+		}
+		else
+		{
+		    flags = 0;
+		    if (sscanf(line,
+			    "flabel %49s %d %d %d %d %d %255s %d %d %d %d %99[^\n]",
+			    layername, &r.r_xbot, &r.r_ybot, &r.r_xtop, &r.r_ytop,
+			    &orient, fontname, &size, &rotate, &offset.p_x,
+			    &offset.p_y, text) != 12)
+		    {
+			TxError("Skipping bad \"flabel\" line: %s", line);
+			goto nextlabel;
+	    	    }
+		}
+	    }
+	    else
 	    {
 		TxError("Skipping bad \"flabel\" line: %s", line);
 		goto nextlabel;
 	    }
+
+	    font = DBNameToFont(fontname);
+	    if (font < -1) font = -1;	/* Force default font if font is unknown */
 	}
 	else if (line[0] == 'p')
 	{
@@ -1692,7 +1727,8 @@ dbReadLabels(cellDef, line, len, f, scalen, scaled)
 		TxError("Skipping bad \"port\" line: %s", line);
 		goto nextlabel;
 	    }
-	    lab->lab_flags = idx;
+	    lab->lab_flags &= ~LABEL_STICKY;
+	    lab->lab_flags |= idx;
 	    for (pptr = &ppos[0]; *pptr != '\0'; pptr++)
 	    {
 		switch(*pptr)
@@ -1768,7 +1804,7 @@ dbReadLabels(cellDef, line, len, f, scalen, scaled)
 	}
 	else	/* deprecated, retained for backward compatibility */
 	{
-	    if (sscanf(line, "label %1023s %d %d %d %99[^\n]",
+	    if (sscanf(line, "label %49s %d %d %d %99[^\n]",
 		    layername, &r.r_xbot, &r.r_ybot, &orient, text) != 5)
 	    {
 		TxError("Skipping bad \"label\" line: %s", line);
@@ -1776,6 +1812,7 @@ dbReadLabels(cellDef, line, len, f, scalen, scaled)
 	    }
 	    r.r_xtop = r.r_xbot;
 	    r.r_ytop = r.r_ybot;
+	    font = -1;
 	}
 
 	if (scalen > 1)
@@ -1784,6 +1821,12 @@ dbReadLabels(cellDef, line, len, f, scalen, scaled)
 	    r.r_ybot *= scalen;
 	    r.r_xtop *= scalen;
 	    r.r_ytop *= scalen;
+	    if (font >= 0)
+	    {
+		size *= scalen;
+		offset.p_x *= scalen;
+		offset.p_y *= scalen;
+	    }
 	}
 	if (scaled > 1)
 	{
@@ -1791,6 +1834,12 @@ dbReadLabels(cellDef, line, len, f, scalen, scaled)
 	    r.r_ybot /= scaled;
 	    r.r_xtop /= scaled;
 	    r.r_ytop /= scaled;
+	    if (font >= 0)
+	    {
+		size /= scaled;
+		offset.p_x /= scaled;
+		offset.p_y /= scaled;
+	    }
 	}
 	type = DBTechNameType(layername);
 	if (type < 0)
@@ -1822,7 +1871,11 @@ dbReadLabels(cellDef, line, len, f, scalen, scaled)
 		if (TTMaskHasType(rmask, rtype))
 		    type = rtype;
 	}
-	DBPutLabel(cellDef, &r, orient, text, type, 0);
+	if (font < 0)
+	    DBPutLabel(cellDef, &r, orient, text, type, flags);
+	else
+	    DBPutFontLabel(cellDef, &r, font, size, rotate, &offset,
+			orient, text, type, flags);
 
 nextlabel:
 	if (dbFgets(line, len, f) == NULL)
@@ -2049,6 +2102,7 @@ DBCellWriteFile(cellDef, f)
     FILE *f;		/* The FILE to write to */
 {
     int dbWritePaintFunc(), dbWriteCellFunc(), dbWritePropFunc();
+    int dbClearCellFunc();
     Label *lab;
     struct writeArg arg;
     int pNum;
@@ -2134,6 +2188,9 @@ DBCellWriteFile(cellDef, f)
     if (DBCellEnum(cellDef, dbWriteCellFunc, (ClientData) &arg))
 	goto ioerror;
 
+    /* Clear flags set in dbWriteCellFunc */
+    DBCellEnum(cellDef, dbClearCellFunc, (ClientData)NULL);
+
     /* Now labels */
     if (cellDef->cd_labels)
     {
@@ -2142,13 +2199,32 @@ DBCellWriteFile(cellDef, f)
 	FPRINTF(f, "<< labels >>\n");
 	for (lab = cellDef->cd_labels; lab; lab = lab->lab_next)
 	{
-	    sprintf(lstring, "rlabel %s %d %d %d %d %d %s\n",
+	    if (strlen(lab->lab_text) == 0) continue;	// Shouldn't happen
+	    if (lab->lab_font < 0)
+	    {
+		sprintf(lstring, "rlabel %s %s%d %d %d %d %d %s\n",
 			DBTypeLongName(lab->lab_type),
+			((lab->lab_flags & LABEL_STICKY) ? "s " : ""),
 			lab->lab_rect.r_xbot / reducer,
 			lab->lab_rect.r_ybot / reducer,
 			lab->lab_rect.r_xtop / reducer,
 			lab->lab_rect.r_ytop / reducer,
 			lab->lab_just, lab->lab_text);
+	    }
+	    else
+	    {
+		sprintf(lstring, "flabel %s %s%d %d %d %d %d %s %d %d %d %d %s\n",
+			DBTypeLongName(lab->lab_type),
+			((lab->lab_flags & LABEL_STICKY) ? "s " : ""),
+			lab->lab_rect.r_xbot / reducer,
+			lab->lab_rect.r_ybot / reducer,
+			lab->lab_rect.r_xtop / reducer,
+			lab->lab_rect.r_ytop / reducer,
+			lab->lab_just, DBFontList[lab->lab_font]->mf_name,
+			lab->lab_size / reducer, lab->lab_rotate,
+			lab->lab_offset.p_x / reducer,
+			lab->lab_offset.p_y / reducer, lab->lab_text);
+	    }
 	    FPRINTF(f, lstring);
 	    if (lab->lab_flags & PORT_DIR_MASK)
 	    {
@@ -2347,12 +2423,14 @@ DBCellWrite(cellDef, fileName)
      */
     if (fileName)
     {
-	realname = (char *) mallocMagic((unsigned) (strlen(fileName)+strlen(DBSuffix)+1));
+	realname = (char *) mallocMagic(strlen(fileName) + strlen(DBSuffix) + 1);
 	(void) sprintf(realname, "%s%s", fileName, DBSuffix);
+
 	/* Bug fix: 7/17/99, Michael D. Godfrey:  Forces		*/
 	/* cd_name and cd_file to ALWAYS be the same, otherwise ugly	*/
 	/* surprises can occur after saving a file as a different	*/
 	/* filename.							*/
+
 	cellDef->cd_file = StrDup(&cellDef->cd_file, realname);
     }
     else if (cellDef->cd_file)
@@ -2391,6 +2469,7 @@ DBCellWrite(cellDef, fileName)
 #else
 	TxPrintf("File %s is read_only and cannot be written\n", realname);
 #endif
+	freeMagic(realname);
 	return(FALSE);
     }
 
@@ -2549,20 +2628,29 @@ DBCellWrite(cellDef, fileName)
 	}
     }
 
-    /* Everything worked. */
+    /* Everything worked so far. */
+
     (void) StrDup(&cellDef->cd_file, expandname);
     result = TRUE;
     {
-    	 struct stat thestat;
-	 realf = fopen(expandname,"r");
-	 fstat(fileno(realf),&thestat);
-	 if (thestat.st_size != DBFileOffset)
-	 {
-              cellDef->cd_flags |= CDMODIFIED;
-	      TxError("Warning: I/O error in writing file\n");
-	 }
-	 fclose(realf);
-	 realf = NULL;
+	struct stat thestat;
+	realf = fopen(expandname,"r");
+	if (realf == NULL)
+	{
+	    cellDef->cd_flags |= CDMODIFIED;
+	    TxError("Warning: Cannot open file for writing!\n");
+	}
+	else
+	{
+	    fstat(fileno(realf),&thestat);
+	    if (thestat.st_size != DBFileOffset)
+	    {
+		cellDef->cd_flags |= CDMODIFIED;
+		TxError("Warning: I/O error in writing file\n");
+	    }
+	    fclose(realf);
+	}
+	realf = NULL;
     }
 
 cleanup:
@@ -2600,9 +2688,8 @@ dbWritePaintFunc(tile, cdarg)
     char pstring[256];
     struct writeArg *arg = (struct writeArg *) cdarg;
     TileType type = TiGetType(tile);
+    TileTypeBitMask *lMask, *rMask;
 
-#ifdef NONMANHATTAN
-    TileTypeBitMask *rMask, *lMask;
     int dir;
 
     if (IsSplit(tile))      
@@ -2627,9 +2714,7 @@ dbWritePaintFunc(tile, cdarg)
 	else
 	    return 0;
     }
-    else  
-#endif   
-    if (type != arg->wa_type)
+    else if (type != arg->wa_type)
     {
 	rMask = DBResidueMask(type);
 	if ((type < DBNumUserLayers) ||
@@ -2646,7 +2731,6 @@ dbWritePaintFunc(tile, cdarg)
 	arg->wa_found = TRUE;
     }
 
-#ifdef NONMANHATTAN
     if (IsSplit(tile))
     {
 	static char *pos_diag[] = {"nw", "sw", "se", "ne"};
@@ -2657,15 +2741,38 @@ dbWritePaintFunc(tile, cdarg)
 		pos_diag[dir]);
     }
     else
-#endif
-
-    sprintf(pstring, "rect %d %d %d %d\n",
+	sprintf(pstring, "rect %d %d %d %d\n",
 	    LEFT(tile) / arg->wa_reducer, BOTTOM(tile) / arg->wa_reducer,
 	    RIGHT(tile) / arg->wa_reducer, TOP(tile) / arg->wa_reducer);
     FPRINTR(arg->wa_file,pstring);
     return 0;
 }
 
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * dbClearCellFunc --
+ *
+ * Filter function that clears flags set by dbWriteCellFunc.
+ *
+ * Results:
+ *	Always returns 0
+ *
+ * Side effects:
+ *	Cell use flags changed.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+int
+dbClearCellFunc(cellUse, cdarg)
+    CellUse *cellUse;	/* Cell use */
+    ClientData cdarg;	/* Not used */
+{
+    cellUse->cu_def->cd_flags &= ~CDVISITED;
+    return 0;
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -2691,15 +2798,36 @@ dbWriteCellFunc(cellUse, cdarg)
     struct writeArg *arg = (struct writeArg *) cdarg;
     Transform *t;
     Rect *b;
-    char     cstring[256];
+    char     cstring[256], *pathend;
 
     t = &(cellUse->cu_transform);
     b = &(cellUse->cu_def->cd_bbox);
-    sprintf(cstring, "use %s %c%s\n", cellUse->cu_def->cd_name,
+
+    if (cellUse->cu_def->cd_file == NULL)
+	pathend = NULL;
+    else
+    {
+	pathend = strrchr(cellUse->cu_def->cd_file, '/');
+	if (pathend != NULL) *pathend = '\0';
+    }
+
+    if ((cellUse->cu_def->cd_flags & CDVISITED) || (pathend == NULL))
+    {
+	sprintf(cstring, "use %s %c%s\n", cellUse->cu_def->cd_name,
 		(cellUse->cu_flags & CU_LOCKED) ? CULOCKCHAR : ' ',
 		cellUse->cu_id);
+    }
+    else
+    {
+	sprintf(cstring, "use %s %c%s %s\n", cellUse->cu_def->cd_name,
+		(cellUse->cu_flags & CU_LOCKED) ? CULOCKCHAR : ' ',
+		cellUse->cu_id, cellUse->cu_def->cd_file);
+    }
     FPRINTR(arg->wa_file, cstring);
 
+    cellUse->cu_def->cd_flags |= CDVISITED;
+    if (pathend != NULL) *pathend = '/';
+
     if ((cellUse->cu_xlo != cellUse->cu_xhi)
 	    || (cellUse->cu_ylo != cellUse->cu_yhi))
     {
