@@ -19,7 +19,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/database/DBtiles.c,v 1.2 2006/11/07 19:48:07 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/database/DBtiles.c,v 1.2 2010/06/24 12:37:15 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -43,8 +43,6 @@ struct dbCheck
 
 int dbCheckMaxHFunc(), dbCheckMaxVFunc();
 
-#ifdef NONMANHATTAN
-
 /*
  * --------------------------------------------------------------------
  *
@@ -313,8 +311,6 @@ enum_next:
     return (0);
 }
 
-#endif	/* NONMANHATTAN */
-
 
 /*
  * --------------------------------------------------------------------
@@ -390,7 +386,6 @@ enumerate:
 	if (SigInterruptPending)
 	    return (1);
 
-#ifdef NONMANHATTAN
 	/* Only perform func() on diagonal tiles if the mask includes the  */
 	/* tile type for either the left or right sides of the tile (could */
 	/* also use top and bottom)---by definition, opposite sides must   */
@@ -443,9 +438,8 @@ enumerate:
 	    }
 	}
 	else
-#endif
-	if (TTMaskHasType(mask, TiGetType(tp)) && (*func)(tp, arg))
-	    return (1);
+	    if (TTMaskHasType(mask, TiGetType(tp)) && (*func)(tp, arg))
+		return (1);
 
 	tpnew = TR(tp);
 	if (LEFT(tpnew) < rect->r_xtop)
@@ -552,7 +546,6 @@ enumerate:
 	if (SigInterruptPending)
 	    return (1);
 
-#ifdef NONMANHATTAN
 	/* Only perform func() on diagonal tiles if the mask includes the  */
 	/* tile type for either the left or right sides of the tile (could */
 	/* also use top and bottom)---by definition, opposite sides must   */
@@ -607,10 +600,9 @@ enumerate:
 	    }
 	}
 	else
-#endif
-	if (TTMaskHasType(mask, TiGetType(tp)) && tp->ti_client == client
-		&& (*func)(tp, arg))
-	    return (1);
+	    if (TTMaskHasType(mask, TiGetType(tp)) && tp->ti_client == client
+				&& (*func)(tp, arg))
+		return (1);
 
 	tpnew = TR(tp);
 	if (LEFT(tpnew) < rect->r_xtop)
