@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/database/DBtpaint2.c,v 1.8 2009/12/30 13:45:48 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/database/DBtpaint2.c,v 1.4 2009/12/30 13:42:33 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -538,19 +538,11 @@ dbComposePaintContact(lpImage, lpPaint)
  *
  * dbComposeSubsetResidues --
  *
- *	Create a mask of unique types left over when the type of lpErase
- *	is erased from the type of lpImage.  The rules are as follows:
- *
- *	1) If lpErase is a non-contact type, the mask contains all contact
- *	   types whose residues are subsets of the "have" type but are not
- *	   supersets of the "erase" type.
- *	2) If lpErase is a contact type, the mask contains all contact types
- *	   whose residues are subsets of the "have" type but do not share
- *	   more than one plane with the "erase" type.
- *	3) All types that are subsets of other types in the mask are removed.
+ *	Create a mask of all contact types whose residues are subsets
+ *	of the "have" type but are not supersets of the "erase" type.
  *
  * Results:
- *	None.
+ *	True if residues of types in outMask overlap, False if not.
  *
  * Side Effects:
  *	outMask is filled with the mask of types.
@@ -558,17 +550,16 @@ dbComposePaintContact(lpImage, lpPaint)
  * ----------------------------------------------------------------------------
  */
 
-void
+bool 
 dbComposeSubsetResidues(lpImage, lpErase, outMask)
     LayerInfo *lpImage, *lpErase;
     TileTypeBitMask *outMask;
 {
     TileTypeBitMask ires;
     TileTypeBitMask smask, overlapmask;
-    LayerInfo *li, *lc;
-    int n, m;
-    TileType t, stype;
-    PlaneMask cmask;
+    LayerInfo *li;
+    int n;
+    bool rval = FALSE;
 
     /* The residues of lpImage must themselves be decomposed if lpImage	*/
     /* is a stacking type.						*/
@@ -600,65 +591,26 @@ dbComposeSubsetResidues(lpImage, lpErase, outMask)
 	TTMaskAndMask3(&smask, &li->l_residues, &ires);
 	if (TTMaskEqual(&smask, &li->l_residues))
 	{
+	    /* The residues of type cannot be a superset of the */
+	    /* residues of the erase type			*/
+
 	    TTMaskAndMask3(&smask, &li->l_residues, &lpErase->l_residues);
-	    if (lpErase->l_isContact)
+	    if (!TTMaskEqual(&smask, &lpErase->l_residues))
 	    {
-		/* The residues of type li cannot share more than one	*/
-		/* plane with the residues of the erase type		*/
+		TTMaskSetType(outMask, li->l_type);
 
-		for (t = TT_TECHDEPBASE; t < DBNumUserLayers; t++)
-		    if (TTMaskHasType(&smask, t))
-		    {
-			TTMaskClearType(&smask, t);
-			break;
-		    }
+		/* Check if the residues of type overlap one of	*/
+		/* the types already generated.			*/
 
-		if (!TTMaskIsZero(&smask)) continue;
-	    }
-	    else
-	    {
-		/* The residues of type li cannot be a superset of the	*/
-		/* residues of the erase type				*/
-
-		if (TTMaskEqual(&smask, &lpErase->l_residues))
-		    continue;
-	    }
-
-	    TTMaskSetType(outMask, li->l_type);
-
-	    /* Remove any types that are a subset of this type		*/
-	    /* Ignore this type if it is a subset of another type	*/
-	    /* Ignore this type if it overlaps another type and	there	*/
-	    /*	 is no stacking type for the two.			*/
-
-	    for (m = 0; m < dbNumContacts; m++)
-	    {
-		lc = dbContactInfo[m];
-		if (TTMaskHasType(outMask, lc->l_type))
-		{
-		    cmask = lc->l_pmask & li->l_pmask;
-		    if (cmask == lc->l_pmask)	
-			TTMaskClearType(outMask, lc->l_type);
-		    else if (cmask == li->l_pmask)	
-		    {
-			TTMaskClearType(outMask, li->l_type);
-			break;
-		    }
-		    else if (cmask != 0)
-		    {
-			stype = DBTechFindStacking(lc->l_type, li->l_type);
-			if (stype < 0)
-			{
-			    TTMaskClearType(outMask, li->l_type);
-			    break;
-			}
-			else
-			    TTMaskSetType(outMask, stype);
-		    }
-		}
+		TTMaskAndMask3(&smask, &overlapmask, &li->l_residues);
+		if (TTMaskIsZero(&smask))
+		    TTMaskSetMask(&overlapmask, &li->l_residues);
+		else
+		    rval = TRUE;
 	    }
 	}
     }
+    return rval;
 }
 
 
@@ -676,10 +628,11 @@ dbComposeEraseContact(lpImage, lpErase)
     LayerInfo *lpImage, *lpErase;
 {
     int pNum;
-    PlaneMask pmask, overlap;
+    PlaneMask pmask;
     LayerInfo *lp;
     TileTypeBitMask cmask;
-    TileType itype, stype, ctype;
+    TileType itype;
+    bool overlap;
 
     /* The erased planes generally end up with space, so we generate	*/
     /* space as default behavior.  This may be altered in specific	*/
@@ -713,41 +666,42 @@ dbComposeEraseContact(lpImage, lpErase)
     /* If those types overlap, leave the existing type alone.	If they	*/
     /* don't, then paint them.						*/
 
-    dbComposeSubsetResidues(lpImage, lpErase, &cmask);
+    overlap = dbComposeSubsetResidues(lpImage, lpErase, &cmask);
 
-    pmask = lpImage->l_pmask & (~(lpErase->l_pmask));
+    if (overlap)
+    {
+	for (pNum = PL_TECHDEPBASE; pNum < DBNumPlanes; pNum++)
+	    if (PlaneMaskHasPlane(lpImage->l_pmask, pNum))
+//		SETERASE(lpImage->l_type, lpErase->l_type, pNum, TT_SPACE);
+		SETERASE(lpImage->l_type, lpErase->l_type, pNum, lpImage->l_type);
+    }
+    else
+    {
+	pmask = lpImage->l_pmask & (~(lpErase->l_pmask));
+	for (itype = TT_TECHDEPBASE; itype < DBNumTypes; itype++)
+	    if (TTMaskHasType(&cmask, itype))
+	    {
+		lp = &dbLayerInfo[itype];
+		pmask &= ~(lp->l_pmask);
+		for (pNum = PL_TECHDEPBASE; pNum < DBNumPlanes; pNum++)
+		    if (PlaneMaskHasPlane(lp->l_pmask, pNum))
+			if ((lpImage->l_type < DBNumUserLayers)
+				|| (pNum == DBPlane(lpImage->l_type)))
+			    SETERASE(lpImage->l_type, lpErase->l_type, pNum, itype);
+	    }
 
-    for (itype = TT_TECHDEPBASE; itype < DBNumUserLayers; itype++)
-	if (TTMaskHasType(&cmask, itype))
-	{
-	    lp = &dbLayerInfo[itype];
-	    pmask &= ~(lp->l_pmask);
-	    for (pNum = PL_TECHDEPBASE; pNum < DBNumPlanes; pNum++)
-		if (PlaneMaskHasPlane(lp->l_pmask, pNum))
-		{
-		    SETERASE(lpImage->l_type, lpErase->l_type, pNum, itype);
-		}
-	}
+	/* If there are any planes in the image which have not been	*/
+	/* accounted for, then we decompose these into the residues	*/
+	/* of the image.						*/
 
-    for (itype = DBNumUserLayers; itype < DBNumTypes; itype++)
-	if (TTMaskHasType(&cmask, itype))
-	{
-	    lp = &dbLayerInfo[itype];
-	    SETERASE(lpImage->l_type, lpErase->l_type, DBPlane(lp->l_type), itype);
-	}
-
-
-    /* If there are any planes in the image which have not been	*/
-    /* accounted for, then we decompose these into the residues	*/
-    /* of the image.						*/
-
-    for (pNum = PL_TECHDEPBASE; pNum < DBNumPlanes; pNum++)
-	if (PlaneMaskHasPlane(pmask, pNum))
-	{
-	    itype = DBPlaneToResidue(lpImage->l_type, pNum);
-	    SETERASE(lpImage->l_type, lpErase->l_type, pNum, itype);
-	}
+	for (pNum = PL_TECHDEPBASE; pNum < DBNumPlanes; pNum++)
+	    if (PlaneMaskHasPlane(pmask, pNum))
+	    {
+		itype = DBPlaneToResidue(lpImage->l_type, pNum);
+		SETERASE(lpImage->l_type, lpErase->l_type, pNum, itype);
+	    }
 	
+    }
 
     /* Diagnostic (to be removed or commented out) */
 
