@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/database/DBundo.c,v 1.4 2008/01/14 23:31:35 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/database/DBundo.c,v 1.5 2010/09/24 19:53:19 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -41,7 +41,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/
 /***
  *** Identifiers for each of the clients defined here
  ***/
-UndoType dbUndoIDPaint;
+UndoType dbUndoIDPaint, dbUndoIDSplit, dbUndoIDJoin;
 UndoType dbUndoIDPutLabel, dbUndoIDEraseLabel;
 UndoType dbUndoIDOpenCell, dbUndoIDCloseCell;
 UndoType dbUndoIDCellUse;
@@ -52,6 +52,9 @@ UndoType dbUndoIDCellUse;
     /* Paint */
 void dbUndoPaintForw(), dbUndoPaintBack();
 
+    /* Split */
+void dbUndoSplitForw(), dbUndoSplitBack();
+
     /* Labels */
 void dbUndoLabelForw(), dbUndoLabelBack();
 
@@ -133,6 +136,13 @@ DBUndoInit()
 			(UndoEvent *(*)()) NULL, (int (*)()) NULL,
 			dbUndoPaintForw, dbUndoPaintBack, "paint");
 
+    /* Tile Splits */
+    dbUndoIDSplit = UndoAddClient(dbUndoInit, dbUndoCloseCell,
+			(UndoEvent *(*)()) NULL, (int (*)()) NULL,
+			dbUndoSplitForw, dbUndoSplitBack, "split");
+    dbUndoIDJoin = UndoAddClient(dbUndoInit, dbUndoCloseCell,
+			(UndoEvent *(*)()) NULL, (int (*)()) NULL,
+			dbUndoSplitBack, dbUndoSplitForw, "join");
 
     /* Labels */
     dbUndoIDPutLabel = UndoAddClient(nullProc, nullProc,
@@ -228,6 +238,48 @@ dbUndoInit()
  * ============================================================================
  */
 
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * dbUndoSplitForw --
+ * dbUndoSplitBack --
+ *
+ * Play forward/backward a tile split event.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Modifies the database.
+ *
+ * ----------------------------------------------------------------------------
+ */
+void
+dbUndoSplitForw(us)
+    splitUE *us;
+{
+    /* Create internal fracture */
+    if (dbUndoLastCell == NULL) return;
+    DBSplitTile(dbUndoLastCell->cd_planes[us->sue_plane], &us->sue_point,
+		us->sue_splitx);
+}
+
+void
+dbUndoSplitBack(us)
+    splitUE *us;
+{
+    Rect srect;
+    if (dbUndoLastCell == NULL) return;
+
+    srect.r_ll = us->sue_point;
+    srect.r_ur.p_x = us->sue_point.p_x + 1;
+    srect.r_ur.p_y = us->sue_point.p_y + 1;
+
+    /* Remove internal fracture and restore original split tile */
+    DBMergeNMTiles0(dbUndoLastCell->cd_planes[us->sue_plane], &srect,
+		(PaintUndoInfo *)NULL, TRUE);
+}
+
 /***
  *** The procedures to record paint undo events have been expanded
  *** in-line in DBPaintPlane() for speed.
@@ -254,11 +306,8 @@ void
 dbUndoPaintForw(up)
     paintUE *up;
 {
-#ifdef NONMANHATTAN 
     TileType loctype, dinfo;
-
-    if ((up->pue_newtype == up->pue_oldtype) && (up->pue_newtype & TT_DIAGONAL))
-	return;		/* ignore 4-way splits on undo forward */
+    if (dbUndoLastCell == NULL) return;
 
     if (up->pue_oldtype & TT_DIAGONAL)
     {
@@ -277,11 +326,10 @@ dbUndoPaintForw(up)
 		(PaintUndoInfo *) NULL);
     }
     else
-#endif
-    DBPaintPlane(dbUndoLastCell->cd_planes[up->pue_plane], &up->pue_rect,
+	DBPaintPlane(dbUndoLastCell->cd_planes[up->pue_plane], &up->pue_rect,
 			DBStdEraseTbl(up->pue_oldtype, up->pue_plane),
 			(PaintUndoInfo *) NULL);
-#ifdef NONMANHATTAN
+
     if (up->pue_newtype & TT_DIAGONAL)
     {
 	loctype = (up->pue_newtype & TT_LEFTMASK);
@@ -299,8 +347,7 @@ dbUndoPaintForw(up)
 		(PaintUndoInfo *) NULL);
     }
     else
-#endif
-    DBPaintPlane(dbUndoLastCell->cd_planes[up->pue_plane], &up->pue_rect,
+	DBPaintPlane(dbUndoLastCell->cd_planes[up->pue_plane], &up->pue_rect,
 			DBStdPaintTbl(up->pue_newtype, up->pue_plane),
 			(PaintUndoInfo *) NULL);
 endPaintFor:
@@ -313,16 +360,8 @@ void
 dbUndoPaintBack(up)
     paintUE *up;
 {
-#ifdef NONMANHATTAN
     TileType loctype, dinfo;
-
-    if ((up->pue_newtype == up->pue_oldtype) && (up->pue_newtype & TT_DIAGONAL))
-    {
-	/* Paint TT_SPACE into area to remove any internal fractures */
-	DBPaintPlane(dbUndoLastCell->cd_planes[up->pue_plane], &up->pue_rect,
-			DBStdPaintTbl(TT_SPACE, up->pue_plane),
-			(PaintUndoInfo *) NULL);
-    }
+    if (dbUndoLastCell == NULL) return;
 
     if (up->pue_newtype & TT_DIAGONAL)
     {
@@ -341,11 +380,10 @@ dbUndoPaintBack(up)
 		(PaintUndoInfo *) NULL);
     }
     else
-#endif
-    DBPaintPlane(dbUndoLastCell->cd_planes[up->pue_plane], &up->pue_rect,
+	DBPaintPlane(dbUndoLastCell->cd_planes[up->pue_plane], &up->pue_rect,
 			DBStdEraseTbl(up->pue_newtype, up->pue_plane),
 			(PaintUndoInfo *) NULL);
-#ifdef NONMANHATTAN
+
     if (up->pue_oldtype & TT_DIAGONAL)
     {
 	loctype = (up->pue_oldtype & TT_LEFTMASK);
@@ -361,10 +399,12 @@ dbUndoPaintBack(up)
 	DBNMPaintPlane(dbUndoLastCell->cd_planes[up->pue_plane], dinfo,
 		&up->pue_rect, DBStdPaintTbl(loctype, up->pue_plane),
 		(PaintUndoInfo *) NULL);
+
+	DBMergeNMTiles0(dbUndoLastCell->cd_planes[up->pue_plane],
+		&up->pue_rect, (PaintUndoInfo *)NULL, TRUE);
     }
     else
-#endif
-    DBPaintPlane(dbUndoLastCell->cd_planes[up->pue_plane], &up->pue_rect,
+	DBPaintPlane(dbUndoLastCell->cd_planes[up->pue_plane], &up->pue_rect,
 			DBStdPaintTbl(up->pue_oldtype, up->pue_plane),
 			(PaintUndoInfo *) NULL);
 
@@ -382,18 +422,18 @@ endPaintBack:
  * ============================================================================
  */
 
-    typedef struct
-    {
-	Rect	 lue_rect;	/* Location of label */
-	int	 lue_pos;	/* Relative position of label */
-	TileType lue_type;	/* Type of tile labelled */
-	unsigned int lue_flags;	/* Flags for label */
-	char	 lue_text[4];	/* Text of label.  This is a place
-				 * holder only; the actual structure
-				 * is allocated to hold all the bytes
-				 * in the label, plus the null byte.
-				 */
-    } labelUE;
+/* Just define a labelUE to be a Label. */
+
+typedef  Label labelUE;
+#define  lue_type   lab_type
+#define  lue_rect   lab_rect
+#define  lue_just   lab_just
+#define  lue_font   lab_font
+#define  lue_size   lab_size
+#define  lue_rotate lab_rotate
+#define  lue_offset lab_offset
+#define  lue_flags  lab_flags
+#define  lue_text   lab_text
 
     /*
      * labelSize(n) is the size of a labelUE large enough to hold
@@ -420,17 +460,9 @@ endPaintBack:
  */
 
 void
-DBUndoPutLabel(cellDef, rect, pos, text, type, flags)
+DBUndoPutLabel(cellDef, lab)
     CellDef *cellDef;	/* CellDef being modified */
-    Rect *rect;/* Box definining label.  The lower left coordinate
-			 * of the box determines the tile to which the label
-			 * is attached, although it needn't be the physically
-			 * lower left coordinate of the box.
-			 */
-    int pos;		/* Relative position to point */
-    char *text;		/* Text of label */
-    TileType type;	/* Type of tile being labelled */
-    int flags;		/* Flags of label */
+    Label *lab;		/* Label being modified */
 {
     labelUE *lup;
 
@@ -439,15 +471,19 @@ DBUndoPutLabel(cellDef, rect, pos, text, type, flags)
 
     if (cellDef != dbUndoLastCell) dbUndoEdit(cellDef);
     lup = (labelUE *) UndoNewEvent(dbUndoIDPutLabel,
-			(unsigned) labelSize(strlen(text)));
+			(unsigned) labelSize(strlen(lab->lab_text)));
     if (lup == (labelUE *) NULL)
 	return;
 
-    lup->lue_rect = *rect;
-    lup->lue_pos = pos;
-    lup->lue_type = type;
-    lup->lue_flags = flags;
-    strcpy(lup->lue_text, text);
+    lup->lue_rect = lab->lab_rect;
+    lup->lue_just = lab->lab_just;
+    lup->lue_type = lab->lab_type;
+    lup->lue_flags = lab->lab_flags;
+    lup->lue_font = lab->lab_font;
+    lup->lue_size = lab->lab_size;
+    lup->lue_rotate = lab->lab_rotate;
+    lup->lue_offset = lab->lab_offset;
+    strcpy(lup->lue_text, lab->lab_text);
 }
 
 /*
@@ -467,17 +503,9 @@ DBUndoPutLabel(cellDef, rect, pos, text, type, flags)
  */
 
 void
-DBUndoEraseLabel(cellDef, rect, pos, text, type, flags)
+DBUndoEraseLabel(cellDef, lab)
     CellDef *cellDef;	/* Cell being modified */
-    Rect *rect;/* Box definining label.  The lower left coordinate
-			 * of the box determines the tile to which the label
-			 * is attached, although it needn't be the physically
-			 * lower left coordinate of the box.
-			 */
-    int pos;		/* Relative position to point */
-    char *text;		/* Text of label */
-    TileType type;	/* Type of tile being labelled */
-    int flags;		/* Flags for label */
+    Label *lab;		/* Label being modified */
 {
     labelUE *lup;
 
@@ -486,15 +514,19 @@ DBUndoEraseLabel(cellDef, rect, pos, text, type, flags)
 
     if (cellDef != dbUndoLastCell) dbUndoEdit(cellDef);
     lup = (labelUE *) UndoNewEvent(dbUndoIDEraseLabel,
-			(unsigned) labelSize(strlen(text)));
+			(unsigned) labelSize(strlen(lab->lab_text)));
     if (lup == (labelUE *) NULL)
 	return;
 
-    lup->lue_rect = *rect;
-    lup->lue_pos = pos;
-    lup->lue_type = type;
-    lup->lue_flags = flags;
-    strcpy(lup->lue_text, text);
+    lup->lue_rect = lab->lab_rect;
+    lup->lue_just = lab->lab_just;
+    lup->lue_type = lab->lab_type;
+    lup->lue_flags = lab->lab_flags;
+    lup->lue_font = lab->lab_font;
+    lup->lue_size = lab->lab_size;
+    lup->lue_rotate = lab->lab_rotate;
+    lup->lue_offset = lab->lab_offset;
+    strcpy(lup->lue_text, lab->lab_text);
 }
 
 /*
@@ -518,10 +550,14 @@ void
 dbUndoLabelForw(up)
     labelUE *up;
 {
-    (void) DBPutLabel(dbUndoLastCell, &up->lue_rect, up->lue_pos,
-	up->lue_text, up->lue_type, up->lue_flags);
-    DBWLabelChanged(dbUndoLastCell, up->lue_text, &up->lue_rect,
-	up->lue_pos, DBW_ALLWINDOWS);
+    Label *lab;
+
+    if (dbUndoLastCell == NULL) return;
+    lab = DBPutFontLabel(dbUndoLastCell, &up->lue_rect,
+	up->lue_font, up->lue_size, up->lue_rotate,
+	&up->lue_offset, up->lue_just, up->lue_text,
+	up->lue_type, up->lue_flags);
+    DBWLabelChanged(dbUndoLastCell, lab, DBW_ALLWINDOWS);
 
     /*
      * Record that this cell def has changed, for bounding box
@@ -538,8 +574,9 @@ void
 dbUndoLabelBack(up)
     labelUE *up;
 {
+    if (dbUndoLastCell == NULL) return;
     (void) DBEraseLabelsByContent(dbUndoLastCell, &up->lue_rect,
-		up->lue_pos, up->lue_type, up->lue_text);
+		up->lue_type, up->lue_text);
 
     /*
      * Record that this cell def has changed, for bounding box
@@ -572,6 +609,7 @@ dbUndoLabelBack(up)
 	CellDef		*cue_def;
 	CellDef		*cue_parent;
 	Rect		 cue_bbox;
+	Rect		 cue_extended;
 	unsigned char	 cue_flags;
 	char		 cue_id[4];
     } cellUE;
@@ -628,6 +666,7 @@ DBUndoCellUse(use, action)
     up->cue_parent = use->cu_parent;
     up->cue_expandMask = use->cu_expandMask;
     up->cue_bbox = use->cu_bbox;
+    up->cue_extended = use->cu_extended;
     up->cue_flags = use->cu_flags;
     strcpy(up->cue_id, use->cu_id);
 }
@@ -663,6 +702,7 @@ dbUndoCellForw(up)
 	    use->cu_array = up->cue_array;
 	    use->cu_expandMask = up->cue_expandMask;
 	    use->cu_bbox = up->cue_bbox;
+	    use->cu_extended = up->cue_extended;
 	    use->cu_flags = up->cue_flags; 
 	    use->cu_id = StrDup((char **) NULL, up->cue_id);
 	    (void) DBLinkCell(use, up->cue_parent);
@@ -731,6 +771,7 @@ dbUndoCellBack(up)
 	    use->cu_array = up->cue_array;
 	    use->cu_expandMask = up->cue_expandMask;
 	    use->cu_bbox = up->cue_bbox;
+	    use->cu_extended = up->cue_extended;
 	    use->cu_flags = up->cue_flags; 
 	    use->cu_id = StrDup((char **) NULL, up->cue_id);
 	    SigDisableInterrupts ();
