@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/dbwind/DBWcommands.c,v 1.1.1.1 2006/04/10 22:03:14 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/dbwind/DBWcommands.c,v 1.3 2010/06/24 12:37:16 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -50,7 +50,7 @@ extern void CmdLabel(), CmdLoad();
 extern void CmdMove(), CmdNetlist(), CmdPaint(), CmdPath(), CmdPlow();
 extern void CmdPolygon(), CmdPort(), CmdProperty();
 extern void CmdSave(), CmdScaleGrid(), CmdSee();
-extern void CmdSelect(), CmdSideways();
+extern void CmdSelect(), CmdSetLabel(), CmdSideways();
 extern void CmdShell(), CmdSnap();
 extern void CmdStretch(), CmdStraighten();
 extern void CmdTech(), CmdTool(), CmdUnexpand();
@@ -61,10 +61,8 @@ extern void CmdGoto(), CmdFlatten(), CmdXload();
  * Nonmanhattan geometry extensions
  */
 
-#ifdef NONMANHATTAN
 extern void CmdSplit();
 extern void CmdSplitErase();
-#endif
 
 /*
  * Wizard commands
@@ -416,6 +414,9 @@ DBWInitCommands()
 	"select [option]		change selection; type \"select help\"\n\
 			for information on options",
 	CmdSelect, FALSE);
+    WindAddCommand(DBWclientID,
+	"setlabel [option [value]]	place a label",
+	CmdSetLabel, FALSE);
     WindAddCommand(DBWclientID,
 	"shell [command]		execute a command in a subshell",
 	CmdShell, FALSE);
@@ -483,7 +484,6 @@ DBWInitCommands()
 #endif
 
 
-#ifdef NONMANHATTAN 
     /* Add the NonManhattan Geometry extension commands */
     WindAddCommand(DBWclientID,
 	"splitpaint dir layer [layer2]\n\
@@ -494,10 +494,9 @@ DBWInitCommands()
 	"spliterase dir [layer]	erase layers from diagonal corner dir of the\n\
 			edit box",
 	CmdSplitErase, FALSE);   
-#endif
 
 #ifdef MAGIC_WRAPPER
-    /* Add the Tcl commands for exttospice and exttosim */
+    /* Add the Tcl commands for exttospice, exttosim, and aliases ext2spice, ext2sim */
 #ifdef EXT2SIM_AUTO
     WindAddCommand(DBWclientID,
 	"exttosim [args]	convert extracted file(s) to a sim format file;"
@@ -518,7 +517,6 @@ DBWInitCommands()
 	CmdExtToSim, FALSE);
 #endif /* EXT2SIM_AUTO */
 #ifdef EXT2SPICE_AUTO
-    /* Add the alias ext2sim */
     WindAddCommand(DBWclientID,
 	"exttospice [args]	convert extracted file(s) to a SPICE format file;"
 	" type\n\t\t\t\"exttospice help\" for information on options",
@@ -536,8 +534,8 @@ DBWInitCommands()
 	"ext2spice [args]	convert extracted file(s) to a SPICE format file;"
 	" type\n\t\t\t\"ext2spice help\" for information on options",
 	CmdExtToSpice, FALSE);
-#endif /* EXT2SPICE_AUTO */
-#endif
+#endif	/* EXT2SPICE_AUTO */
+#endif  /* MAGIC_WRAPPER */
 
 
 #ifdef USE_READLINE
