@@ -360,7 +360,7 @@ DBWElementRedraw(window, plane)
 		switch (elem->type)
 		{
 		    case ELEMENT_RECT:
-			GrFastBox(&screenArea, NULL);
+			GrFastBox(&screenArea);
 			break;
 		    case ELEMENT_TEXT:
 			p.p_x = screenArea.r_xbot;
@@ -373,7 +373,6 @@ DBWElementRedraw(window, plane)
 		    case ELEMENT_LINE:
 			GrClipLine(screenArea.r_xbot, screenArea.r_ybot,
 				screenArea.r_xtop, screenArea.r_ytop);
-#ifdef NONMANHATTAN
 			/* Draw arrowheads on endpoints, if flags are set */
 
 			if (elem->flags & (DBW_ELEMENT_LINE_ARROWL |
@@ -434,7 +433,6 @@ DBWElementRedraw(window, plane)
 				GrFillPolygon(polyp, 4);
 			    }
 			}
-#endif
 			break;
 		}
 	    }
@@ -500,7 +498,7 @@ dbwElementUndraw(mw, elem)
 		(screenArea.r_ybot <= screenArea.r_ytop))
     {
 	GrSetStuff(STYLE_ERASEALL);
-	GrFastBox(&screenArea, NULL);
+	GrFastBox(&screenArea);
         WindAreaChanged(mw, &screenArea);
     }
     GrUnlock(mw, TRUE);
