@@ -21,7 +21,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/dbwind/DBWfdback.c,v 1.1.1.1 2006/04/10 22:03:14 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/dbwind/DBWfdback.c,v 1.2 2010/06/24 12:37:16 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -162,12 +162,10 @@ DBWFeedbackRedraw(window, plane)
 
 	tmp = fb->fb_area;
 
-#ifdef NONMANHATTAN
 	/* Don't clip non-Manhattan tiles, or else the diagonals are	*/
 	/* clipped to an incorrect position.				*/
 
 	if (!(fb->fb_style & TT_DIAGONAL))
-#endif
 	GeoClip(&tmp, &worldArea);
 
 	/*
@@ -196,11 +194,7 @@ DBWFeedbackRedraw(window, plane)
 	    continue;
 	}
 
-#ifdef NONMANHATTAN
 	newStyle = fb->fb_style & (TT_LEFTMASK | TT_RIGHTMASK);
-#else
-	newStyle = fb->fb_style;
-#endif
 
 	/* Another little trick:  when the feedback area is very small ("very 
 	 * small" is a hand-tuned constant), change all stippled styles to
@@ -221,7 +215,6 @@ DBWFeedbackRedraw(window, plane)
 	    GrSetStuff(curStyle);
 	}
 
-#ifdef NONMANHATTAN
 	if (fb->fb_style & TT_DIAGONAL)
 	    GrDiagonal(&screenArea, fb->fb_style);
 
@@ -239,8 +232,7 @@ DBWFeedbackRedraw(window, plane)
 			screenArea.r_xtop, screenArea.r_ytop);
 	}
 	else
-#endif
-	GrFastBox(&screenArea);
+	    GrFastBox(&screenArea);
     }
 }
 
