@@ -20,12 +20,12 @@
  */
 
 #ifndef	lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/drc/DRCbasic.c,v 1.12 2010/09/21 17:08:49 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/drc/DRCbasic.c,v 1.7 2010/09/20 21:13:22 tim Exp $";
 #endif	/* not lint */
 
 #include <sys/types.h>
 #include <stdio.h>
-#include <string.h>		/* for memcpy() */
+#include <string.h>		// for memcpy()
 #include "utils/magic.h"
 #include "utils/geometry.h"
 #include "tiles/tile.h"
@@ -33,6 +33,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/
 #include "database/database.h"
 #include "drc/drc.h"
 #include "utils/signals.h"
+#include "utils/maxrect.h"
 #include "utils/malloc.h"
 
 int dbDRCDebug = 0;
@@ -54,8 +55,7 @@ static DRCCookie drcOverlapCookie = {
 
 extern int areaCheck();
 extern int drcTile();
-
-#ifdef NONMANHATTAN
+extern MaxRectsData *drcCanonicalMaxwidth();
 
 /*
  *-----------------------------------------------------------------------
@@ -107,8 +107,6 @@ point_to_segment(px, py, s1x, s1y, s2x, s2y)
     }
 }
 
-#endif	/* NONMANHATTAN */
-
 /* Define Euclidean distance checks */
 
 #define RADIAL_NW	0x1000
@@ -183,7 +181,6 @@ areaCheck(tile, arg)
 			- arg->dCD_constraint->r_ytop + sdist) >= 0)
 			&& ((sqx * sqx + sqy * sqy) >= ssdist))
 		return 0;
-#ifdef NONMANHATTAN
 	    else if (IsSplit(tile) && !SplitDirection(tile) && !SplitSide(tile))
 	    {
 		sstest = point_to_segment(arg->dCD_constraint->r_xbot + sdist,
@@ -191,7 +188,6 @@ areaCheck(tile, arg)
 			LEFT(tile), BOTTOM(tile), RIGHT(tile), TOP(tile));
 		if (sstest > ssdist) return 0;
 	    }
-#endif
 	}
 	if ((arg->dCD_radial & RADIAL_NE) != 0)
 	{
@@ -200,7 +196,6 @@ areaCheck(tile, arg)
 			- arg->dCD_constraint->r_ytop + sdist) >= 0)
 			&& ((sqx * sqx + sqy * sqy) >= ssdist))
 		return 0;
-#ifdef NONMANHATTAN
 	    else if (IsSplit(tile) && SplitDirection(tile) && SplitSide(tile))
 	    {
 		sstest = point_to_segment(arg->dCD_constraint->r_xtop - sdist,
@@ -208,7 +203,6 @@ areaCheck(tile, arg)
 			LEFT(tile), TOP(tile), RIGHT(tile), BOTTOM(tile));
 		if (sstest > ssdist) return 0;
 	    }
-#endif
 	}
 	if ((arg->dCD_radial & RADIAL_SW) != 0)
 	{
@@ -218,7 +212,6 @@ areaCheck(tile, arg)
 			+ sdist - rect.r_ytop) >= 0)
 			&& ((sqx * sqx + sqy * sqy) >= ssdist))
 		return 0;
-#ifdef NONMANHATTAN
 	    else if (IsSplit(tile) && SplitDirection(tile) && !SplitSide(tile))
 	    {
 		sstest = point_to_segment(arg->dCD_constraint->r_xbot + sdist,
@@ -226,7 +219,6 @@ areaCheck(tile, arg)
 			LEFT(tile), TOP(tile), RIGHT(tile), BOTTOM(tile));
 		if (sstest > ssdist) return 0;
 	    }
-#endif
 	}
 	if ((arg->dCD_radial & RADIAL_SE) != 0)
 	{
@@ -236,7 +228,6 @@ areaCheck(tile, arg)
 			+ sdist - rect.r_ytop) >= 0)
 			&& ((sqx * sqx + sqy * sqy) >= ssdist))
 		return 0;
-#ifdef NONMANHATTAN
 	    else if (IsSplit(tile) && !SplitDirection(tile) && SplitSide(tile))
 	    {
 		sstest = point_to_segment(arg->dCD_constraint->r_xtop - sdist,
@@ -244,7 +235,6 @@ areaCheck(tile, arg)
 			LEFT(tile), BOTTOM(tile), RIGHT(tile), TOP(tile));
 		if (sstest > ssdist) return 0;
 	    }
-#endif
 	}
     }
 
@@ -392,7 +382,8 @@ drcTile (tile, arg)
     Rect errRect;		/* Area checked for an individual rule */
     MaxRectsData *mrd;		/* Used by widespacing rule */
     TileTypeBitMask tmpMask, *rMask;
-    bool trigpending;	/* Hack for widespacing rule */
+    bool trigpending;		/* Hack for widespacing rule */
+    bool firsttile;
     int triggered;
     int cdist, dist, ccdist, result;
 
@@ -419,7 +410,6 @@ drcTile (tile, arg)
         (*(arg->dCD_errors))++;
     }
 
-#ifdef NONMANHATTAN
     if (IsSplit(tile))
     {
 	/* Check rules for DRC_ANGLES rule and process */
@@ -449,7 +439,6 @@ drcTile (tile, arg)
         /* This drc is only for the left edge of the tile */
 	if (SplitSide(tile)) goto checkbottom;
     }
-#endif  
 
     /*
      * Check design rules along a vertical boundary between two tiles.
@@ -494,6 +483,7 @@ drcTile (tile, arg)
         int bottom = MAX(BOTTOM(tile), rect->r_ybot);
 	int edgeX = LEFT(tile);
 
+	firsttile = TRUE;
         for (tpleft = BL(tile); BOTTOM(tpleft) < top; tpleft = RT(tpleft))
         {
 	    /* Get the tile types to the left and right of the edge */
@@ -544,7 +534,8 @@ drcTile (tile, arg)
 
 		if (cptr->drcc_flags & DRC_AREA)
 		{
-		    drcCheckArea(tile, arg, cptr);
+		    if (firsttile)
+			drcCheckArea(tile, arg, cptr);
 		    continue;
 		}
 
@@ -559,8 +550,10 @@ drcTile (tile, arg)
 
 		    if (cptr->drcc_flags & DRC_REVERSE)
 			mrd = drcCanonicalMaxwidth(tpleft, GEO_WEST, arg, cptr);
-		    else
+		    else if (firsttile)
 			mrd = drcCanonicalMaxwidth(tile, GEO_EAST, arg, cptr);
+		    else
+			mrd = NULL;
 		    if (!trigpending) cptr->drcc_dist--;
 		    if (trigpending)
 		    {
@@ -588,7 +581,8 @@ drcTile (tile, arg)
 		else if (cptr->drcc_flags & DRC_MAXWIDTH)
 		{
 		    /* bends_illegal option only */
-		    drcCheckMaxwidth(tile, arg, cptr);
+		    if (firsttile)
+			drcCheckMaxwidth(tile, arg, cptr);
 		    continue;
 		}
 		else if (!triggered) mrd = NULL;
@@ -596,7 +590,7 @@ drcTile (tile, arg)
 		if (cptr->drcc_flags & DRC_RECTSIZE)
 		{
 		    /* only checked for bottom-left tile in a rect area */
-		    if (!TTMaskHasType(&cptr->drcc_mask,
+		    if (firsttile && !TTMaskHasType(&cptr->drcc_mask,
 				TiGetRightType(BL(tile))) &&
 				!TTMaskHasType(&cptr->drcc_mask,
 				TiGetTopType(LB(tile))))
@@ -609,7 +603,7 @@ drcTile (tile, arg)
 		do {
 		    if (triggered)
 		    {
-			/* For triggered rules, we want to look at the	*/
+			/* For triggered rules, we want to look	at the	*/
 			/* clipped region found by the triggering rule	*/
 
 			if (mrd)
@@ -784,7 +778,7 @@ drcTile (tile, arg)
 		    arg->dCD_cptr = cptr;
 		    arg->dCD_entries = 0;
 		    TTMaskCom2(&tmpMask, &cptr->drcc_mask);
-
+		    TTMaskClearType(&tmpMask, TT_ERROR_S);
 		    DBSrPaintArea((Tile *) NULL,
 				arg->dCD_celldef->cd_planes[cptr->drcc_plane],
 				&errRect, &tmpMask, areaCheck, (ClientData) arg);
@@ -802,17 +796,15 @@ drcTile (tile, arg)
 		    triggered = arg->dCD_entries;
 	    }
 	    DRCstatEdges++;
+	    firsttile = FALSE;
         }
     }
 
-#ifdef NONMANHATTAN
-
     /* This drc is only for the bottom edge of the tile */
 
 checkbottom:
     if (IsSplit(tile))
 	if (SplitSide(tile) == SplitDirection(tile)) return 0;
-#endif          
 
     /*
      * Check design rules along a horizontal boundary between two tiles.
@@ -852,6 +844,7 @@ checkbottom:
 	int edgeY = BOTTOM(tile);
 
 	/* Go right across bottom of tile */
+	firsttile = TRUE;
         for (tpbot = LB(tile); LEFT(tpbot) < right; tpbot = TR(tpbot))
         {
 	    /* Get the tile types to the top and bottom of the edge */
@@ -914,8 +907,10 @@ checkbottom:
 
 		    if (cptr->drcc_flags & DRC_REVERSE)
 			mrd = drcCanonicalMaxwidth(tpbot, GEO_SOUTH, arg, cptr);
-		    else
+		    else if (firsttile)
 			mrd = drcCanonicalMaxwidth(tile, GEO_NORTH, arg, cptr);
+		    else
+			mrd = NULL;
 		    if (!trigpending) cptr->drcc_dist--;
 		    if (trigpending)
 		    {
@@ -954,7 +949,7 @@ checkbottom:
 		do {
 		    if (triggered)
 		    {
-			/* For triggered rules, we want to look at the	*/
+			/* For triggered rules, we want to look	at the	*/
 			/* clipped region found by the triggering rule	*/
 
 			if (mrd)
@@ -1088,6 +1083,7 @@ checkbottom:
 		    arg->dCD_cptr = cptr;
 		    arg->dCD_entries = 0;
 		    TTMaskCom2(&tmpMask, &cptr->drcc_mask);
+		    TTMaskClearType(&tmpMask, TT_ERROR_S);
 		    DBSrPaintArea((Tile *) NULL,
 				arg->dCD_celldef->cd_planes[cptr->drcc_plane],
 				&errRect, &tmpMask, areaCheck, (ClientData) arg);
@@ -1105,6 +1101,7 @@ checkbottom:
 		    triggered = arg->dCD_entries;
 	    }
 	    DRCstatEdges++;
+	    firsttile = FALSE;
         }
     }
     return (0);
