@@ -24,7 +24,7 @@
  */
 
 #ifndef	lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/drc/DRCcif.c,v 1.8 2010/10/20 20:31:16 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/drc/DRCcif.c,v 1.5 2010/10/20 20:34:20 tim Exp $";
 #endif	/* not lint */
 
 #include <stdio.h>
@@ -503,9 +503,9 @@ drcCifCheck(arg)
 	    for (drcCifCur = drcCifRules[i][j]; 
 	       		drcCifCur; drcCifCur = drcCifCur->drcc_next)
             {
-     	        Plane *plane = CIFPlanes[i];
 	  	TileTypeBitMask	*mask;
 	  
+		arg->dCD_plane = i;
 	        DBSrPaintArea((Tile *) NULL, CIFPlanes[i], &cifrect,
 			(j == DRC_CIF_SOLID) ? &DBSpaceBits : &CIFSolidBits,
 	  		drcCifTile, arg);
