@@ -20,7 +20,7 @@
  */
 
 #ifndef	lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/drc/DRCcontin.c,v 1.7 2010/03/12 17:19:22 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/drc/DRCcontin.c,v 1.7 2010/03/12 17:19:45 tim Exp $";
 #endif	/* not lint */
 
 #include <stdio.h>
@@ -479,7 +479,7 @@ DRCContinuous()
 				 *  returns 1, meaning that a CHECK tile
 				 *  was found and processed.
 				 */
-	while ((DRCPendingRoot != (DRCPendingCookie *) NULL) &&
+	while ((DRCPendingRoot != (DRCPendingCookie *)NULL) &&
 	    DBSrPaintArea ((Tile *) NULL,
 	    DRCPendingRoot->dpc_def->cd_planes [PL_DRC_CHECK],
 	    &TiPlaneRect, &DBAllButSpaceBits, drcCheckTile, (ClientData) NULL))
@@ -487,7 +487,9 @@ DRCContinuous()
 			     /* check for new user command (without blocking) */
 
 #ifdef MAGIC_WRAPPER
-	    /* Execute pending Tcl events, so the DRC process doesn't block. */
+	    /* Execute pending Tcl events, so the DRC process doesn't block.	*/
+	    /* NOTE:  Exclude file events, or else "drc catchup" will not work	*/
+	    /* in batch mode.							*/
 	    UndoEnable();
 	    while (Tcl_DoOneEvent(TCL_DONT_WAIT))
 	    {
@@ -511,7 +513,7 @@ DRCContinuous()
 
 	/* No check tiles were found, so knock this cell off the list. */
 
-	if (DRCPendingRoot != (DRCPendingCookie *) NULL) {
+	if (DRCPendingRoot != (DRCPendingCookie *)NULL) {
 	    DBReComputeBbox(DRCPendingRoot->dpc_def);
 	    freeMagic((char *) DRCPendingRoot);
 	    DRCPendingRoot = DRCPendingRoot->dpc_next;
@@ -613,9 +615,6 @@ drcCheckTile(tile, arg)
     Rect erasebox;		/* erase old ERROR tiles in this
 				 * region and clip new ERRORs to it
 				 */
-    Rect checkbox;		/* apply rules across all edges in
-				 * this region
-				 */
     CellDef * celldef;		/* First CellDef on DRCPending list. */
     Rect redisplayArea;		/* Area to be redisplayed. */
     extern int drcXorFunc();	/* Forward declarations. */
@@ -645,12 +644,6 @@ drcCheckTile(tile, arg)
 	erasebox.r_xtop, erasebox.r_ytop);
     */
 
-    /* Compute area to recheck in order to recompute all errors in
-     * erasebox.
-     */
-
-    GEO_EXPAND(&erasebox, DRCTechHalo, &checkbox);
-
     /* Use drcDisplayPlane to save all the current errors in the
      * area we're about to recheck.
      */
@@ -663,8 +656,15 @@ drcCheckTile(tile, arg)
 
     DRCErrorType = TT_ERROR_P;
     DBClearPaintPlane(drcTempPlane);
-    (void) DRCBasicCheck (celldef, &checkbox, &erasebox, drcPaintError,
-	(ClientData) drcTempPlane);
+
+    /* May 4, 2008:  Moved DRCBasicCheck into DRCInteractionCheck
+     * to avoid requiring DRC rules to be satisfied independently
+     * of subcells (checkbox was [erasebox + DRCTechHalo], now
+     * computed within DRCInteractionCheck()).
+     */
+
+    /* DRCBasicCheck (celldef, &checkbox, &erasebox, drcPaintError,
+	(ClientData) drcTempPlane); */
 
     /* Check #2:  check interactions between paint and subcells, and
      * also between subcells and other subcells.  If any part of a
@@ -675,8 +675,8 @@ drcCheckTile(tile, arg)
      */
 
     DRCErrorType = TT_ERROR_S;
-    (void) DRCInteractionCheck(celldef, &square, drcPaintError,
-	(ClientData) drcTempPlane);
+    (void) DRCInteractionCheck(celldef, &square, &erasebox,
+		drcPaintError, (ClientData) drcTempPlane);
     
     /* Check #3:  check for array formation errors in the area. */
 
