@@ -1,6 +1,6 @@
 
 #ifndef	lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/drc/DRCextend.c,v 1.8 2009/01/22 03:07:36 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/drc/DRCextend.c,v 1.6 2010/09/20 21:13:22 tim Exp $";
 #endif	
 
 #include <sys/types.h>
@@ -19,6 +19,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/
 #include "drc/drc.h"
 #include "utils/signals.h"
 #include "utils/stack.h"
+#include "utils/maxrect.h"
 
 Stack *DRCstack = (Stack *)NULL;
 
@@ -28,8 +29,6 @@ Stack *DRCstack = (Stack *)NULL;
         STACKPUSH((ClientData) (tp), DRCstack); \
     }
 
-#ifdef NONMANHATTAN
-
 /*
  *-------------------------------------------------------------------------
  *
@@ -71,22 +70,6 @@ drcCheckAngles(tile, arg, cptr)
     }
 }
 
-#else
-
-/* When non-Manhattan geometry is not compiled in, this routine	*/
-/* is merely a placeholder.					*/
-
-void
-drcCheckAngles(tile, arg, cptr)
-    Tile	*starttile;
-    struct drcClientData *arg;
-    DRCCookie	*cptr;
-{
-    /* Do nothing */
-}
-
-#endif	/* NONMANHATTAN */
-
 /*
  *-------------------------------------------------------------------------
  *
@@ -465,7 +448,6 @@ drcCanonicalMaxwidth(starttile, dir, arg, cptr)
     Tile	    *tile,*tp;
     TileTypeBitMask wrongtypes;
     static MaxRectsData *mrd = (MaxRectsData *)NULL;
-    extern int	    findMaxRects();		/* forward declaration */
     Rect	    *boundrect, boundorig;
 
     /* Generate an initial array size of 8 for rlist and swap. */
@@ -476,7 +458,10 @@ drcCanonicalMaxwidth(starttile, dir, arg, cptr)
 	mrd->swap = (Rect *)mallocMagic(8 * sizeof(Rect));
 	mrd->listdepth = 8;
     }
+    if (starttile == NULL) return mrd;
+
     boundrect = &(mrd->rlist[0]);
+    mrd->match = CLIENTDEFAULT;
      
     edgelimit = cptr->drcc_dist;
     arg->dCD_cptr = cptr;
@@ -533,180 +518,10 @@ drcCanonicalMaxwidth(starttile, dir, arg, cptr)
     TTMaskCom2(&wrongtypes, &cptr->drcc_mask);
     boundorig = *boundrect;
     DBSrPaintArea(starttile, arg->dCD_celldef->cd_planes[cptr->drcc_plane],
-		&boundorig, &wrongtypes, findMaxRects, mrd);
+		&boundorig, &wrongtypes, FindMaxRects, mrd);
     if (mrd->entries == 0)
 	return NULL;
     else
 	return (MaxRectsData *)mrd;
 }
 
-int
-findMaxRects(tile, mrd)
-    Tile *tile;
-    MaxRectsData *mrd;
-{
-    Rect area;
-    Rect *rlist, *sr, *tmp;
-    int s, entries = 0;
-
-    TiToRect(tile, &area);
-    rlist = mrd->swap;
-    for (s = 0; s < mrd->entries; s++)
-    {
-	sr = &(mrd->rlist[s]);
-	if (GEO_OVERLAP(sr, &area))
-	{
-	    /* Top */
-	    if (sr->r_ytop >= area.r_ytop + mrd->maxdist)
-	    {
-		rlist[entries] = *sr;
-		rlist[entries].r_ybot = area.r_ytop;
-		entries++;
-	    }
-	    /* Bottom */
-	    if (sr->r_ybot <= area.r_ybot - mrd->maxdist)
-	    {
-		rlist[entries] = *sr;
-		rlist[entries].r_ytop = area.r_ybot;
-		entries++;
-	    }
-	    /* Left */
-	    if (sr->r_xbot <= area.r_xbot - mrd->maxdist)
-	    {
-		rlist[entries] = *sr;
-		rlist[entries].r_xtop = area.r_xbot;
-		entries++;
-	    }
-	    /* Right */
-	    if (sr->r_xtop >= area.r_xtop + mrd->maxdist)
-	    {
-		rlist[entries] = *sr;
-		rlist[entries].r_xbot = area.r_xtop;
-		entries++;
-	    }
-	}
-	else
-	{
-	    /* Copy sr to the new list */
-	    rlist[entries] = *sr;
-	    entries++;
-	}
-
-	/* If we have more rectangles than we allocated space for,	*/
-	/* double the list size and continue.				*/
-
-	if (entries > (mrd->listdepth - 4))
-	{
-	    Rect *newrlist;		
-
-	    mrd->listdepth <<= 1;	/* double the list size */
-
-	    newrlist = (Rect *)mallocMagic(mrd->listdepth * sizeof(Rect));
-	    memcpy((void *)newrlist, (void *)mrd->rlist,
-				(size_t)mrd->entries * sizeof(Rect));
-	    // for (s = 0; s < entries; s++)
-	    //	   newrlist[s] = mrd->rlist[s];
-	    freeMagic(mrd->rlist);
-	    mrd->rlist = newrlist;
-
-	    newrlist = (Rect *)mallocMagic(mrd->listdepth * sizeof(Rect));
-	    memcpy((void *)newrlist, (void *)mrd->swap,
-				(size_t)entries * sizeof(Rect));
-	    // for (s = 0; s < entries; s++)
-	    //     newrlist[s] = mrd->swap[s];
-	    freeMagic(mrd->swap);
-	    mrd->swap = newrlist;
-
-	    rlist = mrd->swap;
-	}
-    }
-
-    /* copy rlist back to mrd by swapping "rlist" and "swap" */
-    mrd->entries = entries;
-    tmp = mrd->rlist;
-    mrd->rlist = rlist;
-    mrd->swap = tmp;
-
-    if (entries > 0)
-	return 0; 	/* keep the search going */
-    else
-	return 1;	/* no more max size areas; stop the search */
-}
-
-/*
- *-------------------------------------------------------------------------
- *
- * DRCMaxRectangle ---
- *
- *	This is a general-purpose routine to be called from anywhere
- *	that expands an area to the largest area rectangle containing
- *	tiles connected to the given types list.  This is kludged
- *	together to make use of DRC structures, but it is meant to
- *	be independent of the DRC code.  It can be called, for example,
- *	from the router code to expand a point label into the maximum
- *	size rectangular terminal.
- *
- * Results:
- *	Pointer to a rectangle containing the maximum size area found.
- *	This pointer should not be deallocated!
- *	
- *-------------------------------------------------------------------------
- */
-
-Rect *
-DRCMaxRectangle(def, startpoint, pNum, expandtypes, expanddist)
-    CellDef *def;
-    Point *startpoint;
-    int pNum;				/* plane of types to expand */
-    TileTypeBitMask *expandtypes;	/* types to expand in */
-    int expanddist;
-{
-    MaxRectsData *mrd;
-    struct drcClientData dcd;
-    DRCCookie	cptr;
-    Tile *starttile;
-    TileType tt;
-    Plane *plane;
-    int rectArea;
-    int maxarea = 0;
-    int s, sidx = -1;
-    Rect origrect;
-
-    cptr.drcc_plane = pNum;
-    cptr.drcc_dist = expanddist;
-    cptr.drcc_mask = *expandtypes;
-
-    dcd.dCD_celldef = def;
-    dcd.dCD_cptr = &cptr;
-
-    /* Find tile in def that surrounds or touches startpoint */
-    plane = def->cd_planes[pNum];
-    starttile = plane->pl_hint;
-    GOTOPOINT(starttile, startpoint);
-    mrd = drcCanonicalMaxwidth(starttile, GEO_CENTER, &dcd, &cptr);
-
-    /* Return only the largest rectangle that contains starttile. */
-
-    TiToRect(starttile, &origrect);
-
-    for (s = 0; s < mrd->entries; s++)
-    {
-	if (GEO_SURROUND(&mrd->rlist[s], &origrect))
-	{
-	    rectArea = (mrd->rlist[s].r_xtop - mrd->rlist[s].r_xbot) *
-			(mrd->rlist[s].r_ytop - mrd->rlist[s].r_ybot);
-	    if (rectArea > maxarea)
-	    {
-		maxarea = rectArea;
-		sidx = s;
-	    }
-	}
-    }
-
-    if (sidx < 0)
-    {
-	sidx = 0;
-	mrd->rlist[0] = origrect;
-    }
-    return &(mrd->rlist[sidx]);
-}
