@@ -19,12 +19,13 @@
  */
 
 #ifndef	lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/drc/DRCmain.c,v 1.4 2010/03/08 13:33:06 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/drc/DRCmain.c,v 1.4 2010/06/24 12:37:16 tim Exp $";
 #endif	/* not lint */
 
 #include <sys/types.h>
 #include <stdio.h>
 
+#include "tcltk/tclmagic.h"
 #include "utils/magic.h"
 #include "utils/malloc.h"
 #include "textio/textio.h"
@@ -177,19 +178,20 @@ drcPaintError(celldef, rect, cptr, plane)
  * ----------------------------------------------------------------------------
  */
 
-/* ARGSUSED */
 void
-drcPrintError (celldef, rect, cptr, area)
+drcPrintError (celldef, rect, cptr, scx)
     CellDef   * celldef;	/* CellDef being checked -- not used here */
     Rect      * rect;		/* Area of error */
     DRCCookie * cptr;  		/* Design rule violated */
-    Rect      * area;		/* Only errors in this area get reported. */
+    SearchContext * scx;	/* Only errors in scx->scx_area get reported. */
 {
     HashEntry *h;
     int i;
+    Rect *area;
 
     ASSERT (cptr != (DRCCookie *) NULL, "drcPrintError");
 
+    area = &scx->scx_area;
     if ((area != NULL) && (!GEO_OVERLAP(area, rect))) return;
     DRCErrorCount += 1;
     h = HashFind(&DRCErrorTable, cptr->drcc_why);
@@ -199,6 +201,83 @@ drcPrintError (celldef, rect, cptr, area)
     i += 1;
     HashSetValue(h, (spointertype)i);
 }
+
+/* Same routine as above, but output goes to a Tcl list and is appended	*/
+/* to the interpreter result.						*/
+
+#ifdef MAGIC_WRAPPER
+
+void
+drcListError (celldef, rect, cptr, scx)
+    CellDef   * celldef;	/* CellDef being checked -- not used here */
+    Rect      * rect;		/* Area of error */
+    DRCCookie * cptr;  		/* Design rule violated */
+    SearchContext * scx;	/* Only errors in scx->scx_area get reported */
+{
+    HashEntry *h;
+    int i;
+    Rect *area;
+
+    ASSERT (cptr != (DRCCookie *) NULL, "drcListError");
+
+    area = &scx->scx_area;
+    if ((area != NULL) && (!GEO_OVERLAP(area, rect))) return;
+    DRCErrorCount += 1;
+    h = HashFind(&DRCErrorTable, cptr->drcc_why);
+    i = (spointertype) HashGetValue(h);
+    if (i == 0)
+    {
+	Tcl_Obj *lobj;
+	lobj = Tcl_GetObjResult(magicinterp);
+	Tcl_ListObjAppendElement(magicinterp, lobj,
+			Tcl_NewStringObj(cptr->drcc_why, -1));
+	Tcl_SetObjResult(magicinterp, lobj);
+    }
+    i += 1;
+    HashSetValue(h, (spointertype)i);
+}
+
+/* Same routine as above, but output for every single error is recorded	*/
+/* along with position information.					*/
+
+void
+drcListallError (celldef, rect, cptr, scx)
+    CellDef   * celldef;	/* CellDef being checked -- not used here */
+    Rect      * rect;		/* Area of error */
+    DRCCookie * cptr;  		/* Design rule violated */
+    SearchContext * scx;	/* Only errors in scx->scx_area get reported. */
+{
+    Tcl_Obj *lobj, *pobj;
+    HashEntry *h;
+    Rect *area, r;
+
+    ASSERT (cptr != (DRCCookie *) NULL, "drcListallError");
+
+    GeoTransRect(&scx->scx_trans, rect, &r);
+    area = &scx->scx_area;
+    if ((area != NULL) && (!GEO_OVERLAP(area, rect))) return;
+    DRCErrorCount += 1;
+    h = HashFind(&DRCErrorTable, cptr->drcc_why);
+    lobj = (Tcl_Obj *) HashGetValue(h);
+    if (lobj == NULL)
+       lobj = Tcl_NewListObj(0, NULL);
+    
+    pobj = Tcl_NewListObj(0, NULL);
+
+    Tcl_ListObjAppendElement(magicinterp, pobj, Tcl_NewIntObj(r.r_xbot));
+    Tcl_ListObjAppendElement(magicinterp, pobj, Tcl_NewIntObj(r.r_ybot));
+    Tcl_ListObjAppendElement(magicinterp, pobj, Tcl_NewIntObj(r.r_xtop));
+    Tcl_ListObjAppendElement(magicinterp, pobj, Tcl_NewIntObj(r.r_ytop));
+    Tcl_ListObjAppendElement(magicinterp, lobj, pobj);
+
+    HashSetValue(h, lobj);
+}
+
+#else
+
+#define drcListError drcPrintError
+
+#endif
 
 /*
  * ----------------------------------------------------------------------------
@@ -297,7 +376,10 @@ DRCPrintStats()
  */
 
 void
-DRCWhy(use, area)
+DRCWhy(dolist, use, area)
+    bool dolist;			/*
+					 * Generate Tcl list for value
+					 */
     CellUse *use;			/* Use in whose definition to start
 					 * the hierarchical check.
 					 */
@@ -323,7 +405,7 @@ DRCWhy(use, area)
     scx.scx_y = use->cu_ylo;
     scx.scx_area = *area;
     scx.scx_trans = GeoIdentityTransform;
-    (void) drcWhyFunc(&scx, (ClientData) NULL);
+    drcWhyFunc(&scx, (pointertype)dolist);
     UndoEnable();
 
     /* Delete the hash table now that we're finished (otherwise there
@@ -342,7 +424,81 @@ DRCWhy(use, area)
 
     if (DRCErrorCount == 0) TxPrintf("No errors found.\n");
 }
-
+
+#ifdef MAGIC_WRAPPER
+
+void
+DRCWhyAll(use, area, fout)
+    CellUse *use;			/* Use in whose definition to start
+					 * the hierarchical check.
+					 */
+    Rect *area;				/* Area, in def's coordinates, that
+					 * is to be checked.
+					 */
+    FILE *fout;				/*
+					 * Write formatted output to fout
+					 */
+{
+    SearchContext scx;
+    Rect box;
+    extern int drcWhyAllFunc();		/* Forward reference. */
+    HashSearch	hs;
+    HashEntry	*he;
+    Tcl_Obj *lobj, *robj;
+
+    /* Create a hash table to used for eliminating duplicate messages. */
+
+    HashInit(&DRCErrorTable, 16, HT_STRINGKEYS);
+    DRCErrorCount = 0;
+    box = DRCdef->cd_bbox;
+
+    /* Undo will only slow things down in here, so turn it off. */
+
+    UndoDisable();
+    scx.scx_use = use;
+    scx.scx_x = use->cu_xlo;
+    scx.scx_y = use->cu_ylo;
+    scx.scx_area = *area;
+    scx.scx_trans = GeoIdentityTransform;
+    drcWhyAllFunc(&scx, NULL);
+    UndoEnable();
+
+    /* Generate results */
+
+    robj = Tcl_NewListObj(0, NULL);
+    
+    HashStartSearch(&hs);
+    while ((he = HashNext(&DRCErrorTable, &hs)) != (HashEntry *)NULL)
+    {
+	lobj = (Tcl_Obj *)HashGetValue(he);
+	if (lobj != NULL)
+	{
+	    Tcl_ListObjAppendElement(magicinterp, robj, 
+			Tcl_NewStringObj((char *)he->h_key.h_name, -1));
+	    Tcl_ListObjAppendElement(magicinterp, robj, lobj);
+	}
+    }
+    Tcl_SetObjResult(magicinterp, robj);
+
+    /* Delete the hash table now that we're finished (otherwise there
+     * will be a core leak.
+     */
+	
+    HashKill(&DRCErrorTable);
+
+    /* Redisplay the DRC yank definition in case anyone is looking
+     * at it.
+     */
+    
+    DBReComputeBbox(DRCdef);
+    (void) GeoInclude(&DRCdef->cd_bbox, &box);
+    DBWAreaChanged(DRCdef, &box, DBW_ALLWINDOWS, &DBAllButSpaceBits);
+
+    if (DRCErrorCount == 0) TxPrintf("No errors found.\n");
+}
+
+#endif	/* MAGIC_WRAPPER */
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -366,28 +522,55 @@ DRCWhy(use, area)
 int
 drcWhyFunc(scx, cdarg)
     SearchContext *scx;		/* Describes current state of search. */
-    ClientData cdarg;		/* Not used. */
+    ClientData cdarg;		/* Used to hold boolean value "dolist" */
+{
+    CellDef *def = scx->scx_use->cu_def;
+    bool dolist = (bool)((pointertype)cdarg);
+
+    /* Check paint and interactions in this subcell. */
+    
+//  (void) DRCBasicCheck(def, &haloArea, &scx->scx_area,
+//		(dolist) ? drcListError : drcPrintError,
+//		(ClientData) scx);
+    (void) DRCInteractionCheck(def, &scx->scx_area, &scx->scx_area,
+		(dolist) ? drcListError : drcPrintError,
+		(ClientData) scx);
+    (void) DRCArrayCheck(def, &scx->scx_area,
+		(dolist) ? drcListError : drcPrintError,
+		(ClientData) scx);
+    
+    /* Also search children. */
+
+    (void) DBCellSrArea(scx, drcWhyFunc, (ClientData)cdarg);
+
+    return 0;
+}
+
+#ifdef MAGIC_WRAPPER
+
+int
+drcWhyAllFunc(scx, cdarg)
+    SearchContext *scx;		/* Describes current state of search. */
+    ClientData cdarg;		/* Unused */
 {
-    Rect haloArea;
     CellDef *def = scx->scx_use->cu_def;
 
     /* Check paint and interactions in this subcell. */
     
-    GEO_EXPAND(&scx->scx_area, DRCTechHalo, &haloArea);
-    (void) DRCBasicCheck(def, &haloArea, &scx->scx_area,
-	drcPrintError, (ClientData) &scx->scx_area);
-    (void) DRCInteractionCheck(def, &scx->scx_area,
-	drcPrintError, (ClientData) &scx->scx_area);
+    (void) DRCInteractionCheck(def, &scx->scx_area, &scx->scx_area,
+		drcListallError, (ClientData)scx);
     (void) DRCArrayCheck(def, &scx->scx_area,
-	drcPrintError, (ClientData) &scx->scx_area);
+		drcListallError, (ClientData)scx);
     
     /* Also search children. */
 
-    (void) DBCellSrArea(scx, drcWhyFunc, (ClientData) NULL);
+    (void) DBCellSrArea(scx, drcWhyAllFunc, (ClientData)cdarg);
 
     return 0;
 }
-
+
+#endif /* MAGIC_WRAPPER */
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -550,7 +733,7 @@ drcCountFunc(scx, dupTable)
      */
 
     def = scx->scx_use->cu_def;
-    h = HashFind(dupTable, def);
+    h = HashFind(dupTable, (char *)def);
     if (HashGetValue(h) != 0) goto done;
     HashSetValue(h, 1);
 
@@ -703,7 +886,7 @@ drcFindFunc(scx, finddata)
     int drcFindFunc2();
 
     def = scx->scx_use->cu_def;
-    h = HashFind(finddata->deft, def);
+    h = HashFind(finddata->deft, (char *)def);
     if (HashGetValue(h) != 0) return 0;
     HashSetValue(h, 1);
 
