@@ -19,7 +19,7 @@
  */
 
 #ifndef	lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/drc/DRCsubcell.c,v 1.5 2008/05/10 20:13:05 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/drc/DRCsubcell.c,v 1.4 2009/05/01 18:59:44 tim Exp $";
 #endif	/* not lint */
 
 #include <stdio.h>
@@ -63,6 +63,8 @@ static DRCCookie drcSubcellCookie = {
     (DRCCookie *) NULL
 };
 
+extern int DRCErrorType;
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -525,14 +527,15 @@ drcExactOverlapTile(tile, cxp)
  */
 
 int
-DRCInteractionCheck(def, area, func, cdarg)
+DRCInteractionCheck(def, area, erasebox, func, cdarg)
     CellDef *def;		/* Definition in which to do check. */
     Rect *area;			/* Area in which all errors are to be found. */
+    Rect *erasebox;		/* Smaller area containing DRC check tiles */
     void (*func)();		/* Function to call for each error. */
     ClientData cdarg;		/* Extra info to be passed to func. */
 {
-    int oldTiles, count, x, y;
-    Rect intArea, square;
+    int oldTiles, count, x, y, errorSaveType;
+    Rect intArea, square, subArea;
     PaintResultType (*savedPaintTable)[NT][NT];
     void (*savedPaintPlane)();
     struct drcClientData arg;
@@ -563,27 +566,95 @@ DRCInteractionCheck(def, area, func, cdarg)
 	     * area we're interested in. */
 
 	    if (!DRCFindInteractions(def, &square, DRCTechHalo, &intArea))
-		continue;
+	    {
+		/* Added May 4, 2008---if there are no subcells, run the
+		 * basic check over the area of the square.
+		 */
+		subArea = *erasebox;
+		GeoClip(&subArea, &square);
+		GEO_EXPAND(&subArea, DRCTechHalo, &intArea);
 
-	    GeoClip(&intArea, area);
+		errorSaveType = DRCErrorType;
+		DRCErrorType = TT_ERROR_P;	// Basic check is always ERROR_P
+                DRCBasicCheck(def, &intArea, &subArea, func, cdarg); 
+		DRCErrorType = errorSaveType;
+		continue;
+	    }
+	    else
+	    {
+		/* Added March 6, 2012:  Any area(s) outside the
+		 * interaction area are processed with the basic
+		 * check.  This avoids unnecessary copying, so it
+		 * speeds up the DRC without requiring that geometry
+		 * passes DRC rules independently of subcell geometry
+	         * around it.
+		 *
+		 * As intArea can be smaller than square, we may have
+		 * to process as many as four independent rectangles.
+		 * NOTE that the area of (intArea + halo) will be checked
+		 * in the subcell interaction check, so we can ignore
+		 * that.
+		 */
+		Rect eraseClip, eraseHalo, subArea;
+
+		errorSaveType = DRCErrorType;
+		DRCErrorType = TT_ERROR_P;	// Basic check is always ERROR_P
+		eraseClip = *erasebox;
+		GeoClip(&eraseClip, &square);
+		subArea = eraseClip;
+
+		/* check above */
+		if (intArea.r_ytop < eraseClip.r_ytop)
+		{
+		    subArea.r_ybot = intArea.r_ytop;
+		    GEO_EXPAND(&subArea, DRCTechHalo, &eraseHalo);
+                    DRCBasicCheck(def, &eraseHalo, &subArea, func, cdarg); 
+		} 
+		/* check below */
+		if (intArea.r_ybot > eraseClip.r_ybot)
+		{
+		    subArea.r_ybot = eraseClip.r_ybot;
+		    subArea.r_ytop = intArea.r_ybot;
+		    GEO_EXPAND(&subArea, DRCTechHalo, &eraseHalo);
+                    DRCBasicCheck(def, &eraseHalo, &subArea, func, cdarg); 
+		} 
+		subArea.r_ytop = intArea.r_ytop;
+		subArea.r_ybot = intArea.r_ybot;
+
+		/* check right */
+		if (intArea.r_xtop < eraseClip.r_xtop)
+		{
+		    subArea.r_xbot = intArea.r_xtop;
+		    GEO_EXPAND(&subArea, DRCTechHalo, &eraseHalo);
+                    DRCBasicCheck(def, &eraseHalo, &subArea, func, cdarg); 
+		}
+		/* check left */
+		if (intArea.r_xbot > eraseClip.r_xbot)
+		{
+		    subArea.r_xtop = intArea.r_xbot;
+		    subArea.r_xbot = eraseClip.r_xbot;
+		    GEO_EXPAND(&subArea, DRCTechHalo, &eraseHalo);
+                    DRCBasicCheck(def, &eraseHalo, &subArea, func, cdarg); 
+		}
+		DRCErrorType = errorSaveType;
+	    }
     
 	    /* Flatten the interaction area. */
 
+	    DRCstatInteractions += 1;
 	    GEO_EXPAND(&intArea, DRCTechHalo, &scx.scx_area);
 	    DRCDummyUse->cu_def = def;
 	    scx.scx_use = DRCDummyUse;
 	    scx.scx_trans = GeoIdentityTransform;
-
 	    DBCellClearDef(DRCdef);
-	    DRCstatInteractions += 1;
 
 	    savedPaintTable = DBNewPaintTable(DRCCurStyle->DRCPaintTable);
-	    /* savedPaintPlane = DBNewPaintPlane(DBPaintPlaneMergeOnce); */
+	    savedPaintPlane = DBNewPaintPlane(DBPaintPlaneMark);
 
 	    (void) DBCellCopyAllPaint(&scx, &DBAllButSpaceBits, 0, DRCuse);
 
 	    (void) DBNewPaintTable(savedPaintTable);
-	    /* (void) DBNewPaintPlane(savedPaintPlane); */
+	    (void) DBNewPaintPlane(savedPaintPlane);
 
 	    /* Run the basic checker over the interaction area. */
 
@@ -645,12 +716,12 @@ DRCFlatCheck(use, area)
 	    DBCellClearDef(DRCdef);
 
 	    savedPaintTable = DBNewPaintTable(DRCCurStyle->DRCPaintTable);
-	    /* savedPaintPlane = DBNewPaintPlane(DBPaintPlaneMergeOnce); */
+	    savedPaintPlane = DBNewPaintPlane(DBPaintPlaneMark);
 
 	    (void) DBCellCopyAllPaint(&scx, &DBAllButSpaceBits, 0, DRCuse);
 
 	    (void) DBNewPaintTable(savedPaintTable);
-	    /* (void) DBNewPaintPlane(savedPaintPlane); */
+	    (void) DBNewPaintPlane(savedPaintPlane);
 
 	    (void) DRCBasicCheck(DRCdef, &scx.scx_area, &chunk,
 		drcIncCount, (ClientData) &drcFlatCount);
