@@ -13,7 +13,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/ext2sim/ext2sim.c,v 1.4 2008/12/02 17:12:35 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/ext2sim/ext2sim.c,v 1.2 2008/12/03 14:12:09 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -59,7 +59,7 @@ bool esNoAttrs = FALSE;
 bool esHierAP = FALSE;
 bool esMergeDevsA = FALSE;	/* merge devices of equal length */
 bool esMergeDevsC = FALSE;	/* merge devices of equal length & width */
-int  esCapAccuracy = 1;
+int esCapAccuracy = 1;
 
 #else
 extern bool esDoExtResis;
@@ -631,7 +631,7 @@ runexttosim:
 	devMergeList = NULL;
     }
 
-    EFVisitDevs(simdevVisit, (ClientData) NULL);
+    EFVisitDevs(simdevVisit, (ClientData)NULL);
     if (flatFlags & EF_FLATCAPS) {
     	(void) sprintf( esCapFormat, " %%.%dlf\n", esCapAccuracy);
 	EFVisitCaps(simcapVisit, (ClientData) NULL);
@@ -957,6 +957,7 @@ HierName *suffix;
 	he = EFHNConcatLook(prefix, suffix, "output");
 	return(((EFNodeName *) HashGetValue(he))->efnn_node);
 }
+
 
 /*
  * ----------------------------------------------------------------------------
@@ -989,14 +990,14 @@ HierName *suffix;
  */
 
 int
-simdevVisit(dev, hierName, trans)
+simdevVisit(dev, hierName, scale, trans)
     Dev *dev;		/* Device being output */
     HierName *hierName;	/* Hierarchical path down to this device */
-    Transform *trans;	/* Coordinate transform for output */
+    float scale;	/* Scale transform for output */
+    Transform *trans;	/* Coordinate transform */
 {
     DevTerm *gate, *source, *drain;
     EFNode  *subnode, *snode, *dnode;
-    int scale;
     int l, w;
     Rect r;
     char name[12];
@@ -1007,6 +1008,12 @@ simdevVisit(dev, hierName, trans)
     if (dev->dev_nterm < 1)
 	return 0;
 
+    /* If only one terminal, can't do much of anything either 	     */
+    /* (may want to provide eventual support for subcircuit devices) */
+
+    if (dev->dev_nterm < 2)
+	return 0;
+
     /* Merged devices */
     if ((esMergeDevsA || esMergeDevsC) && devIsKilled(esFMIndex++))
 	return 0;
@@ -1063,14 +1070,14 @@ simdevVisit(dev, hierName, trans)
     }
     else if (dev->dev_class == DEV_DIODE && dev->dev_nterm == 1 && subnode)
     {
-	sprintf(name, "fet");
-	simdevOutNode(hierName, subnode->efnode_name->efnn_hier, name, esSimF);
+        sprintf(name, "fet");
+        simdevOutNode(hierName, subnode->efnode_name->efnn_hier, name, esSimF); 
     }
     else if (dev->dev_nterm > 2)
         simdevOutNode(hierName, drain->dterm_node->efnode_name->efnn_hier, name, esSimF);
 
     /* Support gemini's substrate comparison */
-    if ( esFormat == LBL && subnode)
+    if (esFormat == LBL && subnode)
     {
 	putc(' ', esSimF);
 	simdevSubstrate(hierName, subnode->efnode_name->efnn_hier,
@@ -1078,7 +1085,6 @@ simdevVisit(dev, hierName, trans)
     }
 
     GeoTransRect(trans, &dev->dev_rect, &r);
-    scale = GeoScale(trans);
 
     if (dev->dev_class == DEV_BJT || EFDevTypes[dev->dev_type][0] == 'b')
     {
@@ -1102,8 +1108,8 @@ simdevVisit(dev, hierName, trans)
 	cl = 0.5 * (chp + sqrt(chp * chp - 4 * ca));
 	cw = ca / cl;
 
-	fprintf(esSimF, " %d %d %d %d", (int)cl, (int)cw,
-		r.r_xbot, r.r_ybot);
+	fprintf(esSimF, " %d %d %g %g", (int)cl, (int)cw,
+		r.r_xbot * scale, r.r_ybot * scale);
     }
     else if (dev->dev_class == DEV_RES) {	/* generate a resistor */
        fprintf(esSimF, " %f", (double)(dev->dev_res));
@@ -1122,8 +1128,8 @@ simdevVisit(dev, hierName, trans)
         * factor at the beginning of the .sim file.
         */
 
-       fprintf(esSimF, " %d %d %d %d",
-		l*scale, w*scale, r.r_xbot, r.r_ybot);
+       fprintf(esSimF, " %g %g %g %g",
+		l * scale, w * scale, r.r_xbot * scale, r.r_ybot * scale);
 
        /* Attributes, if present */
        if (!esNoAttrs)
@@ -1541,7 +1547,7 @@ int simnodeVisit(node, res, cap)
  * ----------------------------------------------------------------------------
  */
 devMerge *simmkDevMerge(l, w, g, s, d, b, hn, dev)
-int     l, w;
+int   l, w;
 EFNode *g, *s, *d, *b;
 HierName *hn;
 Dev    *dev;
@@ -1635,17 +1641,17 @@ Dev    *dev;
  * ----------------------------------------------------------------------------
  */
 int
-simmergeVisit(dev, hierName, trans)
+simmergeVisit(dev, hierName, scale, trans)
 Dev *dev;		/* Dev to examine */
 HierName *hierName;	/* Hierarchical path down to this dev */
-Transform *trans;	/* Coordinate transform not used */
+float scale;		/* Scale transform */
+Transform *trans;	/* Coordinate transform (not used) */
 {
 	DevTerm *gate, *source, *drain;
 	Dev     *cf;
 	DevTerm *cg, *cs, *cd;
 	EFNode	*subnode, *snode, *dnode, *gnode;
-	int      scale, pmode, l, w;
-    	Rect	 r;
+	int      pmode, l, w;
 	float	 m;
 	devMerge *fp, *cfp;
 
@@ -1664,10 +1670,8 @@ Transform *trans;	/* Coordinate transform not used */
 	snode = SimGetNode (hierName, source->dterm_node->efnode_name->efnn_hier);
 	dnode = SimGetNode (hierName, drain->dterm_node->efnode_name->efnn_hier);
 
-    	GeoTransRect(trans, &dev->dev_rect, &r);
-    	scale = GeoScale(trans);
 	EFGetLengthAndWidth(dev, &l, &w);
-	fp = simmkDevMerge(l*scale, w*scale, gnode, snode, 
+	fp = simmkDevMerge((int)(l*scale), (int)(w*scale), gnode, snode, 
 			dnode, subnode, hierName, dev);
 
 	/*
