@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/extflat/EFbuild.c,v 1.6 2008/12/29 22:14:10 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/extflat/EFbuild.c,v 1.6 2010/06/24 12:37:17 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -448,14 +448,15 @@ efBuildEquiv(def, nodeName1, nodeName2)
 
 DevParam *
 efGetDeviceParams(name)
+    char *name;
 {
-   HashEntry *he;
-   DevParam *plist = NULL;
+    HashEntry *he;
+    DevParam *plist = NULL;
 
-   he = HashLookOnly(&efDevParamTable, name);
-   if (he != NULL)
+    he = HashLookOnly(&efDevParamTable, (char *)name);
+    if (he != NULL)
 	plist = (DevParam *)HashGetValue(he);
-   return plist;
+    return plist;
 }
 
 /*
@@ -487,6 +488,8 @@ efBuildDeviceParams(name, argc, argv)
     /* Parse arguments for each parameter */
     for (n = 0; n < argc; n++)
     {
+	char *mult;
+
 	pptr = strchr(argv[n], '=');
 	if (pptr == NULL)
 	{
@@ -495,7 +498,20 @@ efBuildDeviceParams(name, argc, argv)
 	    continue;
 	}
 	newparm = (DevParam *)mallocMagic(sizeof(DevParam));
-	newparm->parm_type = *argv[n];
+	newparm->parm_type[0] = *argv[n];
+	if ((pptr - argv[n]) == 1)
+	    newparm->parm_type[1] = '\0';
+	else
+	    newparm->parm_type[1] = *(argv[n] + 1);
+
+	if ((mult = strchr(pptr + 1, '*')) != NULL)
+	{
+	    *mult = '\0';
+	    newparm->parm_scale = atof(mult + 1);
+	}
+	else
+	    newparm->parm_scale = 1.0;
+
 	newparm->parm_name = StrDup((char **)NULL, pptr + 1);
 	newparm->parm_next = plist;
 	plist = newparm;
@@ -537,7 +553,7 @@ efBuildDevice(def, class, type, r, argc, argv)
 			 * and an attribute list (or the token 0).
 			 */
 {
-    int n, nterminals;
+    int n, nterminals, pn;
     DevTerm *term;
     Dev *newdev, devtmp;
     char ptype, *pptr, **av;
@@ -559,6 +575,7 @@ efBuildDevice(def, class, type, r, argc, argv)
     {
 	case DEV_FET:
 	case DEV_MOSFET:
+	case DEV_ASYMMETRIC:
 	case DEV_BJT:
 	    argstart = 3;
 	    break;
@@ -571,6 +588,7 @@ efBuildDevice(def, class, type, r, argc, argv)
 		argstart = 2;
 	    break;
 	case DEV_SUBCKT:
+	case DEV_MSUBCKT:
 	case DEV_RSUBCKT:
 	    argstart = 0;
 
@@ -581,10 +599,26 @@ efBuildDevice(def, class, type, r, argc, argv)
 		switch(*argv[argstart])
 		{
 		    case 'a':
-			devtmp.dev_area = atoi(pptr);
+			if ((pptr - argv[argstart]) == 2)
+			    devtmp.dev_area = atoi(pptr);
+			else
+			{
+			    pn = *(argv[argstart] + 1) - '0';
+			    if (pn == 0)
+				devtmp.dev_area = atoi(pptr);
+			    /* Otherwise, punt */
+			}
 			break;
 		    case 'p':
-			devtmp.dev_perim = atoi(pptr);
+			if ((pptr - argv[argstart]) == 2)
+			    devtmp.dev_perim = atoi(pptr);
+			else
+			{
+			    pn = *(argv[argstart] + 1) - '0';
+			    if (pn == 0)
+				devtmp.dev_perim = atoi(pptr);
+			    /* Otherwise, punt */
+			}
 			break;
 		    case 'l':
 			devtmp.dev_length = atoi(pptr);
@@ -611,6 +645,7 @@ efBuildDevice(def, class, type, r, argc, argv)
 	case DEV_RES:
 	case DEV_CAP:
 	case DEV_RSUBCKT:
+	case DEV_MSUBCKT:
 	case DEV_SUBCKT:
 	case DEV_DIODE:
 	    n = argc - argstart;
@@ -618,6 +653,7 @@ efBuildDevice(def, class, type, r, argc, argv)
 	    {
 		if (strncmp(argv[argstart], "None", 4) != 0)
 		    devtmp.dev_subsnode = efBuildDevNode(def, argv[argstart], TRUE);
+
 		argstart++;
 	    }
 	    break;
@@ -652,6 +688,7 @@ efBuildDevice(def, class, type, r, argc, argv)
 	    newdev->dev_subsnode = efBuildDevNode(def, argv[2], TRUE);
 	    break;
 	case DEV_MOSFET:	/* new-style "device mosfet" record */
+	case DEV_ASYMMETRIC:
 	case DEV_BJT:
 	    newdev->dev_length = atoi(argv[0]);
 	    newdev->dev_width = atoi(argv[1]);
@@ -728,7 +765,9 @@ efBuildDevice(def, class, type, r, argc, argv)
     {
 	term = &newdev->dev_terms[n];
 	term->dterm_node = efBuildDevNode(def, av[TERM_NAME], FALSE);
-	term->dterm_perim = atoi(av[TERM_PERIM]);
+	term->dterm_length = atoi(av[TERM_PERIM]);
+	term->dterm_area = 0;
+	term->dterm_perim = 0;
 
 	/* If the attr list is '0', this signifies no attributes */
 	if (av[TERM_ATTRS][0] == '0' && av[TERM_ATTRS][1] == '\0')
@@ -789,7 +828,58 @@ efBuildPortNode(def, name, idx, x, y, layername)
     }
 }
 
-
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * EFGetPortMax --
+ *
+ * Find the highest port number in the cell def and return the value.
+ *
+ * Results:
+ *	Value of highest port number in the cell def's node list
+ *
+ * Side effects:
+ *	Larger value including the implicit ports is placed in the
+ *	location of the pointer imp_max.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+int
+EFGetPortMax(def, imp_max)
+   Def *def;
+   int *imp_max;
+{
+    EFNode *snode;
+    EFNodeName *nodeName;
+    int portmax, portorder;
+
+    portmax = -1;
+    if (imp_max) *imp_max = -1;
+
+    for (snode = (EFNode *) def->def_firstn.efnode_next;
+                snode != &def->def_firstn;
+                snode = (EFNode *) snode->efnode_next)
+    {
+        if (imp_max && (snode->efnode_flags & EF_SUBS_PORT))
+	{
+	    nodeName = snode->efnode_name;
+	    portorder = nodeName->efnn_port;
+	    if (portorder > (*imp_max)) (*imp_max) = portorder;
+	}
+        else if (snode->efnode_flags & EF_PORT)
+	{
+	    for (nodeName = snode->efnode_name; nodeName != NULL; nodeName =
+                        nodeName->efnn_next)
+	    {
+		portorder = nodeName->efnn_port;
+		if (portorder > portmax) portmax = portorder;
+	    }
+	}
+    }
+    return portmax;
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -837,15 +927,19 @@ efBuildDevNode(def, name, isSubsNode)
 #ifdef MAGIC_WRAPPER
 		if ((name[0] == '$') && (name[1] != '$'))
 		    efReadError("Substrate node is an undefined Tcl variable.\n");
-		else
+		// else
 #endif
-		    efReadError("Default device substrate node"
-				" \"%s\" is not a global\n", name);
+		//    efReadError("Default device substrate node"
+		//		" \"%s\" is not a global\n", name);
+
+		/* This node is declared to be an implicit port */
+		nn->efnn_node->efnode_flags |= EF_SUBS_PORT;
+		nn->efnn_port = -1;
+		def->def_flags |= DEF_SUBSNODES;
 	    }
 	    nn->efnn_node->efnode_flags |= EF_DEVTERM;
 	}
     }
-
     return nn->efnn_node;
 }
 
@@ -997,6 +1091,7 @@ efBuildConnect(def, nodeName1, nodeName2, deltaC, av, ac)
 		    + (efNumResistClasses - 1) * sizeof (PerimArea);
 
     conn = (Connection *) mallocMagic((unsigned)(size));
+
     if (efConnInitSubs(conn, nodeName1, nodeName2))
     {
 	conn->conn_cap = (EFCapValue) deltaC;
@@ -1119,7 +1214,7 @@ efConnInitSubs(conn, nodeName1, nodeName2)
 
     if (c1->cn_nsubs != c2->cn_nsubs)
     {
-	efReadError("Number of subscripts don't match\n");
+	efReadError("Number of subscripts doesn't match\n");
 	goto bad;
     }
 
@@ -1315,7 +1410,7 @@ efNodeAddName(node, he, hn)
  * make this node2 and free its memory.
  *
  * Results:
- *	None.
+ *	Return 0 if node1 has precedence, 1 if node2 has precedence
  *
  * Side effects:
  *	See above.
