@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/extflat/EFflat.c,v 1.2 2006/05/12 06:21:47 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/extflat/EFflat.c,v 1.5 2010/12/16 18:59:03 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -58,7 +58,7 @@ int efFlatGlobHash(HierName *);
 bool efFlatGlobCmp(HierName *, HierName *);
 char *efFlatGlobCopy(HierName *);
 void efFlatGlobError(EFNodeName *, EFNodeName *);
-void efAddNodes(HierContext *, bool);
+int efAddNodes(HierContext *, bool);
 int efAddOneConn(HierContext *, char *, char *, Connection *);
 
 
@@ -143,6 +143,79 @@ EFFlatBuild(name, flags)
 
  return;
 }
+
+/*----------------------------------------------------------------------*/
+/* EFFlatBuildOneLevel --						*/
+/*									*/
+/* EFFlatBuild for a single hierarchical level.  Note, however that	*/
+/* where subcircuits have no extracted components, the hierarchy of	*/
+/* the subcircuit will be traversed and the subcircuit merged into	*/
+/* the root being flattened.						*/
+/*									*/
+/* This routine used for hierarchical extraction.			*/
+/*----------------------------------------------------------------------*/
+
+HierContext *
+EFFlatBuildOneLevel(def)
+    Def *def;		/* root def being flattened */
+{
+    int usecount;
+    Use *use;
+    int efFlatNodesDeviceless();	/* Forward declaration */
+
+    efFlatRootDef = def;
+
+    /* Keyed by a full HierName */
+    HashInitClient(&efNodeHashTable, INITFLATSIZE, HT_CLIENTKEYS,
+	efHNCompare, (char *(*)()) NULL, efHNHash, (int (*)()) NULL);
+
+    /* Keyed by a pair of HierNames */
+    HashInitClient(&efDistHashTable, INITFLATSIZE, HT_CLIENTKEYS,
+	efHNDistCompare, efHNDistCopy, efHNDistHash, efHNDistKill);
+
+    /* Keyed by pairs of EFNode pointers (i.e., EFCoupleKeys) */
+    HashInit(&efCapHashTable, INITFLATSIZE, HashSize(sizeof (EFCoupleKey)));
+
+    /* Keyed by a string and a HierName */
+    HashInitClient(&efHNUseHashTable, INITFLATSIZE, HT_CLIENTKEYS,
+	efHNUseCompare, (char *(*)()) NULL, efHNUseHash, (int (*)()) NULL);
+
+    /* Circular list of all nodes contains no elements initially */
+    efNodeList.efnode_next = (EFNodeHdr *) &efNodeList;
+    efNodeList.efnode_prev = (EFNodeHdr *) &efNodeList;
+
+    efFlatContext.hc_hierName = (HierName *) NULL;
+    efFlatContext.hc_use = &efFlatRootUse;
+    efFlatContext.hc_trans = GeoIdentityTransform;
+    efFlatContext.hc_x = efFlatContext.hc_y = 0;
+    efFlatRootUse.use_def = efFlatRootDef;
+
+    usecount = 0;
+
+    /* Record all nodes of the next level in the hierarchy */
+    efHierSrUses(&efFlatContext, efAddNodes, (ClientData)FALSE);
+
+    /* Expand all subcells that contain connectivity information but	*/
+    /* no active devices (including those in subcells).			*/
+
+    for (use = efFlatRootUse.use_def->def_uses; use; use = use->use_next)
+	usecount++;
+
+    /* Recursively flatten uses that have no active devices */
+    if (usecount > 0)
+	efHierSrUses(&efFlatContext, efFlatNodesDeviceless, (ClientData)&usecount);
+
+    if ((usecount == 0) && (efFlatRootUse.use_def->def_devs == NULL))
+	efFlatRootUse.use_def->def_flags |= DEF_NODEVICES;
+
+    /* Record all local nodes */
+    efAddNodes(&efFlatContext, FALSE);
+    efAddConns(&efFlatContext);
+
+    efFlatGlob();
+
+    return &efFlatContext;
+}
 
 /*
  * ----------------------------------------------------------------------------
@@ -278,6 +351,38 @@ efFlatNodesStdCell(hc)
     return (0);
 }
 
+int
+efFlatNodesDeviceless(hc, cdata)
+    HierContext *hc;
+    ClientData cdata;
+{
+    int *usecount = (int *)cdata;
+    int newcount = 0;
+    Use *use;
+
+    for (use = hc->hc_use->use_def->def_uses; use; use = use->use_next)
+	newcount++;
+
+    /* Recursively flatten uses that have no active devices */
+    if (newcount > 0)
+	efHierSrUses(hc, efFlatNodesDeviceless, (ClientData)&newcount);
+
+    if ((hc->hc_use->use_def->def_devs == NULL) && (newcount == 0))
+    {
+	/* Add all our own nodes to the table */
+	efAddNodes(hc, TRUE);
+
+	/* Process our own connections and adjustments */
+	efAddConns(hc);
+
+	/* Mark this definition as having no devices, so it will not be visited */
+	hc->hc_use->use_def->def_flags |= DEF_NODEVICES;
+
+	(*usecount)--;
+    }
+    return (0);
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -297,7 +402,7 @@ efFlatNodesStdCell(hc)
  * ----------------------------------------------------------------------------
  */
 
-void
+int
 efAddNodes(hc, stdcell)
     HierContext *hc;
     bool stdcell;
@@ -310,17 +415,9 @@ efAddNodes(hc, stdcell)
     float scale;
     int size, asize;
     HashEntry *he;
-    Transform t;
     bool is_subcircuit = (def->def_flags & DEF_SUBCIRCUIT) ? TRUE : FALSE;
 
     scale = def->def_scale;
-    t = hc->hc_trans;
-    t.t_a = (int)((float)t.t_a * scale);
-    t.t_b = (int)((float)t.t_b * scale);
-    t.t_c = (int)((float)t.t_c * scale);
-    t.t_d = (int)((float)t.t_d * scale);
-    t.t_e = (int)((float)t.t_e * scale);
-    t.t_f = (int)((float)t.t_f * scale);
     size = sizeof (EFNode) + (efNumResistClasses-1) * sizeof (PerimArea);
 
     for (node = (EFNode *) def->def_firstn.efnode_next;
@@ -338,7 +435,12 @@ efAddNodes(hc, stdcell)
 	    asize = ATTRSIZE(strlen(ap->efa_text));
 	    newap = (EFAttr *) mallocMagic((unsigned)(asize));
 	    (void) strcpy(newap->efa_text, ap->efa_text);
-	    GeoTransRect(&t, &ap->efa_loc, &newap->efa_loc);
+	    GeoTransRect(&hc->hc_trans, &ap->efa_loc, &newap->efa_loc);
+	    newap->efa_loc.r_xbot = (int)((float)(newap->efa_loc.r_xbot) * scale);
+	    newap->efa_loc.r_xtop = (int)((float)(newap->efa_loc.r_xtop) * scale);
+	    newap->efa_loc.r_ybot = (int)((float)(newap->efa_loc.r_ybot) * scale);
+	    newap->efa_loc.r_ytop = (int)((float)(newap->efa_loc.r_ytop) * scale);
+
 	    newap->efa_type = ap->efa_type;
 	    newap->efa_next = newnode->efnode_attrs;
 	    newnode->efnode_attrs = newap;
@@ -349,7 +451,15 @@ efAddNodes(hc, stdcell)
 	newnode->efnode_type = node->efnode_type;
 	bcopy((char *) node->efnode_pa, (char *) newnode->efnode_pa,
 		efNumResistClasses * sizeof (PerimArea));
-	GeoTransRect(&t, &node->efnode_loc, &newnode->efnode_loc);
+	GeoTransRect(&hc->hc_trans, &node->efnode_loc, &newnode->efnode_loc);
+
+	/* Scale the result by "scale" --- hopefully we end up with an integer	*/
+	/* We don't scale the transform because the scale may be non-integer	*/
+	/* and the Transform type has integers only.				*/
+	newnode->efnode_loc.r_xbot = (int)((float)(newnode->efnode_loc.r_xbot) * scale);
+	newnode->efnode_loc.r_xtop = (int)((float)(newnode->efnode_loc.r_xtop) * scale);
+	newnode->efnode_loc.r_ybot = (int)((float)(newnode->efnode_loc.r_ybot) * scale);
+	newnode->efnode_loc.r_ytop = (int)((float)(newnode->efnode_loc.r_ytop) * scale);
 
 	/* Prepend to global node list */
 	newnode->efnode_next = efNodeList.efnode_next;
@@ -409,6 +519,7 @@ efAddNodes(hc, stdcell)
 	    }
 	}
     }
+    return 0;
 }
 
 /*
