@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/extflat/EFread.c,v 1.5 2008/12/02 17:12:36 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/extflat/EFread.c,v 1.4 2009/01/30 03:51:02 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -33,6 +33,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/
 #include "utils/hash.h"
 #include "utils/utils.h"
 #include "tiles/tile.h"
+#include "commands/commands.h"
 #include "database/database.h"
 #include "extflat/extflat.h"
 #include "extflat/EFint.h"
@@ -41,8 +42,8 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/
 
 #ifndef MAGIC_WRAPPER
 /* This must match the definition for extDevTable in extract/ExtBasic.c */
-char *extDevTable[] = {"fet", "mosfet", "bjt", "devres", "devcap",
-		"diode", "subckt", "rsubckt", NULL};
+char *extDevTable[] = {"fet", "mosfet", "asymmetric", "bjt", "devres",
+		"devcap", "diode", "subckt", "rsubckt", "msubckt", NULL};
 #endif
 
 /*
@@ -67,7 +68,7 @@ keyTable[] =
     "adjust",		ADJUST,		4,
     "attr",		ATTR,		8,
     "cap",		CAP,		4,
-    "device",		DEVICE,		13,	/* effectively replaces "fet" */
+    "device",		DEVICE,		11,	/* effectively replaces "fet" */
     "distance",		DIST,		4,
     "equiv",		EQUIV,		3,
     "fet",		FET,		12,	/* for backwards compatibility */
@@ -141,7 +142,6 @@ EFReadFile(name, dosubckt, resist, noscale)
     return rc;
 }
 
-
 /*
  * ----------------------------------------------------------------------------
  *
@@ -284,6 +284,7 @@ readfile:
 		switch (n)
 		{
 		    case DEV_MOSFET:
+		    case DEV_ASYMMETRIC:
 		    case DEV_BJT:
 			ac = 10;
 			break;
@@ -298,6 +299,7 @@ readfile:
 			   ac = 9;	/* Has device L and W */
 			break;
 		    case DEV_SUBCKT:
+		    case DEV_MSUBCKT:
 		    case DEV_RSUBCKT:
 			ac = 7;	/* Actually can have many arguments, which */
 			break;	/* we will deal with in efBuildDevice().   */
@@ -431,11 +433,29 @@ resistChanged:
 #ifdef MAGIC_WRAPPER
 		if (strcmp(argv[1], DBTechName))
 		{
-		    TxError("Error: .ext file has different technology %s\n",
+		    /* If we are running in batch mode and no layout is	*/
+		    /* present, then load the new technology.		*/
+
+		    if (CmdCheckForPaintFunc())
+		    {
+			TxError("Error: .ext file has different technology %s\n",
 				argv[1]);
-		    TxError("Load this technology and repeat.\n");
-		    rc = FALSE;
-		    break;
+			TxError("Load this technology and repeat.\n");
+			rc = FALSE;
+			break;
+		    }
+		    else
+		    {
+			TxError("Loading technology %s\n", argv[1]);
+			if (!TechLoad(argv[1], 0))
+			{
+			    TxError("Error in loading technology file\n");
+			    rc = FALSE;
+			    break;
+			}
+			else
+			    EFTech = StrDup((char **) NULL, argv[1]);
+		    }
 		}
 #else
 		if (EFTech && EFTech[0])
