@@ -19,7 +19,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/extract/ExtArray.c,v 1.3 2009/09/11 18:05:37 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/extract/ExtArray.c,v 1.2 2009/05/30 03:14:00 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -69,7 +69,50 @@ void extArrayProcess();
 void extArrayAdjust();
 void extArrayHardSearch();
 
-
+#if 0
+
+/* 
+ * ----------------------------------------------------------------------------
+ * extOutputGeneratedLabels ---
+ *
+ * Write to the .ext file output "node" lines for labels generated in
+ * the parent cell where paint in the subcell is not otherwise
+ * represented by a node in the parent.  These nodes have no material
+ * in the parent, and therefore have no capacitance or resistance
+ * associated with them.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+void
+extOutputGeneratedLabels(parentUse, f)
+    CellUse *parentUse;
+    FILE *f;
+{
+    CellDef *parentDef;
+    Label *lab;
+    int n;
+
+    parentDef = parentUse->cu_def;
+
+    while ((lab = parentDef->cd_labels) != NULL)
+    {
+	if ((lab->lab_flags & LABEL_GENERATE) == 0) return;
+
+	fprintf(f, "node \"%s\" 0 0 %d %d %s",
+		lab->lab_text, lab->lab_rect.r_xbot,
+		lab->lab_rect.r_ybot,
+		DBTypeShortName(lab->lab_type));
+	for (n = 0; n < ExtCurStyle->exts_numResistClasses; n++)
+	    fprintf(f, " 0 0");
+	putc('\n', f);
+	freeMagic(lab);
+	parentDef->cd_labels = lab->lab_next;
+    }
+}
+
+#endif
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -126,6 +169,11 @@ extArray(parentUse, f)
     scx.scx_area = TiPlaneRect;
     (void) DBCellSrArea(&scx, extArrayFunc, (ClientData) &ha);
 
+#if 0
+    /* Output generated labels and remove them from the parent */
+    extOutputGeneratedLabels(parentUse, f);
+#endif
+
     /* Output connections and node adjustments */
     extOutputConns(&ha.ha_connHash, f);
     HashKill(&ha.ha_connHash);
@@ -409,7 +457,8 @@ extArrayPrimaryFunc(use, trans, x, y, ha)
      */
     primDef = extArrayPrimary->et_use->cu_def;
     extArrayPrimary->et_nodes = extFindNodes(primDef, &ha->ha_clipArea);
-    ExtLabelRegions(primDef, ExtCurStyle->exts_nodeConn);
+    ExtLabelRegions(primDef, ExtCurStyle->exts_nodeConn,
+		&extArrayPrimary->et_nodes, &ha->ha_clipArea);
     if ((ExtOptions & (EXT_DOADJUST|EXT_DOCOUPLING))
 		   == (EXT_DOADJUST|EXT_DOCOUPLING))
 	extFindCoupling(primDef, &extArrayPrimary->et_coupleHash,
@@ -547,7 +596,8 @@ extArrayInterFunc(use, trans, x, y, ha)
      */
     oneDef = oneFlat->et_use->cu_def;
     oneFlat->et_nodes = extFindNodes(oneDef, &ha->ha_clipArea),
-    ExtLabelRegions(oneDef, ExtCurStyle->exts_nodeConn);
+    ExtLabelRegions(oneDef, ExtCurStyle->exts_nodeConn, &oneFlat->et_nodes,
+		&ha->ha_clipArea);
     if ((ExtOptions & (EXT_DOADJUST|EXT_DOCOUPLING))
 		   == (EXT_DOADJUST|EXT_DOCOUPLING))
 	extFindCoupling(oneDef, &oneFlat->et_coupleHash, &ha->ha_clipArea);
