@@ -53,8 +53,8 @@ static char sccsid[] = "@(#)ExtBasic.c	4.13 MAGIC (Berkeley) 12/5/85";
 /* ext2spice/ext2sim are compiled as separate programs (i.e., non-Tcl)	*/
 
 #ifdef MAGIC_WRAPPER
-char *extDevTable[] = {"fet", "mosfet", "bjt", "devres", "devcap",
-	"diode", "subckt", "rsubckt", NULL};
+char *extDevTable[] = {"fet", "mosfet", "asymmetric", "bjt", "devres",
+	"devcap", "diode", "subckt", "rsubckt", "msubckt", NULL};
 #endif
 
 /* --------------------- Data local to this file ---------------------- */
@@ -89,7 +89,7 @@ struct transRec
     int		 tr_gatelen;		/* Perimeter of connection to gate */
     NodeRegion	*tr_gatenode;		/* Node region for gate terminal */
     NodeRegion	*tr_termnode[MAXSD];	/* Node region for each diff terminal */
-    NodeRegion	*tr_subsnode;		/* Substrate node */
+    NodeRegion  *tr_subsnode;		/* Substrate node */
     int		 tr_termlen[MAXSD];	/* Length of each diff terminal edge,
 					 * used for computing L/W for the fet.
 					 */
@@ -111,6 +111,18 @@ typedef struct LB1
 
 LinkedBoundary **extSpecialBounds;		/* Linked Boundary List */
 
+/* Structure used for finding substrate connections on implicitly-defined
+ * substrates
+ */
+
+typedef struct TSD1
+{
+    bool found;		/* Set to 1 if a substrate connection was found */
+    Rect rtrans;	/* Rectangle of device */
+    Rect rhalo;		/* Search halo around device */
+    NodeRegion *nreg;	/* Closest substrate region within halo */
+} TransSubsData;
+
 #define	EDGENULL(r)	((r)->r_xbot > (r)->r_xtop || (r)->r_ybot > (r)->r_ytop)
 
 /* Forward declarations */
@@ -118,7 +130,7 @@ void extOutputNodes();
 int extTransTileFunc();
 int extTransPerimFunc();
 NodeRegion *extTransFindSubsNode();
-int extTransFindSubsFunc();
+int extTransFindSubs();
 
 int extAnnularTileFunc();
 int extResistorTileFunc();
@@ -198,7 +210,7 @@ extBasic(def, outFile)
 
     /* Assign the labels to their associated regions */
     if (!SigInterruptPending)
-	ExtLabelRegions(def, ExtCurStyle->exts_nodeConn);
+	ExtLabelRegions(def, ExtCurStyle->exts_nodeConn, &nodeList, &TiPlaneRect);
 
     /*
      * Make sure all geometry with the same label is part of the
@@ -318,7 +330,7 @@ extSetResist(reg)
 	extResistArea[n] = extResistPerim[n] = 0;
     }
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -394,9 +406,24 @@ extOutputNodes(nodeList, outFile)
 	fprintf(outFile, "node \"%s\" %d %lg", text, intR, finC);
 
 	/* Output its location (lower-leftmost point and type name) */
-	fprintf(outFile, " %d %d %s",
+
+	if (reg->nreg_type & TT_DIAGONAL) {
+	    /* Node may be recorded as a diagonal tile if no other	*/
+	    /* non-diagonal tiles are adjoining it.			*/
+
+	    TileType loctype = (reg->nreg_type & TT_SIDE) ? ((reg->nreg_type &
+			TT_RIGHTMASK) >> 14) : (reg->nreg_type & TT_LEFTMASK);
+
+	    fprintf(outFile, " %d %d %s",
+		    reg->nreg_ll.p_x, reg->nreg_ll.p_y,
+		    DBTypeShortName(loctype));
+	}
+	else
+	{
+	    fprintf(outFile, " %d %d %s",
 		    reg->nreg_ll.p_x, reg->nreg_ll.p_y,
 		    DBTypeShortName(reg->nreg_type));
+	}
 
 	/* Output its area and perimeter for each resistivity class */
 	for (n = 0; n < ExtCurStyle->exts_numResistClasses; n++)
@@ -701,7 +728,7 @@ extComputeCapLW(rlengthptr, rwidthptr)
  * first converted to Manhattan geometry.
  *
  * Results:
- *	None
+ *	None.
  *
  * Side Effects:
  *	Puts effective length and width into the pointers
@@ -711,9 +738,9 @@ extComputeCapLW(rlengthptr, rwidthptr)
 
 void
 extComputeEffectiveLW(rlengthptr, rwidthptr, numregions, chop)
-    int *rlengthptr, *rwidthptr;
-    int numregions;
-    float chop;
+   int *rlengthptr, *rwidthptr;
+   int numregions;
+   float chop;
 {
     int i, j, p, jmax;
     LinkedBoundary *lb, *lb2;
@@ -773,7 +800,7 @@ extComputeEffectiveLW(rlengthptr, rwidthptr, numregions, chop)
 
         for (i = 0; i < numregions; i++)
 	{
-	    if (i == p) continue;
+	    if ((i == p) && (i > 1)) continue;
 	    for (lb2 = extSpecialBounds[i]; lb2 != NULL; lb2 = lb2->b_next)
 	    {
 		if (lb2->dir == oppdir)
@@ -893,7 +920,7 @@ extComputeEffectiveLW(rlengthptr, rwidthptr, numregions, chop)
 	cornerw = 0;
         for (i = 0; i < numregions; i++)
 	{
-	    if (i == p) continue;
+	    if ((i == p) && (i > 1)) continue;
 	    for (lb2 = extSpecialBounds[i]; lb2 != NULL; lb2 = lb2->b_next)
 	    {
 		if (lb2->dir == oppdir)
@@ -968,7 +995,14 @@ extComputeEffectiveLW(rlengthptr, rwidthptr, numregions, chop)
     if ((length > 0) && (width > 0))
     {
 	*rlengthptr = length;
-	*rwidthptr = (width >> 1);
+
+	// If numregions == 1 then everything was put in one record,
+	// and we have double-counted the width.
+
+	if (numregions == 1)
+	    *rwidthptr = (width >> 2);
+	else
+	    *rwidthptr = (width >> 1);
 
 	/* fprintf(stderr, "total L = %d, W = %d\n", length, width); */
 	/* fflush(stderr); */
@@ -1100,9 +1134,9 @@ extSeparateBounds(nterm)
  *
  * Scan through the TransRegion in the supplied list, and collect a mask of
  * all transistor types used in the layout.  Then for each transistor type,
- * find if it belongs to a "subcircuit" or "rsubcircuit" definition.  If it
- * does, output a record containing the list of parameter names used by that
- * subcircuit.
+ * find if it belongs to a "subcircuit" (including "rsubcircuit" and
+ * "msubcircuit") definition.  If it does, output a record containing the
+ * list of parameter names used by that subcircuit.
  *
  * Results:
  *	None.
@@ -1137,13 +1171,81 @@ extOutputParameters(def, transList, outFile)
 	    {
 		fprintf(outFile, "parameters %s", ExtCurStyle->exts_transName[t]);
 		for (; plist != NULL; plist = plist->pl_next)
-		    fprintf(outFile, " %c=%s", plist->pl_param, plist->pl_name);
+		{
+		    if (plist->pl_param[1] != '\0')
+		    {
+			if (plist->pl_scale != 1.0)
+			    fprintf(outFile, " %c%c=%s*%g", 
+					plist->pl_param[0], plist->pl_param[1],
+					plist->pl_name, plist->pl_scale);
+			else
+			    fprintf(outFile, " %c%c=%s", plist->pl_param[0],
+					plist->pl_param[1], plist->pl_name);
+		    }
+		    else
+		    {
+			if (plist->pl_scale != 1.0)
+			    fprintf(outFile, " %c=%s*%g", 
+					plist->pl_param[0],
+					plist->pl_name, plist->pl_scale);
+			else
+			    fprintf(outFile, " %c=%s", plist->pl_param[0],
+					plist->pl_name);
+		    }
+		}
 		fprintf(outFile, "\n");
 	    }
 	}
     }
 }
 
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * extGetNativeResistClass() --
+ *
+ * For the purpose of generating a node area and perimeter value to output
+ * to a subcircuit call as a passed parameter.  The value output is assumed
+ * to refer only to the part of the whole eletrical node that is the
+ * actual device node, not to include connected metal, contacts, etc.
+ * Since area and perimeter information about a node is separated into
+ * resist classes, we need to figure out which resist class belongs to
+ * the device terminal type.
+ *
+ * "type" is the type identifier for the device (e.g., gate).  "term" is
+ * the index of the terminal for the device.  Devices with symmetrical
+ * terminals (e.g., MOSFETs), may have fewer type masks than terminals.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+int
+extGetNativeResistClass(type, term)
+    TileType type;
+    int term;
+{
+    TileTypeBitMask *tmask, *rmask;
+    int i, n;
+
+    tmask = NULL;
+    for (i = 0;; i++)
+    {
+	rmask = &ExtCurStyle->exts_transSDTypes[type][i];
+	if (TTMaskHasType(rmask, TT_SPACE)) break;
+	tmask = rmask;
+	if (i == term) break;
+    }
+    if (tmask == NULL) return -1;	/* Error */
+
+    for (n = 0; n < ExtCurStyle->exts_numResistClasses; n++)
+    {
+	rmask = &ExtCurStyle->exts_typesByResistClass[n];
+	if (TTMaskIntersect(rmask, tmask))
+	    return n;
+    }
+    return -1; 		/* Error */
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1187,7 +1289,7 @@ extOutputTrans(def, transList, outFile)
     FindRegion arg;
     LabelList *ll;
     TileType t;
-    int nsd, length, width, n, i, ntiles;
+    int nsd, length, width, n, i, ntiles, corners, tn, rc;
     double dres, dcap;
     char mesg[256];
     bool isAnnular, hasModel;
@@ -1218,9 +1320,10 @@ extOutputTrans(def, transList, outFile)
 
 	/* Set all terminals to NULL to guard against	*/
 	/* asymmetric devices missing a terminal.	*/
+	/* 5/30/09---but, reinitialize the array out to MAXSD,	*/
+	/* or devices declaring minterms < maxterms screw up!	*/
+
 	nsd = ExtCurStyle->exts_transSDCount[t];
-	/* NOTE:  This was nsd, but leaves non-NULL terminals	*/
-	/* when minterms < maxterms. . .			*/
 	for (i = 0; i < MAXSD; i++) extTransRec.tr_termnode[i] = NULL;
 
 	/* Mark with reg and process each perimeter segment */
@@ -1327,6 +1430,8 @@ extOutputTrans(def, transList, outFile)
 	    case DEV_MOSFET:
 	    case DEV_BJT:
 	    case DEV_SUBCKT:
+	    case DEV_MSUBCKT:
+	    case DEV_ASYMMETRIC:
 		length = extTransRec.tr_gatelen / 2;	/* (default) */
 		width = 0;
 		isAnnular = FALSE;
@@ -1364,7 +1469,11 @@ extOutputTrans(def, transList, outFile)
 
 		if (n == 0)
 		{
-		    /* Don't issue warning on devices with no declared terminals */
+		    /* Don't issue a warning on devices such as a 	*/
+		    /* vertical diode that may declare zero terminals	*/
+		    /* because the substrate node (i.e., well) is the	*/
+		    /* other terminal.					*/
+
 		    if (ExtDoWarn && (ExtCurStyle->exts_transSDCount[t] > 0))
 			extTransBad(def, reg->treg_tile,
 				"Could not determine device boundary");
@@ -1407,31 +1516,36 @@ extOutputTrans(def, transList, outFile)
 
 		}
 
-		if (ExtCurStyle->exts_deviceClass[t] == DEV_SUBCKT)
+		if (ExtCurStyle->exts_deviceClass[t] == DEV_SUBCKT ||
+			ExtCurStyle->exts_deviceClass[t] == DEV_MSUBCKT)
 		{
 		    for (chkParam = ExtCurStyle->exts_deviceParams[t]; chkParam
 				!= NULL; chkParam = chkParam->pl_next)
 		    {
-			switch(tolower(chkParam->pl_param))
+			switch(tolower(chkParam->pl_param[0]))
 			{
 			    case 'a':
-				fprintf(outFile, " %c=%d", chkParam->pl_param,
+				if (chkParam->pl_param[1] == '\0' ||
+					chkParam->pl_param[1] == '0')
+				    fprintf(outFile, " %c=%d", chkParam->pl_param[0],
 						reg->treg_area);
 				break;
 			    case 'p':
-				fprintf(outFile, " %c=%d", chkParam->pl_param,
+				if (chkParam->pl_param[1] == '\0' ||
+					chkParam->pl_param[1] == '0')
+				    fprintf(outFile, " %c=%d", chkParam->pl_param[0],
 						extTransRec.tr_perim);
 				break;
 			    case 'l':
-				fprintf(outFile, " %c=%d", chkParam->pl_param,
+				fprintf(outFile, " %c=%d", chkParam->pl_param[0],
 						length);
 				break;
 			    case 'w':
-				fprintf(outFile, " %c=%d", chkParam->pl_param,
+				fprintf(outFile, " %c=%d", chkParam->pl_param[0],
 						width);
 				break;
 			    case 'c':
-				fprintf(outFile, " %c=%g", chkParam->pl_param,
+				fprintf(outFile, " %c=%g", chkParam->pl_param[0],
 					(ExtCurStyle->exts_transGateCap[t]
 					* reg->treg_area) +
 					(ExtCurStyle->exts_transSDCap[t]
@@ -1443,13 +1557,13 @@ extOutputTrans(def, transList, outFile)
 				/* Do nothing;  these values are standard output */
 				break;
 			    default:
-				fprintf(outFile, " %c=", chkParam->pl_param);
+				fprintf(outFile, " %c=", chkParam->pl_param[0]);
 				break;
 			}
 		    }
 
 		}
-		else  /* DEV_MOSFET, DEV_BJT */
+		else  /* DEV_MOSFET, DEV_ASYMMETRIC, DEV_BJT */
 		{
 		    fprintf(outFile, " %d %d", length, width);
 		}
@@ -1467,12 +1581,20 @@ extOutputTrans(def, transList, outFile)
 	    case DEV_RSUBCKT:
 		hasModel = strcmp(ExtCurStyle->exts_transName[t], "None");
 		length = extTransRec.tr_perim;
+		isAnnular = FALSE;
 	
 		/* Boundary perimeter scan for resistors with more than */
 		/* one tile.						*/
 
-		for (n = 0; n < extTransRec.tr_nterm &&
-				extTransRec.tr_termnode[n] != NULL; n++);
+		for (n = 0; n < extTransRec.tr_nterm; n++)
+		{
+		    if (extTransRec.tr_termnode[n] == NULL) continue;
+
+		    /* Mark annular resistors as requiring extra processing */
+		    if (extTransRec.tr_termvector[n].p_x == 0 &&
+				extTransRec.tr_termvector[n].p_y == 0)
+			isAnnular = TRUE;
+		}
 
 		if (n == 0)
 		    width = length = 0;
@@ -1489,14 +1611,21 @@ extOutputTrans(def, transList, outFile)
 
 		    arg.fra_uninit = (ClientData) extTransRec.tr_gatenode;
 		    arg.fra_region = (Region *) reg;
-		    arg.fra_each = extResistorTileFunc;
+		    if (isAnnular)
+		        arg.fra_each = extAnnularTileFunc;
+		    else
+		        arg.fra_each = extResistorTileFunc;
 		    (void) ExtFindNeighbors(reg->treg_tile, arg.fra_pNum, &arg);
 
 		    if (extSpecialBounds[0] != NULL)
 		    {
 			extSeparateBounds(n - 1);
-			extComputeEffectiveLW(&width, &length, n,
-				ExtCurStyle->exts_cornerChop[t]);
+			if (isAnnular)
+			    extComputeEffectiveLW(&length, &width, n, 
+					ExtCurStyle->exts_cornerChop[t]);
+			else
+			    extComputeEffectiveLW(&width, &length, n, 
+					ExtCurStyle->exts_cornerChop[t]);
 		    }
 		    else
 		    {
@@ -1562,30 +1691,34 @@ extOutputTrans(def, transList, outFile)
 		    for (chkParam = ExtCurStyle->exts_deviceParams[t]; chkParam
 				!= NULL; chkParam = chkParam->pl_next)
 		    {
-			switch(tolower(chkParam->pl_param))
+			switch(tolower(chkParam->pl_param[0]))
 			{
 			    case 'a':
-				fprintf(outFile, " %c=%d", chkParam->pl_param,
-					reg->treg_area);
+				if (chkParam->pl_param[1] == '\0' ||
+					chkParam->pl_param[1] == '0')
+				    fprintf(outFile, " %c=%d", chkParam->pl_param[0],
+						reg->treg_area);
 				break;
 			    case 'p':
-				fprintf(outFile, " %c=%d", chkParam->pl_param,
-					extTransRec.tr_perim);
+				if (chkParam->pl_param[1] == '\0' ||
+					chkParam->pl_param[1] == '0')
+				    fprintf(outFile, " %c=%d", chkParam->pl_param[0],
+						extTransRec.tr_perim);
 				break;
 			    case 'l':
-				fprintf(outFile, " %c=%d", chkParam->pl_param,
+				fprintf(outFile, " %c=%d", chkParam->pl_param[0],
 					length);
 				break;
 			    case 'w':
-				fprintf(outFile, " %c=%d", chkParam->pl_param,
+				fprintf(outFile, " %c=%d", chkParam->pl_param[0],
 					width);
 				break;
 			    case 'r':
-				fprintf(outFile, " %c=%g", chkParam->pl_param,
+				fprintf(outFile, " %c=%g", chkParam->pl_param[0],
 					dres / 1000.0);
 				break;
 			    case 'c':
-				fprintf(outFile, " %c=%g", chkParam->pl_param,
+				fprintf(outFile, " %c=%g", chkParam->pl_param[0],
 					(ExtCurStyle->exts_transGateCap[t]
 					* reg->treg_area) +
 					(ExtCurStyle->exts_transSDCap[t]
@@ -1597,7 +1730,7 @@ extOutputTrans(def, transList, outFile)
 				/* Do nothing;  these values are standard output */
 				break;
 			    default:
-				fprintf(outFile, " %c=", chkParam->pl_param);
+				fprintf(outFile, " %c=", chkParam->pl_param[0]);
 				break;
 			}
 		    }
@@ -1629,7 +1762,7 @@ extOutputTrans(def, transList, outFile)
 		    {
 			width = 0;
 			extTransBad(def, reg->treg_tile,
-					"Capacitor has zero size");
+						"Capacitor has zero size");
 			fprintf(outFile, " 0 0");
 		    }
 //		    else if ((ntiles == 1) || (n > 1))
@@ -1732,7 +1865,7 @@ extOutputTrans(def, transList, outFile)
 }
 
 int
-extTransFindSubsFunc1(tile, def, sn)
+extTransFindSubs(tile, def, sn)
     Tile *tile;
     CellDef *def;
     NodeRegion **sn;
@@ -1741,23 +1874,20 @@ extTransFindSubsFunc1(tile, def, sn)
     TileTypeBitMask *mask;
     Rect tileArea;
     int pNum;
-    int extTransFindSubsFunc2();	/* Forward declaration */
+    int extTransFindSubsFunc1();	/* Forward declaration */
 
     TiToRect(tile, &tileArea);
-#ifdef NONMANHATTAN
     if (IsSplit(tile))
-	t = (SplitSide(tile)) ? SplitRightType(tile): SplitLeftType(tile);
+        t = (SplitSide(tile)) ? SplitRightType(tile): SplitLeftType(tile);
     else
-#endif
-	t = TiGetTypeExact(tile);
-
+        t = TiGetTypeExact(tile);
     mask = &ExtCurStyle->exts_transSubstrateTypes[t];
     for (pNum = PL_TECHDEPBASE; pNum < DBNumPlanes; pNum++)
     {
 	if (TTMaskIntersect(&DBPlaneTypes[pNum], mask))
 	{
 	    if (DBSrPaintArea((Tile *) NULL, def->cd_planes[pNum], &tileArea,
-		    mask, extTransFindSubsFunc2, (ClientData)sn))
+		    mask, extTransFindSubsFunc1, (ClientData)sn))
 		return 1;
 	}
     }
@@ -1765,18 +1895,26 @@ extTransFindSubsFunc1(tile, def, sn)
 }
 
 int
-extTransFindSubsFunc2(tile, sn)
+extTransFindSubsFunc1(tile, sn)
     Tile *tile;
     NodeRegion **sn;
 {
+    /* Report split substrate region errors (two different substrate
+     * regions under the same device)
+     */
+
     if (tile->ti_client != (ClientData) extUnInit)
     {
+	if ((*sn != (NodeRegion *)NULL) && (*sn != tile->ti_client))
+	    TxError("Warning:  Split substrate under device at (%d %d)\n",
+			tile->ti_ll.p_x, tile->ti_ll.p_y);
+
 	*sn = (NodeRegion *) tile->ti_client;
 	return 1;
     }
     return 0;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -1803,6 +1941,7 @@ extTransTileFunc(tile, pNum, arg)
 {
     TileTypeBitMask mask;
     TileType loctype;
+    int perim;
 
     LabelList *ll;
     Label *lab;
@@ -1824,32 +1963,28 @@ extTransTileFunc(tile, pNum, arg)
      * Visit each segment of the perimeter of this tile that
      * that borders on something of a different type.
      */
-#ifdef NONMANHATTAN
     if (IsSplit(tile))
     {
-	int width, height;
-
         loctype = (SplitSide(tile)) ? SplitRightType(tile): SplitLeftType(tile);
-	width = RIGHT(tile) - LEFT(tile);
-	height = TOP(tile) - BOTTOM(tile);
-
-	/* Note that extTransPerimFunc, as written, measures perimeter around	*/
-	/* the edge of a split tile, for compatibility with the width/length	*/
-	/* calculating algorithm.  This corrects for that error.		*/
-
-	extTransRec.tr_perim -= (width + height);
-	extTransRec.tr_perim += (int)sqrt((double)(width * width + height * height));
+	// return (0); 	/* Hack alert!  We must properly handle diagonals! */
     }
     else
-#endif
-    loctype = TiGetTypeExact(tile);
+	loctype = TiGetTypeExact(tile);
 
     mask = ExtCurStyle->exts_transConn[loctype];
     TTMaskCom(&mask);
-    extEnumTilePerim(tile, mask, pNum, extTransPerimFunc, (ClientData)NULL);
+
+    /* NOTE:  DO NOT USE extTransRec.tr_perim += extEnumTilePerim(...)	*/
+    /* The AMD target gcc compile works and the Intel target gcc	*/
+    /* compile doesn't!  The following code works the same on both.	*/
+
+    perim = extEnumTilePerim(tile, mask, pNum,
+		extTransPerimFunc, (ClientData)NULL);
+    extTransRec.tr_perim += perim;
 
     if (extTransRec.tr_subsnode == (NodeRegion *)NULL)
-	extTransFindSubsFunc1(tile, arg->fra_def, &extTransRec.tr_subsnode);
+	extTransFindSubs(tile, arg->fra_def, &extTransRec.tr_subsnode);
+
     return (0);
 }
 
@@ -1859,27 +1994,24 @@ extTransPerimFunc(bp)
 {
     TileType tinside, toutside;
     Tile *tile;
-    NodeRegion *diffNode;
-    int i, len;
+    NodeRegion *diffNode = (NodeRegion *) extGetRegion(bp->b_outside);
+    int i, len = BoundaryLength(bp);
     int thisterm;
     LabelList *ll;
     Label *lab;
     Rect r;
     bool SDterm = FALSE;
 
-    diffNode = (NodeRegion *) extGetRegion(bp->b_outside);
-    len = BoundaryLength(bp);
-
     tile = bp->b_inside;
     if (IsSplit(tile))
-	tinside = (SplitSide(tile)) ? SplitRightType(tile): SplitLeftType(tile);
+        tinside = (SplitSide(tile)) ? SplitRightType(tile): SplitLeftType(tile);
     else
-	tinside = TiGetTypeExact(bp->b_inside);
+        tinside = TiGetTypeExact(bp->b_inside);
     tile = bp->b_outside;
     if (IsSplit(tile))
-	toutside = (SplitSide(tile)) ? SplitRightType(tile): SplitLeftType(tile);
+        toutside = (SplitSide(tile)) ? SplitRightType(tile): SplitLeftType(tile);
     else
-	toutside = TiGetTypeExact(bp->b_outside);
+        toutside = TiGetTypeExact(bp->b_outside);
 
     for (i = 0; !TTMaskHasType(&ExtCurStyle->exts_transSDTypes[tinside][i], TT_SPACE);
 		i++)
@@ -1934,7 +2066,9 @@ extTransPerimFunc(bp)
 		}
 	    }
 	    else
+	    {
 		TxError("Error:  Asymmetric device with multiple terminals!\n");
+	    }
 
 	    /* Add the length to this terminal's perimeter */
 	    extTransRec.tr_termlen[thisterm] += len;
@@ -2028,14 +2162,12 @@ extAnnularTileFunc(tile, pNum)
      * Visit each segment of the perimeter of this tile that
      * that borders on something of a different type.
      */
-#ifdef NONMANHATTAN
     if (IsSplit(tile))
     {
         loctype = (SplitSide(tile)) ? SplitRightType(tile): SplitLeftType(tile);
     }
     else
-#endif
-    loctype = TiGetTypeExact(tile);
+	loctype = TiGetTypeExact(tile);
 
     mask = ExtCurStyle->exts_transConn[loctype];
     TTMaskCom(&mask);
@@ -2079,14 +2211,12 @@ extResistorTileFunc(tile, pNum)
      * Visit each segment of the perimeter of this tile that
      * that borders on something of a different type.
      */
-#ifdef NONMANHATTAN
     if (IsSplit(tile))
     {
         loctype = (SplitSide(tile)) ? SplitRightType(tile): SplitLeftType(tile);
     }
     else
-#endif
-    loctype = TiGetTypeExact(tile);
+	loctype = TiGetTypeExact(tile);
 
     mask = ExtCurStyle->exts_transConn[loctype];
     TTMaskSetMask(&mask, &ExtCurStyle->exts_transSDTypes[loctype][0]);
@@ -2114,45 +2244,31 @@ extSpecialPerimFunc(bp, sense)
     LinkedBoundary *newBound, *lb, *lastlb;
     bool needSurvey;
 
-#ifdef NONMANHATTAN
     /* Note that extEnumTilePerim() assumes for the non-Manhattan case	*/
     /* that non-Manhattan tiles should be incorporated into the device	*/
     /* gate for purposes of computing effective length and width.  In	*/
     /* most cases this will be only a slight deviation from the true	*/
     /* result.								*/
 
-    if (IsSplit(bp->b_inside))
+    switch (bp->b_direction)
     {
-	switch (bp->b_direction)
-	{
-	    case BD_TOP:
-		tinside = TiGetBottomType(bp->b_inside);
-		toutside = TiGetBottomType(bp->b_outside);
-		break;
-	    case BD_BOTTOM:
-		tinside = TiGetTopType(bp->b_inside);
-		toutside = TiGetTopType(bp->b_outside);
-		break;
-	    case BD_RIGHT:
-		tinside = TiGetLeftType(bp->b_inside);
-		toutside = TiGetLeftType(bp->b_outside);
-		break;
-	    case BD_LEFT:
-		tinside = TiGetRightType(bp->b_inside);
-		toutside = TiGetRightType(bp->b_outside);
-		break;
-	}
+	case BD_TOP:
+	    tinside = TiGetTopType(bp->b_inside);
+	    toutside = TiGetBottomType(bp->b_outside);
+	    break;
+	case BD_BOTTOM:
+	    tinside = TiGetBottomType(bp->b_inside);
+	    toutside = TiGetTopType(bp->b_outside);
+	    break;
+	case BD_RIGHT:
+	    tinside = TiGetRightType(bp->b_inside);
+	    toutside = TiGetLeftType(bp->b_outside);
+	    break;
+	case BD_LEFT:
+	    tinside = TiGetLeftType(bp->b_inside);
+	    toutside = TiGetRightType(bp->b_outside);
+	    break;
     }
-    else
-    {
-	tinside = TiGetTypeExact(bp->b_inside);
-	toutside = TiGetTypeExact(bp->b_outside);
-    }
-
-#else
-    tinside = TiGetType(bp->b_inside);
-    toutside = TiGetType(bp->b_outside);
-#endif
 
     /* Check all terminal classes for a matching type */
     needSurvey = FALSE;
@@ -2420,19 +2536,18 @@ extLabType(text, typeMask)
 
 /*
  * ----------------------------------------------------------------------------
- *
  * extNodeToTile --
  *
  *	Sets tp to be the tile containing the lower-leftmost point of the
  *	NodeRegion *np, but in the tile planes of the ExtTree *et instead
- *	of the tile places originally containing *np.  This routine used
+ *	of the tile planes originally containing *np.  This routine used
  *	to be defined as the macro NODETOTILE().
  *
  * Results:
  *	Returns a pointer to the tile
  *
  * Side effects:
- *	None. 
+ *	None.
  * ----------------------------------------------------------------------------
  */
 
@@ -2449,7 +2564,6 @@ Tile *extNodeToTile(np, et)
     GOTOPOINT(tp, &np->nreg_ll);
     myplane->pl_hint = tp;
 
-#ifdef NONMANHATTAN
     if (IsSplit(tp))
     {
 	TileType tpt = TiGetTypeExact(tp);
@@ -2458,7 +2572,6 @@ Tile *extNodeToTile(np, et)
 	else
 	    TiSetBody(tp, tpt | TT_SIDE);
     }
-#endif
 
     return tp;
 }
@@ -2492,11 +2605,10 @@ extSetNodeNum(reg, plane, tile)
 {
     TileType type;
 
-#ifdef NONMANHATTAN
     if (IsSplit(tile))
     {
-	/* Only consider split tiles if the lower-left-hand corner	*/
-	/* is only the type under consideration.			*/
+	/* Only consider split tiles if the lower-left-hand corner      */
+	/* is only the type under consideration.                        */
 
 	if (!SplitSide(tile) && SplitDirection(tile))
 	    type = SplitSide(tile) ? SplitRightType(tile) : SplitLeftType(tile);
@@ -2506,7 +2618,6 @@ extSetNodeNum(reg, plane, tile)
 	    return;
     }
     else
-#endif
 	type = TiGetType(tile);
 
     if ((plane < reg->lreg_pnum) || (reg->lreg_type & TT_DIAGONAL))
@@ -2564,15 +2675,13 @@ extTransFirst(tile, arg)
     reg = (TransRegion *) mallocMagic((unsigned) (sizeof (TransRegion)));
     reg->treg_next = (TransRegion *) NULL;
     reg->treg_labels = (LabelList *) NULL;
-    reg->treg_pnum = DBNumPlanes;
     reg->treg_area = 0;
     reg->treg_tile = tile;
+    reg->treg_pnum = DBNumPlanes;
 
-#ifdef NONMANHATTAN
     if (IsSplit(tile))
 	reg->treg_type = SplitSide(tile) ? SplitRightType(tile) : SplitLeftType(tile);
     else
-#endif
 	reg->treg_type = TiGetTypeExact(tile);
 
     /* Prepend it to the region list */
@@ -2591,7 +2700,6 @@ extTransEach(tile, pNum, arg)
     TransRegion *reg = (TransRegion *) arg->fra_region;
     int area = TILEAREA(tile);
 
-#ifdef NONMANHATTAN
     if (IsSplit(tile)) area /= 2;	/* Split tiles are 1/2 area! */
     else if (IsSplit(reg->treg_tile))
     {
@@ -2599,12 +2707,11 @@ extTransEach(tile, pNum, arg)
 	reg->treg_tile = tile;
 	reg->treg_type = TiGetTypeExact(tile);
     }
-#endif
 
     /* The following is non-ideal.  It assumes that the lowest plane of	*/
-    /* types connected to a devices is the plane of the device itself.	*/
+    /* types connected to a device is the plane of the device itself.	*/
     /* Otherwise, the area of the device will be miscalculated.		*/
-
+    
     if (pNum < reg->treg_pnum) reg->treg_area = 0;
 
     extSetNodeNum((LabRegion *) reg, pNum, tile);
@@ -2693,13 +2800,11 @@ extNodeAreaFunc(tile, arg)
     Rect r;
     PlaneAndArea pla;
 
-#ifdef NONMANHATTAN
     if (IsSplit(tile))
     {
 	type = (SplitSide(tile)) ? SplitRightType(tile) : SplitLeftType(tile);
 	if (type == TT_SPACE) return 0;		/* Should not happen */
     }
-#endif
 
     /* Compute the resistance for the previous region */
     if (old = (NodeRegion *) arg->fra_region)
@@ -2742,15 +2847,13 @@ extNodeAreaFunc(tile, arg)
 	if (DebugIsSet(extDebugID, extDebNeighbor))
 	    extShowTile(tile, "neighbor", 1);
 
-#ifdef NONMANHATTAN
         if (IsSplit(tile))
         {           
             type = (SplitSide(tile)) ? SplitRightType(tile):
                         SplitLeftType(tile);
         }           
         else        
-#endif              
-        type = TiGetTypeExact(tile);
+            type = TiGetTypeExact(tile);
 
 	/* Contacts are replaced by their residues when calculating */
 	/* area/perimeter capacitance and resistance.		    */
@@ -2783,35 +2886,33 @@ extNodeAreaFunc(tile, arg)
 	}
 	else area = TILEAREA(tile);
 
-#ifdef NONMANHATTAN
 	if (IsSplit(tile)) area /= 2;	/* Split tiles are 1/2 area! */
-#endif
 
 	if (resistClass != -1)
 	    extResistArea[resistClass] += area;
 	reg->nreg_cap += area * ExtCurStyle->exts_areaCap[residue];
 
-#ifdef NONMANHATTAN
+	/* Compute perimeter of nonManhattan edges */
 	if (IsSplit(tile))
 	{
 	    len = ((RIGHT(tile) - LEFT(tile)) * (RIGHT(tile) - LEFT(tile))) +
-			((TOP(tile) - BOTTOM(tile)) * (TOP(tile) - BOTTOM(tile)));
+		 	((TOP(tile) - BOTTOM(tile)) * (TOP(tile) - BOTTOM(tile)));
 	    len = (int)sqrt((double)len);
 
 	    if (extNodeClipArea)
 	    {
 		/* To-do:  Find perimeter length of clipped edge */
-	    }	
+	    }
 
 	    /* Find the type on the other side of the tile */
-	    t = (SplitSide(tile)) ? SplitLeftType(tile): SplitRightType(tile);
+            t = (SplitSide(tile)) ? SplitLeftType(tile):
+                        SplitRightType(tile);
 	    tres = (DBIsContact(t)) ? DBPlaneToResidue(t, tilePlaneNum) : t;
 	    if ((capval = ExtCurStyle->exts_perimCap[residue][tres]) != (CapValue) 0)
 		reg->nreg_cap += capval * len;
 	    if (TTMaskHasType(resMask, tres) && resistClass != -1)
 		extResistPerim[resistClass] += len;
 	}
-#endif
 
 	/*
 	 * Walk along all four sides of tile.
@@ -2823,9 +2924,7 @@ extNodeAreaFunc(tile, arg)
 	/* Top */
 topside:
 
-#ifdef NONMANHATTAN
 	if (IsSplit(tile) && (SplitSide(tile) ^ SplitDirection(tile))) goto leftside;
-#endif
 
 	for (tp = RT(tile); RIGHT(tp) > LEFT(tile); tp = BL(tp))
 	{
@@ -2838,7 +2937,6 @@ topside:
 		len = EDGENULL(&r) ? 0 : r.r_xtop - r.r_xbot;
 	    }
 	    else len = MIN(RIGHT(tile), RIGHT(tp)) - MAX(LEFT(tile), LEFT(tp));
-#ifdef NONMANHATTAN
             if (IsSplit(tp))
 	    {
         	t = SplitBottomType(tp);
@@ -2846,9 +2944,14 @@ topside:
 		{
 		    PUSHTILEBOTTOM(tp, tilePlaneNum);
 		}
+		else if (tp->ti_client != (ClientData)reg && TTMaskHasType(mask, t))
+		{
+		    /* Count split tile twice, once for each node it belongs to. */
+		    tp->ti_client = extUnInit;
+		    PUSHTILEBOTTOM(tp, tilePlaneNum);
+		}
 	    }
             else
-#endif
 	    {
 		t = TiGetTypeExact(tp);
 		if (tp->ti_client == extUnInit && TTMaskHasType(mask, t))
@@ -2866,9 +2969,7 @@ topside:
 	/* Left */
 leftside:
 
-#ifdef NONMANHATTAN
 	if (IsSplit(tile) && SplitSide(tile)) goto bottomside;
-#endif
 
 	for (tp = BL(tile); BOTTOM(tp) < TOP(tile); tp = RT(tp))
 	{
@@ -2881,7 +2982,6 @@ leftside:
 		len = EDGENULL(&r) ? 0 : r.r_ytop - r.r_ybot;
 	    }
 	    else len = MIN(TOP(tile), TOP(tp)) - MAX(BOTTOM(tile), BOTTOM(tp));
-#ifdef NONMANHATTAN
             if (IsSplit(tp))
 	    {
                 t = SplitRightType(tp);
@@ -2889,9 +2989,14 @@ leftside:
 		{
 		    PUSHTILERIGHT(tp, tilePlaneNum);
 		}
+		else if (tp->ti_client != (ClientData)reg && TTMaskHasType(mask, t))
+		{
+		    /* Count split tile twice, once for each node it belongs to. */
+		    tp->ti_client = extUnInit;
+		    PUSHTILERIGHT(tp, tilePlaneNum);
+		}
 	    }
             else
-#endif
 	    {
 		t = TiGetTypeExact(tp);
 		if (tp->ti_client == extUnInit && TTMaskHasType(mask, t))
@@ -2909,10 +3014,8 @@ leftside:
 	/* Bottom */
 bottomside:
 
-#ifdef NONMANHATTAN
 	if (IsSplit(tile) && (!(SplitSide(tile) ^ SplitDirection(tile))))
 	    goto rightside;
-#endif
 
 	for (tp = LB(tile); LEFT(tp) < RIGHT(tile); tp = TR(tp))
 	{
@@ -2925,7 +3028,6 @@ bottomside:
 		len = EDGENULL(&r) ? 0 : r.r_xtop - r.r_xbot;
 	    }
 	    else len = MIN(RIGHT(tile), RIGHT(tp)) - MAX(LEFT(tile), LEFT(tp));
-#ifdef NONMANHATTAN
             if (IsSplit(tp))
 	    {
                 t = SplitTopType(tp);
@@ -2933,9 +3035,14 @@ bottomside:
 		{
 		    PUSHTILETOP(tp, tilePlaneNum);
 		}
+		else if (tp->ti_client != (ClientData)reg && TTMaskHasType(mask, t))
+		{
+		    /* Count split tile twice, once for each node it belongs to. */
+		    tp->ti_client = extUnInit;
+		    PUSHTILETOP(tp, tilePlaneNum);
+		}
 	    }
             else
-#endif
 	    {
 		t = TiGetTypeExact(tp);
 		if (tp->ti_client == extUnInit && TTMaskHasType(mask, t))
@@ -2953,9 +3060,7 @@ bottomside:
 	/* Right */
 rightside:
 
-#ifdef NONMANHATTAN
 	if (IsSplit(tile) && !SplitSide(tile)) goto donesides;
-#endif
 
 	for (tp = TR(tile); TOP(tp) > BOTTOM(tile); tp = LB(tp))
 	{
@@ -2968,7 +3073,6 @@ rightside:
 		len = EDGENULL(&r) ? 0 : r.r_ytop - r.r_ybot;
 	    }
 	    else len = MIN(TOP(tile), TOP(tp)) - MAX(BOTTOM(tile), BOTTOM(tp));
-#ifdef NONMANHATTAN
             if (IsSplit(tp))
 	    {
                 t = SplitLeftType(tp);
@@ -2976,9 +3080,14 @@ rightside:
 		{
 		    PUSHTILELEFT(tp, tilePlaneNum);
 		}
+		else if (tp->ti_client != (ClientData)reg && TTMaskHasType(mask, t))
+		{
+		    /* Count split tile twice, once for each node it belongs to	*/
+		    tp->ti_client = extUnInit;
+		    PUSHTILELEFT(tp, tilePlaneNum);
+		}
 	    }
             else
-#endif
 	    {
 		t = TiGetTypeExact(tp);
 		if (tp->ti_client == extUnInit && TTMaskHasType(mask, t))
@@ -3014,7 +3123,6 @@ donesides:
 
 		    if (tp->ti_client != extUnInit) continue;
 
-#ifdef NONMANHATTAN
 		    /* tp and tile should have the same geometry for a contact */
 		    if (IsSplit(tile) && IsSplit(tp))
 		    {           
@@ -3050,7 +3158,6 @@ donesides:
 			}
 		    }
 		    else
-#endif              
 		    {
 			t = TiGetTypeExact(tp);
 			if (TTMaskHasType(mask, t))
@@ -3073,6 +3180,7 @@ donesides:
 	if (pMask = DBAllConnPlanes[type])
 	{
 	    Rect biggerArea;
+	    bool is_split = IsSplit(tile);
 
 	    extNbrUn = extUnInit;
 	    TITORECT(tile, &pla.area);
@@ -3081,9 +3189,17 @@ donesides:
 		if ((pNum != tilePlaneNum) && PlaneMaskHasPlane(pMask, pNum))
 		{
 		    pla.plane = pNum;
-		    (void) DBSrPaintArea((Tile *) NULL,
-			    arg->fra_def->cd_planes[pNum], &biggerArea,
-			    mask, extNbrPushFunc, (ClientData) &pla);
+		    if (is_split)
+		        DBSrPaintNMArea((Tile *) NULL,
+				arg->fra_def->cd_planes[pNum],
+				TiGetTypeExact(tile) &
+				(TT_DIAGONAL | TT_SIDE | TT_DIRECTION),
+				&biggerArea, mask, extNbrPushFunc,
+				(ClientData) &pla);
+		    else
+		        DBSrPaintArea((Tile *) NULL,
+				arg->fra_def->cd_planes[pNum], &biggerArea,
+				mask, extNbrPushFunc, (ClientData) &pla);
 		}
 	}
     }
