@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/extract/ExtCouple.c,v 1.4 2007/03/03 19:22:42 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/extract/ExtCouple.c,v 1.2 2010/06/24 12:37:17 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -274,13 +274,11 @@ extBasicOverlap(tile, ecs)
     int thisPlane = ecs->plane;
     extCoupleStruct ecpls;
 
-#ifdef NONMANHATTAN
     if (IsSplit(tile))
 	thisType = (SplitSide(tile)) ? SplitRightType(tile) :
 		SplitLeftType(tile);
     else
-#endif
-    thisType = TiGetTypeExact(tile);
+	thisType = TiGetTypeExact(tile);
 
     if (DBIsContact(thisType))
 	thisType = DBPlaneToResidue(thisType, thisPlane);
