@@ -20,10 +20,11 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/extract/ExtHier.c,v 1.5 2009/09/11 18:05:37 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/extract/ExtHier.c,v 1.3 2010/06/24 12:37:17 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
+#include <string.h>
 #include <math.h>
 
 #include "utils/magic.h"
@@ -120,7 +121,9 @@ extHierConnectFunc1(oneTile, ha)
     Rect r;
     TileTypeBitMask mask, *connected;
     TileType rtype;
+    Label *lab, *newlab;
     int i;
+    unsigned n;
 
     /*
      * Find all tiles that connect to 'srcTile', but in the
@@ -130,14 +133,12 @@ extHierConnectFunc1(oneTile, ha)
     ha->hierOneTile = oneTile;
     ha->hierType = TiGetTypeExact(oneTile);
 
-#ifdef NONMANHATTAN
     if (IsSplit(oneTile))
     {
 	rtype = ha->hierType;
 	ha->hierType = (rtype & TT_SIDE) ? SplitRightType(oneTile) :
 		SplitLeftType(oneTile);
     }  
-#endif
 
     connected = &(ExtCurStyle->exts_nodeConn[ha->hierType]);
     TITORECT(oneTile, &r);
@@ -149,19 +150,90 @@ extHierConnectFunc1(oneTile, ha)
 	TTMaskAndMask3(&mask, connected, &DBPlaneTypes[i]);
 	if (!TTMaskIsZero(&mask))
 	{
-#ifdef NONMANHATTAN
 	    if (IsSplit(oneTile))
 		DBSrPaintNMArea((Tile *) NULL, cumDef->cd_planes[i],
 			rtype, &r,
 			((i == ha->hierPNum) ? &DBAllButSpaceBits : connected),
 			extHierConnectFunc2, (ClientData) ha);
 	    else
-#endif
 		DBSrPaintArea((Tile *) NULL, cumDef->cd_planes[i], &r,
 			((i == ha->hierPNum) ? &DBAllButSpaceBits : connected),
 			extHierConnectFunc2, (ClientData) ha);
 	}
     }
+
+    /* Where labels have been saved from the parent cell, look for any	*/
+    /* that are inside the cell boundary and would connect to the tile.	*/
+    /* This allows the extractor to catch "sticky" labels that are not	*/
+    /* attached to a physical layer in the parent cell.			*/
+
+    // NOTE by Tim, 9/10/2014:  This generates phantom nodes when the
+    // labels are created by the "hard" node search;  I think this code
+    // should be restricted to sticky labels only.  But not certain.
+    // Definitely this causes problems in arrays, because the array node
+    // name may refer to a range of array elements, and the generated
+    // node only describes a single point.
+
+    for (lab = cumDef->cd_labels;  lab;  lab = lab->lab_next)
+	if (GEO_TOUCH(&r, &lab->lab_rect) && (lab->lab_flags & LABEL_STICKY))
+	    if (TTMaskHasType(connected, lab->lab_type))
+	    {
+		HashTable *table = &ha->ha_connHash;
+		HashEntry *he;
+		NodeName *nn;
+		Node *node1, *node2;
+		char *name;
+
+		/* Register the name, like is done in extHierConnectFunc2 */
+		he = HashFind(table, lab->lab_text);
+		nn = (NodeName *) HashGetValue(he);
+		node1 = nn ? nn->nn_node : extHierNewNode(he);
+
+		name = (*ha->ha_nodename)(ha->hierOneTile, ha->hierPNum,
+			extHierOneFlat, ha, TRUE);
+		he = HashFind(table, name);
+		nn = (NodeName *) HashGetValue(he);
+		node2 = nn ? nn->nn_node : extHierNewNode(he);
+
+		if (node1 != node2)
+		{
+		    /*
+		     * Both sets of names will now point to node1.
+		     * We don't need to update node_cap since it
+		     * hasn't been computed yet.
+		     */
+		    for (nn = node2->node_names; nn->nn_next; nn = nn->nn_next)
+			nn->nn_node = node1;
+		    nn->nn_node = node1;
+		    nn->nn_next = node1->node_names;
+		    node1->node_names = node2->node_names;
+		    freeMagic((char *) node2);
+		}
+
+#if 0
+		/* Copy this label to the parent def with a	*/
+		/* special flag, so we can output it as a node	*/
+	 	/* and then delete it.  Don't duplicate labels	*/
+		/* that are already in the parent.		*/
+
+		for (newlab = ha->ha_parentUse->cu_def->cd_labels;
+				newlab; newlab = newlab->lab_next)
+		    if (!strcmp(newlab->lab_text, lab->lab_text))
+			break;
+
+		if (newlab == NULL)
+		{
+		    n = sizeof(Label) + strlen(lab->lab_text)
+				- sizeof lab->lab_text + 1;
+		    newlab = (Label *)mallocMagic((unsigned)n);
+		    bcopy((char *)lab, (char *)newlab, (int)n);
+		
+		    newlab->lab_next = ha->ha_parentUse->cu_def->cd_labels;
+		    ha->ha_parentUse->cu_def->cd_labels = newlab;
+		}
+#endif
+	    }
+
     return (0);
 }
 
@@ -215,10 +287,8 @@ extHierConnectFunc2(cum, ha)
 
     ttype = TiGetTypeExact(cum);
 
-#ifdef NONMANHATTAN
     if (IsSplit(cum))
 	ttype = (ttype & TT_SIDE) ? SplitRightType(cum) : SplitLeftType(cum);
-#endif
 
     if (extConnectsTo(ha->hierType, ttype, ExtCurStyle->exts_nodeConn))
     {
