@@ -142,7 +142,7 @@ static keydesc keyTable[] = {
 "resistance-scalefactor",
 
     "sidehalo",		SIDEHALO,	2,	2,
-"halo(lambda)",
+"halo",
 
     "sideoverlap",	SIDEOVERLAP,	5,	6,
 "intypes outtypes ovtypes capacitance [shieldtypes]",
@@ -151,7 +151,7 @@ static keydesc keyTable[] = {
 "intypes outtypes neartypes fartypes capacitance",
 
     "step",		STEP,		2,	2,
-"size(lambda)",
+"size",
 
     "style",		STYLE,		2,	4,
 "stylename",
@@ -170,7 +170,7 @@ static keydesc keyTable[] = {
  * Table used for parsing the "device" keyword types
  *
  * (Note: "10" for max types in subcircuit is arbitrary---the parser
- * ignores max types for DEV_SUBCKT).
+ * ignores max types for DEV_SUBCKT and DEV_MSUBCKT).
  */
 
 /* types are enumerated in extract.h */
@@ -197,6 +197,9 @@ static keydesc devTable[] = {
     "rsubcircuit",	DEV_RSUBCKT,		4,	7,
 "name dev-types terminal-types [sub-types|None sub-node] [options]",
 
+    "msubcircuit",	DEV_MSUBCKT,		4,	11,
+"name dev-types [N] [term1-types ... termN-types [sub-types|None sub-node]] [options]",
+
     0
 };
 
@@ -598,7 +601,9 @@ extTechStyleInit(style)
 	}
 
 	TTMaskZero(&style->exts_perimCapMask[r]);
+#ifdef ARIEL
 	TTMaskZero(&style->exts_subsTransistorTypes[r]);
+#endif
 	if (style->exts_transSDTypes[r] != NULL)
 	    freeMagic(style->exts_transSDTypes[r]);
 	style->exts_transSDTypes[r] = NULL;
@@ -955,7 +960,7 @@ ExtTechSimplePerimCap(argc, argv)
     /* Part 1: Perimeter cap */
 
     for (s = TT_TECHDEPBASE; s < DBNumTypes; s++)
-	for (t = TT_TECHDEPBASE; t < DBNumTypes; t++)
+	for (t = 0; t < DBNumTypes; t++)
 	    if (TTMaskHasType(&types, s) && TTMaskHasType(&nottypes, t))
 	    {
 		ExtCurStyle->exts_perimCap[s][t] = capVal;
@@ -1504,6 +1509,7 @@ ExtTechLine(sectionName, argc, argv)
     ExtKeep *es, *newStyle;
     ParamList *subcktParams, *newParam;
     int refcnt;
+    double dhalo;
     bool bad;
 
     if (argc < 1)
@@ -1820,6 +1826,7 @@ ExtTechLine(sectionName, argc, argv)
 		    ExtCurStyle->exts_transSubstrateName[t] =
 			    StrDup((char **) NULL, subsName);
 		    ExtCurStyle->exts_transSubstrateTypes[t] = subsTypes;
+#ifdef ARIEL
 		    {
 			int z;
 
@@ -1830,6 +1837,7 @@ ExtTechLine(sectionName, argc, argv)
 					t);
 			}
 		    }
+#endif
 		}
 	    break;
 
@@ -1867,12 +1875,14 @@ ExtTechLine(sectionName, argc, argv)
 	    if ((argc - 1) < dv->k_minargs)
 		goto usage;
 
-	    /* No limit on arguments in DEV_SUBCKT! */
+	    /* No limit on arguments in DEV_SUBCKT and DEV_MSUBCKT! */
 	    /* And. . . check DEV_RSUBCKT later, after parsing parameter names */
-	    switch (dv->k_key)
+	    class = dv->k_key;
+	    switch (class)
 	    {
 		case DEV_SUBCKT:
 		case DEV_RSUBCKT:
+		case DEV_MSUBCKT:
 		    break;
 		default:
 		    if ((argc - 1) > dv->k_maxargs)
@@ -1905,7 +1915,7 @@ ExtTechLine(sectionName, argc, argv)
 			TTMaskAndMask3(&termtypes[2], &termtypes[0], &termtypes[1]);
 
 			if (TTMaskEqual(&termtypes[0], &termtypes[1]))
-			    termtypes[1] = DBSpaceBits;	/* Make is symmetric */
+			    termtypes[1] = DBSpaceBits;	/* Make it symmetric */
 			else if (!TTMaskIsZero(&termtypes[2]))
 			{
 			    TechError("Device mosfet %s has overlapping drain"
@@ -1919,6 +1929,7 @@ ExtTechLine(sectionName, argc, argv)
 			if (argc > 8) gscap = aToCap(argv[8]);
 			if (argc > 9) gccap = aToCap(argv[9]);
 			nterm = 2;
+			class = DEV_ASYMMETRIC;
 		    }
 		    else
 		    {
@@ -1978,15 +1989,41 @@ ExtTechLine(sectionName, argc, argv)
 		    break;
 
 		case DEV_SUBCKT:
+		case DEV_MSUBCKT:
 		    /* Check final arguments for "x=y" statements showing what	*/
 		    /* parameter names the subcircuit uses.			*/
 
 		    while ((paramName = strchr(argv[argc - 1], '=')) != NULL)
 		    {
+			char *mult;
+
 			paramName++;
 			newParam = (ParamList *)mallocMagic(sizeof(ParamList));
 			newParam->pl_count = 0;
-			newParam->pl_param = *argv[argc - 1];
+			newParam->pl_param[0] = *argv[argc - 1];
+			newParam->pl_param[1] = '\0';
+
+			if (paramName - argv[argc - 1] == 3)
+			    newParam->pl_param[1] = *(argv[argc - 1] + 1);
+
+			else if (paramName - argv[argc - 1] > 3)
+			    TechError("Parameter name %s can be no more than"
+				"two characters.\n", argv[argc - 1]);
+
+			// Parameter syntax "<type>=<name>*<scale>" indicates
+			// that the subcircuit has internal scaling, and the
+			// extractor should multiply the parameter by this value
+			// before passing it to the subcircuit.
+
+			if ((mult = strchr(paramName, '*')) != NULL)
+			{
+			    *mult = '\0';
+			    mult++;
+			    newParam->pl_scale = atof(mult);
+			}
+			else
+			    newParam->pl_scale = 1.0;
+
 			newParam->pl_name = StrDup((char **)NULL, paramName);
 			newParam->pl_next = subcktParams;
 			subcktParams = newParam;
@@ -2011,6 +2048,12 @@ ExtTechLine(sectionName, argc, argv)
 
 		    if (nterm == 0) i++;
 
+		    // Type MSUBCKT:  Source and drain are symmetric.  The
+		    // number of unique terminals in the definition is 1,
+		    // but nterm needs to be set to 2 for proper extraction.
+
+		    if ((dv->k_key == DEV_MSUBCKT) && (nterm == 1)) nterm = 2;
+
 		    if (argc > i)
 			DBTechNoisyNameMask(argv[i], &subsTypes); /* substrate */
 		    if (argc > (i + 1)) subsName = argv[i + 1];
@@ -2022,10 +2065,32 @@ ExtTechLine(sectionName, argc, argv)
 
 		    while ((paramName = strchr(argv[argc - 1], '=')) != NULL)
 		    {
+			char *mult;
+
 			paramName++;
 			newParam = (ParamList *)mallocMagic(sizeof(ParamList));
 			newParam->pl_count = 0;
-			newParam->pl_param = *argv[argc - 1];
+			newParam->pl_param[0] = *argv[argc - 1];
+			newParam->pl_param[1] = '\0';
+
+			if (paramName - argv[argc - 1] == 3)
+			    newParam->pl_param[1] = *(argv[argc - 1] + 1);
+
+			else if (paramName - argv[argc - 1] > 3)
+			    TechError("Parameter name %s can be no more than"
+				"two characters.\n", argv[argc - 1]);
+
+			// See comments for DEV_SUBCKT/DEV_MSUBCKT above.
+
+			if ((mult = strchr(paramName, '*')) != NULL)
+			{
+			    *mult = '\0';
+			    mult++;
+			    newParam->pl_scale = atof(mult);
+			}
+			else
+			    newParam->pl_scale = 1.0;
+
 			newParam->pl_name = StrDup((char **)NULL, paramName);
 			newParam->pl_next = subcktParams;
 			subcktParams = newParam;
@@ -2074,12 +2139,14 @@ ExtTechLine(sectionName, argc, argv)
 		    ExtCurStyle->exts_transSDCount[t] = nterm;
 		    ExtCurStyle->exts_transSDCap[t] = gscap;
 		    ExtCurStyle->exts_transGateCap[t] = gccap;
-		    ExtCurStyle->exts_deviceClass[t] = dv->k_key;
+		    ExtCurStyle->exts_deviceClass[t] = class;
 		    ExtCurStyle->exts_transName[t] =
 			    StrDup((char **) NULL, transName);
-		    ExtCurStyle->exts_transSubstrateName[t] =
-			    StrDup((char **) NULL, subsName);
+		    if (subsName != NULL)
+			ExtCurStyle->exts_transSubstrateName[t] =
+				StrDup((char **) NULL, subsName);
 		    ExtCurStyle->exts_transSubstrateTypes[t] = subsTypes;
+#ifdef ARIEL
 		    {
 			int z;
 
@@ -2090,6 +2157,7 @@ ExtTechLine(sectionName, argc, argv)
 					exts_subsTransistorTypes[z], t);
 			}
 		    }
+#endif
 		    if (subcktParams != NULL)
 		    {
 			ExtCurStyle->exts_deviceParams[t] = subcktParams;
@@ -2149,7 +2217,7 @@ ExtTechLine(sectionName, argc, argv)
 			"default value (lambda).\n");
 	    break;
 	case LAMBDA:
-	    ExtCurStyle->exts_unitsPerLambda = (float)MagAtof(argv[1]);
+	    ExtCurStyle->exts_unitsPerLambda = (float)atof(argv[1]);
 	    break;
 	case OVERC:
 	    DBTechNoisyNameMask(argv[2], &types2);
@@ -2358,7 +2426,14 @@ ExtTechLine(sectionName, argc, argv)
 	case SIDEHALO:
 	    /* Allow floating-point and increase by factor of 1000	*/
 	    /* to accomodate "units microns".				*/
-	    ExtCurStyle->exts_sideCoupleHalo = (int)(1000.0 * (float)MagAtof(argv[1]));
+
+	    /* Warning:  Due to some gcc bug with an i686 FPU, using a	*/
+	    /* result from atof() with a static value like 1000		*/
+	    /* produces a NaN result!  sscanf() seems to be safe. . .	*/
+
+	    sscanf(argv[1], "%lg", &dhalo);
+	    dhalo *= (double)1000.0;
+	    ExtCurStyle->exts_sideCoupleHalo = (int)dhalo;
 	    break;
 	case PERIMC:
 	    DBTechNoisyNameMask(argv[2], &types2);
@@ -2378,7 +2453,7 @@ ExtTechLine(sectionName, argc, argv)
 
 	    val = atoi(argv[2]);
 	    if (argc == 4)
-		chop = MagAtof(argv[3]);
+		chop = atof(argv[3]);
 	    class = ExtCurStyle->exts_numResistClasses++;
 	    for (t = TT_TECHDEPBASE; t < DBNumTypes; t++)
 		if (TTMaskHasType(&types1, t))
@@ -2395,7 +2470,7 @@ ExtTechLine(sectionName, argc, argv)
 	    ExtCurStyle->exts_resistScale = atoi(argv[1]);
 	    break;
 	case STEP:
-	    val = (int)MagAtof(argv[1]);
+	    val = (int)atof(argv[1]);
 	    if (val <= 0)
 	    {
 		TechError("Hierarchical interaction step size must be > 0\n");
@@ -2667,7 +2742,7 @@ zinit:
 	/* exts_unitsPerLambda is in centimicrons, so divide by		*/
 	/* 100 to get microns.						*/
 
-	CapValue scalefac = style->exts_unitsPerLambda / 100.0;
+	CapValue scalefac = (CapValue)style->exts_unitsPerLambda / 100.0;
 	CapValue sqfac = scalefac * scalefac;
 
 	for (r = 0; r < DBNumTypes; r++)
@@ -2684,14 +2759,24 @@ zinit:
 		style->exts_overlapCap[r][s] *= sqfac;
 		for (ec = style->exts_sideOverlapCap[r][s]; ec != NULL;
 				ec = ec->ec_next)
-		    ec->ec_cap *= sqfac;
+		    ec->ec_cap *= scalefac;
+
+		// Sidewall capacitance is referred to distance,
+		// so cap value does not scale.  However, the lambda
+                // reference for sidewall cap is 2 lambda, so if
+                // the reference is to be interpreted as 1 micron,
+                // the value needs to be divided by 2 (the factor of
+                // 2 is made up by the fact that the sidewall is
+                // independently accumulated on each plate of the
+                // capacitor).
+
 		for (ec = style->exts_sideCoupleCap[r][s]; ec != NULL;
 				ec = ec->ec_next)
-		    ec->ec_cap *= scalefac;
+		   ec->ec_cap *= 0.5;
 	    }
 	}
 
-	/* side halo and step size are also assumed to be in microns */
+	/* side halo and step size are also in microns */
 
 	style->exts_sideCoupleHalo = (int)(((CapValue)style->exts_sideCoupleHalo
 		/ scalefac) + 0.5);
@@ -2699,11 +2784,11 @@ zinit:
 		/ scalefac) + 0.5);
     }
 
-    /* Avoid setting stepSize to zero, or extraction will hang */
+    /* Avoid setting stepSize to zero, or extraction will hang! */
     if (style->exts_stepSize <= 0)
     {
-	TxError("Error:  zero step size!  Resetting to default.\n");
-	style->exts_stepSize = 100;
+	TxError("Warning:  zero step size!  Resetting to default.\n");
+	style->exts_stepSize = 100;		/* Revert to default */
     }
 
     /* We had multiplied sideCoupleHalo by 1000 to accomodate a 	*/
@@ -2762,14 +2847,18 @@ ExtTechScale(scalen, scaled)
 	    style->exts_perimCap[i][j] /= scaled;
 	    style->exts_overlapCap[i][j] *= sqn;
 	    style->exts_overlapCap[i][j] /= sqd;    /* Typo fixed in 7.2.57 */
+
+	    // Sidewall capacitance is referred to distance,
+	    // so cap value does not scale.
+
+	    // for (ec = style->exts_sideCoupleCap[i][j]; ec != NULL;
+	    //			ec = ec->ec_next)
+	    // {
+	    //	ec->ec_cap *= scalen;
+	    //	ec->ec_cap /= scaled;
+	    // }
 	    for (ec = style->exts_sideOverlapCap[i][j]; ec != NULL;
 				ec = ec->ec_next)
-	    {
-		ec->ec_cap *= sqn;
-		ec->ec_cap /= sqd;
-	    }
-	    for (ec = style->exts_sideCoupleCap[i][j]; ec != NULL;
-				ec = ec->ec_next)
 	    {
 		ec->ec_cap *= scalen;
 		ec->ec_cap /= scaled;
