@@ -4,7 +4,7 @@
  * Defines things shared internally by the extract module of Magic,
  * but not generally needed outside the extract module.
  *
- * rcsid "$Header: /usr/cvsroot/magic-7.5/extract/extractInt.h,v 1.14 2009/09/11 18:05:37 tim Exp $"
+ * rcsid "$Header: /usr/cvsroot/magic-8.0/extract/extractInt.h,v 1.7 2010/08/10 00:18:46 tim Exp $"
  *
  *     ********************************************************************* 
  *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
@@ -53,14 +53,20 @@ typedef int ResValue;	/* Warning:  in some places resistances are stored
 /* These lists keep track of what parameter names subcircuit definitions
  * use for parameters that magic knows how to generate.  Valid pl_param
  * values are a (area), p (perimeter), w (width), l (length), s (substrate),
- * x (position), and y (position).
+ * x (position), and y (position).  Values "a" and "p" may be followed by
+ * an additional integer indicating the terminal from which the value is
+ * used (e.g., source area, drain perimeter, etc.).  An integer "0"
+ * indicates the device identifier region (e.g., gate) and is equivalent
+ * to having no integer at all.  Integers "1" and up indicate terminals,
+ * in order.
  */
 
 typedef struct pl
 {
     int		 pl_count;	/* Share this list. . . */
-    char	 pl_param;	/* Default character for parameter */
+    char	 pl_param[2];	/* Default character for parameter */
     char	*pl_name;	/* Full name for parameter */
+    double	 pl_scale;	/* Scaling of parameter, if specified */
     struct pl	*pl_next;	/* Next parameter in list */
 } ParamList;
 
@@ -274,10 +280,10 @@ typedef struct
 
 /* Directions in which we can be following the boundary of a perimeter	*/
 
-#define	BD_LEFT		0	/* Inside is to right */
-#define	BD_TOP		1	/* Inside is below */
-#define	BD_RIGHT	2	/* Inside is to left */
-#define	BD_BOTTOM	3	/* Inside is above */
+#define	BD_LEFT		1	/* Inside is to right */
+#define	BD_TOP		2	/* Inside is below */
+#define	BD_RIGHT	4	/* Inside is to left */
+#define	BD_BOTTOM	8	/* Inside is above */
 
 /* -------- Yank buffers for hierarchical and array extraction -------- */
 
@@ -544,7 +550,7 @@ typedef struct extstyle
 
     /*
      * Amount to scale resistance of a material on a corner.
-     * Defaults to 1.0.  Often set to 0.5.
+     * Defauts to 1.0.  Often set to 0.5.
      */
     float		exts_cornerChop[NT];
 
@@ -806,11 +812,13 @@ typedef struct extstyle
 	 * if the mask exts_transSubstrateTypes[t] is non-zero, and if
 	 * the transistor overlaps material of one of the types in the
 	 * mask, then the transistor substrate node is the node of the
-	 * material it overlaps.
+	 * material it overlaps.  If exts_transSub
 	 */
     char		*exts_transSubstrateName[NT];
     TileTypeBitMask	 exts_transSubstrateTypes[NT];
+#ifdef ARIEL
     TileTypeBitMask	 exts_subsTransistorTypes[NT];
+#endif	/* ARIEL */
 
     /* Scaling */
 	/*
@@ -896,8 +904,6 @@ extern ClientData extUnInit;
 /* For non-recursive flooding algorithm */
 #define	VISITPENDING	((ClientData) NULL)	/* Marks tiles on stack */
 
-#ifdef NONMANHATTAN
-
 /* Note that this macro depends on MAXPLANES being small	*/
 /* compared to the bit position of TT_SIDE.  Since tens of	*/
 /* thousands of planes is inconceivable, this should not be a	*/
@@ -945,19 +951,6 @@ extern ClientData extUnInit;
 	STACKPUSH((ClientData)(pointertype)(pl | TT_SIDE), extNodeStack); \
 	STACKPUSH((ClientData)(pointertype)tp, extNodeStack)
 
-#else
-
-#define	PUSHTILE(tp, pl) \
-	(tp)->ti_client = VISITPENDING; \
-	STACKPUSH((ClientData)pl, extNodeStack); \
-	STACKPUSH((ClientData)tp, extNodeStack)
-
-#define POPTILE(tp, pl) \
-	tp = (Tile *) STACKPOP(extNodeStack); \
-	pl = (int) STACKPOP(extNodeStack)
-
-#endif
-
 /* ------------------------- Region finding --------------------------- */
 
 extern Region *ExtFindRegions();
@@ -970,7 +963,7 @@ extern Region *extHierLabFirst();	extern int extHierLabEach();
 
 extern Tile *extNodeToTile();
 
-/* -------- Search for matching tile/node in another ExtTree ---------- */
+/* -------- Search for matching node in another ExtTree ---------- */
 
 /*
  * NODETONODE(nold, et, nnew)
