@@ -19,7 +19,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/garouter/gaSimple.c,v 1.1.1.1 2006/04/10 22:03:13 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/garouter/gaSimple.c,v 1.2 2009/05/13 15:03:16 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -52,7 +52,9 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/
 bool gaIsClear();
 
 /*--- Externals referenced in this file and not declared in include files ---*/
-extern gaMetalClear, gaPolyClear, gaContactClear;
+extern int gaMetalClear;
+extern int gaPolyClear;
+extern int gaContactClear;
 
 
 /*
