@@ -138,7 +138,6 @@ w3dFillEdge(bbox, r, ztop, zbot)
     glEnd();
 }
 
-#ifdef NONMANHATTAN
 void
 w3dFillPolygon(p, np, zval, istop)
     Point *p;
@@ -158,7 +157,6 @@ w3dFillPolygon(p, np, zval, istop)
 
     glEnd();
 }
-#endif
 
 void
 w3dFillTile(r, zval, istop)
@@ -214,8 +212,6 @@ w3dFillYSide(xval, ystart, yend, ztop, zbot)
     glEnd();
 }
 
-#ifdef NONMANHATTAN
-
 /* This routine assumes that vector (x1,y1)->(x2,y2) is in the   */
 /* counterclockwise direction with respect to the tile interior. */
 
@@ -231,8 +227,6 @@ w3dFillDiagonal(x1, y1, x2, y2, ztop, zbot)
     glEnd();
 }
 
-#endif
-
 void
 w3dFillOps(trans, tile, cliprect, ztop, zbot)
     Transform *trans;
@@ -244,11 +238,8 @@ w3dFillOps(trans, tile, cliprect, ztop, zbot)
     LinkedRect *tilesegs, *segptr;
     Rect r, r2;
     float xbot, ybot, xtop, ytop;
-
-#ifdef NONMANHATTAN
     Point p[5];
     int np;
-#endif
 
     r2.r_xbot = LEFT(tile);
     r2.r_ybot = BOTTOM(tile);
@@ -257,7 +248,6 @@ w3dFillOps(trans, tile, cliprect, ztop, zbot)
 
     GeoTransRect(trans, &r2, &r);
 
-#ifdef NONMANHATTAN
     if (IsSplit(tile))
     {
 	Rect fullr;
@@ -279,23 +269,18 @@ w3dFillOps(trans, tile, cliprect, ztop, zbot)
     }
     else
     {
-#endif
+	/* Clip the tile area to the clipping area */
+	if (cliprect != NULL)
+	    GeoClip(&r, cliprect);
 
-    /* Clip the tile area to the clipping area */
-    if (cliprect != NULL)
-	GeoClip(&r, cliprect);
-
-    /* draw tile top and bottom */
-    if (!GEO_RECTNULL(&r))
-    {
-	w3dFillTile(&r, ztop, TRUE);
-	w3dFillTile(&r, zbot, FALSE);
+	/* draw tile top and bottom */
+	if (!GEO_RECTNULL(&r))
+	{
+	    w3dFillTile(&r, ztop, TRUE);
+	    w3dFillTile(&r, zbot, FALSE);
+	}
     }
 
-#ifdef NONMANHATTAN
-    }
-#endif
-
     /* If height is zero, don't bother to draw sides */
     if (ztop == zbot) return;
 
@@ -337,7 +322,6 @@ w3dFillOps(trans, tile, cliprect, ztop, zbot)
 	    freeMagic(segptr);
 	}
 
-#ifdef NONMANHATTAN
 	/* For non-manhattan tiles, GrBoxOutline only returns	*/
 	/* the manhattan edges.  This leaves the (possibly	*/
 	/* clipped) diagonal edge to render.			*/
@@ -361,7 +345,6 @@ w3dFillOps(trans, tile, cliprect, ztop, zbot)
 				(float)p[0].p_x, (float)p[0].p_y,
 				ztop, zbot);
 	}
-#endif
 
 	/* Render edges cut by the clipping area, if they're	*/
 	/* inside the tile, so the tile doesn't look "hollow".	*/
@@ -1554,7 +1537,7 @@ W3Dcreate(window, argc, argv)
     if (tkwind != 0)
     {
 	window->w_grdata = (ClientData) tkwind;
-	entry = HashFind(&grTOGLWindowTable, tkwind);
+	entry = HashFind(&grTOGLWindowTable, (char *)tkwind);
 	HashSetValue(entry, window);
 
 	if (name != NULL)
