@@ -19,7 +19,7 @@
 
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/graphics/grClip.c,v 1.4 2010/05/13 21:28:13 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/graphics/grClip.c,v 1.4 2010/06/24 12:37:18 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -179,6 +179,7 @@ bool grClipAddFunc(box, cd)
     lr->r_next = **lllr;
     **lllr = lr;
     *lllr = &lr->r_next;
+
     return TRUE;
 }
 
@@ -585,158 +586,146 @@ GrBoxOutline(tile, tilesegs)
     *tilesegs = NULL;
     TiToRect(tile, &rect);
 
-#ifdef NONMANHATTAN
     if (IsSplit(tile) && SplitSide(tile))
 	isolate |= 0x1;
     else
     {
-#endif
-    ttype = TiGetLeftType(tile);   /* = TiGetType when NONMANHATTAN not def'd */
-
-    edgeBot = rect.r_ybot;
-    sense = TRUE;
-    for (tpleft = BL(tile); BOTTOM(tpleft) < rect.r_ytop; tpleft = RT(tpleft))
-    {
-	if (TiGetRightType(tpleft) != ttype)
+	ttype = TiGetLeftType(tile);
+	edgeBot = rect.r_ybot;
+	sense = TRUE;
+	for (tpleft = BL(tile); BOTTOM(tpleft) < rect.r_ytop; tpleft = RT(tpleft))
 	{
-	    if (!sense)
+	    if (TiGetRightType(tpleft) != ttype)
 	    {
-		edgeBot = BOTTOM(tpleft);
-		if (TOP(tpleft) >= rect.r_ytop)
-		    grAddSegment(rect.r_xbot, edgeBot, rect.r_xbot, rect.r_ytop,
+		if (!sense)
+		{
+		    edgeBot = BOTTOM(tpleft);
+		    if (TOP(tpleft) >= rect.r_ytop)
+			grAddSegment(rect.r_xbot, edgeBot, rect.r_xbot, rect.r_ytop,
+					tilesegs);
+		    sense = TRUE;
+		}
+	    }
+	    else
+	    {
+		if (sense)
+		{
+		    edgeTop = BOTTOM(tpleft);
+		    if (edgeTop > edgeBot)
+			grAddSegment(rect.r_xbot, edgeBot, rect.r_xbot, edgeTop,
 				tilesegs);
-		sense = TRUE;
+		    isolate |= 0x1;
+		    sense = FALSE;
+		}
 	    }
 	}
-	else
-	{
-	    if (sense)
-	    {
-		edgeTop = BOTTOM(tpleft);
-		if (edgeTop > edgeBot)
-		    grAddSegment(rect.r_xbot, edgeBot, rect.r_xbot, edgeTop,
-				tilesegs);
-		isolate |= 0x1;
-		sense = FALSE;
-	    }
-	}
-    }
-
-#ifdef NONMANHATTAN
     }
     if (IsSplit(tile) && !SplitSide(tile))
 	isolate |= 0x2;
     else
     {
 	ttype = TiGetRightType(tile);
-#endif
-    edgeTop = rect.r_ytop;
-    sense = TRUE;
-    for (tpright = TR(tile); TOP(tpright) > rect.r_ybot; tpright = LB(tpright))
-    {
-	if (TiGetLeftType(tpright) != ttype)
+	edgeTop = rect.r_ytop;
+	sense = TRUE;
+	for (tpright = TR(tile); TOP(tpright) > rect.r_ybot; tpright = LB(tpright))
 	{
-	    if (!sense)
+	    if (TiGetLeftType(tpright) != ttype)
 	    {
-		edgeTop = TOP(tpright);
-		if (BOTTOM(tpright) <= rect.r_ybot)
-		    grAddSegment(rect.r_xtop, rect.r_ybot, rect.r_xtop, edgeTop,
-				tilesegs);
-		sense = TRUE;
+		if (!sense)
+		{
+		    edgeTop = TOP(tpright);
+		    if (BOTTOM(tpright) <= rect.r_ybot)
+			grAddSegment(rect.r_xtop, rect.r_ybot, rect.r_xtop, edgeTop,
+					tilesegs);
+		    sense = TRUE;
+		}
 	    }
-	}
-	else
-	{
-	    if (sense)
+	    else
 	    {
-		edgeBot = TOP(tpright);
-		if (edgeBot < edgeTop)
-		    grAddSegment(rect.r_xtop, edgeBot, rect.r_xtop, edgeTop,
-				tilesegs);
-		isolate |= 0x2;
-		sense = FALSE;
+		if (sense)
+		{
+		    edgeBot = TOP(tpright);
+		    if (edgeBot < edgeTop)
+			grAddSegment(rect.r_xtop, edgeBot, rect.r_xtop, edgeTop,
+					tilesegs);
+		    isolate |= 0x2;
+		    sense = FALSE;
+		}
 	    }
 	}
     }
 
-#ifdef NONMANHATTAN
-    }
     if (IsSplit(tile) &&
 		(SplitSide(tile) == SplitDirection(tile)))
 	isolate |= 0x4;
     else
     {
 	ttype = TiGetBottomType(tile);
-#endif
-    edgeLeft = rect.r_xbot;
-    sense = TRUE;
-    for (tpbot = LB(tile); LEFT(tpbot) < rect.r_xtop; tpbot = TR(tpbot))
-    {
-	if (TiGetTopType(tpbot) != ttype)
+	edgeLeft = rect.r_xbot;
+	sense = TRUE;
+	for (tpbot = LB(tile); LEFT(tpbot) < rect.r_xtop; tpbot = TR(tpbot))
 	{
-	    if (!sense)
+	    if (TiGetTopType(tpbot) != ttype)
 	    {
-		edgeLeft = LEFT(tpbot);
-		if (RIGHT(tpbot) >= rect.r_xtop)
-		    grAddSegment(edgeLeft, rect.r_ybot, rect.r_xtop, rect.r_ybot,
-				tilesegs);
-		sense = TRUE;
+		if (!sense)
+		{
+		    edgeLeft = LEFT(tpbot);
+		    if (RIGHT(tpbot) >= rect.r_xtop)
+			grAddSegment(edgeLeft, rect.r_ybot, rect.r_xtop, rect.r_ybot,
+					tilesegs);
+		    sense = TRUE;
+		}
 	    }
-	}
-	else
-	{
-	    if (sense)
+	    else
 	    {
-		edgeRight = LEFT(tpbot);
-		if (edgeRight > edgeLeft)
-		    grAddSegment(edgeLeft, rect.r_ybot, edgeRight, rect.r_ybot,
-				tilesegs);
-		isolate |= 0x4;
-		sense = FALSE;
+		if (sense)
+		{
+		    edgeRight = LEFT(tpbot);
+		    if (edgeRight > edgeLeft)
+			grAddSegment(edgeLeft, rect.r_ybot, edgeRight, rect.r_ybot,
+					tilesegs);
+		    isolate |= 0x4;
+		    sense = FALSE;
+		}
 	    }
 	}
     }
 
-#ifdef NONMANHATTAN
-    }
     if (IsSplit(tile) &&
 		(SplitSide(tile) != SplitDirection(tile)))
 	isolate |= 0x8;
     else
     {
 	ttype = TiGetTopType(tile);
-#endif
-    edgeRight = rect.r_xtop;
-    sense = TRUE;
-    for (tptop = RT(tile); RIGHT(tptop) > rect.r_xbot; tptop = BL(tptop))
-    {
-	if (TiGetBottomType(tptop) != ttype)
+	edgeRight = rect.r_xtop;
+	sense = TRUE;
+	for (tptop = RT(tile); RIGHT(tptop) > rect.r_xbot; tptop = BL(tptop))
 	{
-	    if (!sense)
+	    if (TiGetBottomType(tptop) != ttype)
 	    {
-		edgeRight = RIGHT(tptop);
-		if (LEFT(tptop) <= rect.r_xbot)
-		    grAddSegment(rect.r_xbot, rect.r_ytop, edgeRight, rect.r_ytop,
-				tilesegs);
-		sense = TRUE;
+		if (!sense)
+		{
+		    edgeRight = RIGHT(tptop);
+		    if (LEFT(tptop) <= rect.r_xbot)
+			grAddSegment(rect.r_xbot, rect.r_ytop, edgeRight, rect.r_ytop,
+					tilesegs);
+		    sense = TRUE;
+		}
 	    }
-	}
-	else
-	{
-	    if (sense)
+	    else
 	    {
-		edgeLeft = RIGHT(tptop);
-		if (edgeLeft < edgeRight)
-		    grAddSegment(edgeLeft, rect.r_ytop, edgeRight, rect.r_ytop,
-				tilesegs);
-		isolate |= 0x8;
-		sense = FALSE;
+		if (sense)
+		{
+		    edgeLeft = RIGHT(tptop);
+		    if (edgeLeft < edgeRight)
+			grAddSegment(edgeLeft, rect.r_ytop, edgeRight, rect.r_ytop,
+					tilesegs);
+		    isolate |= 0x8;
+		    sense = FALSE;
+		}
 	    }
 	}
     }
-#ifdef NONMANHATTAN
-    }
-#endif
 
     if (isolate == 0)	/* Common case */
 	return TRUE;
@@ -798,12 +787,10 @@ GrBox(MagWindow *mw, Transform *trans, Tile *tile)
     r.r_ytop = TOP(tile);
 
     GeoTransRect(trans, &r, &r2);
-#ifdef NONMANHATTAN
     if (IsSplit(tile))
 	WindSurfaceToScreenNoClip(mw, &r2, &r);
     else
-#endif
-    WindSurfaceToScreen(mw, &r2, &r);
+	WindSurfaceToScreen(mw, &r2, &r);
 
     GR_CHECK_LOCK();
     if (!grDriverInformed) grInformDriver();
@@ -819,7 +806,6 @@ GrBox(MagWindow *mw, Transform *trans, Tile *tile)
     for (ob = grCurObscure; ob != NULL; ob = ob->r_next)
 	needObscure |= GEO_TOUCH(&r, &(ob->r_r));
 
-#ifdef NONMANHATTAN
     /* Nonmanhattan tiles: */
     /* Expects one of the two tile types to be masked out before */
     /* this procedure is called.                                 */
@@ -849,26 +835,21 @@ GrBox(MagWindow *mw, Transform *trans, Tile *tile)
     }
     else
     {
-#endif
 
-    /* do solid areas (same as GrFastBox) */
+	/* do solid areas (same as GrFastBox) */
 
-    if ((grCurFill == GR_STSOLID) || 
-	(grCurFill == GR_STSTIPPLE))
-    {
-	/* We have a filled area to deal with */
-	clipr = r;
-	if (needClip)
-	    GeoClip(&clipr, &grCurClip);
-	if (needObscure)
-	    grObsBox(&clipr);
-	else
-	    (void) (*grFillRectPtr)(&clipr);
-    } 
-
-#ifdef NONMANHATTAN
+	if ((grCurFill == GR_STSOLID) || (grCurFill == GR_STSTIPPLE))
+	{
+	    /* We have a filled area to deal with */
+	    clipr = r;
+	    if (needClip)
+		GeoClip(&clipr, &grCurClip);
+	    if (needObscure)
+		grObsBox(&clipr);
+	    else
+		(void) (*grFillRectPtr)(&clipr);
+	} 
     }
-#endif
 
     /* return if outline is too small to be worth drawing */
 
@@ -946,7 +927,6 @@ GrBox(MagWindow *mw, Transform *trans, Tile *tile)
 		freeMagic(segptr);
 	    }
 
-#ifdef NONMANHATTAN
 	    /* For non-manhattan tiles, the manhattan parts of the	*/
 	    /* boundary have already been drawn.  The diagonal boundary */
 	    /* is guaranteed to be continuous by definition, and it has	*/
@@ -974,20 +954,24 @@ GrBox(MagWindow *mw, Transform *trans, Tile *tile)
 				polyp[0].p_x, polyp[0].p_y);
 		}
 	    }
-#endif
 	}
     }
 }
 
 /*---------------------------------------------------------
- * GrFastBox:
- *	GrFastBox will draw a rectangle on the screen in the
+ * GrDrawFastBox:
+ *	GrDrawFastBox will draw a rectangle on the screen in the
  *	style set by the previous call to GrSetStuff.  It will also
  *	be clipped against the rectangles passed to GrSetStuff.
  *
- *	If GrClipBox is called between GrFastBox calls then GrSetStuff
+ *	If GrClipBox is called between GrDrawFastBox calls then GrSetStuff
  *	must be called to set the parameters back.
  *
+ *	Usually this is called as GrFastBox, defined as GrDrawFastBox(p, 0).
+ *	The "scale" is only used to reduce the size of crosses drawn at
+ *	point positions, to prevent point labels from dominating a layout
+ *	in top-level views. 
+ *
  * Results:	None.
  *
  * Side Effects:
@@ -997,10 +981,15 @@ GrBox(MagWindow *mw, Transform *trans, Tile *tile)
  */
 
 void
-GrFastBox(prect)
+GrDrawFastBox(prect, scale)
     Rect *prect;	/* The rectangle to be drawn, given in
 			 * screen coordinates.
 			 */
+    int scale;		/* If < 0, we reduce the cross size for
+			 * points according to the (negative) scale,
+			 * so point labels don't dominate a top-level
+			 * layout.
+			 */
 {
     Rect *r;
     bool needClip, needObscure;
@@ -1061,6 +1050,14 @@ GrFastBox(prect)
 	if ( (grCurFill == GR_STOUTLINE) && (r->r_xbot == r->r_xtop) &&
 		(r->r_ybot == r->r_ytop) )
 	{
+	    int crossSize = GR_CROSSSIZE;
+
+	    if (scale < 0)
+	    {
+		crossSize += scale;
+		if (crossSize < 0)
+		    goto endit;
+	    }
 
 	    /* turn the outline into a cross */
 	    if (needClip || needObscure) 
@@ -1071,10 +1068,10 @@ GrFastBox(prect)
 		Rect crossBox;
 
 		/* check the larger cross area for clipping */
-		crossBox.r_xbot = r->r_xbot - GR_CROSSSIZE;
-		crossBox.r_ybot = r->r_ybot - GR_CROSSSIZE;
-		crossBox.r_xtop = r->r_xtop + GR_CROSSSIZE;
-		crossBox.r_ytop = r->r_ytop + GR_CROSSSIZE;
+		crossBox.r_xbot = r->r_xbot - crossSize;
+		crossBox.r_ybot = r->r_ybot - crossSize;
+		crossBox.r_xtop = r->r_xtop + crossSize;
+		crossBox.r_ytop = r->r_ytop + crossSize;
 
 		crossClip = !GEO_SURROUND(&grCurClip, &crossBox);
 		crossObscure = FALSE;
@@ -1088,17 +1085,17 @@ GrFastBox(prect)
 	    }
 
 	    clipit:
-		GrClipLine(r->r_xbot, r->r_ybot - GR_CROSSSIZE,
-		    r->r_xtop, r->r_ytop + GR_CROSSSIZE);
-		GrClipLine(r->r_xbot - GR_CROSSSIZE, r->r_ybot,
-		    r->r_xtop + GR_CROSSSIZE, r->r_ytop);
+		GrClipLine(r->r_xbot, r->r_ybot - crossSize,
+		    r->r_xtop, r->r_ytop + crossSize - 1 + GrPixelCorrect);
+		GrClipLine(r->r_xbot - crossSize, r->r_ybot,
+		    r->r_xtop + crossSize - 1 + GrPixelCorrect, r->r_ytop);
 		goto endit;
 
 	    noclipit:
-		(*grDrawLinePtr)(r->r_xbot, r->r_ybot - GR_CROSSSIZE,
-		    r->r_xtop, r->r_ytop + GR_CROSSSIZE);
-		(*grDrawLinePtr)(r->r_xbot - GR_CROSSSIZE, r->r_ybot,
-		    r->r_xtop + GR_CROSSSIZE, r->r_ytop);
+		(*grDrawLinePtr)(r->r_xbot, r->r_ybot - crossSize,
+		    r->r_xtop, r->r_ytop + crossSize - 1 + GrPixelCorrect);
+		(*grDrawLinePtr)(r->r_xbot - crossSize, r->r_ybot,
+		    r->r_xtop + crossSize - 1 + GrPixelCorrect, r->r_ytop);
 
 	    endit:  ;
 	} 
@@ -1137,8 +1134,6 @@ GrFastBox(prect)
     }
 }
 
-#ifdef NONMANHATTAN
-
 /*---------------------------------------------------------
  * GrClipTriangle:
  *	Returns an array of points representing a clipped
@@ -1377,7 +1372,7 @@ GrClipTriangle(r, c, clipped, dinfo, points, np)
 		    points[3].p_y = points[2].p_y - yround(dely);
 		    *np = 4;
 
-		    if (c->r_ybot > points[3].p_y)  /* clip BOTTOM, rectangle */
+		    if (c->r_ybot >= points[3].p_y)  /* clip BOTTOM, rectangle */
 		    {
 			points[3].p_y = c->r_ybot;
 			points[0].p_y = c->r_ybot;
@@ -1553,7 +1548,6 @@ GrFillPolygon(polyp, np)
     }
 }
 
-#endif /* NONMANHATTAN */
 
 /*---------------------------------------------------------
  * GrClipBox:
