@@ -694,6 +694,9 @@ oglSetDisplay (dispType, outFileName, mouseFileName)
     /* local indirections */
     grSetSPatternPtr = groglSetSPattern;
     grPutTextPtr = groglPutText;
+#ifdef VECTOR_FONTS
+    grFontTextPtr = groglFontText;
+#endif
     grDefineCursorPtr = groglDefineCursor;
     grDrawGridPtr = groglDrawGrid;
     grDrawLinePtr = groglDrawLine;
@@ -702,9 +705,7 @@ oglSetDisplay (dispType, outFileName, mouseFileName)
     grSetStipplePtr = groglSetStipple;
     grSetLineStylePtr = groglSetLineStyle;
     grSetCharSizePtr = groglSetCharSize;
-#ifdef NONMANHATTAN
     grFillPolygonPtr = groglFillPolygon;
-#endif
 
     if (execFailed) {
         TxError("Execution failed!\n");
