@@ -23,6 +23,7 @@
 
 #include <GL/gl.h>
 #include <GL/glx.h>
+#include <GL/glu.h>
 
 #include "utils/magic.h"
 #include "utils/geometry.h"
@@ -34,6 +35,7 @@
 #include "utils/utils.h"
 #include "utils/hash.h"
 #include "dbwind/dbwind.h"
+#include "database/fonts.h"
 #include "grOGLInt.h"
 
 extern Display *grXdpy;
@@ -348,6 +350,170 @@ GrOGLBitBlt(r, p)
 		r->r_ytop - r->r_ybot + 1, GL_COLOR);
 }
 
+#ifdef VECTOR_FONTS
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * Technically, there should be no self-intersecting polygons in outline
+ * fonts.  However, decomposition of bezier curves into line segments
+ * may occasionally produce one, so it needs to be handled.
+ *----------------------------------------------------------------------
+ */
+
+void
+myCombine(GLdouble coords[3], GLdouble *vertex_data[4],
+          GLfloat weight[4], GLdouble **outData, void *dataptr)
+{
+    /* This needs to be free'd at the end of gluTessEndPolygon()! */
+    GLdouble *new = (GLdouble *)mallocMagic(2 * sizeof(GLdouble));
+    new[0] = coords[0];
+    new[1] = coords[1];
+    *outData = new;
+    /* Diagnostic */
+    TxError("Intersecting polygon in char \"%c\" at %g %g!\n", 
+	*((char *)dataptr), coords[0], coords[1]);
+}
+
+/*
+ *----------------------------------------------------------------------
+ * Draw a text character
+ * This routine differs from grtoglFillPolygon() in that it uses the
+ * glu library to handle non-convex polygons as may appear in font
+ * outlines.
+ *----------------------------------------------------------------------
+ */
+
+void
+groglDrawCharacter(clist, tc, pixsize)
+    FontChar *clist;
+    unsigned char tc;
+    int pixsize;
+{
+    Point *tp;
+    int np, nptotal;
+    int i, j;
+    static GLUtesselator *tess = NULL;
+    static GLdouble *v = NULL;
+    static int maxnp = 0;
+    FontChar *ccur;
+
+    if (pixsize < 5) return;	/* Label too small to be useful */
+
+    if (tess == NULL)
+    {
+	tess = gluNewTess();
+	gluTessCallback(tess, GLU_TESS_BEGIN, (_GLUfuncptr)glBegin);
+	gluTessCallback(tess, GLU_TESS_VERTEX, (_GLUfuncptr)glVertex3dv);
+	gluTessCallback(tess, GLU_TESS_END, (_GLUfuncptr)glEnd);
+	gluTessCallback(tess, GLU_TESS_COMBINE_DATA, (_GLUfuncptr)myCombine);
+    }
+    // Boundary-only does not look particularly good. . .
+    gluTessProperty(tess, GLU_TESS_BOUNDARY_ONLY, GL_FALSE);
+
+    nptotal = 0;
+    for (ccur = clist; ccur != NULL; ccur = ccur->fc_next)
+	nptotal += ccur->fc_numpoints;
+
+    if (nptotal > maxnp)
+    {
+	if (v != NULL) freeMagic((char *)v);
+	maxnp = nptotal;
+	v = (GLdouble *)mallocMagic(nptotal * 3 * sizeof(GLdouble));
+    }
+
+    j = 0;
+    for (ccur = clist; ccur != NULL; ccur = ccur->fc_next)
+    {
+	tp = ccur->fc_points;
+	np = ccur->fc_numpoints;
+
+	for (i = 0; i < np; i++, j += 3) {
+	    v[j] = tp[i].p_x;
+	    v[j + 1] = tp[i].p_y;
+	    v[j + 2] = 0;
+	}
+    }
+
+    gluTessBeginPolygon(tess, (GLvoid *)(&tc));
+    j = 0;
+    for (ccur = clist; ccur != NULL; ccur = ccur->fc_next)
+    {
+	np = ccur->fc_numpoints;
+	gluTessBeginContour(tess);
+	for (i = 0; i < np; i++, j += 3) {
+	    gluTessVertex(tess, &v[j], &v[j]);
+	}
+	gluTessEndContour(tess);
+    }
+    gluTessEndPolygon(tess);
+}
+
+/*---------------------------------------------------------
+ * groglFontText:
+ *
+ *	This routine draws text from font vectors using the
+ *	font vector routines in DBlabel.c.  Text is clipped
+ *	to the clipping rectangle.
+ *
+ *	For speed, we should be transferring the font
+ *	vectors into OpenGL display lists!
+ *	
+ *---------------------------------------------------------
+ */
+
+void
+groglFontText(text, font, size, rotate, pos, clip, obscure)
+    char *text;			/* The text to be drawn */
+    int   font;			/* Font to use from fontList */
+    int   size;			/* Pixel size of the font */
+    int	  rotate;		/* Text rotation */
+    Point *pos;			/* Text base position */
+    Rect  *clip;		/* Clipping area */
+    LinkedRect *obscure;	/* List of obscuring areas */ 
+{
+    char *tptr;
+    Point *coffset;		/* vector to next character */
+    Rect *cbbox;
+    GLfloat fsize, matvals[16];
+    FontChar *clist;
+    int cheight, baseline;
+    float tmp;
+
+    /* Keep it simple for now---ignore clip and obscure */
+
+    glDisable(GL_BLEND);
+    glPushMatrix();
+    glTranslated(pos->p_x, pos->p_y, 0);
+    glRotated(rotate, 0, 0, 1);
+
+    /* Get label size */
+    cbbox = &DBFontList[font]->mf_extents;
+
+    fsize = (GLfloat)size / (GLfloat)cbbox->r_ytop;
+    glScalef(fsize, fsize, 1.0);
+
+    /* Adjust to baseline */
+    baseline = 0;
+    for (tptr = text; *tptr != '\0'; tptr++)
+    {
+	DBFontChar(font, *tptr, NULL, NULL, &cbbox);
+	if (cbbox->r_ybot < baseline)
+	    baseline = cbbox->r_ybot;
+    }
+    glTranslated(0, -baseline, 0);
+
+    for (tptr = text; *tptr != '\0'; tptr++)
+    {
+	DBFontChar(font, *tptr, &clist, &coffset, NULL);
+	groglDrawCharacter(clist, *tptr, size);
+	glTranslated(coffset->p_x, coffset->p_y, 0);
+    }
+    glPopMatrix();
+}
+
+#endif /* VECTOR_FONTS */
+
 static GC grXcopyGC = (GC)NULL;
 
 /*
