@@ -69,6 +69,7 @@ extern int  GrTOGLWindowId();
 extern char *GrTkWindowName();
 
 extern void toglSetProjection();
+
 
 /*---------------------------------------------------------
  * grtoglSetWMandC:
@@ -307,8 +308,6 @@ GrTOGLInit ()
     glLineWidth(1.0);
     glShadeModel (GL_FLAT);
     glPixelStorei(GL_PACK_LSB_FIRST, TRUE);
-    glMatrixMode(GL_MODELVIEW);
-    glLoadIdentity();
 
     /* OpenGL sets its own names for colormap and dstyle file types */
     grCMapType = "OpenGL";
@@ -370,7 +369,7 @@ GrTOGLFlush ()
  *---------------------------------------------------------
  */
 
-static GLXPixmap glpmap;
+static GLXPixmap glpmap = None;
 
 #define glTransYs(n) (DisplayHeight(grXdpy, grXscrn)-(n))
 
@@ -386,6 +385,7 @@ toglSetProjection(llx, lly, width, height)
 {
     if (toglCurrent.mw->w_flags & WIND_OFFSCREEN)
     {
+	if (glpmap != None) glXDestroyGLXPixmap(grXdpy, glpmap);
 	glpmap = glXCreateGLXPixmap(grXdpy, grVisualInfo,
 			(Pixmap)toglCurrent.windowid);
 	glXMakeCurrent(grXdpy, (GLXDrawable)glpmap, grXcontext);
@@ -418,6 +418,11 @@ toglSetProjection(llx, lly, width, height)
     /* magic origin maps to window center; move to window origin */
 
     glTranslated(-(GLsizei)(width >> 1), -(GLsizei)(height >> 1), 0);
+
+    /* Remaining transformations are done on the modelview matrix */
+
+    glMatrixMode(GL_MODELVIEW);
+    glLoadIdentity();
 }
 
 
@@ -488,7 +493,7 @@ TOGLEventProc(clientData, xevent)
 		}
 		nbytes = 0;
 
-		entry = HashLookOnly(&grTOGLWindowTable, tkwind);
+		entry = HashLookOnly(&grTOGLWindowTable, (char *)tkwind);
 		mw = (entry)?(MagWindow *)HashGetValue(entry):0;
 
 		if (mw && (mw->w_flags & WIND_SCROLLBARS))
@@ -511,20 +516,16 @@ TOGLEventProc(clientData, xevent)
 
 		if (IsModifierKey(keysym)) break;	/* Don't handle modifiers */
 
-		entry = HashLookOnly(&grTOGLWindowTable, tkwind);
+		entry = HashLookOnly(&grTOGLWindowTable, (char *)tkwind);
 		mw = (entry)?(MagWindow *)HashGetValue(entry):0;
 
 keys_and_buttons:
 
 		keymod = (LockMask | ControlMask | ShiftMask)
 				& KeyPressedEvent->state;
-
 #ifdef __APPLE__
-		/* Change by Philippe---Allows magic to deal with Mac OS-X
-		 * mapping Meta to Mod2 instead of Mod1
-		 */
-		if (KeyPressedEvent->state & (Mod1Mask | Mod2Mask | Mod3Mask
-				| Mod4Mask | Mod5Mask))
+		if (KeyPressedEvent->state & (Mod1Mask | Mod2Mask |
+				Mod3Mask | Mod4Mask | Mod5Mask))
 		    keymod |= Mod1Mask;
 #else
 		keymod |= (Mod1Mask & KeyPressedEvent->state);
@@ -779,7 +780,7 @@ keys_and_buttons:
 		width = ConfigureEvent->width;
 		height = ConfigureEvent->height;
 
-		entry = HashLookOnly(&grTOGLWindowTable, tkwind);
+		entry = HashLookOnly(&grTOGLWindowTable, (char *)tkwind);
 		mw = (entry)?(MagWindow *)HashGetValue(entry):0;
 
 		screenRect.r_xbot = ConfigureEvent->x;
@@ -798,7 +799,7 @@ keys_and_buttons:
 	    {
 		XVisibilityEvent *VisEvent = (XVisibilityEvent*) xevent;
 
-		entry = HashLookOnly(&grTOGLWindowTable, tkwind);
+		entry = HashLookOnly(&grTOGLWindowTable, (char *)tkwind);
 		mw = (entry)?(MagWindow *)HashGetValue(entry):0;
 
 		switch(VisEvent->state)
@@ -807,7 +808,7 @@ keys_and_buttons:
 			mw->w_flags &= ~WIND_OBSCURED;
 			if (mw->w_backingStore == (ClientData)NULL)
 			{
-			    grtkCreateBackingStore(mw);
+			    (*GrCreateBackingStorePtr)(mw);
 			    if (mw->w_backingStore != (ClientData)NULL)
 			    {
 				WindAreaChanged(mw, &mw->w_allArea);
@@ -827,7 +828,7 @@ keys_and_buttons:
 		XExposeEvent *ExposeEvent = (XExposeEvent*) xevent;
 		Rect screenRect;
 
-		entry = HashLookOnly(&grTOGLWindowTable, tkwind);
+		entry = HashLookOnly(&grTOGLWindowTable, (char *)tkwind);
 		mw = (entry)?(MagWindow *)HashGetValue(entry):0;
 
 		screenRect.r_xbot = ExposeEvent->x;
@@ -934,6 +935,9 @@ oglSetDisplay (dispType, outFileName, mouseFileName)
     /* local indirections */
     grSetSPatternPtr = grtoglSetSPattern;
     grPutTextPtr = grtoglPutText;
+#ifdef VECTOR_FONTS
+    grFontTextPtr = grtoglFontText;
+#endif
     grDefineCursorPtr = grTkDefineCursor;
     grFreeCursorPtr = grTkFreeCursors;
     GrBitBltPtr = GrTOGLBitBlt;
@@ -944,15 +948,21 @@ oglSetDisplay (dispType, outFileName, mouseFileName)
     grSetStipplePtr = grtoglSetStipple;
     grSetLineStylePtr = grtoglSetLineStyle;
     grSetCharSizePtr = grtoglSetCharSize;
-#ifdef NONMANHATTAN
     grFillPolygonPtr = grtoglFillPolygon;
-#endif
 
+#ifdef X11_BACKING_STORE
     GrFreeBackingStorePtr = grtkFreeBackingStore;
     GrCreateBackingStorePtr = grtkCreateBackingStore;
     GrGetBackingStorePtr = grtkGetBackingStore;
     GrPutBackingStorePtr = grtkPutBackingStore;
     GrScrollBackingStorePtr = grtkScrollBackingStore;
+#else
+    GrFreeBackingStorePtr = grtoglFreeBackingStore;
+    GrCreateBackingStorePtr = grtoglCreateBackingStore;
+    GrGetBackingStorePtr = grtoglGetBackingStore;
+    GrPutBackingStorePtr = grtoglPutBackingStore;
+    GrScrollBackingStorePtr = grtoglScrollBackingStore;
+#endif
     
     if (execFailed) {
 	TxError("Execution failed!\n");
@@ -1074,7 +1084,7 @@ GrTOGLCreate(w, name)
 
 	w->w_grdata = (ClientData) tkwind;
 	
-	entry = HashFind(&grTOGLWindowTable, tkwind);
+	entry = HashFind(&grTOGLWindowTable, (char *)tkwind);
 	HashSetValue(entry,w);
 
 	/* ensure that the visual is what we wanted, if possible to change */
@@ -1147,7 +1157,7 @@ GrTOGLDelete(w)
     HashEntry	*entry;
 
     xw = (Tk_Window) w->w_grdata;
-    entry = HashLookOnly(&grTOGLWindowTable, xw);
+    entry = HashLookOnly(&grTOGLWindowTable, (char *)xw);
     HashSetValue(entry,NULL);
 
     Tcl_DeleteCommand(magicinterp, Tk_PathName(xw));
@@ -1305,11 +1315,6 @@ GrTOGLUnlock(w)
 {
     /* GR_TOGL_FLUSH_BATCH(); */
     GrTOGLFlush();	/* (?) Adds glFlush and glFinish to the above. */
-
-    if (w != GR_LOCK_SCREEN)
-	if (w->w_flags & WIND_OFFSCREEN)
-	    glXDestroyGLXPixmap(grXdpy, glpmap);
-
     grSimpleUnlock(w);
 }
 
@@ -1422,7 +1427,7 @@ GrTOGLWindowId(tkname)
     tkwind = Tk_NameToWindow(magicinterp, tkname, Tk_MainWindow(magicinterp));
     if (tkwind != NULL)
     {   
-	entry = HashLookOnly(&grTOGLWindowTable, tkwind);
+	entry = HashLookOnly(&grTOGLWindowTable, (char *)tkwind);
 	mw = (entry) ? (MagWindow *)HashGetValue(entry) : 0;
 	if (mw) id = mw->w_wid;
     }       
