@@ -771,7 +771,7 @@ MagicEventProc(clientData, xevent)
 		}
 		nbytes = 0;
 
-		entry = HashLookOnly(&grTkWindowTable, wind);
+		entry = HashLookOnly(&grTkWindowTable, (char *)wind);
 		mw = (entry)?(MagWindow *)HashGetValue(entry):0;
 
 		/* Hack---non-wrapper versions of magic have	*/
@@ -796,7 +796,7 @@ MagicEventProc(clientData, xevent)
 
 		if (IsModifierKey(keysym)) break;	/* Don't handle modifiers */
 
-		entry = HashLookOnly(&grTkWindowTable, wind);
+		entry = HashLookOnly(&grTkWindowTable, (char *)wind);
 		mw = (entry)?(MagWindow *)HashGetValue(entry):0;
 
 keys_and_buttons:
@@ -806,12 +806,13 @@ keys_and_buttons:
 
 		keymod = (LockMask | ControlMask | ShiftMask)
 				& KeyPressedEvent->state;
+
 #ifdef __APPLE__
 		if (KeyPressedEvent->state & (Mod1Mask | Mod2Mask |
 				Mod3Mask | Mod4Mask | Mod5Mask))
 		    keymod |= Mod1Mask;
 #else
-                keymod |= (Mod1Mask & KeyPressedEvent->state);
+		keymod |= (Mod1Mask & KeyPressedEvent->state);
 #endif
 
 		if (nbytes == 0)
@@ -1066,7 +1067,7 @@ keys_and_buttons:
 	    {
 		XVisibilityEvent *VisEvent = (XVisibilityEvent*) xevent;
 
-		entry = HashLookOnly(&grTkWindowTable, wind);
+		entry = HashLookOnly(&grTkWindowTable, (char *)wind);
 		mw = (entry)?(MagWindow *)HashGetValue(entry):0;
 
 		switch(VisEvent->state)
@@ -1095,7 +1096,7 @@ keys_and_buttons:
 		XExposeEvent *ExposeEvent = (XExposeEvent*) xevent;
 		Rect screenRect;
 
-		entry = HashLookOnly(&grTkWindowTable, wind);
+		entry = HashLookOnly(&grTkWindowTable, (char *)wind);
 		mw = (entry)?(MagWindow *)HashGetValue(entry):0;
 		grCurrent.window = wind;
 		grCurrent.windowid = Tk_WindowId(wind);
@@ -1125,7 +1126,7 @@ keys_and_buttons:
 		XConfigureEvent *ConfigureEvent = (XConfigureEvent*) xevent;
 		Rect screenRect;
 		    
-		entry = HashLookOnly(&grTkWindowTable, wind);
+		entry = HashLookOnly(&grTkWindowTable, (char *)wind);
 		mw = (entry)?(MagWindow *)HashGetValue(entry):0;
 	        grCurrent.window = wind;
 		grCurrent.windowid = Tk_WindowId(wind);
@@ -1222,6 +1223,7 @@ x11SetDisplay (dispType, outFileName, mouseFileName)
     /* local indirections */
     grSetSPatternPtr = grtkSetSPattern;
     grPutTextPtr = grtkPutText;
+    grFontTextPtr = grtkFontText;
     grDefineCursorPtr = grTkDefineCursor;
     grFreeCursorPtr = grTkFreeCursors;
     GrBitBltPtr = GrTkBitBlt;
@@ -1232,9 +1234,7 @@ x11SetDisplay (dispType, outFileName, mouseFileName)
     grSetStipplePtr = grtkSetStipple;
     grSetLineStylePtr = grtkSetLineStyle;
     grSetCharSizePtr = grtkSetCharSize;
-#ifdef NONMANHATTAN
     grFillPolygonPtr = grtkFillPolygon;
-#endif
 
     GrFreeBackingStorePtr = grtkFreeBackingStore;
     GrCreateBackingStorePtr = grtkCreateBackingStore;
@@ -1361,7 +1361,7 @@ GrTkCreate(w, name)
 	grCurrent.mw = w;
 	w->w_grdata = (ClientData) tkwind;
 
-	entry = HashFind(&grTkWindowTable, tkwind);
+	entry = HashFind(&grTkWindowTable, (char *)tkwind);
 	HashSetValue(entry,w);
 
 	/* set the window attributes, since Tk doesn't do this in the	*/
@@ -1511,7 +1511,7 @@ GrTkDelete(w)
     HashEntry	*entry;
 
     xw = (Tk_Window) w->w_grdata;
-    entry = HashLookOnly(&grTkWindowTable, xw);
+    entry = HashLookOnly(&grTkWindowTable, (char *)xw);
     HashSetValue(entry,NULL);
 
     Tcl_DeleteCommand(magicinterp, Tk_PathName(xw));
@@ -1774,7 +1774,7 @@ GrTkWindowId(tkname)
     tkwind = Tk_NameToWindow(magicinterp, tkname, Tk_MainWindow(magicinterp));
     if (tkwind != NULL)
     {
-	entry = HashLookOnly(&grTkWindowTable, tkwind);
+	entry = HashLookOnly(&grTkWindowTable, (char *)tkwind);
 	mw = (entry) ? (MagWindow *)HashGetValue(entry) : 0;
 	if (mw) id = mw->w_wid;
     }
