@@ -1038,6 +1038,7 @@ x11SetDisplay (dispType, outFileName, mouseFileName)
     /* local indirections */
     grSetSPatternPtr = grx11SetSPattern;
     grPutTextPtr = grx11PutText;
+    grFontTextPtr = grx11FontText;
     grDefineCursorPtr = grx11DefineCursor;
     GrBitBltPtr = GrX11BitBlt;
 
@@ -1054,9 +1055,7 @@ x11SetDisplay (dispType, outFileName, mouseFileName)
     grSetStipplePtr = grx11SetStipple;
     grSetLineStylePtr = grx11SetLineStyle;
     grSetCharSizePtr = grx11SetCharSize;
-#ifdef NONMANHATTAN
     grFillPolygonPtr = grx11FillPolygon;
-#endif
     
     if (execFailed)
     {
