@@ -33,6 +33,7 @@
 #include "utils/utils.h"
 #include "utils/hash.h"
 #include "dbwind/dbwind.h"
+#include "database/fonts.h"
 #include "grX11Int.h"
 
 /* locals */
@@ -594,6 +595,159 @@ grx11RectConvert(mr, xr)
 	xr->height = mr->r_ytop - mr->r_ybot + 1;
 }
 
+/*
+ *---------------------------------------------------------
+ * grx11FontText:
+ *
+ *	This routine is a fancier version of grx11PutText used for
+ *	drawing vector outline fonts from the fontList records.
+ *
+ *---------------------------------------------------------
+ */
+
+void
+grx11FontText(text, font, size, rotate, pos, clip, obscure)
+    char *text;
+    int font;
+    int size;			/* pixel size of the text */
+    int rotate;			/* text rotation */
+    Point *pos;			/* text base position */
+    Rect *clip;
+    LinkedRect *obscure;
+{
+    char *tptr;
+    FontChar *ccur, *clist;
+    Point *coffset, *tp, loffset, locoffset, corners[4], lpos;
+    Rect *cbbox, charbbox, *frect;
+    int np, i, j, w, h, llx, lly, baseline;
+    XPoint *xp;
+    Pixmap pxm;
+    double fscale, scx, scy, tmpx, tmpy, rrad, cr, sr;
+    static GC fontgc = (GC)NULL;
+
+    frect = &DBFontList[font]->mf_extents;
+    fscale = (double)size / (double)frect->r_ytop;
+    rrad = (double)rotate * 0.0174532925;
+    cr = cos(rrad);
+    sr = sin(rrad);
+    lpos = GeoOrigin;
+
+    /* 1st pass: find the descent of the string */
+
+    baseline = 0;
+    for (tptr = text; *tptr != '\0'; tptr++)
+    {
+        DBFontChar(font, *tptr, NULL, NULL, &cbbox);
+	if (cbbox->r_ybot < -baseline)
+	    baseline = -cbbox->r_ybot;
+    }
+    baseline = (int)((double)baseline * fscale);
+
+    for (tptr = text; *tptr != '\0'; tptr++)
+    {
+	scx = (double)lpos.p_x * fscale;
+	scy = (double)lpos.p_y * fscale;
+
+	tmpx = scx * cr + scy * sr;
+	tmpy = scy * cr - scx * sr;
+
+	loffset.p_x = pos->p_x + (int)round(tmpx);
+	loffset.p_y = grMagicToX(pos->p_y + baseline) + (int)round(tmpy);
+
+        DBFontChar(font, *tptr, &clist, &coffset, &cbbox);
+	np = 0;
+	for (ccur = clist; ccur != NULL; ccur = ccur->fc_next)
+	    np += ccur->fc_numpoints;
+
+	xp = (XPoint *)mallocMagic(np * sizeof(XPoint));
+
+	j = 0;
+	for (ccur = clist; ccur != NULL; ccur = ccur->fc_next)
+	{
+	    tp = ccur->fc_points;
+	    for (i = 0; i < ccur->fc_numpoints; i++, j++)
+	    {
+		scx = (double)tp[i].p_x * fscale;
+		scy = (double)tp[i].p_y * fscale;
+
+		tmpx = scx * cr - scy * sr;
+		tmpy = scx * sr + scy * cr;
+
+		xp[j].x = (int)round(tmpx);
+		xp[j].y = (int)round(tmpy);
+
+		/* Initialize bounding box */
+		if (j == 0)
+		{
+		    charbbox.r_xbot = charbbox.r_xtop = xp[j].x;
+		    charbbox.r_ybot = charbbox.r_ytop = xp[j].y;
+		}
+		else
+		{
+		    if (xp[j].x < charbbox.r_xbot)
+			charbbox.r_xbot = xp[j].x;
+		    else if (xp[j].x > charbbox.r_xtop)
+			charbbox.r_xtop = xp[j].x;
+		    if (xp[j].y < charbbox.r_ybot)
+			charbbox.r_ybot = xp[j].y;
+		    else if (xp[j].y > charbbox.r_ytop)
+			charbbox.r_ytop = xp[j].y;
+		}
+	    }
+	}
+
+	/* Create a bitmap */
+
+	w = charbbox.r_xtop - charbbox.r_xbot + 1;
+	h = charbbox.r_ytop - charbbox.r_ybot + 1;
+
+	/* Adjust all points to the bounding box origin, and invert Y */
+	for (j = 0; j < np; j++)
+	{
+	    xp[j].x -= charbbox.r_xbot;
+	    xp[j].y = charbbox.r_ytop - xp[j].y;
+	}
+
+	pxm = XCreatePixmap(grXdpy, grCurrent.window, w, h, 1);
+
+	if (fontgc == (GC)NULL)
+	{
+	    XGCValues values;
+	    values.foreground = 0;
+	    values.background = 0;
+	    fontgc = XCreateGC(grXdpy, pxm, GCForeground | GCBackground, &values);
+	}
+
+	locoffset.p_x = loffset.p_x + charbbox.r_xbot;
+	locoffset.p_y = loffset.p_y - charbbox.r_ytop;
+
+	XSetForeground(grXdpy, fontgc, 0);
+	XSetFunction(grXdpy, fontgc, GXcopy);
+	XFillRectangle(grXdpy, pxm, fontgc, 0, 0, w, h);
+	XSetFunction(grXdpy, fontgc, GXxor);
+	XSetForeground(grXdpy, fontgc, 1);
+
+	j = 0;
+	for (ccur = clist; ccur != NULL; ccur = ccur->fc_next)
+	{
+	    np = ccur->fc_numpoints;
+	    XFillPolygon(grXdpy, pxm, fontgc, &xp[j], np, Complex, CoordModeOrigin);
+	    j += np;
+	}
+	freeMagic((char *)xp);
+
+	XSetClipMask(grXdpy, grGCText, pxm);
+	XSetClipOrigin(grXdpy, grGCText, locoffset.p_x, locoffset.p_y);
+	XFillRectangle(grXdpy, grCurrent.window, grGCText, locoffset.p_x,
+		locoffset.p_y, w, h);
+
+	lpos.p_x += coffset->p_x;
+	lpos.p_y += coffset->p_y;
+
+        XFreePixmap(grXdpy, pxm);
+    }
+}
+
 /*---------------------------------------------------------
  * grxPutText:
  *      (modified on SunPutText)
