@@ -9,7 +9,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/lef/defRead.c,v 1.10 2008/05/10 20:13:05 tim Exp $";            
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/lef/defRead.c,v 1.2 2008/06/01 18:37:43 tim Exp $";            
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -122,11 +122,13 @@ DefAddRoutes(rootDef, f, oscale, special, defLayerMap)
 		if (w != 0)
 		    paintWidth = (int)roundf(w / oscale);
 		else
-		    paintWidth = (lefl) ? lefl->info.route.width : DEFAULT_WIDTH;
+		    paintWidth = (lefl) ? lefl->info.route.width :
+				DEFAULT_WIDTH * DBLambda[1] / DBLambda[0];
 		saveWidth = paintWidth;
 	    }
 	    else
-		paintWidth = (lefl) ? lefl->info.route.width : DEFAULT_WIDTH;
+		paintWidth = (lefl) ? lefl->info.route.width :
+				DEFAULT_WIDTH * DBLambda[1] / DBLambda[0];
 	}
 	else if (*token != '(')	/* via name */
 	{
@@ -236,7 +238,7 @@ DefAddRoutes(rootDef, f, oscale, special, defLayerMap)
 				    paintWidth = saveWidth;
 				else
 				    paintWidth = (lefl) ? lefl->info.route.width
-					: DEFAULT_WIDTH;
+					: DEFAULT_WIDTH * DBLambda[1] / DBLambda[0];
 				break;
 			    }
 		}
@@ -671,7 +673,8 @@ parse_error:
 enum def_pins_keys {DEF_PINS_START = 0, DEF_PINS_END};
 enum def_pins_prop_keys {
 	DEF_PINS_PROP_NET = 0, DEF_PINS_PROP_DIR,
-	DEF_PINS_PROP_LAYER, DEF_PINS_PROP_PLACED};
+	DEF_PINS_PROP_LAYER, DEF_PINS_PROP_USE,
+	DEF_PINS_PROP_PLACED, DEF_PINS_PROP_FIXED};
 
 void
 DefReadPins(f, rootDef, sname, oscale, total)
@@ -686,10 +689,11 @@ DefReadPins(f, rootDef, sname, oscale, total)
     int keyword, subkey, values;
     int processed = 0;
     int pinDir = PORT_CLASS_DEFAULT;
-    TileType curlayer;
+    TileType curlayer = -1;
     Rect *currect, topRect;
     Transform t;
     lefLayer *lefl;
+    bool pending = FALSE;
 
     static char *pin_keys[] = {
 	"-",
@@ -701,7 +705,9 @@ DefReadPins(f, rootDef, sname, oscale, total)
 	"NET",
 	"DIRECTION",
 	"LAYER",
+	"USE",
 	"PLACED",
+	"FIXED",
 	NULL
     };
 
@@ -739,6 +745,14 @@ DefReadPins(f, rootDef, sname, oscale, total)
 	{
 	    case DEF_PINS_START:		/* "-" keyword */
 
+		// Flag an error if a pin was waiting on a layer
+		// specification that was never given.
+
+		if (pending)
+		{
+		    LefError("Pin specified without layer, was not placed.\n");
+		}
+
 		/* Update the record of the number of pins		*/
 		/* processed and spit out a message for every 5% done.	*/
  
@@ -752,6 +766,7 @@ DefReadPins(f, rootDef, sname, oscale, total)
 		    LefEndStatement(f);
 		    break;
 		}
+		pending = FALSE;
 
 		/* Now do a search through the line for "+" entries	*/
 		/* And process each.					*/
@@ -771,6 +786,7 @@ DefReadPins(f, rootDef, sname, oscale, total)
 		    }
 		    switch (subkey)
 		    {
+			case DEF_PINS_PROP_USE:
 			case DEF_PINS_PROP_NET:
 			    /* Get the net name, but ignore it */
 			    token = LefNextToken(f, TRUE);
@@ -786,13 +802,27 @@ DefReadPins(f, rootDef, sname, oscale, total)
 			case DEF_PINS_PROP_LAYER:
 			    curlayer = LefReadLayer(f, FALSE);
 			    currect = LefReadRect(f, curlayer, oscale);
+			    if (pending)
+			    {
+				GeoTransRect(&t, currect, &topRect);
+				DBPaint(rootDef, &topRect, curlayer);
+				DBPutLabel(rootDef, &topRect, -1, pinname, curlayer,
+					pinDir);
+				pending = FALSE;
+			    }
 			    break;
+			case DEF_PINS_PROP_FIXED:
 			case DEF_PINS_PROP_PLACED:
 			    DefReadLocation(NULL, f, oscale, &t);
-			    GeoTransRect(&t, currect, &topRect);
-			    DBPaint(rootDef, &topRect, curlayer);
-			    DBPutLabel(rootDef, &topRect, -1, pinname, curlayer,
+			    if (curlayer == -1)
+				pending = TRUE;
+			    else
+			    {
+				GeoTransRect(&t, currect, &topRect);
+				DBPaint(rootDef, &topRect, curlayer);
+				DBPutLabel(rootDef, &topRect, -1, pinname, curlayer,
 					pinDir);
+			    }
 			    break;
 		    }
 		}
@@ -1082,6 +1112,7 @@ DefReadComponents(f, rootDef, sname, oscale, total)
 		    LefEndStatement(f);
 		    break;
 		}
+		DBLinkCell(defUse, rootDef);
 
 		/* Now do a search through the line for "+" entries	*/
 		/* And process each.					*/
