@@ -14,7 +14,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/lef/lefRead.c,v 1.11 2006/11/02 20:00:07 tim Exp $";            
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/lef/lefRead.c,v 1.2 2008/12/17 18:40:04 tim Exp $";            
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -242,9 +242,26 @@ LefNextToken(f, ignore_eol)
 	curtoken = nexttoken;
 
     /* Find the next token; set to NULL if none (end-of-line). */
+    /* Treat quoted material as a single token. */
 
-    while (!isspace(*nexttoken) && (*nexttoken != '\0') && (*nexttoken != '\n'))
-	nexttoken++;	/* skip non-whitespace (move past current token) */
+    if (*nexttoken == '\"') {
+	nexttoken++;
+	while (((*nexttoken != '\"') || (*(nexttoken - 1) == '\\')) &&
+		(*nexttoken != '\0')) {
+	    if (*nexttoken == '\n') {
+		if (fgets(nexttoken + 1, LEF_LINE_MAX -
+				(size_t)(nexttoken - line), f) == NULL)
+		    return NULL;
+	    }
+	    nexttoken++;
+	}
+	if (*nexttoken == '\"')
+	    nexttoken++;
+    }
+    else {
+	while (!isspace(*nexttoken) && (*nexttoken != '\0') && (*nexttoken != '\n'))
+	    nexttoken++;	/* skip non-whitespace (move past current token) */
+    }
 
     /* Terminate the current token */
     if (*nexttoken != '\0') *nexttoken++ = '\0';
@@ -567,10 +584,11 @@ LefRedefined(lefl, redefname)
  */
 
 TileType
-LefReadLayers(f, obstruct, lreturn)
+LefReadLayers(f, obstruct, lreturn, rreturn)
     FILE *f;
     bool obstruct;
     TileType *lreturn;
+    Rect **rreturn;
 {
     char *token;
     TileType curlayer = -1;
@@ -601,6 +619,16 @@ LefReadLayers(f, obstruct, lreturn)
 	    {
 		if (lefl->lefClass != CLASS_IGNORE)
 		    curlayer = lefl->type;
+		if (lefl->lefClass == CLASS_VIA)
+		    if (rreturn) *rreturn = &(lefl->info.via.area);
+	    }
+
+	    if (rreturn)
+	    {
+		if (lefl->lefClass == CLASS_VIA)
+		    *rreturn = &(lefl->info.via.area);
+		else
+		    *rreturn = &GeoNullRect;
 	    }
 	}
 	else
@@ -648,7 +676,7 @@ LefReadLayer(f, obstruct)
     FILE *f;
     bool obstruct;
 {
-    return LefReadLayers(f, obstruct, (TileType *)NULL);
+    return LefReadLayers(f, obstruct, (TileType *)NULL, (Rect **)NULL);
 }
 
 /*
@@ -741,6 +769,126 @@ parse_error:
     return (Rect *)NULL;
 }
 
+/*
+ *------------------------------------------------------------
+ * LefReadPolygon --
+ *
+ *	Read a LEF "POLYGON" record from the file, and
+ *	return a linked point structure.  
+ *
+ * Results:
+ *	Returns a pointer to a Rect containing the magic
+ *	coordinates, or NULL if an error occurred.
+ *
+ * Side Effects:
+ *	Reads input from file f;
+ *
+ * Note:
+ *	LEF/DEF does NOT define a RECT record as having (...)
+ *	pairs, only routes.  However, at least one DEF file
+ *	contains this syntax, so it is checked.
+ *
+ *------------------------------------------------------------
+ */
+
+Point *
+LefReadPolygon(f, curlayer, oscale, ppoints)
+    FILE *f;
+    TileType curlayer;
+    float oscale;
+    int *ppoints;
+{
+    linkedRect *lr = NULL, *newRect;
+    Point *plist = NULL;
+    char *token;
+    float px, py;
+    int lpoints = 0;
+
+    while (1)
+    {
+	token = LefNextToken(f, TRUE);
+	if (token == NULL || *token == ';') break;
+ 	if (sscanf(token, "%f", &px) != 1)
+	{
+	    LefError("Bad X value in polygon.\n");
+	    LefEndStatement(f);
+	    break;
+	}
+
+	token = LefNextToken(f, TRUE);
+	if (token == NULL || *token == ';')
+	{
+	    LefError("Missing Y value in polygon point!\n");
+	    break;
+	}
+	if (sscanf(token, "%f", &py) != 1)
+	{
+	    LefError("Bad Y value in polygon.\n");
+	    LefEndStatement(f);
+	    break;
+	}
+
+	/* Use the rect structure for convenience;  we only	*/
+	/* use the r_ll point of the rect to store each point	*/
+	/* as we read it in.					*/
+
+	newRect = (linkedRect *)mallocMagic(sizeof(linkedRect));
+	newRect->area.r_xbot = (int)roundf(px / oscale);
+	newRect->area.r_ybot = (int)roundf(py / oscale);
+	newRect->rect_next = lr;
+	lr = newRect;
+	lpoints++;
+    }
+
+    *ppoints = lpoints;
+    if (lpoints == 0) return NULL;
+
+    /* Convert linkedRect structure into a simple point list */
+
+    plist = (Point *)mallocMagic(lpoints * sizeof(Point));
+    lpoints = 0;
+    while (lr != NULL)
+    {
+	plist[*ppoints - lpoints - 1].p_x = lr->area.r_xbot;
+	plist[*ppoints - lpoints - 1].p_y = lr->area.r_ybot;
+	freeMagic(lr);
+	lpoints++;
+	lr = lr->rect_next;
+    }	
+    return plist;	
+}
+
+/*
+ *------------------------------------------------------------
+ * LefPaintPolygon --
+ *
+ *	Paint a polygon into the CellDef indicated by lefMacro.
+ * 
+ *------------------------------------------------------------
+ */
+
+void
+LefPaintPolygon(lefMacro, pointList, points, curlayer)
+    CellDef *lefMacro;
+    Point *pointList;
+    int points;
+    TileType curlayer;
+{
+    int pNum;
+    PaintUndoInfo ui;
+
+    ui.pu_def = lefMacro;
+    for (pNum = PL_PAINTBASE; pNum < DBNumPlanes; pNum++)
+    {
+	if (DBPaintOnPlane(curlayer, pNum))
+	{
+	    ui.pu_pNum = pNum;
+	    PaintPolygon(pointList, points, lefMacro->cd_planes[pNum],
+			DBStdPaintTbl(curlayer, pNum), &ui);
+	}
+    }
+}
+
 /*
  *------------------------------------------------------------
  * LefReadGeometry --
@@ -781,7 +929,9 @@ LefReadGeometry(lefMacro, f, oscale, do_list)
     char *token;
     int keyword;
     linkedRect *newRect, *rectList;
-    Rect *paintrect;
+    Point *pointList;
+    int points;
+    Rect *paintrect, *contact;
 
     static char *geometry_keys[] = {
 	"LAYER",
@@ -808,7 +958,8 @@ LefReadGeometry(lefMacro, f, oscale, do_list)
 	switch (keyword)
 	{
 	    case LEF_LAYER:
-		curlayer = LefReadLayers(f, (do_list) ? FALSE : TRUE, &otherlayer);
+		curlayer = LefReadLayers(f, (do_list) ? FALSE : TRUE, &otherlayer,
+				&contact);
 		LefEndStatement(f);
 		break;
 	    case LEF_WIDTH:
@@ -824,7 +975,23 @@ LefReadGeometry(lefMacro, f, oscale, do_list)
 		    /* Paint the area, if a CellDef is defined */
 		    if (lefMacro)
 		    {
+			// Cut layers defined as contacts use the contact
+			// dimensions, not the dimension from the LEF file
+			if (DBIsContact(curlayer) && !(GEO_RECTNULL(contact)))
+			{
+			    paintrect->r_xbot = (paintrect->r_xbot + paintrect->r_xtop);
+			    paintrect->r_ybot = (paintrect->r_ybot + paintrect->r_ytop);
+			    paintrect->r_xtop = paintrect->r_xbot + contact->r_xtop;
+			    paintrect->r_ytop = paintrect->r_ybot + contact->r_ytop;
+			    paintrect->r_xbot += contact->r_xbot;
+			    paintrect->r_ybot += contact->r_ybot;
+			    paintrect->r_xbot >>= 1;
+			    paintrect->r_ybot >>= 1;
+			    paintrect->r_xtop >>= 1;
+			    paintrect->r_ytop >>= 1;
+			}
 			DBPaint(lefMacro, paintrect, curlayer);
+
 			if ((!do_list) && (otherlayer != -1))
 				DBPaint(lefMacro, paintrect, otherlayer);
 		    }
@@ -842,7 +1009,17 @@ LefReadGeometry(lefMacro, f, oscale, do_list)
 		LefEndStatement(f);
 		break;
 	    case LEF_POLYGON:
-		LefEndStatement(f);
+		pointList = LefReadPolygon(f, curlayer, oscale, &points);
+		if (pointList)
+		{
+		    if (lefMacro)
+		    {
+			LefPaintPolygon(lefMacro, pointList, points, curlayer);
+			if ((!do_list) && (otherlayer != -1))
+			    LefPaintPolygon(lefMacro, pointList, points, otherlayer);
+		    }
+		    freeMagic(pointList);
+		}
 		break;
 	    case LEF_VIA:
 		LefEndStatement(f);
@@ -1288,7 +1465,10 @@ origin_error:
 	    DBReComputeBbox(lefMacro);
 	}
 	else
+	{
 	    lefMacro->cd_bbox = lefBBox;
+	    lefMacro->cd_extended = lefBBox;
+	}
 
 	/* Fix the bounding box and do not allow edits */
 	lefMacro->cd_flags |= /* CDNOEDIT | */ CDFIXEDBBOX;
@@ -1330,7 +1510,7 @@ LefAddViaGeometry(f, lefl, curlayer, oscale)
     currect = LefReadRect(f, curlayer, (oscale / 2));
     if (currect == NULL) return;
 
-    /* Don't save geometry for which the layer type is not known! */
+    /* Don't create any geometry for unknown layers! */
     if (curlayer < 0) return;
 
     /* For LEF contact types matching magic contact types,	*/
@@ -1338,7 +1518,7 @@ LefAddViaGeometry(f, lefl, curlayer, oscale)
     /* rectangle in the other layers that satisfies the		*/
     /* CIF/GDS contact generation.				*/
 
-    if (DBIsContact(curlayer))
+    if (DBIsContact(curlayer) && CIFCurStyle != NULL)
     {
 	int edgeSize = 0, contSize, halfSize;
 	float fcontSize;
@@ -1498,25 +1678,29 @@ LefReadLayerSection(f, lname, mode, lefl)
 	    case LEF_LAYER_WIDTH:
 		token = LefNextToken(f, TRUE);
 		sscanf(token, "%f", &fvalue);
-		lefl->info.route.width = (int)roundf(fvalue / oscale);
+		if (lefl->lefClass == CLASS_ROUTE)
+		    lefl->info.route.width = (int)roundf(fvalue / oscale);
 		LefEndStatement(f);
 		break;
 	    case LEF_LAYER_SPACING:
 		token = LefNextToken(f, TRUE);
 		sscanf(token, "%f", &fvalue);
-		lefl->info.route.spacing = (int)roundf(fvalue / oscale);
+		if (lefl->lefClass == CLASS_ROUTE)
+		    lefl->info.route.spacing = (int)roundf(fvalue / oscale);
 		LefEndStatement(f);
 		break;
 	    case LEF_LAYER_PITCH:
 		token = LefNextToken(f, TRUE);
 		sscanf(token, "%f", &fvalue);
-		lefl->info.route.pitch = (int)roundf(fvalue / oscale);
+		if (lefl->lefClass == CLASS_ROUTE)
+		    lefl->info.route.pitch = (int)roundf(fvalue / oscale);
 		LefEndStatement(f);
 		break;
 	    case LEF_LAYER_DIRECTION:
 		token = LefNextToken(f, TRUE);
 		LefLower(token);
-		lefl->info.route.hdirection = (token[0] == 'h') ? TRUE : FALSE;
+		if (lefl->lefClass == CLASS_ROUTE)
+		    lefl->info.route.hdirection = (token[0] == 'h') ? TRUE : FALSE;
 		LefEndStatement(f);
 		break;
 	    case LEF_LAYER_OFFSET:
