@@ -13,7 +13,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/lef/lefWrite.c,v 1.3 2008/02/10 19:06:32 tim Exp $";            
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/lef/lefWrite.c,v 1.3 2010/06/24 12:37:18 tim Exp $";            
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -195,10 +195,14 @@ lefWriteHeader(def, f)
 
 		/* Ignore obstruction-only layers */
 		if (lefl->type == -1) continue;	
+
 		/* Ignore VIA types, report only CUT types here */
 		else if ((lefl->lefClass == CLASS_VIA)
 			&& lefl->info.via.cell != NULL) continue;
 
+		/* Ignore boundary types */
+		else if (lefl->lefClass == CLASS_BOUND) continue;
+
 		fprintf(f, "LAYER %s\n", lefl->canonName);
 		if (lefl->lefClass == CLASS_VIA)
 		{
@@ -235,7 +239,7 @@ lefWriteHeader(def, f)
 		{
 		    fprintf(f, "   TYPE OVERLAP ;\n");
 		}
-		fprintf(f, "END %s ;\n\n", lefl->canonName);
+		fprintf(f, "END %s\n\n", lefl->canonName);
 	    }
 	}
 
@@ -264,6 +268,8 @@ typedef struct
     TileType	*lastType;	/* last type output, so we minimize LAYER
 				 * statements.
 				 */
+    CellDef	*lefYank;	/* CellDef to write into */
+    LefMapping  *lefMagicMap;	/* Layer inverse mapping table */
     TileTypeBitMask rmask;	/* mask of routing layer types */
     Point	origin;		/* origin of cell */
     float	oscale;		/* units scale conversion factor */
@@ -278,6 +284,129 @@ typedef struct
 				 */
 } lefClient;
 
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * Callback function to find the cell boundary based on the specified
+ * boundary layer type.  Typically this will be a single rectangle on
+ * its own plane, but for completeness, all geometry in the cell is
+ * checked, and the bounding rectangle adjusted to fit that area.
+ *
+ * Return 0 to keep the search going.
+ * ----------------------------------------------------------------------------
+ */
+
+int
+lefGetBound(tile, cdata)
+    Tile *tile;
+    ClientData cdata;
+{
+    Rect *boundary = (Rect *)cdata;
+    Rect area;
+
+    TiToRect(tile, &area);
+    GeoInclude(&area, boundary);
+    return 0;
+}
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * lefYankGeometry --
+ *
+ * Function called from SimSrConnect() that copies geometry from
+ * the cell into a yank buffer cell, one pin connection at a time.
+ *
+ * Return 0 to keep the search going.
+ * ----------------------------------------------------------------------------
+ */
+
+int
+lefYankGeometry(tile, cdata)
+    Tile *tile;
+    ClientData cdata;
+{
+    lefClient *lefdata = (lefClient *)cdata;
+    Rect area;
+    TileType ttype;
+    LefMapping *lefMagicToLefLayer;
+    TileTypeBitMask *sMask;
+
+    /* To enumerate obstructions, we search all tiles in the	*/
+    /* cell, and ignore all of those which were electrically	*/
+    /* connected to any pin.  These will have the tile's	*/
+    /* ti_client record set to 1.				*/
+
+    /* Because DBSrPaintArea will look at each tile once and	*/
+    /* only once, we can reset the ti_client record here, so	*/
+    /* everything is back to normal after LEF output.		*/
+
+    if (lefdata->lefMode == LEF_MODE_OBSTRUCT)
+	if (tile->ti_client == (ClientData)1)
+	{
+	    tile->ti_client = (ClientData)CLIENTDEFAULT;
+	    return 0;
+	}
+
+    ttype = TiGetTypeExact(tile);
+
+    if (IsSplit(tile))
+	ttype = (ttype & TT_SIDE) ? SplitRightType(tile) :
+			SplitLeftType(tile);
+
+    /* Output geometry only for defined routing layers	*/
+    /* If we have encountered a contact type, then	*/
+    /* decompose into constituent layers and see if any	*/
+    /* of them are in the route layer masks.		*/
+
+    if (DBIsContact(ttype))
+    {
+	sMask = DBResidueMask(ttype);
+
+	/* First check if lefdata->lastType is in the residue,	*/
+	/* and process first.					*/
+
+	if (TTMaskHasType(sMask, *(lefdata->lastType)))
+	    ttype = *(lefdata->lastType);
+	else
+	{
+	    /* And if not, then switch to the first routing	*/
+	    /* layer that is represented in sMask.  If none,	*/
+	    /* then return.					*/
+
+	    for (ttype = TT_TECHDEPBASE; ttype < DBNumTypes; ttype++)
+		if (TTMaskHasType(&lefdata->rmask, ttype))
+		    if (TTMaskHasType(sMask, ttype))
+			break;
+
+	    if (ttype == DBNumTypes) return 0;
+	}
+    }
+    else
+    {
+	if (!TTMaskHasType(&lefdata->rmask, ttype)) return 0;
+	sMask = NULL;
+    }
+    lefMagicToLefLayer = lefdata->lefMagicMap;
+
+    TiToRect(tile, &area);
+
+    while (ttype < DBNumTypes)
+    {
+	if (lefMagicToLefLayer[ttype].lefInfo != NULL)
+	    if (!IsSplit(tile))
+		DBPaint(lefdata->lefYank, &area, ttype);
+
+	if (sMask == NULL) break;
+
+	for (++ttype; ttype < DBNumTypes; ttype++)
+	    if (TTMaskHasType(&lefdata->rmask, ttype))
+		if (TTMaskHasType(sMask, ttype))
+			break;
+    }
+    return 0;
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -299,62 +428,26 @@ lefWriteGeometry(tile, cdata)
     lefClient *lefdata = (lefClient *)cdata;
     FILE *f = lefdata->file;
     float scale = lefdata->oscale;
-    TileType ttype;
+    TileType ttype = TiGetTypeExact(tile);
+    LefMapping *lefMagicToLefLayer = lefdata->lefMagicMap;
 
-    /* To enumerate obstructions, we search all tiles in the	*/
-    /* cell, and ignore all of those which were electrically	*/
-    /* connected to any pin.  These will have the tile's	*/
-    /* ti_client record set to 1.				*/
+    /* Only LEF routing layer types will be in the yank buffer */
 
-    /* Because DBSrPaintArea will look at each tile once and	*/
-    /* only once, we can reset the ti_client record here, so	*/
-    /* everything is back to normal after LEF output.		*/
-
-    if (lefdata->lefMode == LEF_MODE_OBSTRUCT)
-	if (tile->ti_client == (ClientData)1)
-	{
-	    tile->ti_client = (ClientData)CLIENTDEFAULT;
-	    return 0;
-	}
-
-    ttype = TiGetTypeExact(tile);
-
-#ifdef NONMANHATTAN
-    if (IsSplit(tile))
-	ttype = (ttype & TT_SIDE) ? SplitRightType(tile) :
-			SplitLeftType(tile);
-#endif
-
-    /* Output geometry only for defined routing layers */
     if (!TTMaskHasType(&lefdata->rmask, ttype)) return 0;
 
     if (ttype != *(lefdata->lastType))
     {
-	/* This is a temporary hack---avoid printing the image	*/
-	/* plane designation for contact types.  Preferably,	*/
-	/* the type names for LEF routing layers should be	*/
-	/* spelled out in the tech file, or at least in the	*/
-	/* LEF file header.					*/
-
-	char layername[100], *slashptr;
-
-	sprintf(layername, "%.99s", DBTypeLongName(ttype));
-	if ((slashptr = strchr(layername, '/')) != NULL) *slashptr = '\0';
-
-	fprintf(f, "         LAYER %s ;\n", layername);
-
+	if (lefMagicToLefLayer[ttype].lefInfo != NULL)
+		fprintf(f, "         LAYER %s ;\n",
+				lefMagicToLefLayer[ttype].lefName);
 	*(lefdata->lastType) = ttype;
     }
 
-#ifdef NONMANHATTAN
-    if (!IsSplit(tile))
-#endif
-
     fprintf(f, "	    RECT %.4f %.4f %.4f %.4f ;\n",
-			scale * (float)(LEFT(tile) - lefdata->origin.p_x),
-			scale * (float)(BOTTOM(tile) - lefdata->origin.p_y),
-			scale * (float)(RIGHT(tile) - lefdata->origin.p_x),
-			scale * (float)(TOP(tile) - lefdata->origin.p_y));
+		scale * (float)(LEFT(tile) - lefdata->origin.p_x),
+		scale * (float)(BOTTOM(tile) - lefdata->origin.p_y),
+		scale * (float)(RIGHT(tile) - lefdata->origin.p_x),
+		scale * (float)(TOP(tile) - lefdata->origin.p_y));
 
     return 0;
 }
@@ -365,12 +458,12 @@ lefWriteGeometry(tile, cdata)
  */
 
 int
-lefWriteGeometry2(tile, plane, cdata)
+lefYankGeometry2(tile, plane, cdata)
     Tile *tile;
     Plane *plane;
     ClientData cdata;
 {
-    return lefWriteGeometry(tile, cdata);
+    return lefYankGeometry(tile, cdata);
 }
 
 /*
@@ -451,7 +544,8 @@ lefWriteMacro(def, f, scale)
     bool propfound;
     char *propvalue, *class = NULL;
     Label *lab, *clab;
-    TileTypeBitMask lmask;
+    Rect boundary;
+    TileTypeBitMask lmask, boundmask;
     TileType ttype;
     lefClient lc;
     int idx, pNum;
@@ -468,16 +562,21 @@ lefWriteMacro(def, f, scale)
     lc.file = f;
     lc.lastType = &ttype;
     lc.oscale = scale;
+    lc.lefMagicMap = defMakeInverseLayerMap();
+    lc.lefYank = DBCellNewDef("lefYank", (char *)NULL);
+    DBCellSetAvail(lc.lefYank);
+    lc.lefYank->cd_flags |= CDINTERNAL;
 
     TxPrintf("Diagnostic:  Scale value is %f\n", lc.oscale);
 
     /* Which layers are routing layers are defined in the tech file.	*/
 
     TTMaskZero(&lc.rmask);
+    TTMaskZero(&boundmask);
     HashStartSearch(&hs);
     while (he = HashNext(&LefInfo, &hs))
     {
-	TileTypeBitMask *lmask;
+	TileTypeBitMask *lrmask;
 	lefLayer *lefl = (lefLayer *)HashGetValue(he);
 	if (lefl && (lefl->lefClass == CLASS_ROUTE || lefl->lefClass == CLASS_VIA))
 	    if (lefl->type != -1)
@@ -485,10 +584,25 @@ lefWriteMacro(def, f, scale)
 		TTMaskSetType(&lc.rmask, lefl->type);
 		if (DBIsContact(lefl->type))
 		{
-		    lmask = DBResidueMask(lefl->type);
-		    TTMaskSetMask(&lc.rmask, lmask);
+		    lrmask = DBResidueMask(lefl->type);
+		    TTMaskSetMask(&lc.rmask, lrmask);
+		}
+		else
+		{
+		    // Contact types whose residues contain a route
+		    // layer are included
+		    for (ttype = TT_TECHDEPBASE; ttype < DBNumTypes; ttype++)
+			if (DBIsContact(ttype))
+			{
+			    lrmask = DBResidueMask(ttype);
+			    TTMaskSetMask(&lc.rmask, lrmask);
+			}
 		}
 	    }
+
+	if (lefl && (lefl->lefClass == CLASS_BOUND))
+	    if (lefl->type != -1)
+		TTMaskSetType(&boundmask, lefl->type);
     }
 
     /* Any layer which has a port label attached to it should by	*/
@@ -525,19 +639,36 @@ lefWriteMacro(def, f, scale)
 
     fprintf(f, "   FOREIGN %s ;\n", def->cd_name);
 
+    /* If a boundary class was declared in the LEF section, then use	*/
+    /* that layer type to define the boundary.  Otherwise, the cell	*/
+    /* boundary is defined by the magic database.  If the boundary	*/
+    /* class is used, and the boundary layer corner is not on the	*/
+    /* origin, then shift all geometry by the difference.		*/
+
+    if (!TTMaskIsZero(&boundmask))
+    {
+	for (pNum = PL_PAINTBASE; pNum < DBNumPlanes; pNum++)
+	    DBSrPaintArea((Tile *)NULL, def->cd_planes[pNum], 
+			&TiPlaneRect, &boundmask, lefGetBound,
+			(ClientData)(&boundary));
+    }
+    else
+	boundary = def->cd_bbox;
+
     /* Apparently ORIGIN is not reliable!  Assume origin must be (0,0)	*/
     /* and adjust all geometry instead of adjusting the origin.		*/
 
 /*
     fprintf(f, "   ORIGIN %.4f %.4f ;\n",
-		lc.oscale * (float)def->cd_bbox.r_xbot,
-		lc.oscale * (float)def->cd_bbox.r_ybot);
+		lc.oscale * (float)boundary.r_xbot,
+		lc.oscale * (float)boundary.r_ybot);
 */
-    lc.origin = def->cd_bbox.r_ll;
+    fprintf(f, "   ORIGIN 0.00 0.00 ;\n");
+    lc.origin = boundary.r_ll;
 
     fprintf(f, "   SIZE %.4f BY %.4f ;\n",
-		lc.oscale * (float)(def->cd_bbox.r_xtop - def->cd_bbox.r_xbot),
-		lc.oscale * (float)(def->cd_bbox.r_ytop - def->cd_bbox.r_ybot));
+		lc.oscale * (float)(boundary.r_xtop - boundary.r_xbot),
+		lc.oscale * (float)(boundary.r_ytop - boundary.r_ybot));
 
     propvalue = (char *)DBPropGet(def, "LEFsymmetry", &propfound);
     if (propfound)
@@ -645,7 +776,15 @@ lefWriteMacro(def, f, scale)
 
 		    ttype = TT_SPACE;
 		    SimSrConnect(def, &labr, &lmask, DBConnectTbl,
-			&TiPlaneRect, lefWriteGeometry2, (ClientData) &lc);
+			&TiPlaneRect, lefYankGeometry2, (ClientData) &lc);
+
+		    for (pNum = PL_PAINTBASE; pNum < DBNumPlanes; pNum++)
+		    {
+			DBSrPaintArea((Tile *)NULL, lc.lefYank->cd_planes[pNum], 
+				&TiPlaneRect, &lc.rmask,
+				lefWriteGeometry, (ClientData) &lc);
+			DBClearPaintPlane(lc.lefYank->cd_planes[pNum]);
+		    }
 
 		    fprintf(f, "      END\n");	/* end of port geometries */
 		    clab->lab_flags |= PORT_VISITED;
@@ -671,13 +810,25 @@ lefWriteMacro(def, f, scale)
 
     fprintf(f, "   OBS\n");
     for (pNum = PL_PAINTBASE; pNum < DBNumPlanes; pNum++)
-	(void) DBSrPaintArea((Tile *)NULL, def->cd_planes[pNum], 
+    {
+	DBSrPaintArea((Tile *)NULL, def->cd_planes[pNum], 
+		&TiPlaneRect, &lc.rmask,
+		lefYankGeometry, (ClientData) &lc);
+
+	DBSrPaintArea((Tile *)NULL, lc.lefYank->cd_planes[pNum], 
 		&TiPlaneRect, &lc.rmask,
 		lefWriteGeometry, (ClientData) &lc);
+    }
+
     fprintf(f, "   END\n");	/* end of obstruction geometries */
 
     fprintf(f, "END %s\n", def->cd_name);	/* end of macro */
 
+    SigDisableInterrupts();
+    freeMagic(lc.lefMagicMap);
+    DBCellClearDef(lc.lefYank);
+    SigEnableInterrupts();
+
     UndoEnable();
 }
 
