@@ -23,7 +23,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/mzrouter/mzMain.c,v 1.19 2008/03/06 17:56:53 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/mzrouter/mzMain.c,v 1.3 2010/06/24 12:37:19 tim Exp $";
 #endif  /* not lint */
 
 /*--- includes --- */
@@ -113,9 +113,8 @@ int mzVerbosity;
 /* if positive, upper bound on number of blooms */
 int mzBloomLimit;
 
-/* maskdata tiles, and unexpanded subcells, marked because they are part of 
+/* maskdata unexpanded subcells, marked because they are part of 
  * dest. nodes. */
-List *mzMarkedTilesList;
 List *mzMarkedCellsList;
 
 /* start terminals */
@@ -235,15 +234,15 @@ bool mzPathsDirty = FALSE;
 #define ADDR_TBL_EQUIV(a1,a2) \
 if(TRUE) \
 { \
-   HashSetValue(HashFind(&aT, (ClientData) (a1)), (ClientData) (a2)); \
-   HashSetValue(HashFind(&aT, (ClientData) (a2)), (ClientData) (a1)); \
+   HashSetValue(HashFind(&aT, (char *) (a1)), (char *) (a2)); \
+   HashSetValue(HashFind(&aT, (char *) (a2)), (char *) (a1)); \
 } else
 
 /* macro for translating address to address paired with it in address table */
 #define ADDR_TBL(type,a) \
 if (TRUE) \
 { \
-  HashEntry *he = HashLookOnly(&aT, (ClientData) (a)); \
+  HashEntry *he = HashLookOnly(&aT, (char *) (a)); \
   if(he) \
   { \
       a = (type) HashGetValue(he); \
@@ -1275,19 +1274,6 @@ MZClean()
 	{
 	    List *l;
 
-	    /* Reset Marked tiles client fields to CLIENTDEFAULT */
-	    for(l=mzMarkedTilesList; l!=NULL; l=LIST_TAIL(l))
-	    {
-		Tile *tile = (Tile *) LIST_FIRST(l);
-		
-		/* Restore tile client field to its "unmarked" value */
-		tile->ti_client = (ClientData) CLIENTDEFAULT;
-	    }
-
-	    /* Dealloc list of marked tiles */
-	    ListDealloc(mzMarkedTilesList);
-	    mzMarkedTilesList = NULL;
-
 	    /* Reset Marked subcell client fields to CLIENTDEFAULT */
 	    for(l=mzMarkedCellsList; l!=NULL; l=LIST_TAIL(l))
 	    {
