@@ -21,7 +21,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/netmenu/NMmain.c,v 1.1.1.1 2006/04/10 22:03:14 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/netmenu/NMmain.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -231,7 +231,7 @@ NMreposition(window, newScreenArea, final)
  *	window lock.
  *
  * Results:
- *	Return 0 always
+ *	Return 0 always.
  *
  * Side effects:
  *	The given area of the window is redisplayed.
@@ -348,7 +348,7 @@ NMredisplay(w, rootArea, clipArea)
  *	button is pressed with the cursor inside the netlist menu.
  *
  * Results:
- *	Return 0 always
+ *	Return 0 always.
  *
  * Side effects:
  *	If the cursor is over one of the buttons, the corresponding
