@@ -20,7 +20,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/plot/plotPixels.c,v 1.4 2008/01/29 02:41:57 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/plot/plotPixels.c,v 1.2 2008/06/01 18:37:45 tim Exp $";
 #endif  /* not lint */
 
 #include <errno.h>
@@ -1040,7 +1040,7 @@ PlotPixels(scx, layers, xMask, width)
 	    TTMaskSetType(curMask, TT_SPACE);
 	    GeoTransRect(&tinv, &labelArea, &scx2.scx_area);
 	    (void)DBTreeSrLabels(&scx2, curMask, xMask, (TerminalPath *) NULL,
-				 plotPixLabel, (ClientData) NULL);
+				 TF_LABEL_ATTACH, plotPixLabel, (ClientData) NULL);
 	}
 
 	/* Output subcell bounding boxes, if they are wanted. */
