@@ -21,7 +21,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/plot/plotRutils.c,v 1.6 2008/04/29 17:17:40 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/plot/plotRutils.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -472,7 +472,6 @@ PlotFillRaster(raster, area, stipple)
 
 	/* Fill the leftmost word on this line. */
 
-	*left &= 0x77777777 | ~leftMask;
 	*left |= curStipple & leftMask;
 
 	if (left != right)
@@ -480,14 +479,10 @@ PlotFillRaster(raster, area, stipple)
 	    /* Fill the center of the line. */
 
 	    for (cur = left+1; cur < right; cur += 1)
-	    {
-		*cur &= 0x77777777;
 		*cur |= curStipple;
-	    }
 	    
 	    /* Fill the rightmost word on this line. */
 
-	    *cur &= 0x77777777 | ~rightMask;
 	    *cur |= curStipple & rightMask;
 	}
 
@@ -862,23 +857,18 @@ PlotRasterText(raster, clip, font, string, point)
  */
 
 void
-PlotRastPoint(raster, x, y, stipple)
+PlotRastPoint(raster, x, y)
     Raster *raster;		/* Raster containing pixel to be
 					 * filled.
 					 */
     int x, y;				/* Coordinates of pixel. */
-    Stipple stipple;
 {
-    int curByte;
-
     if ((x < 0) || (x >= raster->ras_width)) return;
     y = (raster->ras_height - 1) - y;
     if ((y < 0) || (y >= raster->ras_height)) return;
 
-    curByte = singleBit[x&037];
-    if (stipple[(-y)&017] & curByte)
-	raster->ras_bits[((y*raster->ras_intsPerLine) + (x>>5))]
-		|= curByte;
+    raster->ras_bits[((y*raster->ras_intsPerLine) + (x>>5))]
+	    |= singleBit[x&037];
 }
 
 /*
@@ -897,23 +887,14 @@ PlotRastPoint(raster, x, y, stipple)
  *	may lie outside the raster (this feature is necessary to draw
  *	straight lines that cross multiple swaths).
  *
- * Notes:
- *	Modified April 23, 2008 by Tim Edwards, to include stippling.
- *	Otherwise, lines can only be drawn in the 7 solid colors RGBCMYK
- *	depending on the components of the tile color.  Stippling lines
- *	allows the boundary color to be arbitrary.  However, line stipple
- *	patterns need to be random to avoid having orientation- and position-
- *	dependent color.
- *
  * ----------------------------------------------------------------------------
  */
 
 void
-PlotRastLine(raster, src, dst, stipple)
+PlotRastLine(raster, src, dst)
     Raster *raster;		/* Where to render the line. */
     Point *src;			/* One endpoint of line, in raster coords. */
     Point *dst;			/* The other endpoint, in raster coords. */
-    Stipple *stipple;		/* Stipple pattern for line */
 {
     int x, y, dx, dy, xinc, incr1, incr2, d, done;
 
@@ -956,7 +937,7 @@ PlotRastLine(raster, src, dst, stipple)
 	done = dst->p_x;
 	for ( ; x != done ; x += xinc)
 	{
-	    PlotRastPoint(raster, x, y, stipple);
+	    PlotRastPoint(raster, x, y);
 	    if (d < 0)
 		d += incr1;
 	    else
@@ -974,7 +955,7 @@ PlotRastLine(raster, src, dst, stipple)
 	done = dst->p_y;
 	for ( ; y != done ; y += 1)
 	{
-	    PlotRastPoint(raster, x, y, stipple);
+	    PlotRastPoint(raster, x, y);
 	    if (d < 0)
 		d += incr1;
 	    else
@@ -984,7 +965,7 @@ PlotRastLine(raster, src, dst, stipple)
 	    }
 	}
     }
-    PlotRastPoint(raster, x, y, stipple);
+    PlotRastPoint(raster, x, y);
 }
 
 /*
@@ -1012,7 +993,7 @@ PlotRastLine(raster, src, dst, stipple)
  */
 
 void
-PlotRastFatLine(raster, src, dst, widen, stipple)
+PlotRastFatLine(raster, src, dst, widen)
     Raster *raster;		/* Where to render the line. */
     Point *src;			/* One endpoint of line, in raster coords. */
     Point *dst;			/* The other endpoint, in raster coords. */
@@ -1020,7 +1001,6 @@ PlotRastFatLine(raster, src, dst, widen, stipple)
 				 * line is one pixel wide, 1 means it's 3
 				 * pixels wide, and so on.
 				 */
-    Stipple *stipple;		/* Pattern in which to draw line */
 {
     double dx, dy, x, y;
     int nLines;
@@ -1055,7 +1035,7 @@ PlotRastFatLine(raster, src, dst, widen, stipple)
 	newSrc.p_x += src->p_x;
 	newSrc.p_y += src->p_y;
 
-	PlotRastLine(raster, &newSrc, &newDst, stipple);
+	PlotRastLine(raster, &newSrc, &newDst);
     }
 }
 
