@@ -18,13 +18,11 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/plot/plotVers.c,v 1.17 2008/11/16 22:06:54 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/plot/plotVers.c,v 1.3 2010/06/24 12:37:25 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
-#include <stdlib.h>		/* For random() */
 #include <string.h>
-#include <math.h>		/* For dot cluster stipple generation */
 
 #include "utils/magic.h"
 #include "utils/geometry.h"
@@ -43,10 +41,6 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/
 #include "dbwind/dbwind.h"
 #include "cif/cif.h"		/* for CIFGetOutputScale() */
 
-#include "graphics/graphics.h"	/* for automatic color-versatec color setup */
-#include "utils/styles.h"
-#include "dbwind/dbwtech.h"
-
 #ifdef VERSATEC
 
 /* Library imports: */
@@ -61,8 +55,7 @@ extern int rasFileByteCount;
 typedef struct versatecstyle
 {
     TileTypeBitMask vs_layers;		/* Layers to plot in this style. */
-    Stipple vs_stipple;			/* Area stipple pattern to use. */
-    Stipple vs_lineStipple;		/* Line stipple pattern to use. */
+    Stipple vs_stipple;			/* Stipple pattern to use. */
     int vs_flags;			/* Flag bits, see below. */
     struct versatecstyle *vs_next;	/* Pointer to next style in list. */
 
@@ -107,11 +100,6 @@ char *plotVersatecColorNames[] = {
 
 unsigned char PlotVersPlotType = HPRTL;
 
-float PlotVersGamma = 1.0;	/* When auto-generating stipples, use
-				 * this value for gamma (color value
-				 * scaling; values > 1 make colors lighter)
-				 */
-
 int PlotVersWidth = 2400;	/* Number of dots across Versatec page.
 				 * Should be a multiple of 32.
 				 */
@@ -249,8 +237,6 @@ PlotVersTechInit()
 
 #endif /* VERSATEC */
 
-#ifdef VERSATEC
-
 
 /*
  * ----------------------------------------------------------------------------
@@ -267,6 +253,8 @@ PlotVersTechInit()
  * ----------------------------------------------------------------------------
  */
 
+#ifdef VERSATEC
+
 void
 PlotColorVersTechInit()
 {
@@ -292,264 +280,12 @@ PlotColorVersTechInit()
 	StrDup(&PlotVersLabelFont, defaultLabelFont);
 }
 
-/*
- * ----------------------------------------------------------------------------
- *	PlotColorVersTechFinal --
- *
- * 	Called once at end of technology file read-in to create a default
- *	style if none was defined, and to fill in line stipple patterns.
- *
- * Results:
- *	None.
- *
- * Side effects:
- *	Clears out the list of things to plot.
- * ----------------------------------------------------------------------------
- */
-
-void
-PlotColorVersTechFinal()
-{
-    VersatecStyle *new, *style;
-    int cs, i, j, k, l, m, fill, stipple;
-    int cc, ff, red, green, blue, colors[3];
-
-    if (plotColorVersStyles == NULL)
-    {
-	/* Generate temporary halftone screens for each color.	*/
-
-	int p[YELLOW + 1];
-	int CMYstip[16][16];
-	int CMYorder[YELLOW + 1][256][2];
-	double t, r, x, y;
-
-	for (k = 0; k < 4; k++)
-	{
-	    p[k] = 0;
-	    for (i = 0; i < 256; i++)
-		for (j = 0; j < 2; j++)
-		    CMYorder[k][i][j] = 0;
-	}
-	for (i = 0; i < 16; i++)
-	    for (j = 0; j < 16; j++)
-		CMYstip[i][j] = 0;
-
-	t = r = 0.0;
-	for (k = 0; k < 16384; k++)
-	{ 
-	   x = r * cos(t);
-	   y = r * sin(t);	/* Expanding spiral */
-
-	   /* Magenta center dot */
-	   i = (int)(x + 8.5);
-	   j = (int)(y + 8.5);
-	   if (i >= 0 && i < 16 && j >= 0 && j < 16 && (CMYstip[i][j] & 1) == 0) {
-	      CMYstip[i][j] |= 1;
-	      CMYorder[MAGENTA][p[MAGENTA]][0] = i;
-	      CMYorder[MAGENTA][p[MAGENTA]][1] = j;
-	      p[MAGENTA]++;
-	   }
-
-	   /* Magenta corner dot */
-	   i = (int)((x < 0) ? x + 17 : x);
-	   j = (int)((y < 0) ? y + 17 : y);
-	   if (i >= 0 && i < 16 && j >= 0 && j < 16 && (CMYstip[i][j] & 1) == 0) {
-	      CMYstip[i][j] |= 1;
-	      CMYorder[MAGENTA][p[MAGENTA]][0] = i;
-	      CMYorder[MAGENTA][p[MAGENTA]][1] = j;
-	      p[MAGENTA]++;
-	   }
-
-	   /* Cyan top/bottom dot */
-	   i = (int)(x + 8.5);
-	   j = (int)((y < 0) ? y + 17 : y);
-	   if (i >= 0 && i < 16 && j >= 0 && j < 16 && (CMYstip[i][j] & 2) == 0) {
-	      CMYstip[i][j] |= 2;
-	      CMYorder[CYAN][p[CYAN]][0] = i;
-	      CMYorder[CYAN][p[CYAN]][1] = j;
-	      p[CYAN]++;
-	   }
-
-	   /* Cyan left/right dot */
-
-	   i = (int)((x < 0) ? x + 17 : x);
-	   j = (int)(y + 8.5);
-	   if (i >= 0 && i < 16 && j >= 0 && j < 16 && (CMYstip[i][j] & 2) == 0) {
-	      CMYstip[i][j] |= 2;
-	      CMYorder[CYAN][p[CYAN]][0] = i;
-	      CMYorder[CYAN][p[CYAN]][1] = j;
-	      p[CYAN]++;
-	   }
-
-	   /* Yellow lower-left dot */
-
-	   i = (int)((x < -4.25) ? x + 12.75 : x + 4.25);
-	   j = (int)((y < -4.25) ? y + 12.75 : y + 4.25);
-	   if (i >= 0 && i < 16 && j >= 0 && j < 16 && (CMYstip[i][j] & 4) == 0) {
-	      CMYstip[i][j] |= 4;
-	      CMYorder[YELLOW][p[YELLOW]][0] = i;
-	      CMYorder[YELLOW][p[YELLOW]][1] = j;
-	      p[YELLOW]++;
-	   }
-
-	   /* Yellow upper-right dot */
-
-	   i = (int)((x > 4.25) ? x - 4.25 : x + 12.75);
-	   j = (int)((y > 4.25) ? y - 4.25 : y + 12.75);
-	   if (i >= 0 && i < 16 && j >= 0 && j < 16 && (CMYstip[i][j] & 4) == 0) {
-	      CMYstip[i][j] |= 4;
-	      CMYorder[YELLOW][p[YELLOW]][0] = i;
-	      CMYorder[YELLOW][p[YELLOW]][1] = j;
-	      p[YELLOW]++;
-	   }
-
-	   /* Increase radius and angle to move outward on spiral */
-	   t += 0.05;
-	   // r += 0.005;
-	   r += 0.002;
-	}
-	
-	for (j = 0; j < DBWNumStyles; j++)
-	{
-	    cc = GrStyleTable[j + TECHBEGINSTYLES].color;
-	    GrGetColor(cc, &red, &green, &blue);
-	    // colors[0] = 255 - red;
-	    // colors[1] = 255 - green;
-	    // colors[2] = 255 - blue;
-	    colors[0] = (int)(255.0 * pow((1.0 - ((double)red / 255.0)),
-			PlotVersGamma));
-	    colors[1] = (int)(255.0 * pow((1.0 - ((double)green / 255.0)),
-			PlotVersGamma));
-	    colors[2] = (int)(255.0 * pow((1.0 - ((double)blue / 255.0)),
-			PlotVersGamma));
-
-	    for (cc = CYAN; cc <= YELLOW; cc++)
-	    {
-		new = (VersatecStyle *)mallocMagic(sizeof(VersatecStyle));
-		TTMaskZero(&new->vs_layers);
-		TTMaskSetMask(&new->vs_layers, DBWStyleToTypes(j));
-
-		/* Set the Versatec style to border or cross if the	*/
-		/* display style is so defined.				*/
-
-		new->vs_color = cc;
-		new->vs_flags = 0;
-		switch (GrStyleTable[j + TECHBEGINSTYLES].fill)
-		{
-		    case GR_STCROSS:
-			new->vs_flags = VS_CROSS;
-			break;
-		    case GR_STOUTLINE:
-			new->vs_flags = VS_BORDER;
-			break;
-		    case GR_STSTIPPLE:
-			stipple = GrStyleTable[j + TECHBEGINSTYLES].stipple;
-			break;
-		    case GR_STSOLID:
-			stipple = -1;
-			break;
-		}
-
-		/* Clear the stipple */
-		for (i = 0; i < 16; i++) new->vs_stipple[i] = 0;
-
-		/* Fill bits of vs_stipple to match the color, using	*/
-		/* dot clusters computed above.  To avoid aliasing	*/
-		/* effects from the display stipple masks, we ensure	*/
-		/* that the correct density of each color plane is	*/
-		/* met for the area defined by the display stipple.	*/
-
-		/* Display style stipple is 8x8; PlotVers stipple is	*/
-		/* 16 x 16, so we correct for the factor of 2.		*/
-
-		fill = colors[cc - CYAN];
-
-		/* Find the number of pixels to fill, and prorate fill	*/
-		/* factor accordingly.					*/
-
-		if (stipple >= 0)
-		{
-		    ff = 0;
-		    for (i = 0; i < 8; i++)
-			for (k = 0; k < 8; k++)
-			{
-			    l = GrStippleTable[stipple][i];
-			    if (((l >> k) & 0x1) != 0)
-				ff += 4;
-			}
-		    fill *= ff;
-		    fill /= 256;
-		}
-
-		/* Fill the color plane stipples where they		*/
-		/* intersect with the display stipple, up to the	*/
-		/* color density required.				*/
-
-		ff = 0;		/* cumulative fill factor */
-		for (m = 0; m < 256; m++)
-		{
-		    i = CMYorder[cc][m][0];
-		    k = CMYorder[cc][m][1];
-		    if (stipple >= 0)
-		    {
-			l = GrStippleTable[stipple][i >> 1];
-			if (((l >> ((k % 16) >> 1)) & 0x1) == 0)
-			    continue;
-		    }
-
-		    new->vs_stipple[i] |= (0x1 << k);
-		    new->vs_stipple[i] |= (0x1 << (16 + k));
-		    if (++ff > fill) break;
-		}
-
-		new->vs_next = plotColorVersStyles;
-		plotColorVersStyles = new;
-	    }
-	}
-    }
-
-    for (style = plotColorVersStyles; style != NULL; style = style->vs_next)
-    {
-	fill = 0;
-	for (i = 0; i < 16; i++)
-	{
-	    cs = style->vs_stipple[i];
-	    for (j = 0; j < 8 * sizeof(int); j++)
-	    {
-		if (cs & 0x1) fill++;
-		cs >>= 1;
-	    }
-	}
-
-	/* "fill" is the number of bits of this color out of 512. */
-	/* Now make it 30% darker */
-
-	fill = (512 + 2 * fill) / 3;
-
-	/* Randomly fill bits of vs_lineStipple to the density of "fill" */
-	for (i = 0; i < 16; i++)
-	{
-	    cs = 0;
-	    for (j = 0; j < 8 * sizeof(int); j++)
-	    {
-		cs <<= 1;
-		if ((random() % 512) < fill) cs |= 0x1;
-	    }
-	    style->vs_lineStipple[i] = cs;
-	}
-    }
-}
-
 #else
 
 void
 PlotColorVersTechInit()
 {}
 
-void
-PlotColorVersTechFinal()
-{}
-
 #endif /* VERSATEC */
 
 
@@ -798,7 +534,7 @@ plotTransToSwath(src, dst)
  */
 
 void
-plotVersLine(area, widen, raster, stipple)
+plotVersLine(area, widen, raster)
     Rect *area;			/* The "corner" points of this rectangle
 				 * give the endpoints of the line, in
 				 * Magic root coordinates.
@@ -808,7 +544,6 @@ plotVersLine(area, widen, raster, stipple)
 				 * pixels wide, etc.
 				 */
     Raster *raster;		/* Raster to write to */
-    Stipple *stipple;		/* Pattern to use to get the right line color */
 {
     Rect swathArea;
 
@@ -823,10 +558,10 @@ plotVersLine(area, widen, raster, stipple)
 	GEOCLIP(&swathArea, &swathClip);
 	if ((swathArea.r_xbot <= swathArea.r_xtop) &&
 		(swathArea.r_ybot <= swathArea.r_ytop))
-	    PlotFillRaster(raster, &swathArea, stipple);
+	    PlotFillRaster(raster, &swathArea, PlotBlackStipple);
     }
     else
-    PlotRastFatLine(raster, &swathArea.r_ll, &swathArea.r_ur, widen, stipple);
+    PlotRastFatLine(raster, &swathArea.r_ll, &swathArea.r_ur, widen);
 }
 
 /*
@@ -848,7 +583,7 @@ plotVersLine(area, widen, raster, stipple)
  */
 
 void
-plotVersRect(area, widen, raster, stipple)
+plotVersRect(area, widen, raster)
     Rect *area;	/* Rectangular area to draw, in root
 				 * coordinates.
 				 */
@@ -858,7 +593,6 @@ plotVersRect(area, widen, raster, stipple)
 				 * side.
 				 */
     Raster *raster;		/* Raster to plot to */
-    Stipple *stipple;		/* Pattern to use to get the right line color */
 {
     Rect side;
 
@@ -868,7 +602,7 @@ plotVersRect(area, widen, raster, stipple)
     {
 	side = *area;
 	side.r_ytop = side.r_ybot;
-	plotVersLine(&side, widen, raster, stipple);
+	plotVersLine(&side, widen, raster);
 
 	/* Now the top side, if it doesn't coincide with the bottom. */
 
@@ -876,7 +610,7 @@ plotVersRect(area, widen, raster, stipple)
 	{
 	    side = *area;
 	    side.r_ybot = side.r_ytop;
-	    plotVersLine(&side, widen, raster, stipple);
+	    plotVersLine(&side, widen, raster);
 	}
     }
 
@@ -886,7 +620,7 @@ plotVersRect(area, widen, raster, stipple)
     {
 	side = *area;
 	side.r_xtop = side.r_xbot;
-	plotVersLine(&side, widen, raster, stipple);
+	plotVersLine(&side, widen, raster);
 
 	/* Now the right side, if it doesn't coincide with the left. */
 
@@ -894,7 +628,7 @@ plotVersRect(area, widen, raster, stipple)
 	{
 	    side = *area;
 	    side.r_xbot = side.r_xtop;
-	    plotVersLine(&side, widen, raster, stipple);
+	    plotVersLine(&side, widen, raster);
 	}
     }
 }
@@ -937,23 +671,20 @@ plotVersTile(tile, cxp)
     
     if (curStyle->vs_flags & VS_CROSS)
     {
-#ifdef NONMANHATTAN
 	if (!IsSplit(tile))
-#endif
 	    if (((swathArea.r_xtop - swathArea.r_xbot) > 6)
 			&& ((swathArea.r_ytop - swathArea.r_ybot) > 6))
 	    {
 		Rect r2;
-		plotVersLine(&rootArea, 0, raster, curStyle->vs_lineStipple);
+		plotVersLine(&rootArea, 0, raster);
 		r2.r_xtop = rootArea.r_xbot;
 		r2.r_ybot = rootArea.r_ybot;
 		r2.r_xbot = rootArea.r_xtop;
 		r2.r_ytop = rootArea.r_ytop;
-		plotVersLine(&r2, 0, raster, curStyle->vs_lineStipple);
+		plotVersLine(&r2, 0, raster);
 	    }
     }
 
-#ifdef NONMANHATTAN
     if (IsSplit(tile))
     {
 	int i, j;
@@ -974,23 +705,20 @@ plotVersTile(tile, cxp)
 	    r.r_xbot = r.r_xtop;
 	    r.r_xtop = rootArea.r_xbot;
 	}
-	plotVersLine(&r, 0, raster, curStyle->vs_lineStipple);
+	plotVersLine(&r, 0, raster);
     }
     else
     {
-#endif
 
-    /* Clip and then stipple. */
+	/* Clip and then stipple. */
 
-    GEOCLIP(&swathArea, &swathClip);
-    if (swathArea.r_xbot > swathArea.r_xtop) return 0;
-    if (swathArea.r_ybot > swathArea.r_ytop) return 0;
-    if (!(curStyle->vs_flags & VS_BORDER) && !(curStyle->vs_flags & VS_CROSS))
-	PlotFillRaster(raster, &swathArea, curStyle->vs_stipple);
+	GEOCLIP(&swathArea, &swathClip);
+	if (swathArea.r_xbot > swathArea.r_xtop) return 0;
+	if (swathArea.r_ybot > swathArea.r_ytop) return 0;
+	if (!(curStyle->vs_flags & VS_BORDER) && !(curStyle->vs_flags & VS_CROSS))
+		PlotFillRaster(raster, &swathArea, curStyle->vs_stipple);
 
-#ifdef NONMANHATTAN
     }
-#endif
 
     /* Now output lines for any edges between material of the type
      * currently being drawn and material of other types.  This is
@@ -999,10 +727,8 @@ plotVersTile(tile, cxp)
      * (unless it is at infinity).
      */
     
-#ifdef NONMANHATTAN
     if (IsSplit(tile) && (!(SplitSide(tile) ^ SplitDirection(tile))))
 	goto searchleft;	/* nothing on bottom of split */
-#endif
 
     if (tileArea.r_ybot > TiPlaneRect.r_ybot)
     {
@@ -1017,15 +743,13 @@ plotVersTile(tile, cxp)
 	    if (edge.r_xbot < tileArea.r_xbot) edge.r_xbot = tileArea.r_xbot;
 	    if (edge.r_xtop > tileArea.r_xtop) edge.r_xtop = tileArea.r_xtop;
 	    GEOTRANSRECT(trans, &edge, &rootArea);
-	    plotVersLine(&rootArea, 0, raster, curStyle->vs_lineStipple);
+	    plotVersLine(&rootArea, 0, raster);
 	}
     }
 
 searchleft:
-#ifdef NONMANHATTAN
     if (IsSplit(tile) && (SplitSide(tile)))
 	goto searchtop;		/* Nothing on left side of split */
-#endif
 
     /* Now go along the tile's left border, doing the same thing.   Ignore
      * edges that are at infinity.
@@ -1044,17 +768,15 @@ searchleft:
 	    if (edge.r_ybot < tileArea.r_ybot) edge.r_ybot = tileArea.r_ybot;
 	    if (edge.r_ytop > tileArea.r_ytop) edge.r_ytop = tileArea.r_ytop;
 	    GEOTRANSRECT(trans, &edge, &rootArea);
-	    plotVersLine(&rootArea, 0, raster, curStyle->vs_lineStipple);
+	    plotVersLine(&rootArea, 0, raster);
 	}
     }
 
     /* Same thing for the tile's top border. */
 
 searchtop:
-#ifdef NONMANHATTAN
     if (IsSplit(tile) && (SplitSide(tile) ^ SplitDirection(tile)))
 	goto searchright;		/* Nothing on top side of tile */
-#endif
 
     if (tileArea.r_ytop < TiPlaneRect.r_ytop)
     {
@@ -1069,17 +791,15 @@ searchtop:
 	    if (edge.r_xbot < tileArea.r_xbot) edge.r_xbot = tileArea.r_xbot;
 	    if (edge.r_xtop > tileArea.r_xtop) edge.r_xtop = tileArea.r_xtop;
 	    GEOTRANSRECT(trans, &edge, &rootArea);
-	    plotVersLine(&rootArea, 0, raster, curStyle->vs_lineStipple);
+	    plotVersLine(&rootArea, 0, raster);
 	}
     }
 
     /* Finally, the right border. */
 
 searchright:
-#ifdef NONMANHATTAN
     if (IsSplit(tile) && !(SplitSide(tile)))
 	return 0;		/* Nothing on right side of tile */
-#endif
 
     if (tileArea.r_xtop < TiPlaneRect.r_xtop)
     {
@@ -1094,7 +814,7 @@ searchright:
 	    if (edge.r_ybot < tileArea.r_ybot) edge.r_ybot = tileArea.r_ybot;
 	    if (edge.r_ytop > tileArea.r_ytop) edge.r_ytop = tileArea.r_ytop;
 	    GEOTRANSRECT(trans, &edge, &rootArea);
-	    plotVersLine(&rootArea, 0, raster, curStyle->vs_lineStipple);
+	    plotVersLine(&rootArea, 0, raster);
 	}
     }
     return 0;
@@ -1216,7 +936,7 @@ plotVersLabel(scx, label, tpath, raster)
     {
 	/* Line or rectangle.  Draw outline. */
 
-	plotVersRect(&rootArea, 1, raster, PlotBlackStipple);
+	plotVersRect(&rootArea, 1, raster);
     }
 
     /* Output the text for the label.  Before outputting the label,
@@ -1270,7 +990,7 @@ plotVersCell(scx, raster)
 
     def = scx->scx_use->cu_def;
     GeoTransRect(&scx->scx_trans, &def->cd_bbox, &rootArea);
-    plotVersRect(&rootArea, 2, raster, PlotBlackStipple);
+    plotVersRect(&rootArea, 2, raster);
 
     if (!PlotShowCellNames)
 	return (0);
@@ -1690,8 +1410,9 @@ PlotVersatec(scx, layers, xMask, user_scale)
 		curMask = *layers;
 		TTMaskSetType(&curMask, TT_SPACE);
 		GeoTransRect(&tinv, &labelArea, &scx2.scx_area);
-		(void) DBTreeSrLabels(&scx2, &curMask, xMask, (TerminalPath *) NULL,
-				      plotVersLabel, (ClientData) raster);
+		(void) DBTreeSrLabels(&scx2, &curMask, xMask,
+				(TerminalPath *) NULL, TF_LABEL_ATTACH,
+				plotVersLabel, (ClientData) raster);
 	    }
 
 	    /* Output subcell bounding boxes, if they are wanted. */
