@@ -19,7 +19,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/plow/PlowMain.c,v 1.1.1.1 2006/04/10 22:03:13 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/plow/PlowMain.c,v 1.2 2008/12/11 04:20:12 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -45,6 +45,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/
 #include "utils/signals.h"
 #include "utils/main.h"
 #include "select/select.h"
+#include "graphics/graphics.h"
 
 #if defined(SYSV) || defined(__APPLE__)
 # define	NO_RUSAGE
@@ -155,7 +156,6 @@ PlowRule *plowBuildWidthRules();
 void plowMergeBottom(Tile *, Plane *);
 void plowInitRule();
 
-extern void PlowExtendJogHorizon();
 extern void PlowRedrawBound();
 extern void PlowClearBound();
 extern void plowUpdate();
