@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/plow/PlowRules1.c,v 1.1.1.1 2006/04/10 22:03:13 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/plow/PlowRules1.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -469,7 +469,7 @@ plowPenumbraRule(impactedEdge, ar)
  * The second pass actually queues the edges to be moved.
  *
  * Results:
- *	Always return 0
+ *	Return 0 always
  *
  * Side effects:
  *	May add an edge to the queue of edges to be processed.
