@@ -1,5 +1,5 @@
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/resis/ResMain.c,v 1.4 2009/01/30 03:11:31 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/resis/ResMain.c,v 1.4 2010/06/24 12:37:56 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -73,16 +73,17 @@ ResInitializeConn()
 	{
 	    for (diff = TT_TECHDEPBASE; diff < TT_MAXTYPES; diff++)
 	    {
-		if TTMaskHasType(&(ExtCurStyle->exts_transSDTypes[tran][0]),diff)
+		if TTMaskHasType(&(ExtCurStyle->exts_transSDTypes[tran][0]), diff)
 		    TTMaskSetType(&ResConnectWithSD[diff],tran);
+
 		if TTMaskHasType(&(ExtCurStyle->exts_transSubstrateTypes[tran]),diff)
 		    TTMaskSetType(&ResConnectWithSD[diff],tran);
 	    }
-	    TTMaskSetMask(&ResConnectWithSD[tran],&DBConnectTbl[tran]);
 	}
+	TTMaskSetMask(&ResConnectWithSD[tran],&DBConnectTbl[tran]);
     }
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -280,12 +281,8 @@ ResFindNewContactTiles(contacts)
 		       TxError("Error: setting contact tile to null\n");
 		  }
 #endif
-#ifdef NONMANHATTAN
 		  if ((IsSplit(tile) && TTMaskHasType(&mask, TiGetRightType(tile)))
 			|| TTMaskHasType(&mask, TiGetType(tile)))
-#else
-		  if TTMaskHasType(&mask, TiGetType(tile))
-#endif
 		  {
 		       tileJunk	*j = (tileJunk *)tile->ti_client;
 		       cElement *ce;
@@ -819,7 +816,6 @@ FindStartTile(goodies, SourcePoint)
     tile = ResUse->cu_def->cd_planes[pnum]->pl_hint;
     GOTOPOINT(tile, &workingPoint);
 
-#ifdef NONMANHATTAN
     if (IsSplit(tile))
     {
         if (TTMaskHasType(&ExtCurStyle->exts_transMask, TiGetLeftType(tile)) != 0)
@@ -839,9 +835,7 @@ FindStartTile(goodies, SourcePoint)
 	    return(NULL);
 	}
     }
-    else
-#endif
-    if (TTMaskHasType(&ExtCurStyle->exts_transMask, TiGetType(tile)) == 0)
+    else if (TTMaskHasType(&ExtCurStyle->exts_transMask, TiGetType(tile)) == 0)
     {
 	TxError("Couldn't find transistor at %d %d\n",
 		goodies->rg_tranloc->p_x, goodies->rg_tranloc->p_y);
@@ -941,16 +935,13 @@ ResGetTransistor(pt)
 	  tile = ResUse->cu_def->cd_planes[pnum]->pl_hint;
 	  GOTOPOINT(tile,&workingPoint);
 
-#ifdef NONMANHATTAN
 	  if (IsSplit(tile))
 	  {
               if (TTMaskHasType(&ExtCurStyle->exts_transMask, TiGetLeftType(tile))
               	   || TTMaskHasType(&ExtCurStyle->exts_transMask, TiGetRightType(tile)))
                   return(((tileJunk *)tile->ti_client)->transistorList);
 	  }
-	  else
-#endif
-          if (TTMaskHasType(&ExtCurStyle->exts_transMask, TiGetType(tile)))
+	  else if (TTMaskHasType(&ExtCurStyle->exts_transMask, TiGetType(tile)))
           {
                return(((tileJunk *)tile->ti_client)->transistorList);
           }
