@@ -1,6 +1,6 @@
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/resis/ResMakeRes.c,v 1.3 2009/01/30 03:11:31 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/resis/ResMakeRes.c,v 1.3 2010/06/24 12:37:56 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -238,7 +238,6 @@ ResCalcEastWest(tile, pendingList, doneList, resList)
 	    resistor->rr_cl = (TOP(tile) + BOTTOM(tile)) >> 1;
 	    resistor->rr_width = height;
 
-#ifdef NONMANHATTAN
 	    if (IsSplit(tile))
 	    {
 		resistor->rr_tt = (SplitSide(tile)) ? SplitRightType(tile)
@@ -253,10 +252,6 @@ ResCalcEastWest(tile, pendingList, doneList, resList)
 		resistor->rr_status = RES_EW;
 		resistor->rr_tt = TiGetTypeExact(tile);
 	    }
-#else
-	    resistor->rr_status = RES_EW;
-	    resistor->rr_tt = TiGetType(tile);
-#endif
 #ifdef ARIEL
 	    resistor->rr_csArea = height *
 				ExtCurStyle->exts_thick[resistor->rr_tt];
@@ -411,7 +406,6 @@ ResCalcNorthSouth(tile, pendingList, doneList, resList)
 	    p2->br_this->rn_re = element;
 	    resistor->rr_cl = (LEFT(tile) + RIGHT(tile)) >> 1;
 	    resistor->rr_width = width;
-#ifdef NONMANHATTAN
 	    if (IsSplit(tile))
 	    {
 		resistor->rr_tt = (SplitSide(tile)) ? SplitRightType(tile)
@@ -425,10 +419,6 @@ ResCalcNorthSouth(tile, pendingList, doneList, resList)
 		resistor->rr_status = RES_NS;
 		resistor->rr_tt = TiGetTypeExact(tile);
 	    }
-#else
-	    resistor->rr_status = RES_NS;
-	    resistor->rr_tt = TiGetType(tile);
-#endif
 #ifdef ARIEL
 	    resistor->rr_csArea = width
 			* ExtCurStyle->exts_thick[resistor->rr_tt];
@@ -793,8 +783,8 @@ ResDoContacts(contact, nodes, resList)
     resNode	 *resptr;
     cElement	 *ccell;
     int		 tilenum, squaresx, squaresy, viawidth;
-    float	 squaresf;
     int 	 minside, spacing, border;
+    float	 squaresf;
     resResistor	 *resistor;
     resElement	 *element;
     static int	 too_small = 1;
