@@ -20,7 +20,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/router/rtrStem.c,v 1.4 2006/08/02 03:34:29 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/router/rtrStem.c,v 1.2 2008/12/04 16:21:44 tim Exp $";
 #endif  /* not lint */
 
 
@@ -44,6 +44,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/
 #include "utils/malloc.h"
 #include "debug/debug.h"
 #include "utils/signals.h"
+#include "utils/maxrect.h"
 
 /* Used when searching for stems */
 typedef struct
@@ -406,7 +407,6 @@ rtrStemExpandFunc(Tile *t, TreeContext *cxp)
     Rect *rtest;
     TileType ttype;
     Point p;
-    extern Rect *DRCMaxRectangle(CellDef *, Point *, int, TileTypeBitMask *, int);
 
     TiToRect(t, &rsrc);
     ttype = TiGetType(t);
@@ -414,9 +414,8 @@ rtrStemExpandFunc(Tile *t, TreeContext *cxp)
     p.p_x = (rsrc.r_xtop + rsrc.r_xbot) / 2;
     p.p_y = (rsrc.r_ytop + rsrc.r_ybot) / 2;
 
-    /* TO DO!  Replace 32 with pin width + spacing rule */
-
-    rtest = DRCMaxRectangle(def, &p, DBPlane(ttype), &DBConnectTbl[ttype], 32);
+    rtest = FindMaxRectangle(&TiPlaneRect, def->cd_planes[cxp->tc_plane], &p,
+			&DBConnectTbl[ttype]);
     if (rtest)
     {
         GeoTransRect(&scx->scx_trans, rtest, r); /* Copy the final rectangle */
