@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/select/selCreate.c,v 1.17 2008/11/25 13:40:06 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/select/selCreate.c,v 1.10 2010/06/24 12:37:56 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -165,12 +165,13 @@ void
 SelectClear()
 {
     SearchContext scx;
-    Rect r;
+    Rect r, expand;
     extern int selClearFunc();		/* Forward declaration. */
 
     if (SelectRootDef == NULL) return;
 
     scx.scx_area = SelectDef->cd_bbox;
+    expand = scx.scx_area;
 
     /* Special handling when the cell is flagged as a net selection */
 
@@ -194,7 +195,7 @@ SelectClear()
 
 	/* Erase all of the labels from the select cell. */
 
-	(void) DBEraseLabel(SelectDef, &TiPlaneRect, &DBAllTypeBits);
+	(void) DBEraseLabel(SelectDef, &TiPlaneRect, &DBAllTypeBits, &expand);
 
 	/* Erase all of the subcells from the select cell.  This is a bit tricky,
 	 * because we can't erase the subcells while searching for them (it will
@@ -223,17 +224,18 @@ SelectClear()
 
 	SelRememberForUndo(FALSE, SelectRootDef, &scx.scx_area);
     }
+
     TTMaskZero(&SelectDef->cd_types);
 
     /* Reset the transform, if we have been moving the selection around */
-    GeoTransRect(&SelectUse->cu_transform, &scx.scx_area, &r);
+    GeoTransRect(&SelectUse->cu_transform, &expand, &r);
     SelectUse->cu_transform = GeoIdentityTransform;
 
     /* Erase the selection from the screen. */
     DBWHLRedraw(SelectRootDef, &r, TRUE);
 
     DBReComputeBbox(SelectDef);
-    DBWAreaChanged(SelectDef, &scx.scx_area, DBW_ALLWINDOWS,
+    DBWAreaChanged(SelectDef, &expand, DBW_ALLWINDOWS,
 	(TileTypeBitMask *) NULL);
 }
 
@@ -338,7 +340,7 @@ SelectArea(scx, types, xMask)
     SelRememberForUndo(FALSE, SelectRootDef, &labelArea);
     DBReComputeBbox(SelectDef);
     DBWHLRedraw(SelectRootDef, &labelArea, TRUE);
-    DBWAreaChanged(SelectDef, &SelectDef->cd_bbox, DBW_ALLWINDOWS,
+    DBWAreaChanged(SelectDef, &SelectDef->cd_extended, DBW_ALLWINDOWS,
 	&DBAllButSpaceBits);
 }
 
@@ -362,16 +364,6 @@ SelectArea(scx, types, xMask)
  * Side effects:
  *	None.
  *
- * Notes:
- *	This routine is O(2^N) in the worst-case (caused by stair-stepped
- *	geometry at single-unit spacing).  Limiting the recursion level to
- *	22 causes the routine to fail to find the maximum-sized rectangle
- *	in such cases but also ensures a completion time of less than a
- *	second or so on a decent machine (e.g., 2GHz CPU).  The inability
- *	to find the maximum-area rectangle is not really such a big deal
- *	in these (rare) cases.  However, there must be better area-
- *	partitioning algorithms out there, and a replacement would be nice.
- *
  * ----------------------------------------------------------------------------
  */
 
@@ -405,9 +397,8 @@ selFindChunk(plane, wrongTypes, searchArea, containedArea, bestMin,
 					 * so far, if we find one better than
 					 * bestMin and bestMax.
 					 */
-    int level;				/* Recursion level, to avoid deep
-					 * recursion (this routine is worst-
-					 * case O(2^N)!)
+    int level;				/* Recursion level (to avoid worst-
+					 * case scenarios)
 					 */
 {
     Rect smaller;
@@ -483,7 +474,6 @@ selFindChunk(plane, wrongTypes, searchArea, containedArea, bestMin,
 	    bestMin, bestMax, bestChunk, level + 1);
 }
 
-#ifdef NONMANHATTAN
 /* Search function to find split tiles in the area of interest.
  */
 
@@ -505,8 +495,6 @@ selSplitFunc(tile, cxp)
     return 0;
 }
 
-#endif
-
 /* This procedure is called for each tile of the wrong type in an
  * area that is supposed to contain only tiles of other types.  It
  * just returns the area of the wrong material and aborts the search.
@@ -568,10 +556,7 @@ SelectChunk(scx, type, xMask, pArea, less)
     TileType ttype;
     Rect bestChunk;
     int bestMin, bestMax, width, height;
-
-#ifdef NONMANHATTAN
     extern int selSplitFunc();		/* Forward reference. */
-#endif
 
     /* If the source definition is changing, clear the old selection. */
 
@@ -602,7 +587,6 @@ SelectChunk(scx, type, xMask, pArea, less)
 
     newscx = *scx;
 
-#ifdef NONMANHATTAN
     /* If the tile under the area of interest is a split tile, 	*/
     /* then we use that tile as the chunk.  This routine should	*/
     /* work like the while() loop underneath, looking for the	*/
@@ -617,8 +601,6 @@ SelectChunk(scx, type, xMask, pArea, less)
         goto chunkdone;
     }
 
-#endif
-
     bestMin = bestMax = 0;
     bestChunk = GeoNullRect;
 
@@ -628,11 +610,7 @@ SelectChunk(scx, type, xMask, pArea, less)
 	/* Extract a bunch of junk. */
 
 	DBCellClearDef(Select2Def);
-#ifdef NONMANHATTAN
 	DBCellCopyManhattanPaint(&newscx, &typeMask, xMask, Select2Use);
-#else
-	DBCellCopyAllPaint(&newscx, &typeMask, xMask, Select2Use);
-#endif
 
 	/* Now find the best chunk in the area. */
 
@@ -654,9 +632,9 @@ SelectChunk(scx, type, xMask, pArea, less)
 	
 	if (GEO_SURROUND_STRONG(&newscx.scx_area, &bestChunk)) break;
 
-	/* The chunk extends to the edge of the area.  Anyplace that the
+	/* The chunk extends to the edge of the area.  Any place that the
 	 * chunk touches an edge, move that edge out by a factor of two.
-	 * Anyplace it doesn't touch, move the edge in to be just one
+	 * Any place it doesn't touch, move the edge in to be just one
 	 * unit out from the chunk.
 	 */
 	
@@ -679,11 +657,12 @@ SelectChunk(scx, type, xMask, pArea, less)
 
 chunkdone:
 
-    /* Flag the cell as a chunk selection so we can treat       */
-    /* the contents differently when doing a move op.           */
+    /* Flag the cell as a chunk selection so we can treat	*/
+    /* the contents differently when doing a move op.		*/
 
-    SelectUse->cu_flags |= CU_SELECT_CHUNK; 
+    SelectUse->cu_flags |= CU_SELECT_CHUNK;
     UndoEnable();
+
     if (less)
       {
 	SelRemoveArea(&bestChunk, &typeMask);
@@ -800,11 +779,11 @@ SelectRegion(scx, type, xMask, pArea, less)
     DBReComputeBbox(SelectDef);
     DBComputeUseBbox(SelectUse);
 
-    DBWHLRedraw(SelectRootDef, &Select2Def->cd_bbox, TRUE);
-    DBWAreaChanged(SelectDef, &Select2Def->cd_bbox, DBW_ALLWINDOWS,
+    DBWHLRedraw(SelectRootDef, &Select2Def->cd_extended, TRUE);
+    DBWAreaChanged(SelectDef, &Select2Def->cd_extended, DBW_ALLWINDOWS,
 	 &DBAllButSpaceBits);
 
-    if (pArea != NULL) *pArea = Select2Def->cd_bbox;
+    if (pArea != NULL) *pArea = Select2Def->cd_extended;
 }
 
 /*
@@ -896,22 +875,21 @@ SelectNet(scx, type, xMask, pArea, less)
 			(Rect *) NULL);
     }
 
-    /* Set the cell use flags to mark this as a net selection,  */
-    /* so we can treat it differently when it comes time to     */
+    /* Set the cell use flags to mark this as a net selection,	*/
+    /* so we can treat it differently when it comes time to	*/
     /* unselect it.						*/
 
     SelectUse->cu_flags |= CU_SELECT_NET;
     UndoEnable(); 
 
-    /* Display the newly-selected material. */
     DBReComputeBbox(SelectDef);
     DBComputeUseBbox(SelectUse);
 
-    DBWHLRedraw(SelectRootDef, &Select2Def->cd_bbox, TRUE);
-    DBWAreaChanged(SelectDef, &Select2Def->cd_bbox, DBW_ALLWINDOWS,
+    DBWHLRedraw(SelectRootDef, &Select2Def->cd_extended, TRUE);
+    DBWAreaChanged(SelectDef, &Select2Def->cd_extended, DBW_ALLWINDOWS,
 	&DBAllButSpaceBits);
 
-    if (pArea != NULL) *pArea = Select2Def->cd_bbox;
+    if (pArea != NULL) *pArea = Select2Def->cd_extended;
 }
 
 /*
@@ -1091,7 +1069,7 @@ SelectAndCopy2(newSourceDef)
 					 */
 {
     SearchContext scx;
-    Rect editArea, labelArea;
+    Rect editArea, labelArea, expanded;
     int plane;
     void (*savedPaintPlane)();
     extern int selACPaintFunc();	/* Forward reference. */
@@ -1106,7 +1084,7 @@ SelectAndCopy2(newSourceDef)
     scx.scx_trans = RootToEditTransform;
     savedPaintPlane = DBNewPaintPlane(DBPaintPlaneActive);
     (void) DBCellCopyAllPaint(&scx, &DBAllButSpaceAndDRCBits, CU_DESCEND_SPECIAL,
-    		EditCellUse);
+		EditCellUse);
     DBNewPaintPlane(savedPaintPlane);
     (void) DBCellCopyAllLabels(&scx, &DBAllTypeBits, CU_DESCEND_SPECIAL, EditCellUse,
 		(Rect *) NULL);
@@ -1138,10 +1116,12 @@ SelectAndCopy2(newSourceDef)
 	(void) DBSrPaintArea((Tile *) NULL, Select2Def->cd_planes[plane],
 		&TiPlaneRect, &DBAllButSpaceAndDRCBits, selACPaintFunc,
 		(ClientData) plane);
+	DBMergeNMTiles(Select2Def->cd_planes[plane], &TiPlaneRect,
+		(PaintUndoInfo *)NULL);
     }
 
-    (void) DBCellCopyAllLabels(&scx, &DBAllTypeBits, CU_DESCEND_SPECIAL, SelectUse,
-	    &labelArea);
+    (void) DBCellCopyAllLabels(&scx, &DBAllTypeBits, CU_DESCEND_SPECIAL,
+		SelectUse, NULL);
 
     /* We also have to be careful about copying subcells into the
      * main selection cell.  It might not have been possible to copy
@@ -1155,6 +1135,7 @@ SelectAndCopy2(newSourceDef)
 	    (ClientData) NULL);
 
     DBReComputeBbox(SelectDef);
+    DBComputeUseBbox(SelectUse);
     
     /* A little hack here:  don't do explicit redisplay of the selection,
      * or record a very large redisplay area for undo-ing.  It's not
@@ -1165,7 +1146,7 @@ SelectAndCopy2(newSourceDef)
      */
 
     SelRememberForUndo(FALSE, SelectRootDef, (Rect *) NULL);
-    DBWAreaChanged(SelectDef, &SelectDef->cd_bbox, DBW_ALLWINDOWS,
+    DBWAreaChanged(SelectDef, &SelectDef->cd_extended, DBW_ALLWINDOWS,
 	(TileTypeBitMask *) NULL);
 }
 
@@ -1196,9 +1177,6 @@ selACPaintFunc(tile, plane)
     selACarg.editClip = &area;
     GeoTransRect(&RootToEditTransform, &area, &editArea);
 
-#ifndef NONMANHATTAN
-    selACarg.ttype = TiGetType(tile);
-#else
     selACarg.ttype = TiGetTypeExact(tile);
 
     if (IsSplit(tile))
@@ -1208,10 +1186,9 @@ selACPaintFunc(tile, plane)
 		selACPaintFunc2, (ClientData) &selACarg);
     }
     else
-#endif
-    (void) DBSrPaintArea((Tile *) NULL, EditCellUse->cu_def->cd_planes[plane],
-	    &editArea, &DBAllButSpaceAndDRCBits, selACPaintFunc2,
-	    (ClientData) &selACarg);
+	(void) DBSrPaintArea((Tile *) NULL, EditCellUse->cu_def->cd_planes[plane],
+		&editArea, &DBAllButSpaceAndDRCBits, selACPaintFunc2,
+		(ClientData) &selACarg);
     return 0;
 }
 
@@ -1236,20 +1213,17 @@ selACPaintFunc2(tile, selACarg)
     TileType type = TiGetTypeExact(tile);
     TileTypeBitMask tmask, *rmask;
     TileType ttype, rtype;
-
-#ifdef NONMANHATTAN
     TileType dinfo = selACarg->ttype & (TT_DIAGONAL | TT_DIRECTION | TT_SIDE);
-#endif
 
     TiToRect(tile, &area);
     GeoTransRect(&EditToRootTransform, &area, &selArea);
 
-#ifdef NONMANHATTAN
-
-    if (dinfo & TT_DIAGONAL)
+    if ((dinfo & TT_DIAGONAL) || (type & TT_DIAGONAL))
     {
 	/* If the select area is triangular, then we need to	*/
 	/* clip in a more complicated manner.			*/
+	/* Likewise if the edit cell tile is triangular and	*/
+	/* the select area not.					*/
 
 	Point points[5];
 	Rect rrect, orect;
@@ -1276,7 +1250,10 @@ selACPaintFunc2(tile, selACarg)
 	type = (dinfo & TT_SIDE) ? (rtype << 14) : rtype;
 	type |= dinfo;
 
-	GrClipTriangle(editClip, &selArea, TRUE, dinfo, points, &np);
+	if (dinfo & TT_DIAGONAL)
+	    GrClipTriangle(editClip, &selArea, TRUE, dinfo, points, &np);
+	else
+	    GrClipTriangle(&selArea, editClip, TRUE, type, points, &np);
 
 	if (np == 0)
 	    return 0;
@@ -1371,19 +1348,6 @@ topbottom:
 	}
 	TTMaskSetOnlyType(&tmask, rtype);
     }
-#else	/* !NONMANHATTAN */
-
-    ttype = selACarg->ttype;
-    if (type >= DBNumUserLayers)
-    {
-	rmask = DBResidueMask(type);
-	if (TTMaskHasType(rmask, ttype))
-	    type = ttype;
-    }
-    TTMaskSetOnlyType(&tmask, type);
-
-#endif
-
     GeoClip(&selArea, editClip);	/* editClip already in root coords */
 
 splitdone:
