@@ -20,7 +20,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/select/selEnum.c,v 1.20 2010/05/18 14:21:18 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/select/selEnum.c,v 1.13 2010/06/24 12:37:56 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -28,9 +28,9 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/
 
 #include "utils/magic.h"
 #include "utils/geometry.h"
+#include "database/database.h"
 #include "tiles/tile.h"
 #include "utils/hash.h"
-#include "database/database.h"
 #include "windows/windows.h"
 #include "utils/main.h"
 #include "select/select.h"
@@ -38,15 +38,6 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/
 #include "utils/malloc.h"
 #include "textio/textio.h"
 
-/* Linked rectangle structure including tiletype information */
-
-typedef struct extRectList
-{
-    TileType r_type;
-    Rect r_r;
-    struct extRectList *r_next;
-} ExtRectList;
-
 /* Structure passed from top-level enumeration procedures to lower-level
  * ones:
  */
@@ -155,6 +146,7 @@ SelEnumPaint(layers, editOnly, foundNonEdit, func, clientData)
     arg.sea_cdarg = clientData;
     arg.sea_flags = (editOnly) ? SEA_EDIT_ONLY : 0;
     arg.sea_nonEdit = foundNonEdit;
+    arg.sea_rectList = NULL;
     if (foundNonEdit != NULL) *foundNonEdit = FALSE;
 
     /* First, find all the paint in the selection that has the right
@@ -193,7 +185,6 @@ selEnumPFunc1(tile, arg)
 
     TiToRect(tile, &arg->sea_rect);
 
-#ifdef NONMANHATTAN
     if (IsSplit(tile))
     {
 	arg->sea_type = TiGetTypeExact(tile) & (TT_DIAGONAL | TT_SIDE | TT_DIRECTION);
@@ -203,14 +194,11 @@ selEnumPFunc1(tile, arg)
 	    loctype = SplitLeftType(tile);
     }
     else 
-#endif
-    loctype = TiGetType(tile);
+	loctype = TiGetType(tile);
 
-#ifdef NONMANHATTAN
     if (IsSplit(tile))
 	arg->sea_type |= loctype;
     else
-#endif
 	arg->sea_type = loctype;
 
     /* If the paint doesn't have to be in the edit cell, life's pretty
@@ -243,14 +231,12 @@ selEnumPFunc1(tile, arg)
     if (!TTMaskIsZero(&SelectDef->cd_types))
 	arg->sea_flags |= SEA_AREA;
 
-#ifdef NONMANHATTAN
     if (IsSplit(tile))
 	DBSrPaintNMArea((Tile *)NULL,
 		EditCellUse->cu_def->cd_planes[arg->sea_plane],
 		arg->sea_type, &editRect, &DBAllTypeBits, selEnumPFunc2,
 		(ClientData)arg);
     else
-#endif
 	DBSrPaintArea((Tile *)NULL,
 		EditCellUse->cu_def->cd_planes[arg->sea_plane],
 		&editRect, &DBAllTypeBits, selEnumPFunc2,
@@ -287,18 +273,12 @@ selEnumPFunc2(tile, arg)
     int ttype;
     TileType seltype;
 
-#ifdef NONMANHATTAN
     if (IsSplit(tile))
 	ttype = SplitSide(tile) ? SplitRightType(tile) : SplitLeftType(tile);
     else
 	ttype = TiGetTypeExact(tile);
     seltype = arg->sea_type & TT_LEFTMASK;
 
-#else
-    ttype = TiGetType(tile);
-    seltype = arg->sea_type;
-#endif
-
     /* Handle stacked contact types */
     if ((ttype != seltype) && (ttype >= DBNumUserLayers))
     {
@@ -314,20 +294,20 @@ selEnumPFunc2(tile, arg)
     }
 
     /* For chunk selection, the types have to match exactly, so we don't
-     * pick up bits of compatibly material in another cell that were not
+     * pick up bits of compatible material in another cell that were not
      * intended (a chunk can only exist in one cell)
      */
-    if (arg->sea_flags & SEA_CHUNK) 
+    if (arg->sea_flags & SEA_CHUNK)
 	if (ttype != seltype)
 	    return 0;
 
     /* If we selected with "select area <types>", then the type		*/
     /* must belong to <types> (after decomposing stacked types).	*/
-  
+
     if (arg->sea_flags & SEA_AREA)
 	if (ttype != seltype)
 	    if (!TTMaskHasType(&SelectDef->cd_types, ttype))
-		return 0;    
+		return 0;
 
     /* Check for compatible types.  That is:  paint X (in use) over Y	*/
     /* (in select) yields Y means that it is okay to add X to the list.	*/
@@ -335,8 +315,8 @@ selEnumPFunc2(tile, arg)
     /* are X and Y.							*/
 
     if ((ttype != seltype) && (((ttype == TT_SPACE) &&
-	    TTMaskHasType(&DBHomePlaneTypes[arg->sea_plane], seltype))) ||
-	    (DBPaintResultTbl[arg->sea_plane][ttype][seltype] != seltype))
+		TTMaskHasType(&DBHomePlaneTypes[arg->sea_plane], seltype))) ||
+		(DBPaintResultTbl[arg->sea_plane][ttype][seltype] != seltype))
     {
 	TileType chktype = DBPaintResultTbl[arg->sea_plane][ttype][seltype];
 	TileTypeBitMask *cmask = DBResidueMask(chktype);
@@ -373,20 +353,18 @@ selEnumPFunc2(tile, arg)
 	    /* Continue, with the tile type recast as rtype */
 
 	    ttype = rtype;
-    	}
+	}
     }
 
     lr = (ExtRectList *)mallocMagic(sizeof(ExtRectList));
     TiToRect(tile, &lr->r_r);
 
-#ifdef NONMANHATTAN
     if (arg->sea_type & TT_DIAGONAL)
     {
 	Point points[5];
 	int np, i, j;
 	Rect area, orect;
 
-	lr->r_type = ttype | (arg->sea_type & (TT_DIAGONAL | TT_SIDE | TT_DIRECTION));
 	if (!IsSplit(tile))
 	{
 	    /* Things get messy here.  Clip against the select triangle	*/
@@ -487,12 +465,22 @@ topbottom:
 		}
 	    }
 	}
+	else
+	{
+	    /* NOTE:  Need general-purpose triangle-triangle	*/
+	    /* intersection routine here!  However, for most	*/
+	    /* purposes, it suffices to copy the tile if the	*/
+	    /* triangles are similar and to compute the		*/
+	    /* rectangular union are if opposite.		*/
+	    if ((arg->sea_type & TT_SIDE) == (TiGetTypeExact(tile) & TT_SIDE))
+		lr->r_type = ttype | (arg->sea_type &
+			(TT_DIAGONAL | TT_SIDE | TT_DIRECTION));
+	    else
+		lr->r_type = ttype;
+	}
     }
     else
 	lr->r_type = ttype;
-#else
-    lr->r_type = ttype;
-#endif
 
 splitdone:
     lr->r_next = arg->sea_rectList;
@@ -794,6 +782,7 @@ SelEnumLabels(layers, editOnly, foundNonEdit, func, clientData)
     SearchContext scx;
     struct searg arg;
     extern int selEnumLFunc();	/* Forward reference. */
+    extern int selEnumLFunc2();
 
     if (foundNonEdit != NULL) *foundNonEdit = FALSE;
 
@@ -814,12 +803,20 @@ SelEnumLabels(layers, editOnly, foundNonEdit, func, clientData)
 	arg.sea_label = selLabel;
 	arg.sea_foundLabel = NULL;
 	(void) DBTreeSrLabels(&scx, &DBAllTypeBits, 0, (TerminalPath *) NULL,
-	    selEnumLFunc, (ClientData) &arg);
+	    TF_LABEL_ATTACH, selEnumLFunc, (ClientData) &arg);
 	if (arg.sea_foundLabel == NULL)
 	{
-	    TxError("Internal error:  couldn't find selected label %s.\n",
-		selLabel->lab_text);
-	    continue;
+	    /* Take a 2nd try with relaxed criteria.  We may have	*/
+	    /* altered the text or label justification.			*/
+
+	    DBTreeSrLabels(&scx, &DBAllTypeBits, 0, (TerminalPath *) NULL,
+			TF_LABEL_ATTACH, selEnumLFunc2, (ClientData) &arg);
+	    if (arg.sea_foundLabel == NULL)
+	    {
+		TxError("Internal error:  couldn't find selected label %s.\n",
+				selLabel->lab_text);
+		continue;
+	    }
 	}
 
 	/* If only edit-cell labels are wanted, check this label's
@@ -871,3 +868,42 @@ selEnumLFunc(scx, label, tpath, arg)
     arg->sea_foundTrans = scx->scx_trans;
     return 1;
 }
+
+/*
+ * selEnumLFunc2() is like selEnumLFunc(), but it allows one of the justification
+ * or the text string to differ between the source and target labels.  This
+ * prevents magic from failing to find the label if "setlabel text" or
+ * "setlabel just" is applied to the selection.  Both functions are a lousy
+ * substitute for what ought to be an identifier tag. . .
+ */
+
+int
+selEnumLFunc2(scx, label, tpath, arg)
+    SearchContext *scx;		/* Describes current cell for search. */
+    Label *label;		/* Describes label that is in right area
+				 * and has right type.
+				 */
+    TerminalPath *tpath;	/* Ignored. */
+    struct searg *arg;		/* Indicates what we're looking for. */
+{
+    Rect *want, got;
+    int mismatch = 0;
+
+    GeoTransRect(&scx->scx_trans, &label->lab_rect, &got);
+    want = &arg->sea_label->lab_rect;
+    if (want->r_xbot != got.r_xbot) return 0;
+    if (want->r_ybot != got.r_ybot) return 0;
+    if (want->r_xtop != got.r_xtop) return 0;
+    if (want->r_ytop != got.r_ytop) return 0;
+    if (arg->sea_label->lab_just != GeoTransPos(&scx->scx_trans, label->lab_just))
+	mismatch++;
+    if (strcmp(label->lab_text, arg->sea_label->lab_text) != 0)
+	mismatch++;
+
+    if (mismatch == 2) return 0;
+
+    arg->sea_foundLabel = label;
+    arg->sea_foundUse = scx->scx_use;
+    arg->sea_foundTrans = scx->scx_trans;
+    return 1;
+}
