@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/select/selOps.c,v 1.19 2010/08/22 21:58:55 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/select/selOps.c,v 1.11 2010/08/22 21:58:26 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -48,12 +48,12 @@ static int selStretchX, selStretchY;	/* Stretch distances.  Only one should
 					 */
 static TileType selStretchType;		/* Type of material being stretched. */
 
-typedef struct planeArea
+typedef struct planeAndArea
 {
-    int  pa_plane;			/* Plane of interest */
+    int pa_plane;			/* Plane of interest */
     Rect *pa_area;			/* Area affected */
     TileTypeBitMask *pa_mask;		/* Mask used in plane search */
-} planeArea;
+} planeAndArea;
 
 /* The following structure type is used to build up a list of areas
  * to be painted.  It's used to save information while a search of
@@ -126,7 +126,7 @@ SelectDelete(msg, do_clear)
     }
 
     DBReComputeBbox(EditCellUse->cu_def);
-    GeoTransRect(&RootToEditTransform, &SelectDef->cd_bbox, &editArea);
+    GeoTransRect(&RootToEditTransform, &SelectDef->cd_extended, &editArea);
     DBWAreaChanged(EditCellUse->cu_def, &editArea, DBW_ALLWINDOWS,
 	(TileTypeBitMask *) NULL);
     DRCCheckThis(EditCellUse->cu_def, TT_CHECKPAINT, &editArea);
@@ -144,8 +144,6 @@ selDelPaintFunc(rect, type)
     TileTypeBitMask tmask;
     TileType dinfo;
 
-#ifdef NONMANHATTAN
-
     /* Change diagonal side & direction according to the transform */
 
     if (type & TT_DIAGONAL)
@@ -154,7 +152,6 @@ selDelPaintFunc(rect, type)
 	TTMaskSetOnlyType(&tmask, type & TT_LEFTMASK);   
     }
     else
-#endif
     {
 	dinfo = 0;
 	TTMaskSetOnlyType(&tmask, type);   
@@ -191,8 +188,8 @@ int
 selDelLabelFunc(label)
     Label *label;		/* Label to delete. */
 {
-    DBEraseLabelsByContent(EditCellUse->cu_def, &label->lab_rect,
-	    label->lab_just, -1, label->lab_text);
+    DBEraseLabelsByContent(EditCellUse->cu_def, &label->lab_rect, -1,
+		label->lab_text);
     return 0;
 }
 
@@ -300,12 +297,469 @@ SelectFlat()
     SelRememberForUndo(FALSE, SelectRootDef, &SelectUse->cu_bbox);
 
     /* Redisplay the select cell */
-    DBWHLRedraw(SelectRootDef, &SelectDef->cd_bbox, TRUE);
-    DBWAreaChanged(SelectDef, &SelectDef->cd_bbox, DBW_ALLWINDOWS,
+    DBWHLRedraw(SelectRootDef, &SelectDef->cd_extended, TRUE);
+    DBWAreaChanged(SelectDef, &SelectDef->cd_extended, DBW_ALLWINDOWS,
 	&DBAllButSpaceBits);
 
 }
 
+/*
+ * ----------------------------------------------------------------------------
+ *
+ *  selShortFindPath --
+ *
+ *	Trace back through a path found by selShortFindNext from destination
+ *	to source, picking the lowest cost return path, and adding each tile
+ *	found to the linked list.
+ *
+ *	Algorithm notes (for this and selShortFindNext):  Note that by not
+ *	using one of the standard database search routines, TT_SIDE is NOT
+ *	set on any tile.  To find out what side we're looking at, we keep
+ *	a record of what direction we were traveling from the previous
+ *	tile.  Given this and the diagonal direction, we know the two
+ *	valid sides to search.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+int
+selShortFindPath(tile, pnum, rlist, fdir)
+   Tile *tile;
+   int pnum;
+   ExtRectList **rlist;
+   int fdir;
+{
+    Tile *tp, *mintp;
+    // int mincost = (int)tile->ti_client;
+    int mincost = MAXINT;
+    ExtRectList *newrrec;
+    int minp, p, mindir;
+    TileType ttype;
+
+    newrrec = mallocMagic(sizeof(ExtRectList));
+
+    if (IsSplit(tile))
+    {
+	newrrec->r_type = TiGetTypeExact(tile) & ~TT_SIDE;
+	switch(fdir)
+	{
+	    case GEO_NORTH:
+		ttype = SplitBottomType(tile);
+		if (!SplitDirection(tile)) newrrec->r_type |= TT_SIDE;
+		break;
+	    case GEO_SOUTH:
+		ttype = SplitTopType(tile);
+		if (SplitDirection(tile)) newrrec->r_type |= TT_SIDE;
+		break;
+	    case GEO_EAST:
+		ttype = SplitLeftType(tile);
+		break;
+	    case GEO_WEST:
+		ttype = SplitRightType(tile);
+		newrrec->r_type |= TT_SIDE;
+		break;
+	    default:
+		ttype = SplitRightType(tile);
+		if (ttype == TT_SPACE)
+		    ttype = SplitLeftType(tile);
+		else
+		    newrrec->r_type |= TT_SIDE;
+		break;
+	}
+    }
+    else
+    {
+	ttype = TiGetTypeExact(tile);
+	newrrec->r_type = ttype;
+    }
+
+    /* Add this tile (area and type) to the linked list */
+
+    TiToRect(tile, &newrrec->r_r);
+    newrrec->r_next = *rlist;
+    *rlist = newrrec;
+
+    if ((int)tile->ti_client == 0) return 0;	/* We're done */
+    // if (mincost == 0) return 0;		/* We're done */
+    minp = pnum;
+
+    /* Search top */
+    if (IsSplit(tile))
+    {
+	if (fdir == GEO_NORTH) goto leftside;
+	else if (SplitDirection(tile) && fdir == GEO_EAST) goto leftside;
+	else if (!SplitDirection(tile) && fdir == GEO_WEST) goto leftside;
+    }
+
+    for (tp = RT(tile); RIGHT(tp) > LEFT(tile); tp = BL(tp))
+    {
+	if (tp->ti_client == (ClientData)CLIENTDEFAULT) continue;
+	if ((int)tp->ti_client < mincost)
+	{
+	    mincost = (int)tp->ti_client;
+	    mintp = tp;
+	    mindir = GEO_NORTH;
+	}
+    }
+
+    /* Search left */
+leftside:
+    if (IsSplit(tile))
+    {
+	if (fdir == GEO_WEST) goto bottomside;
+	else if (SplitDirection(tile) && fdir == GEO_SOUTH) goto bottomside;
+	else if (!SplitDirection(tile) && fdir == GEO_NORTH) goto bottomside;
+    }
+
+    for (tp = BL(tile); BOTTOM(tp) < TOP(tile); tp = RT(tp))
+    {
+	if (tp->ti_client == (ClientData)CLIENTDEFAULT) continue;
+	if ((int)tp->ti_client < mincost)
+	{
+	    mincost = (int)tp->ti_client;
+	    mintp = tp;
+	    mindir = GEO_WEST;
+	}
+    }
+
+    /* Search bottom */
+bottomside:
+    if (IsSplit(tile))
+    {
+	if (fdir == GEO_SOUTH) goto rightside;
+	else if (SplitDirection(tile) && fdir == GEO_WEST) goto rightside;
+	else if (!SplitDirection(tile) && fdir == GEO_EAST) goto rightside;
+    }
+
+    for (tp = LB(tile); LEFT(tp) < RIGHT(tile); tp = TR(tp))
+    {
+	if (tp->ti_client == (ClientData)CLIENTDEFAULT) continue;
+	if ((int)tp->ti_client < mincost)
+	{
+	    mincost = (int)tp->ti_client;
+	    mintp = tp;
+	    mindir = GEO_SOUTH;
+	}
+    }
+
+    /* Search right */
+rightside:
+    if (IsSplit(tile))
+    {
+	if (fdir == GEO_EAST) goto donesides;
+	else if (SplitDirection(tile) && fdir == GEO_NORTH) goto donesides;
+	else if (!SplitDirection(tile) && fdir == GEO_SOUTH) goto donesides;
+    }
+
+    for (tp = TR(tile); TOP(tp) > BOTTOM(tile); tp = LB(tp))
+    {
+	if (tp->ti_client == (ClientData)CLIENTDEFAULT) continue;
+	if ((int)tp->ti_client < mincost)
+	{
+	    mincost = (int)tp->ti_client;
+	    mintp = tp;
+	    mindir = GEO_EAST;
+	}
+    }
+
+    /* Search other connecting planes */
+donesides:
+    if (DBIsContact(ttype))
+    {
+	PlaneMask pmask;
+
+	pmask = DBConnPlanes[ttype];
+	for (p = PL_TECHDEPBASE; p < DBNumPlanes; p++)
+	{
+	    if (PlaneMaskHasPlane(pmask, p) && (p != pnum))
+	    {
+		tp = SelectDef->cd_planes[p]->pl_hint;
+		GOTOPOINT(tp, &tile->ti_ll);
+		if (tp->ti_client == (ClientData)CLIENTDEFAULT) continue;
+		if ((int)tp->ti_client < mincost)
+		{
+		    mincost = (int)tp->ti_client;
+		    mintp = tp;
+		    minp = p;
+		    mindir = GEO_CENTER;
+		}
+	    }
+	}
+    }
+
+    /* If mincost is still set to MAXINT we have a real serious problem! */
+    if (mincost == MAXINT) return 1;
+
+    /* If no tile had lower cost than this one, then we have an error */
+    // if (mincost == (int)tile->ti_client) return 1;
+
+    /* Stopgap measure:  Error should not happen, but it does!	*/
+    /* Remove client data of current tile and take minimum.	*/
+    if (mincost == (int)tile->ti_client) TiSetClient(tile, CLIENTDEFAULT);
+
+    /* Now we have the minimum cost neighboring tile;  recursively search it */
+
+    return selShortFindPath(mintp, minp, rlist, mindir);
+}
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * selShortFindNext --
+ *
+ *	Recursive function for finding shorts.  This routine makes strong
+ *	assumptions;  namely, that all non-space material in the cell being
+ *	searched belongs to the same net.  The cell searched is always
+ *	SelectDef.
+ *
+ * Results:
+ *	Return 0 to keep going;  return 1 to stop when the tile contains
+ *	the destination point.
+ *
+ * Side effects:
+ *	Each tile visited has its ClientData record set to the current
+ *	cost, in units equal to the steps from the source.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+int
+selShortFindNext(tile, pnum, ldest, cost, best, fdir, mask)
+    Tile *tile;
+    int pnum;
+    Label *ldest;
+    int cost, *best, fdir;
+    TileTypeBitMask *mask;
+{
+    TileType ttype;
+    TileTypeBitMask *lmask;
+    Tile *tp;
+
+    if (IsSplit(tile))
+    {
+	switch(fdir)
+	{
+	    case GEO_NORTH:
+		ttype = SplitBottomType(tile);
+		break;
+	    case GEO_SOUTH:
+		ttype = SplitTopType(tile);
+		break;
+	    case GEO_EAST:
+		ttype = SplitLeftType(tile);
+		break;
+	    case GEO_WEST:
+		ttype = SplitRightType(tile);
+		break;
+	    default:
+		ttype = SplitLeftType(tile);
+		if (ttype == TT_SPACE) ttype = SplitRightType(tile);
+		break;
+	}
+    }
+    else
+	ttype = TiGetTypeExact(tile);
+
+    /* Ignore space tiles */
+    if (ttype == TT_SPACE) return 0;
+
+    /* Ignore non-connecting tiles */
+    if (!TTMaskHasType(mask, ttype)) return 0;
+
+    /* If this tile is unvisited, or has a lower cost, then return and	*/
+    /* keep going.  Otherwise, return 1 to stop the search this direction */
+
+    if (tile->ti_client == (ClientData)CLIENTDEFAULT)
+	TiSetClient(tile, cost);
+    else if ((int)tile->ti_client > cost)
+	TiSetClient(tile, cost);
+    else
+	return 0;
+
+    /* If this tile contains the destination point, do not search further */
+
+    if ((ttype == ldest->lab_type) && EnclosePoint(tile, &ldest->lab_rect.r_ll))
+    {
+	if (*best >= cost) *best = (cost - 1);
+	return 0;
+    }
+
+    /* If we're more costly than the best known path to destination, do	*/
+    /* not search further.						*/
+
+    if (cost >= *best) return 0;
+    lmask = &DBConnectTbl[ttype];
+
+    /* Search top */
+    if (IsSplit(tile))
+    {
+	if (fdir == GEO_NORTH) goto srchleft;
+	else if (SplitDirection(tile) && fdir == GEO_EAST) goto srchleft;
+	else if (!SplitDirection(tile) && fdir == GEO_WEST) goto srchleft;
+    }
+
+    for (tp = RT(tile); RIGHT(tp) > LEFT(tile); tp = BL(tp))
+    {
+	selShortFindNext(tp, pnum, ldest, cost + 1, best, GEO_NORTH, lmask);
+    }
+
+    /* Search left */
+srchleft:
+    if (IsSplit(tile))
+    {
+	if (fdir == GEO_WEST) goto srchbot;
+	else if (SplitDirection(tile) && fdir == GEO_SOUTH) goto srchbot;
+	else if (!SplitDirection(tile) && fdir == GEO_NORTH) goto srchbot;
+    }
+
+    for (tp = BL(tile); BOTTOM(tp) < TOP(tile); tp = RT(tp))
+    {
+	selShortFindNext(tp, pnum, ldest, cost + 1, best, GEO_WEST, lmask);
+    }
+
+    /* Search bottom */
+srchbot:
+    if (IsSplit(tile))
+    {
+	if (fdir == GEO_SOUTH) goto srchright;
+	else if (SplitDirection(tile) && fdir == GEO_WEST) goto srchright;
+	else if (!SplitDirection(tile) && fdir == GEO_EAST) goto srchright;
+    }
+
+    for (tp = LB(tile); LEFT(tp) < RIGHT(tile); tp = TR(tp))
+    {
+	selShortFindNext(tp, pnum, ldest, cost + 1, best, GEO_SOUTH, lmask);
+    }
+
+    /* Search right */
+srchright:
+    if (IsSplit(tile))
+    {
+	if (fdir == GEO_EAST) goto donesrch;
+	else if (SplitDirection(tile) && fdir == GEO_NORTH) goto donesrch;
+	else if (!SplitDirection(tile) && fdir == GEO_SOUTH) goto donesrch;
+    }
+
+    for (tp = TR(tile); TOP(tp) > BOTTOM(tile); tp = LB(tp))
+    {
+	selShortFindNext(tp, pnum, ldest, cost + 1, best, GEO_EAST, lmask);
+    }
+
+    /* Search other connecting planes */
+donesrch:
+    if (DBIsContact(ttype))
+    {
+	PlaneMask pmask;
+	int p;
+
+	pmask = DBConnPlanes[ttype];
+	for (p = PL_TECHDEPBASE; p < DBNumPlanes; p++)
+	{
+	    if (PlaneMaskHasPlane(pmask, p) && (p != pnum))
+	    {
+		tp = SelectDef->cd_planes[p]->pl_hint;
+		GOTOPOINT(tp, &tile->ti_ll);
+		selShortFindNext(tp, p, ldest, cost + 1, best, GEO_CENTER, lmask);
+	    }
+	}
+    }
+    return 0;
+}
+    
+
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * SelectShort --
+ *
+ *	This procedure, given two labels, finds the location of those
+ *	labels in SelectDef.  One is marked as source, the other as
+ *	destination.  Then the SelectDef paint (which is assumed to be
+ *	a net selection that is all local to SelectDef) is recursively
+ *	searched for connecting material.  Each tile ClientData is
+ *	given a cost which is the number of steps from the source.
+ *	At the end, the minimum cost path is traced from the destination
+ * 	back to the source, and the path is saved as a linked list and
+ *	passed back to the calling procedure.
+ *
+ * Results:
+ *	A linked list of tiles representing the connecting path with the
+ *	fewest steps between source and destination.
+ *
+ * Side effects:
+ *	Tile database left with non-default ClientData.  It may be necessary
+ *	to re-run the search routine to return all tiles to the default
+ *	ClientData value.
+ * ----------------------------------------------------------------------------
+ */
+
+ExtRectList *
+SelectShort(char *lab1, char *lab2)
+{
+    Label *selLabel, *srclab = NULL, *destlab = NULL;
+    Tile *tile;
+    Plane *plane;
+    int pnum, best;
+    PlaneMask pmask;
+    ExtRectList *rlist;
+
+    /* Step one: find the tiles containing the labels.  If not found,	*/
+    /* return NULL.							*/
+
+    for (selLabel = SelectDef->cd_labels; selLabel != NULL; selLabel =
+	selLabel->lab_next)
+    {
+	if ((srclab == NULL) && Match(lab1, selLabel->lab_text))
+	    srclab = selLabel;
+
+	if ((destlab == NULL) && Match(lab2, selLabel->lab_text))
+	    destlab = selLabel;
+    }
+
+    /* Must be able to find both labels */
+    if (srclab == NULL || destlab == NULL) return NULL;
+
+    /* Must be able to find tiles associated with each label */
+    
+    pmask = DBTypePlaneMaskTbl[srclab->lab_type];
+    for (pnum = PL_TECHDEPBASE; pnum < DBNumPlanes; pnum++)
+    {
+	if (PlaneMaskHasPlane(pmask, pnum))
+	{
+	    plane = SelectDef->cd_planes[pnum];
+	    tile = plane->pl_hint;
+	    GOTOPOINT(tile, &srclab->lab_rect.r_ll)
+	    if (TiGetType(tile) == srclab->lab_type) break;
+	}
+    }
+    best = MAXINT;
+    selShortFindNext(tile, pnum, &destlab->lab_rect.r_ll, 0, &best, GEO_CENTER,
+		&DBConnectTbl[srclab->lab_type]);
+
+    /* Now see if destination has been counted */
+
+    pmask = DBTypePlaneMaskTbl[destlab->lab_type];
+    for (pnum = PL_TECHDEPBASE; pnum < DBNumPlanes; pnum++)
+    {
+	if (PlaneMaskHasPlane(pmask, pnum))
+	{
+	    plane = SelectDef->cd_planes[pnum];
+	    tile = plane->pl_hint;
+	    GOTOPOINT(tile, &destlab->lab_rect.r_ll)
+	    if (TiGetType(tile) == destlab->lab_type) break;
+	}
+    }
+
+    if (tile->ti_client == (ClientData)CLIENTDEFAULT) return NULL;
+
+    /* Now find the shortest path between source and destination */
+    rlist = NULL;
+    selShortFindPath(tile, pnum, &rlist, GEO_CENTER);
+
+    return rlist;
+}
+
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -358,8 +812,6 @@ selTransPaintFunc(rect, type, transform)
     Rect newarea;
     TileType loctype;
 
-#ifdef NONMANHATTAN
-
     /* Change diagonal direction according to the transform */
     if (type & TT_DIAGONAL)
     {
@@ -368,7 +820,6 @@ selTransPaintFunc(rect, type, transform)
 		(type & TT_LEFTMASK);
     }
     else
-#endif
 	loctype = type;
 
     GeoTransRect(transform, rect, &newarea);
@@ -431,13 +882,22 @@ selTransLabelFunc(label, cellUse, defTransform, transform)
 {
     Rect rootArea, finalArea;
     int rootJust, finalJust;
+    Point rootOffset, finalOffset;
+    int rootRotate, finalRotate;
 
     GeoTransRect(defTransform, &label->lab_rect, &rootArea);
     rootJust = GeoTransPos(defTransform, label->lab_just);
+    GeoTransPointDelta(defTransform, &label->lab_offset, &rootOffset);
+    rootRotate = GeoTransAngle(defTransform, label->lab_rotate);
+
     GeoTransRect(transform, &rootArea, &finalArea);
     finalJust = GeoTransPos(transform, rootJust);
-    (void) DBPutLabel(Select2Def, &finalArea, finalJust, label->lab_text,
-	    label->lab_type, label->lab_flags);
+    GeoTransPointDelta(transform, &rootOffset, &finalOffset);
+    finalRotate = GeoTransAngle(transform, rootRotate);
+
+    (void) DBPutFontLabel(Select2Def, &finalArea, label->lab_font,
+	    label->lab_size, finalRotate, &finalOffset, finalJust,
+	    label->lab_text, label->lab_type, label->lab_flags);
     return 0;
 }
 
@@ -703,8 +1163,10 @@ selArrayLFunc(label, use, transform, arrayInfo)
     Transform *transform;	/* Transform from coords of def to root. */
     ArrayInfo *arrayInfo;	/* How to replicate. */
 {
-    int y, nx, ny, rootJust;
+    int y, nx, ny, rootJust, rootRotate;
+    Point rootOffset;
     Rect original, current;
+    char *astr;
     int labx, laby, xi, yi, only1;	/* numeric parts of label */
 
     char *nmPutNums();			/* Forward reference */
@@ -716,6 +1178,8 @@ selArrayLFunc(label, use, transform, arrayInfo)
 
     GeoTransRect(transform, &label->lab_rect, &original);
     rootJust = GeoTransPos(transform, label->lab_just);
+    rootRotate = GeoTransAngle(transform, label->lab_rotate);
+    GeoTransPointDelta(transform, &label->lab_offset, &rootOffset);
     current = original;
 
     /* get the existing numbers in the label */
@@ -738,11 +1202,11 @@ selArrayLFunc(label, use, transform, arrayInfo)
 	     * the same text and position is considered a duplicate.
 	     */
 	    yi = ny - y;
-	    DBEraseLabelsByContent(Select2Def, &current, rootJust,
-		-1, nmPutNums(label->lab_text, labx + xi, laby +yi));
-	    (void) DBPutLabel(Select2Def, &current, rootJust,
-		nmPutNums(label->lab_text, labx + xi, laby + yi),
-		label->lab_type, label->lab_flags);
+	    astr = nmPutNums(label->lab_text, labx + xi, laby + yi);
+	    DBEraseLabelsByContent(Select2Def, &current, -1, astr);
+	    DBPutFontLabel(Select2Def, &current, label->lab_font,
+			label->lab_size, rootRotate, &rootOffset,
+			rootJust, astr, label->lab_type, label->lab_flags);
 	    current.r_ybot += arrayInfo->ar_ysep;
 	    current.r_ytop += arrayInfo->ar_ysep;
 	    xi += only1;
@@ -798,8 +1262,8 @@ SelectStretch(x, y)
      * displayer and DRC about it later.
      */
 
-    modifiedArea = Select2Def->cd_bbox;
-    (void) GeoInclude(&SelectDef->cd_bbox, &modifiedArea);
+    modifiedArea = Select2Def->cd_extended;
+    (void) GeoInclude(&SelectDef->cd_extended, &modifiedArea);
     GeoTransRect(&RootToEditTransform, &modifiedArea, &editModified);
 
     /* Delete the selection itself. */
@@ -841,14 +1305,10 @@ SelectStretch(x, y)
 	TileTypeBitMask tmask;
 	TileType type = selStretchList->sa_type;
 
-#ifdef NONMANHATTAN
 	TileType tloc;
 	tloc = (type & TT_DIAGONAL) ?  ((type & TT_SIDE) ?
 		(type & TT_RIGHTMASK) >> 14 : (type & TT_LEFTMASK)) : type;
 	TTMaskSetOnlyType(&tmask, tloc);
-#else
-	TTMaskSetOnlyType(&tmask, type);
-#endif
 
 	DBPaintValid(EditCellUse->cu_def, &selStretchList->sa_area, &tmask, type);
 	freeMagic((char *) selStretchList);
@@ -889,20 +1349,18 @@ selStretchEraseFunc(tile, plane)
 {
     Rect area, editArea;
     int planeNum;
-    planeArea pa;
+    planeAndArea pa;
     TileType type, t;
-    TileTypeBitMask tmpmask, mask;
+    TileTypeBitMask tmpmask, mask, *residueMask;
     PaintUndoInfo ui;
     PaintResultType selStretchEraseTbl[NT];
     extern int selStretchEraseFunc2();
 
-#ifdef NONMANHATTAN
     if (IsSplit(tile))
 	type = (SplitSide(tile)) ? SplitRightType(tile) :
 			SplitLeftType(tile);
     else
-#endif
-    type = TiGetType(tile);
+	type = TiGetType(tile);
 
     TiToRect(tile, &area);
 
@@ -925,9 +1383,10 @@ selStretchEraseFunc(tile, plane)
     GeoTransRect(&RootToEditTransform, &area, &editArea);
  
     /* We need to erase all types that interact with "type", *not* all	*/
-    /* types on	"plane", due to the way stacked contacts are handled.	*/
-    /* But contacts on different planes may stretch across one another	*/
-    /* without effect, if the types are stackable.			*/
+    /* types on "plane", due to the way stacked contacts are handled.	*/
+    /* Contacts on different planes may stretch across one another	*/
+    /* without effect, if the types are stackable.  Likewise, layer	*/
+    /* types may stretch across contacts for which they are a residue.	*/
 
     planeNum = *plane;
     mask = DBPlaneTypes[planeNum];
@@ -948,7 +1407,7 @@ selStretchEraseFunc(tile, plane)
 		    TTMaskClearMask(&mask, &tmpmask);
 		}
 	    }
-        }
+	}
     }
 
     /* For stacked type tiles do not erase common residue
@@ -971,7 +1430,7 @@ selStretchEraseFunc(tile, plane)
      */
 
     TTMaskZero(&tmpmask);
-
+    	
     selStretchEraseTbl[TT_SPACE] = (PaintResultType)TT_SPACE;
     for (t = TT_SPACE + 1; t < DBNumUserLayers; t++)
     {
@@ -1011,9 +1470,8 @@ selStretchEraseFunc(tile, plane)
 int
 selStretchEraseFunc2(tile, pa)
     Tile *tile;
-    planeArea *pa;
+    planeAndArea *pa;
 {
-#ifdef NONMANHATTAN
     if (IsSplit(tile))
     {
 	if (TTMaskHasType(pa->pa_mask, TiGetLeftType(tile)))
@@ -1024,13 +1482,11 @@ selStretchEraseFunc2(tile, pa)
 			DBPlaneToResidue(TiGetRightType(tile), pa->pa_plane));
     }
     else
-#endif
 	DBErase(EditCellUse->cu_def, pa->pa_area,
-    		DBPlaneToResidue(TiGetType(tile), pa->pa_plane));
+		DBPlaneToResidue(TiGetType(tile), pa->pa_plane));
     return 0;
 }
 
-
 /*
  * ----------------------------------------------------------------------------
  *	selStretchFillFunc --
@@ -1086,7 +1542,6 @@ selStretchFillFunc(tile, plane)
 	area.r_ytop += 1;
     }
 
-#ifdef NONMANHATTAN
     if (IsSplit(tile))
     {
 	if (selStretchX > 0)
@@ -1102,8 +1557,7 @@ selStretchFillFunc(tile, plane)
 	if (selStretchType == TT_SPACE) return 0;
     }
     else
-#endif
-    selStretchType = TiGetType(tile);
+	selStretchType = TiGetType(tile);
 
     /* The search functions invoked indirectly by the following procedure
      * call build up a list of areas to paint.
@@ -1179,7 +1633,7 @@ selStretchFillFunc3(tile, area)
 {
     Rect editArea, rootArea;
     TileType type, stype;
-    TileTypeBitMask *mask, *smask;
+    TileTypeBitMask *mask, *sMask, *tMask;
     StretchArea *sa;
 
     /* Compute the area to be painted. */
@@ -1213,7 +1667,6 @@ selStretchFillFunc3(tile, area)
      * must use the master image.
      */
     
-#ifdef NONMANHATTAN
     if (IsSplit(tile))
     {
 	if (selStretchX > 0)
@@ -1229,10 +1682,8 @@ selStretchFillFunc3(tile, area)
 	if (type == TT_SPACE) return 0;   /* nothing to stretch */
     }
     else
-#endif
-    type = TiGetType(tile);
-
-    smask = DBResidueMask(selStretchType);
+	type = TiGetType(tile);
+    sMask = DBResidueMask(selStretchType);
 
     /* If have type and stretch type are both contacts (fixed types)	*/
     /* and there exists a stacking contact type between them, then	*/
@@ -1246,7 +1697,7 @@ selStretchFillFunc3(tile, area)
 	/* fill with type.  Otherwise, we check the list of plow	*/
 	/* types to see what is and isn't fixed.			*/
 
-	if (!TTMaskHasType(smask, type))
+	if (!TTMaskHasType(sMask, type))
 	    if (TTMaskHasType(&PlowFixedTypes, type)
 			|| !TTMaskHasType(&PlowFixedTypes, selStretchType))
 		type = selStretchType;
@@ -1266,9 +1717,9 @@ selStretchFillFunc3(tile, area)
 	    ((selStretchY < 0) && (editArea.r_ytop == area->r_ybot)) ||
 	    ((selStretchY > 0) && (editArea.r_ybot == area->r_ytop)))
 	{
-	    TileTypeBitMask rmask, *tmask = DBResidueMask(type);
-	
-	    TTMaskAndMask3(&rmask, tmask, smask);
+	    TileTypeBitMask rmask;
+	    tMask = DBResidueMask(type);
+	    TTMaskAndMask3(&rmask, tMask, sMask);
 	    for (type = TT_TECHDEPBASE; type < DBNumUserLayers; type++)
 		if (TTMaskHasType(&rmask, type))
 		    break;
