@@ -75,9 +75,7 @@ typedef struct
 {
     Rect		area;		/* Area to process */
     TileTypeBitMask	*connectMask;	/* Connection mask for search */
-#ifdef NONMANHATTAN
     TileType		dinfo;		/* Info about triangular search areas */
-#endif
 } conSrArea;
 
 struct conSrArg2
@@ -145,9 +143,7 @@ SimConnectFunc(tile, cx)
     SearchContext	*scx = cx->tc_scx;
     TileTypeBitMask	notConnectMask, *connectMask;
     TileType		loctype, ctype;
-#ifdef NONMANHATTAN
     TileType		dinfo = 0;
-#endif
     int 		i, pNum;
     static char		nodeName[256];
     CellDef 		*def;
@@ -202,7 +198,6 @@ SimConnectFunc(tile, cx)
 
     loctype = TiGetTypeExact(tile);
 
-#ifdef NONMANHATTAN
     /* Resolve geometric transformations on diagonally-split tiles */
    
     if (IsSplit(tile))
@@ -210,7 +205,6 @@ SimConnectFunc(tile, cx)
 	dinfo = DBTransformDiagonal(loctype, &scx->scx_trans);
 	loctype = (SplitSide(tile)) ? SplitRightType(tile) : SplitLeftType(tile);
     }
-#endif
 
     /* See if the destination cell contains stuff over the whole
      * current area (on its home plane) that is connected to it.
@@ -231,9 +225,12 @@ SimConnectFunc(tile, cx)
 	/* Differenct contact types may share residues (6/18/04) */     
 	for (ctype = TT_TECHDEPBASE; ctype < DBNumUserLayers; ctype++)
 	{
-	    cMask = DBResidueMask(ctype);
-	    if (TTMaskIntersect(rMask, cMask))
-		TTMaskSetType(&notConnectMask, ctype);
+	    if (DBIsContact(ctype))
+	    {
+		cMask = DBResidueMask(ctype);
+		if (TTMaskIntersect(rMask, cMask))
+		    TTMaskSetType(&notConnectMask, ctype);
+	    }
 	}
 
 	/* The mask of contact types must include all stacked contacts */
@@ -252,27 +249,15 @@ SimConnectFunc(tile, cx)
     }
 
     def = csa2->csa2_use->cu_def;
-#ifdef NONMANHATTAN
     if (DBSrPaintNMArea((Tile *) NULL, def->cd_planes[pNum],
 		dinfo, &newarea, &notConnectMask, dbcUnconnectFunc,
 		(ClientData) connectMask) == 0)
 	return 0;
-#else
-    if (DBSrPaintArea((Tile *) NULL, def->cd_planes[pNum],
-	    &newarea, &notConnectMask, dbcUnconnectFunc,
-	    (ClientData) connectMask) == 0)
-	return 0;
-#endif
 
     /* Paint this tile into the destination cell. */
 	
-#ifdef NONMANHATTAN
     DBNMPaintPlane(def->cd_planes[pNum], dinfo, &newarea,
 		DBStdPaintTbl(loctype, pNum), (PaintUndoInfo *) NULL);
-#else
-    DBPaintPlane(def->cd_planes[pNum], &newarea,
-		DBStdPaintTbl(loctype, pNum), (PaintUndoInfo *) NULL);
-#endif
 
     /* Since the whole area of this tile hasn't been recorded,
      * we must process its area to find any other tiles that
@@ -283,7 +268,6 @@ SimConnectFunc(tile, cx)
      * even just touch.
      */
 
-#ifdef NONMANHATTAN
     /* Only extend those sides bordering the diagonal tile */
 
     if (dinfo & TT_DIAGONAL)
@@ -300,14 +284,11 @@ SimConnectFunc(tile, cx)
     }
     else
     {
-#endif
-    newarea.r_xbot -= 1;
-    newarea.r_ybot -= 1;
-    newarea.r_xtop += 1;
-    newarea.r_ytop += 1;
-#ifdef NONMANHATTAN
+	newarea.r_xbot -= 1;
+	newarea.r_ybot -= 1;
+	newarea.r_xtop += 1;
+	newarea.r_ytop += 1;
     }
-#endif
 
     /* Abort the name search if the name is in the abort name search table 
      * or if the name is global and the SimIgnoreGlobals flag is not set.
@@ -339,10 +320,7 @@ SimConnectFunc(tile, cx)
 	{
 	    newlist[i].area = csa2->csa2_list[i].area;
 	    newlist[i].connectMask = csa2->csa2_list[i].connectMask;
-#ifdef NONMANHATTAN
 	    newlist[i].dinfo = csa2->csa2_list[i].dinfo;
-#endif
-	    
 	}
 	freeMagic((char *)csa2->csa2_list);
 	csa2->csa2_list = newlist;
@@ -350,10 +328,7 @@ SimConnectFunc(tile, cx)
 
     csa2->csa2_list[csa2->csa2_top].area = newarea;
     csa2->csa2_list[csa2->csa2_top].connectMask = connectMask;
-#ifdef NONMANHATTAN
     csa2->csa2_list[csa2->csa2_top].dinfo = dinfo;
-#endif
-
     return 0;
 }
 
@@ -426,9 +401,7 @@ SimTreeCopyConnect(scx, mask, xMask, connect, area, destUse, Node_Name)
     char pathName[MAXPATHNAME];
     TileTypeBitMask *newmask;
     struct conSrArg2 csa2;
-#ifdef NONMANHATTAN
     TileType newtype;
-#endif
 
     csa2.csa2_use = destUse;
     csa2.csa2_bounds = area;
@@ -451,19 +424,15 @@ SimTreeCopyConnect(scx, mask, xMask, connect, area, destUse, Node_Name)
     {
 	newmask = csa2.csa2_list[csa2.csa2_top].connectMask;
 	scx->scx_area = csa2.csa2_list[csa2.csa2_top].area;
-#ifdef NONMANHATTAN
 	newtype = csa2.csa2_list[csa2.csa2_top].dinfo;
-#endif
 	csa2.csa2_top--;
 
-#ifdef NONMANHATTAN
 	if (newtype & TT_DIAGONAL)
 	    SimTreeSrNMTiles(scx, newtype, newmask, xMask, &tpath,
 			SimConnectFunc, (ClientData) &csa2);
 	else
-#endif
-	SimTreeSrTiles(scx, newmask, xMask, &tpath, SimConnectFunc,
-		(ClientData) &csa2);
+	    SimTreeSrTiles(scx, newmask, xMask, &tpath, SimConnectFunc,
+			(ClientData) &csa2);
     }
     freeMagic((char *)csa2.csa2_list);
 
@@ -751,15 +720,11 @@ SimTreeSrTiles(scx, mask, xMask, tpath, func, cdarg)
     filter.tf_xmask = xMask;
     filter.tf_planes = DBTechTypesToPlanes(mask);
     filter.tf_tpath = tpath;
-#ifdef NONMANHATTAN
     filter.tf_dinfo = 0;
-#endif
 
     return SimCellTileSrFunc(scx, &filter);
 }
 
-#ifdef NONMANHATTAN
-
 /*
  * SimTreeSrNMTiles ---
  *	This is a variant of the above in which the search is over
@@ -805,8 +770,6 @@ SimTreeSrNMTiles(scx, dinfo, mask, xMask, tpath, func, cdarg)
     return SimCellTileSrFunc(scx, &filter);
 }
 
-#endif	/* NONMANHATTAN */
-
 /*
  * Filter procedure applied to subcells by SimTreeSrTiles().
  */
@@ -857,7 +820,6 @@ SimCellTileSrFunc(scx, fp)
     for (pNum = PL_PAINTBASE; pNum < DBNumPlanes; pNum++)
 	if (PlaneMaskHasPlane(fp->tf_planes, pNum))
 	{
-#ifdef NONMANHATTAN
 	    if (fp->tf_dinfo & TT_DIAGONAL)
 	    {
 		TileType dinfo = DBTransformDiagonal(fp->tf_dinfo, &scx->scx_trans);
@@ -870,14 +832,13 @@ SimCellTileSrFunc(scx, fp)
 		}
 	    }
 	    else
-#endif
-	    if (DBSrPaintArea((Tile *) NULL, def->cd_planes[pNum],
-		    &scx->scx_area, fp->tf_mask,
-		    fp->tf_func, (ClientData) &context))
-	    {
-		result = 1;
-		goto cleanup;
-	    }
+		if (DBSrPaintArea((Tile *) NULL, def->cd_planes[pNum],
+			&scx->scx_area, fp->tf_mask,
+			fp->tf_func, (ClientData) &context))
+		{
+		    result = 1;
+		    goto cleanup;
+		}
 	}
 
     /*
@@ -999,7 +960,7 @@ SimPutLabel(cellDef, rect, align, text, type)
     }
     cellDef->cd_lastLabel = lab;
 
-    DBUndoPutLabel(cellDef, rect, align, text, type, 0);
+    DBUndoPutLabel(cellDef, lab);
     return align;
 }
 
