@@ -335,21 +335,18 @@ SimSelectFunc(tile, pHead)
 	return(0);
     }
 
-#ifdef NONMANHATTAN
     if (IsSplit(tile))
     {
 	type = (SplitSide(tile)) ? SplitRightType(tile):
 			SplitLeftType(tile);
     }  
     else
-#endif 
-    type = TiGetTypeExact(tile);
+	type = TiGetTypeExact(tile);
 
     /* get the tile's area, and initialize the tile's search context. */
 
     TITORECT(tile, &scx.scx_area);
 
-#ifdef NONMANHATTAN
     /* make sure that search context isn't placed on an empty	*/
     /* corner of a split tile.					*/
     if (IsSplit(tile))
@@ -359,7 +356,6 @@ SimSelectFunc(tile, pHead)
 	if (!(SplitDirection(tile) ^ SplitSide(tile)))
 	    scx.scx_area.r_ybot = scx.scx_area.r_ytop - 1;
     }
-#endif
 
     scx.scx_area.r_xtop = scx.scx_area.r_xbot + 1;
     scx.scx_area.r_ytop = scx.scx_area.r_ybot + 1;
@@ -820,9 +816,9 @@ SimEraseLabels()
     for (p = SimCellLabList; p; p = p->dl_next) {
 	if (p->dl_isMarked) {
 	    p->dl_isMarked = FALSE;
-	    DBEraseLabelsByContent(p->dl_def, (Rect *)NULL, -1, -1, "RSIM=X");
-	    DBEraseLabelsByContent(p->dl_def, (Rect *)NULL, -1, -1, "RSIM=1");
-	    DBEraseLabelsByContent(p->dl_def, (Rect *)NULL, -1, -1, "RSIM=0");
+	    DBEraseLabelsByContent(p->dl_def, (Rect *)NULL, -1, "RSIM=X");
+	    DBEraseLabelsByContent(p->dl_def, (Rect *)NULL, -1, "RSIM=1");
+	    DBEraseLabelsByContent(p->dl_def, (Rect *)NULL, -1, "RSIM=0");
 	}
     }
 }
