@@ -9,15 +9,13 @@
 # revision C: Adds a layer manager toolbar on the left side
 # revision D: Adds a menubar on top with cell and tech manager tools
 
-global treeViews
 global windowsopen
+global tk_version
 global Glyph
 global Opts
 global Winopts
 
-set treeViews 0
-if {![catch {package require BLT}]} {set treeViews 1}
-
+set tk_version $::tk_version
 # Simple console commands (like TkCon, but much simpler)
 
 if {[lsearch [namespace children] ::tkshell] < 0} {
@@ -92,6 +90,27 @@ proc magic::promptload {type} {
 		magic::load [file root $Layoutfilename]
 	    }
 	  }
+
+      getcell { set Layoutfilename [ tk_getOpenFile -filetypes \
+		{{Magic {.mag {.mag}}} {"All files" {*}}}]
+	    if {$Layoutfilename != ""} {
+		set fdir [file dirname $Layoutfilename]
+		set lidx [lsearch [path search] $fdir]
+		if {$lidx < 0} {path search +$fdir}
+		magic::getcell [file tail $Layoutfilename]
+
+		# Append path to cell search path if it's not there already
+
+		if {[string index $Layoutfilename 0] != "/"} {
+		   set $Layoutfilename "./$Layoutfilename"
+		}
+		set sidx [string last "/" $Layoutfilename]
+		if {$sidx > 0} {
+		   set cellpath [string range $Layoutfilename 0 $sidx]
+		   magic::path cell +$cellpath
+		}
+	    }
+	  }
    }
 }
 
@@ -131,69 +150,6 @@ proc magic::promptsave {type} {
    }
 }
 
-# Callback to the cell manager
-
-proc magic::instcallback {command} {
-   global Opts
-
-   set rpath [join [.cellmgr.box.view get -full anchor]]
-   set rootdef [lindex $rpath 0]
-   set cellpath [lrange $rpath 1 end]
-
-   if { $Opts(target) == "default" } {
-      set winlist [magic::windownames layout]
-      set winname [lindex $winlist 0]
-   } else {
-      set winname $Opts(target)
-   }
-
-   if { $cellpath == {} } {
-      switch $command {
-	 load {$winname load $rootdef}
-	 default {
-	    magic::select top cell
-	    switch $command {
-	       edit {$winname expand; $winname edit}
-	       expand {$winname expand}
-	       zoom {$winname view}
-	    }
-	 }
-      }
-   } else {
-      set celluse [join $cellpath "/"]
-      set celldef [magic::instance list celldef $celluse]
-
-      set curpath [$winname windowcaption]
-      set curname [lindex $curpath 2]
-      set curroot [lindex $curpath 0]
-
-      switch $command {
-         load {$winname load $celldef}
-         default {
-	    # Here: need to check first for the select cell belonging to the
-	    # current loaded root cell (get the first use).
-	    set defpath [list $rootdef]
-	    foreach i $cellpath {
-	       lappend defpath [magic::instance list celldef $i]
-	    }
-	    set rootpos [lsearch $defpath $curroot]
-	    if {$rootpos < 0} {
-	       $winname load $rootdef
-	       set rootpos 0
-	    }
-	    set usepath [join [lrange $cellpath $rootpos end] "/"]
-	    $winname select cell ${usepath}
-
-	    switch $command {
-	       edit {$winname expand; $winname edit}
-	       expand {$winname expand toggle}
-	       zoom {$winname findbox zoom}
-	    }
-	 }
-      }
-   }
-}
-
 # Callback functions used by the DRC.
 
 proc magic::drcupdate { option } {
@@ -212,7 +168,7 @@ proc magic::drcstate { status } {
       set framename [winfo parent $lwin]
       switch $status {
          idle { 
-		set dct [*bypass drc count total]
+		set dct [*bypass drc list count total]
 		if {$dct > 0} {
 	           ${framename}.titlebar.drcbutton configure -selectcolor red
 		} else {
@@ -229,98 +185,16 @@ proc magic::drcstate { status } {
 # and linked into it by the "clone" command.
 
 menu .winmenu -tearoff 0
-# The cell manager
 
-proc magic::makecellmanager { mgrpath } {
-   global treeViews
-
-   if {$treeViews} {
-      toplevel ${mgrpath}
-      wm withdraw ${mgrpath}
-      frame ${mgrpath}.actionbar
-      frame ${mgrpath}.box
-      frame ${mgrpath}.target
-      blt::hiertable ${mgrpath}.box.view -bg white -allowduplicates false \
-		-hideroot true -yscrollcommand "${mgrpath}.box.vert set" \
-		-xscrollcommand "${mgrpath}.box.vert set"
-      scrollbar ${mgrpath}.box.vert -orient vertical -command "${mgrpath}.box.view yview"
-
-      pack ${mgrpath}.actionbar -side top -fill x
-      pack ${mgrpath}.box.view -side left -fill both -expand true
-      pack ${mgrpath}.box.vert -side right -fill y
-      pack ${mgrpath}.box -side top -fill both -expand true
-      pack ${mgrpath}.target -side top -fill x
-
-      button ${mgrpath}.actionbar.done -text "Zoom" -command {magic::instcallback zoom}
-      button ${mgrpath}.actionbar.edit -text "Edit" -command {magic::instcallback edit}
-      button ${mgrpath}.actionbar.load -text "Load" -command {magic::instcallback load}
-      button ${mgrpath}.actionbar.expand -text "Expand" -command \
-		{magic::instcallback expand}
-
-      pack ${mgrpath}.actionbar.load -side left
-      pack ${mgrpath}.actionbar.edit -side left
-      pack ${mgrpath}.actionbar.expand -side left
-      pack ${mgrpath}.actionbar.done -side right
-
-      label ${mgrpath}.target.name -text "Target window:"
-      menubutton ${mgrpath}.target.list -text "default" \
-		-menu ${mgrpath}.target.list.winmenu
- 
-      pack ${mgrpath}.target.name -side left -padx 2
-      pack ${mgrpath}.target.list -side left
-
-      .winmenu clone ${mgrpath}.target.list.winmenu
-   }
-}
-
-proc magic::addlistinstance {instin} {
-   set hierinst [join [lrange $instin 1 end] "/"]
-   set defname [magic::instance list celldef $hierinst]
-   set instnum [lindex [split [lindex $instin end] _ ] end]
-   .cellmgr.box.view insert end $instin -label "${defname} (${instnum})"
-
-   set sublist [magic::instance list children $hierinst]
-   foreach i $sublist {
-      set instout [concat $instin $i]
-      magic::addlistinstance $instout
-   }
-}
-
-proc magic::cellmanager {{option "update"}} {
-   global treeViews
-   if {$treeViews} {
-
-      # Check for existence of the manager widget
-      if {[catch {wm state .cellmgr}]} {
-	 if {$option == "create"} {
-	    magic::makecellmanager .cellmgr
-	 } else {
-	    return
-	 }
-      } elseif { $option == "create"} {
-	 return
-      }
-
-      # determine the full cell heirarchy
-      set tl [magic::cellname list topcells]
-      # remove the previous entry and create a new one
-      .cellmgr.box.view delete all
-      foreach i $tl {
-	 if { [file extension $i] == ".mag" } {
-	    set nameroot [file rootname $i]
-	 } else {
-	    set nameroot $i
-	 }
-	 set nameroot [file tail $nameroot]
-	 .cellmgr.box.view insert end $i -foreground red -label $nameroot
-
-	 set cd [magic::cellname list childinst $i]
-	 foreach j $cd {
-	    set ci [concat $i $j]
-	    magic::addlistinstance $ci
-	 }
-      }
-   }
+proc magic::setgrid {gridsize} {
+   set techlambda [magic::tech lambda]
+   set tech1 [lindex $techlambda 1]
+   set tech0 [lindex $techlambda 0]
+   set tscale [expr {$tech1 / $tech0}]
+   set lambdaout [expr {[magic::cif scale output] * $tscale}]
+   set gridlambda [expr {$gridsize/$lambdaout}]
+   magic::grid ${gridlambda}l
+   magic::snap on
 }
 
 # Technology manager callback functions
@@ -420,10 +294,18 @@ proc magic::maketechmanager { mgrpath } {
    bind ${mgrpath}.lambda1.lval0 <Return> magic::techparseunits
    bind ${mgrpath}.lambda1.lval1 <Return> magic::techparseunits
 
-   wm protocol ${mgrpath} WM_DELETE_WINDOW "set Opts(techmgr) 0; \
-		wm withdraw ${mgrpath}"
+   #Withdraw the window when the close button is pressed
+   wm protocol ${mgrpath} WM_DELETE_WINDOW  "set Opts(techmgr) 0 ; wm withdraw ${mgrpath}"
 }
 
+# Generate the cell manager
+
+catch {source ${CAD_ROOT}/magic/tcl/cellmgr.tcl}
+
+# Generate the text helper
+
+catch {source ${CAD_ROOT}/magic/tcl/texthelper.tcl}
+
 # Create or redisplay the technology manager
 
 proc magic::techmanager {{option "update"}} {
@@ -508,10 +390,10 @@ proc magic::techmanager {{option "update"}} {
    set techstuff [magic::tech version]
    .techmgr.title.tvers configure -text "(version [lindex $techstuff 0])"
    .techmgr.subtitle configure -text [lindex $techstuff 1]
-   set lotext [format "%g" [expr {[magic::cif lambda output] * $tscale}]]
+   set lotext [format "%g" [expr {[magic::cif scale output] * $tscale}]]
    .techmgr.lambda0.lval configure -text $lotext
    .techmgr.cif0.lstyle configure -text [magic::cif list istyle]
-   set litext [format "%g" [expr {[magic::cif lambda input] * $tscale}]]
+   set litext [format "%g" [expr {[magic::cif scale input] * $tscale}]]
    .techmgr.cif0.llambda configure -text $litext
    .techmgr.cif1.lstyle configure -text [magic::cif list ostyle]
    .techmgr.cif1.llambda configure -text $lotext
@@ -532,7 +414,6 @@ proc magic::captions {{subcommand {}}} {
    }
    set winlist [magic::windownames layout]
    foreach winpath $winlist {
-
       set framename [winfo parent $winpath]
       set caption [$winpath windowcaption]
       set subcaption1 [lindex $caption 0]
@@ -556,8 +437,7 @@ proc magic::captions {{subcommand {}}} {
 # Note that the "box" tag doesn't apply to mouse-button events, so this function
 # is duplicated by Tk binding of mouse events in the layout window.
 
-magic::tag load "magic::cellmanager; magic::captions"
-magic::tag getcell "magic::cellmanager"
+magic::tag load "[magic::tag load]; magic::captions"
 magic::tag edit "magic::captions"
 magic::tag save "magic::captions"
 magic::tag down "magic::captions"
@@ -571,19 +451,13 @@ magic::tag see "magic::toolupdate %W %1 %2"
 magic::tag tech "magic::techrebuild %W %1; magic::captions %1"
 magic::tag drc "magic::drcupdate %1"
 magic::tag path "magic::techmanager update"
-if {$treeViews} {
-   magic::tag cellname "magic::mgrupdate %W %1"
-   magic::tag select "magic::mgrselect %R"
-   magic::tag cif "magic::mgrupdate %W %1"
-   magic::tag gds "magic::mgrupdate %W %1"
-} else {
-   magic::tag cellname "magic::captions %1"
-}
+magic::tag cellname "magic::mgrupdate %W %1"
+magic::tag cif      "magic::mgrupdate %W %1"
+magic::tag gds      "magic::mgrupdate %W %1"
 
 # This should be a list. . . do be done later
 set windowsopen 0
 
-set Opts(cellmgr) 0
 set Opts(techmgr) 0
 set Opts(target)  default
 set Opts(netlist) 0
@@ -592,6 +466,21 @@ set Opts(wind3d) 0
 set Opts(crosshair) 0
 set Opts(drc) 1
 
+# Update cell and tech managers in response to a cif or gds read command
+
+proc magic::mgrupdate {win {cmdstr ""}} {
+   if {${cmdstr} == "read"} {
+      catch {magic::cellmanager}
+      magic::captions
+      magic::techmanager update
+   } elseif {${cmdstr} == "delete" || ${cmdstr} == "rename"} {
+      catch {magic::cellmanager}
+      magic::captions
+   } elseif {${cmdstr} == "writeable"} {
+      magic::captions
+   }
+}
+
 # Set default width and height to be 3/4 of the screen size.
 set Opts(geometry) \
 "[expr 3 * [winfo screenwidth .] / 4]x[expr 3 * [winfo screenheight .] \
@@ -735,70 +624,6 @@ proc magic::toolupdate {win {yesno "yes"} {layerlist "none"}} {
    }
 }
 
-# Redirect and reformat Tcl output of "select" command
-
-if {$treeViews} {
-   proc magic::mgrselect {{sstr ""}} {
-      # Make sure we have a valid option, and the cell manager exists.
-      if {$sstr == ""} {
-	 return
-      } elseif {[catch {wm state .cellmgr}]} {
-	 return
-      }
-
-      set savetag [magic::tag select]
-      magic::tag select ""
-      .cellmgr.box.view selection clearall
-      if {[llength $sstr] == 5} {
-	 # sstr is "Topmost cell in the window"
-	 set clist [magic::cellname list window]
-	 set isrch [magic::cellname list instances $clist]
-	 foreach i $isrch {
-	    if {[llength $i] != 5} {
-	       set gsrch $i
-	       break
-	    } else {
-	       set gsrch $clist
-	    }
-         }
-         set idx [lindex [.cellmgr.box.view find -glob -full *${gsrch}*] 0]
-	 set cdef $clist
-      } else {
-	 regsub -all {\[.*\]} $sstr {[^a-z]+} newstr
-         set gsrch [string map {"/" " "} ".*${newstr}\$"]
-	 set cdef [magic::instance list celldef $sstr]
-         set idx [.cellmgr.box.view find -regexp -full "$gsrch"]
-      }
-      if {$idx != "" } {
-         .cellmgr.box.view open $idx
-         if {[wm state .cellmgr] == "normal"} { .cellmgr.box.view see $idx }
-         .cellmgr.box.view selection anchor $idx
-         .cellmgr.box.view selection set $idx
-         if {$sstr != ""} {
-	    puts stdout "Selected cell is $cdef ($sstr)"
-	 }
-      }
-      magic::tag select $savetag
-   }
-}
-
-# Update cell and tech managers in response to a cif or gds read command
-
-proc magic::mgrupdate {win {cmdstr ""}} {
-   global treeViews
-
-   if {${cmdstr} == "read"} {
-      if {$treeViews} { magic::cellmanager }
-      magic::captions
-      magic::techmanager update
-   } elseif {${cmdstr} == "delete" || ${cmdstr} == "rename"} {
-      if {$treeViews} { magic::cellmanager }
-      magic::captions
-   } elseif {${cmdstr} == "writeable"} {
-      magic::captions
-   }
-}
-
 # Generate the toolbar for the wrapper
 
 proc magic::maketoolbar { framename } {
@@ -1072,7 +897,7 @@ proc magic::makescrollbar { fname orient win } {
 
 proc magic::openwrapper {{cell ""} {framename ""}} {
    global windowsopen
-   global treeViews
+   global tk_version
    global Glyph
    global Opts
    global Winopts
@@ -1118,9 +943,15 @@ proc magic::openwrapper {{cell ""} {framename ""}} {
 
    menubutton ${framename}.titlebar.mbuttons.file -text File -relief raised \
 		-menu ${framename}.titlebar.mbuttons.file.filemenu -borderwidth 2
+   menubutton ${framename}.titlebar.mbuttons.edit -text Edit -relief raised \
+		-menu ${framename}.titlebar.mbuttons.edit.editmenu -borderwidth 2
+   menubutton ${framename}.titlebar.mbuttons.view -text View -relief raised \
+		-menu ${framename}.titlebar.mbuttons.view.viewmenu -borderwidth 2
    menubutton ${framename}.titlebar.mbuttons.opts -text Options -relief raised \
 		-menu ${framename}.titlebar.mbuttons.opts.optsmenu -borderwidth 2
    pack ${framename}.titlebar.mbuttons.file -side left
+   pack ${framename}.titlebar.mbuttons.edit -side left
+   pack ${framename}.titlebar.mbuttons.view -side left
    pack ${framename}.titlebar.mbuttons.opts -side left
 
    # DRC status button
@@ -1183,6 +1014,8 @@ proc magic::openwrapper {{cell ""} {framename ""}} {
 		[${winname} cellname list window]"
    $m add command -label "Close" -command "magic::closewrapper ${framename}"
    $m add separator
+   $m add command -label "Place Cell" -command {magic::promptload getcell}
+   $m add separator
    $m add command -label "Load layout" -command {magic::promptload magic}
    $m add command -label "Read CIF" -command {magic::promptload cif}
    $m add command -label "Read GDS" -command {magic::promptload gds}
@@ -1197,6 +1030,49 @@ proc magic::openwrapper {{cell ""} {framename ""}} {
    $m add separator
    $m add command -label "Quit" -command {magic::quit}
 
+   # Generate the edit menu
+
+   set m [menu ${framename}.titlebar.mbuttons.edit.editmenu -tearoff 0]
+   $m add command -label "Undo          (u)" -command {magic::undo}
+   $m add command -label "Redo          (U)" -command {magic::redo}
+   $m add separator
+   $m add command -label "Rotate 90 degree " -command {magic::clock}
+   $m add command -label "Flip Horizontal" -command {magic::sideways}
+   $m add command -label "Flip Vertical"   -command {magic::upsidedown}
+   $m add separator
+   $m add command -label "Text ..." \
+		-command [subst { magic::update_texthelper; \
+		wm deiconify .texthelper ; raise .texthelper } ]
+   $m add separator
+   $m add command -label "Lock   Cell      " -command {magic::instance lock}
+   $m add command -label "Unlock Cell      " -command {magic::instance unlock}
+   $m add separator
+   $m add command -label "Unlock Base Layers" -command {magic::tech layer unlock *}
+
+   # Generate the view menu
+
+   set m [menu ${framename}.titlebar.mbuttons.view.viewmenu -tearoff 0]
+   $m add command -label "Zoom In   " -command {magic::zoom 0.5}
+   $m add command -label "Zoom Out      (Z)" -command {magic::zoom 2}
+   $m add command -label "Zoom Sel      (z)" -command {magic::findbox zoom}
+   $m add command -label "Full" -command {magic::select top cell ; findbox zoom}
+   $m add separator
+   $m add command -label "Grid 0.10u        " -command {magic::setgrid 0.1}
+   $m add command -label "Grid 0.20u        " -command {magic::setgrid 0.2}
+   $m add command -label "Grid 0.25u        " -command {magic::setgrid 0.25}
+   $m add command -label "Grid 0.40u        " -command {magic::setgrid 0.4}
+   $m add command -label "Grid 0.5u        " -command {magic::setgrid 0.5}
+   $m add command -label "Grid 1.0u        " -command {magic::setgrid 1}
+   $m add command -label "Grid 2.0u        " -command {magic::setgrid 2}
+   $m add command -label "Grid 4.0u        " -command {magic::setgrid 4}
+   $m add command -label "Grid 5.0u        " -command {magic::setgrid 5}
+   $m add command -label "Grid 10.0u        " -command {magic::setgrid 10}
+   $m add command -label "Grid 50.0u        " -command {magic::setgrid 50}
+   $m add separator
+   $m add command -label "Expand Toggle (/)" -command {magic::expand toggle}
+   $m add command -label "Expand        (x)" -command {magic::expand }
+   $m add command -label "Unexpand      (X)" -command {magic::unexpand }
+
    # Generate the options menu
 
    set m [menu ${framename}.titlebar.mbuttons.opts.optsmenu -tearoff 0]
@@ -1208,18 +1084,17 @@ proc magic::openwrapper {{cell ""} {framename ""}} {
 		grid forget ${framename}.toolbar } }]
 
    .winmenu add radio -label ${framename} -variable Opts(target) -value ${winname}
-   if {$treeViews} {
-      $m add check -label "Cell Manager" -variable Opts(cellmgr) \
-		-command [subst { magic::cellmanager create; \
-		if { \$Opts(cellmgr) } { \
-		wm deiconify .cellmgr ; raise .cellmgr \
-		} else { \
-		wm withdraw .cellmgr } }]
+   if {$tk_version >= 8.5} {
+     $m add check -label "Cell Manager" -variable Opts(cellmgr) \
+	-command [subst { magic::cellmanager create; \
+	if { \$Opts(cellmgr) } { \
+	   wm deiconify .cellmgr ; raise .cellmgr \
+	} else { \
+	   wm withdraw .cellmgr } }]
       .winmenu entryconfigure last -command ".cellmgr.target.list configure \
-		-text ${framename}"
+         -text ${framename}"
    }
 
-
    $m add check -label "Tech Manager" -variable Opts(techmgr) \
 	-command [subst { magic::techmanager create; \
 		if { \$Opts(techmgr) } { \
@@ -1263,9 +1138,6 @@ proc magic::openwrapper {{cell ""} {framename ""}} {
 
    catch {addmazehelper $m}
 
-   # Update the cell manager (if it exists)
-   magic::cellmanager update
-
    # Set the default view
 
    update idletasks
@@ -1298,21 +1170,22 @@ proc magic::openwrapper {{cell ""} {framename ""}} {
 # Delete the wrapper and the layout window in it.
 
 proc magic::closewrapper { framename } {
-   global treeViews
+   global tk_version
    global Opts
 
-   # Remove this window from the target list in cellmanager
+   # Remove this window from the target list in .winmenu
+   # (used by, e.g., cellmanager)
 
    if { $Opts(target) == "${framename}.magic" } {
       set Opts(target) "default"
-      if {$treeViews} {
-         if {![catch {wm state .cellmgr}]} {
+      if {$tk_version >= 8.5} {
+	 if {![catch {wm state .cellmgr}]} {
 	    .cellmgr.target.list configure -text "default"
-	 }
+         } 
       }
    }
 
-   set idx [.winmenu index $framename ]
+   set idx [.winmenu index $framename]
    .winmenu delete $idx
 
    ${framename}.magic magic::closewindow
