@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) ="$Header: /usr/cvsroot/magic-7.5/textio/txCommands.c,v 1.6 2009/09/11 16:15:05 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) ="$Header: /usr/cvsroot/magic-8.0/textio/txCommands.c,v 1.6 2009/09/11 16:09:49 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -62,7 +62,7 @@ static fd_set txInputDescriptors;
 /* A table of all input devices.
  */
 static txInputDevRec txInputDevice[TX_MAX_OPEN_FILES];
-static txLastInputEntry = -1;
+static int txLastInputEntry = -1;
 
 /* The current point -- reset by the 'setpoint' command and for each
  * interactive command.  Calls to TxClearPoint clear previous setpoints,
