@@ -17,7 +17,7 @@
  */
 
 #ifndef	lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/tiles/tile.c,v 1.8 2008/03/06 03:54:51 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/tiles/tile.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
 #endif	/* not lint */
 
 #include <stdio.h>
@@ -696,11 +696,6 @@ getTileFromTileStore()
     if (TileStoreFreeList)
     {
 	_return_tile = TileStoreFreeList;
-	if (TileStoreFreeList->ti_client == CLIENTDEFAULT)
-	{
-	    TxError("Corrupted tile list at tile 0x%x!\n", TileStoreFreeList);
-	    return NULL;
-	}
 	TileStoreFreeList = (Tile *)TileStoreFreeList->ti_client;
 	return _return_tile;
     }
@@ -816,9 +811,9 @@ void
 tiPrint(tp)
     Tile *tp;
 {
-    printf("tp=0x%x LL=(%d,%d) body=0x%x\n",
-	tp, LEFT(tp), BOTTOM(tp), tp->ti_body);
-    printf("BL=0x%x LB=0x%x RT=0x%x TR=0x%x\n", BL(tp), LB(tp), RT(tp), TR(tp));
+    printf("tp=%p LL=(%d,%d) body=0x%"DLONG_PREFIX"x\n",
+	tp, LEFT(tp), BOTTOM(tp), (dlong) tp->ti_body);
+    printf("BL=%p LB=%p RT=%p TR=%p\n", BL(tp), LB(tp), RT(tp), TR(tp));
 }
 
 void
