@@ -19,7 +19,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/utils/hash.c,v 1.2 2006/04/15 01:05:01 anonymous Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/utils/hash.c,v 1.2 2009/05/13 15:03:18 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -36,7 +36,7 @@ void rebuild();
  * The following defines the ratio of # entries to # buckets
  * at which we rebuild the table to make it larger.
  */
-static rebuildLimit = 3;
+static int rebuildLimit = 3;
 
 /*
  * An invalid pointer, guaranteed to cause a coredump if 
