@@ -19,7 +19,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/utils/main.c,v 1.6 2008/02/07 17:33:00 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/utils/main.c,v 1.2 2008/02/07 17:33:19 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -212,8 +212,7 @@ mainDoArgs(argc, argv)
 
     RCFileName = StrDup((char **) NULL, ".magicrc");
 
-    argc--;
-    while (argc-- > 0)
+    while (--argc > 0)
     {
 	argv++;
 	if (**argv == '-')
