@@ -4,7 +4,7 @@
  *	See malloc.c for a description of magic's allocation functions.
  *	Magic's built-in malloc() function has been removed.
  *
- * rcsid "$Header: /usr/cvsroot/magic-7.5/utils/malloc.h,v 1.2 2006/04/15 01:05:01 anonymous Exp $"
+ * rcsid "$Header: /usr/cvsroot/magic-8.0/utils/malloc.h,v 1.2 2009/09/10 20:32:55 tim Exp $"
  *
  *     ********************************************************************* 
  *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
@@ -22,8 +22,8 @@
 #ifndef _MALLOC_H
 #define _MALLOC_H
 
-extern void *mallocMagic();
-extern void *callocMagic();
-extern void freeMagic();
+extern void *mallocMagic(unsigned int);
+extern void *callocMagic(unsigned int);
+extern void freeMagic(void *);
 
 #endif
