@@ -19,7 +19,7 @@
  */
 
 #ifndef	lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/utils/netlist.c,v 1.2 2006/07/27 16:29:53 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/utils/netlist.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
 #endif	/* lint */
 
 #include <stdio.h>
@@ -378,14 +378,14 @@ NLNetName(net)
     /* Handle case of small integers, for debugging the channel router */
     if (net <= (NLNet *)(&etext))
     {
-	(void) sprintf(tempId, "#%d", net);
+	(void) sprintf(tempId, "#%"DLONG_PREFIX"d", (dlong) net);
 	return tempId;
     }
 
     term = net->nnet_terms;
     if (term == NULL || term->nterm_name == NULL)
     {
-	(void) sprintf(tempId, "[0x%x]", net);
+	(void) sprintf(tempId, "[%p]", net);
 	return tempId;
     }
 
