@@ -17,7 +17,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/utils/path.c,v 1.3 2008/05/02 15:57:00 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/utils/path.c,v 1.3 2009/05/13 15:03:18 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -43,7 +43,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/
  */
 
 static HashTable expansionTable;
-static noTable = TRUE;
+static bool noTable = TRUE;
 
 /* Limit on how long a single file name may be: */
 
