@@ -19,7 +19,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/utils/runstats.c,v 1.2 2006/04/15 01:05:01 anonymous Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/utils/runstats.c,v 1.2 2009/05/13 15:03:18 tim Exp $";
 #endif  /* not lint */
 
 #include <sys/types.h>
@@ -34,11 +34,13 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/
 
 #ifndef __APPLE__
 extern char *sbrk();
-extern end;
+extern int end;
 #else
+#ifndef CYGWIN
 extern void *sbrk();
 int end;
 #endif
+#endif
 
 /*
  * ----------------------------------------------------------------------------
@@ -83,7 +85,7 @@ RunStats(flags, lastt, deltat)
     struct tms buffer;
     static char string[100];
     int umins, usecs, smins, ssecs, udsecs, sdsecs;
-    long size;
+    pointertype size;
     char *sp = string;
 
     *sp = '\0';
@@ -130,13 +132,16 @@ RunStats(flags, lastt, deltat)
 	while (*sp) sp++;
     }
 
+#ifndef CYGWIN
+    // Ignoring this under cygwin instead of trying to find a workaround
     if (flags & RS_MEM)
     {
-	size = (long)(((pointertype)sbrk(0) - (pointertype)(&end)) + 512)/1024;
+	size = (((pointertype)sbrk(0) - (pointertype) &end) + 512)/1024;
 	if (sp != string)
 	    *sp++ = ' ';
-	sprintf(sp, "%ldk", size);
+	sprintf(sp, "%dk", (int)size);
     }
+#endif
 
     return (string);
 }
@@ -204,7 +209,7 @@ RunStatsRealTime()
     while (inct >= 10) { inct -= 10; incs++; }
     while (incs >= 60) { incs -= 60;  incm++; }
 
-    sprintf(buf, "%ld:%02ld.%ld %ld:%02d.%ld", 
+    sprintf(buf, "%ld:%02ld.%ld %ld:%02ld.%ld",
 	totm, tots, tott, incm, incs, inct);
 
     lasttime = curtime;
