@@ -16,7 +16,7 @@
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-7.5/windows/windDisp.c,v 1.1.1.1 2006/04/10 22:03:14 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/windows/windDisp.c,v 1.1.1.1 2008/02/03 20:43:50 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -76,7 +76,8 @@ bool windSomeSeparateRedisplay = FALSE;
  * ----------------------------------------------------------------------------
  */
 
-int windCheckOnlyWindow(MagWindow **w, WindClient client)
+int
+windCheckOnlyWindow(MagWindow **w, WindClient client)
 {
     MagWindow *sw, *tw;
     int wct = 0;
